@@ -13,6 +13,7 @@ merge() {
 }
 
 merge bugfix
+merge testing
 merge development
 merge development-freebsd
 merge development-voidlinux
@@ -20,3 +21,9 @@ merge freebsd
 merge voidlinux
 merge ubuntu
 merge debian-testing
+merge rrr-mysql-5.7
+merge rrr-mysql-8.0
+merge alpine
+
+git checkout master || exit 1
+
