@@ -30,6 +30,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "read_constants.h"
 #include "cmdlineparser/cmdline.h"
 #include "util/linked_list.h"
+#include "rrr_inttypes.h"
 
 #define RRR_ARRAY_VERSION 7
 
@@ -37,9 +38,11 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #define RRR_ARRAY_HARD_ERROR		RRR_READ_HARD_ERROR
 #define RRR_ARRAY_SOFT_ERROR		RRR_READ_SOFT_ERROR
 #define RRR_ARRAY_PARSE_INCOMPLETE	RRR_READ_INCOMPLETE
+#define RRR_ARRAY_ITERATE_STOP	RRR_READ_EOF
 
 struct rrr_map;
 struct rrr_msg_msg;
+struct rrr_nullsafe_str;
 
 struct rrr_array_value_packed {
 	rrr_type type;
@@ -63,6 +66,10 @@ int rrr_array_append_from (
 		struct rrr_array *target,
 		const struct rrr_array *source
 );
+int rrr_array_push_value_vain_with_tag (
+		struct rrr_array *collection,
+		const char *tag
+);
 int rrr_array_push_value_u64_with_tag (
 		struct rrr_array *collection,
 		const char *tag,
@@ -90,6 +97,16 @@ int rrr_array_push_value_blob_with_tag_with_size (
 		const char *value,
 		size_t value_size
 );
+int rrr_array_push_value_blob_with_tag_nullsafe (
+		struct rrr_array *collection,
+		const char *tag,
+		const struct rrr_nullsafe_str *str
+);
+int rrr_array_push_value_str_with_tag_nullsafe (
+		struct rrr_array *collection,
+		const char *tag,
+		const struct rrr_nullsafe_str *str
+);
 int rrr_array_push_value_str_with_tag (
 		struct rrr_array *collection,
 		const char *tag,
@@ -101,9 +118,16 @@ int rrr_array_get_value_unsigned_64_by_tag (
 		const char *tag,
 		int index
 );
+void rrr_array_strip_type (
+		struct rrr_array *collection,
+		const struct rrr_type_definition *definition
+);
 void rrr_array_clear (
 		struct rrr_array *collection
 );
+void rrr_array_clear_void (
+		void *collection
+);
 void rrr_array_clear_by_tag (
 		struct rrr_array *collection,
 		const char *tag
@@ -123,6 +147,13 @@ const struct rrr_type_value *rrr_array_value_get_by_tag_const (
 ssize_t rrr_array_get_packed_length (
 		const struct rrr_array *definition
 );
+int rrr_array_selected_tags_split (
+		int *found_tags,
+		const struct rrr_array *definition,
+		const struct rrr_map *tags,
+		int (*callback)(const struct rrr_type_value *node_orig, const struct rrr_array *node_values, void *arg),
+		void *callback_arg
+);
 int rrr_array_selected_tags_export (
 		char **target,
 		ssize_t *target_size,
@@ -135,9 +166,32 @@ int rrr_array_new_message_from_collection (
 		const struct rrr_array *definition,
 		uint64_t time,
 		const char *topic,
-		ssize_t topic_length
+		rrr_u16 topic_length
+);
+int rrr_array_message_iterate (
+		const struct rrr_msg_msg *message_orig,
+		int (*callback)(RRR_TYPE_RAW_FIELDS, void *arg),
+		void *callback_arg
+);
+int rrr_array_message_iterate_values (
+		const struct rrr_msg_msg *message_orig,
+		int (*callback)(
+				const struct rrr_type_value *value,
+				void *arg
+		),
+		void *callback_arg
+);
+int rrr_array_message_has_tag (
+		const struct rrr_msg_msg *message_orig,
+		const char *tag
+);
+int rrr_array_message_clone_value_by_tag (
+		struct rrr_type_value **target,
+		const struct rrr_msg_msg *message_orig,
+		const char *tag
 );
 int rrr_array_message_append_to_collection (
+		uint16_t *array_version,
 		struct rrr_array *target,
 		const struct rrr_msg_msg *message_orig
 );
