@@ -27,41 +27,37 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "buffer.h"
 #include "log.h"
+#include "allocator.h"
 #include "util/posix.h"
 #include "util/slow_noop.h"
 #include "util/rrr_time.h"
 
+//#define RRR_FIFO_BUFFER_RATELIMIT_DEBUG 1
 //#define RRR_FIFO_BUFFER_DEBUG 1
 
 static inline void rrr_fifo_write_lock(struct rrr_fifo_buffer *buffer) {
-//	printf ("buffer %p write lock wait thread %lu\n", buffer, pthread_self());
 	while (pthread_rwlock_trywrlock(&buffer->rwlock) != 0) {
 		pthread_testcancel();
 		rrr_posix_usleep(10);
 	}
-//	printf ("buffer %p write lock done thread %lu\n", buffer, pthread_self());
 }
 
 static inline int rrr_fifo_write_trylock(struct rrr_fifo_buffer *buffer) {
 	if (pthread_rwlock_trywrlock(&buffer->rwlock) != 0) {
 		return 1;
 	}
-//	printf ("buffer %p write trylock thread %lu\n", buffer, pthread_self());
 	return 0;
 }
 
 static inline void rrr_fifo_read_lock(struct rrr_fifo_buffer *buffer) {
-//	printf ("buffer %p read lock wait thread %lu\n", buffer, pthread_self());
 	while (pthread_rwlock_tryrdlock(&buffer->rwlock) != 0) {
 		pthread_testcancel();
 		rrr_posix_usleep(10);
 	}
-//	printf ("buffer %p read lock done thread %lu\n", buffer, pthread_self());
 }
 
 static inline void rrr_fifo_unlock(struct rrr_fifo_buffer *buffer) {
 	pthread_rwlock_unlock(&buffer->rwlock);
-//	printf ("buffer %p     unlock thread %lu\n", buffer, pthread_self());
 }
 
 static inline void rrr_fifo_unlock_void(void *arg) {
@@ -105,28 +101,28 @@ static int __rrr_fifo_verify_counter(struct rrr_fifo_buffer *buffer) {
 	return (counter != claimed_count);
 }
 
-#define RRR_FIFO_BUFFER_CONSISTENCY_CHECK()				\
-	__rrr_fifo_consistency_check(buffer);				\
-	__rrr_fifo_verify_counter(buffer)
+#define RRR_FIFO_BUFFER_CONSISTENCY_CHECK()                    \
+    __rrr_fifo_consistency_check(buffer);                      \
+    __rrr_fifo_verify_counter(buffer)
 
-#define RRR_FIFO_BUFFER_CONSISTENCY_CHECK_WRITE_LOCK() 	\
-	rrr_fifo_write_lock(buffer);						\
-	RRR_FIFO_BUFFER_CONSISTENCY_CHECK();				\
-	rrr_fifo_unlock(buffer)
+#define RRR_FIFO_BUFFER_CONSISTENCY_CHECK_WRITE_LOCK()         \
+    rrr_fifo_write_lock(buffer);                               \
+    RRR_FIFO_BUFFER_CONSISTENCY_CHECK();                       \
+    rrr_fifo_unlock(buffer)
 
 #else
 
-#define RRR_FIFO_BUFFER_CONSISTENCY_CHECK() \
-	do { } while (0)
-#define RRR_FIFO_BUFFER_CONSISTENCY_CHECK_WRITE_LOCK() \
-	do { } while (0)
+#define RRR_FIFO_BUFFER_CONSISTENCY_CHECK()                    \
+    do { } while (0)
+#define RRR_FIFO_BUFFER_CONSISTENCY_CHECK_WRITE_LOCK()         \
+    do { } while (0)
 
 #endif
 
-#define RRR_FIFO_BUFFER_WITH_STATS_LOCK_DO(action)		\
-	pthread_mutex_lock(&buffer->stats_mutex);					\
-	action;												\
-	pthread_mutex_unlock(&buffer->stats_mutex)
+#define RRR_FIFO_BUFFER_WITH_STATS_LOCK_DO(action)             \
+    pthread_mutex_lock(&buffer->stats_mutex);                  \
+    action;                                                    \
+    pthread_mutex_unlock(&buffer->stats_mutex)
 
 static inline void __rrr_fifo_buffer_stats_add_written (struct rrr_fifo_buffer *buffer, int num) {
 	RRR_FIFO_BUFFER_WITH_STATS_LOCK_DO(buffer->stats.total_entries_written += num);
@@ -136,6 +132,15 @@ static inline void __rrr_fifo_buffer_stats_add_deleted (struct rrr_fifo_buffer *
 	RRR_FIFO_BUFFER_WITH_STATS_LOCK_DO(buffer->stats.total_entries_deleted += num);
 }
 
+void rrr_fifo_buffer_get_stats_populate (
+		struct rrr_fifo_buffer_stats *target,
+		uint64_t entries_written,
+		uint64_t entries_deleted
+) {
+	target->total_entries_written = entries_written;
+	target->total_entries_deleted = entries_deleted;
+}
+
 int rrr_fifo_buffer_get_stats (
 		struct rrr_fifo_buffer_stats *stats,
 		struct rrr_fifo_buffer *buffer
@@ -168,7 +173,7 @@ static void __rrr_fifo_buffer_entry_destroy_unlocked (
 	}
 	__rrr_fifo_buffer_entry_unlock(entry);
 	pthread_mutex_destroy(&entry->lock);
-	free(entry);
+	rrr_free(entry);
 }
 
 static void __rrr_fifo_buffer_entry_destroy_simple_void (
@@ -176,7 +181,7 @@ static void __rrr_fifo_buffer_entry_destroy_simple_void (
 ) {
 	struct rrr_fifo_buffer_entry *entry = ptr;
 	pthread_mutex_destroy(&entry->lock);
-	free(entry);
+	rrr_free(entry);
 }
 
 static void __rrr_fifo_buffer_entry_destroy_data_unlocked (
@@ -206,7 +211,7 @@ static int __rrr_fifo_buffer_entry_new_unlocked (
 
 	*result = NULL;
 
-	struct rrr_fifo_buffer_entry *entry = malloc(sizeof(*entry));
+	struct rrr_fifo_buffer_entry *entry = rrr_allocate(sizeof(*entry));
 	if (entry == NULL) {
 		RRR_MSG_0("Could not allocate entry in __rrr_fifo_buffer_entry_new_unlocked \n");
 		ret = 1;
@@ -226,7 +231,7 @@ static int __rrr_fifo_buffer_entry_new_unlocked (
 	goto out;
 
 	out_free:
-		free(entry);
+		rrr_free(entry);
 	out:
 		return ret;
 }
@@ -265,22 +270,12 @@ static void __rrr_fifo_merge_write_queue_nolock (
 
 		RRR_FIFO_BUFFER_CONSISTENCY_CHECK();
 
-	//	int merge_entries = 0;
-	//  int merge_result = 0;
-
 		__rrr_fifo_buffer_stats_add_written(buffer, buffer->write_queue_entry_count);
 
 		pthread_mutex_lock(&buffer->ratelimit_mutex);
-	//	merge_entries = buffer->write_queue_entry_count;
 		buffer->entry_count += buffer->write_queue_entry_count;
 		buffer->write_queue_entry_count = 0;
-	//	merge_result = buffer->entry_count;
 		pthread_mutex_unlock(&buffer->ratelimit_mutex);
-
-	//	VL_DEBUG_MSG_3("Buffer %p merged %i entries from write queue, buffer size is now %i\n",
-	//			buffer, merge_entries, merge_result);
-	//	VL_DEBUG_MSG_1("Buffer %p merged %i entries from write queue, buffer size is now %i\n",
-	//			buffer, merge_entries, merge_result);
 	}
 
 	pthread_mutex_unlock(&buffer->write_queue_mutex);
@@ -300,7 +295,7 @@ void rrr_fifo_buffer_destroy (
 static void __rrr_fifo_default_free (
 		void *ptr
 ) {
-	free(ptr);
+	rrr_free(ptr);
 }
 
 int rrr_fifo_buffer_init (
@@ -344,6 +339,8 @@ int rrr_fifo_buffer_init (
 	pthread_rwlock_unlock(&buffer->rwlock);
 
 	goto out;
+//	out_destroy_sem:
+//		sem_destroy(&buffer->new_data_available);
 	out_destroy_stats_mutex:
 		pthread_mutex_destroy(&buffer->stats_mutex);
 	out_destroy_ratelimit_mutex:
@@ -421,8 +418,6 @@ static inline int rrr_fifo_wait_for_data (
 		return 0;
 	}
 
-//	printf ("Waiting for %u milliseconds\n", wait_milliseconds);
-
 	uint64_t time_start = rrr_time_get_64();
 	uint64_t time_end = time_start + (wait_milliseconds * 1000);
 
@@ -434,17 +429,6 @@ static inline int rrr_fifo_wait_for_data (
 	wait_time.tv_nsec = microseconds * 1000;
 	int res = sem_timedwait(&buffer->new_data_available, &wait_time);
 
-/*	uint64_t time_end_real = time_get_64();
-
-	printf ("Waiting time was %" PRIu64 " result was %i\n", (time_end_real - time_start) / 1000, res);*/
-/*	if (res != 0) {
-		char buf[1024];
-		buf[0] = '\0';
-		strerror_r(errno, buf, sizeof(buf));
-		VL_MSG_0("Could wait on semaphore in buffer: %s\n", buf);
-		VL_MSG_0("Start time was %" PRIu64 " end time was %" PRIu64 "\n", time_start, time_end);
-	}
-*/
 	return res;
 }
 
@@ -474,7 +458,7 @@ void rrr_fifo_buffer_clear_with_callback (
 	int freed_counter = 0;
 	while (entry != NULL) {
 		struct rrr_fifo_buffer_entry *next = entry->next;
-		RRR_DBG_4 ("Buffer %p free entry %p with data %p order %" PRIu64 "\n", buffer, entry, entry->data, entry->order);
+		RRR_DBG_4 ("buffer %p free entry %p with data %p order %" PRIu64 "\n", buffer, entry, entry->data, entry->order);
 
 		__rrr_fifo_buffer_entry_lock(entry);
 		pthread_cleanup_push(__rrr_fifo_buffer_entry_unlock_void, entry);
@@ -493,7 +477,7 @@ void rrr_fifo_buffer_clear_with_callback (
 
 	__rrr_fifo_buffer_stats_add_deleted(buffer, freed_counter);
 
-	RRR_DBG_4 ("Buffer %p freed %i entries\n", buffer, freed_counter);
+	RRR_DBG_4 ("buffer %p freed %i entries\n", buffer, freed_counter);
 
 	buffer->gptr_first = NULL;
 	buffer->gptr_last = NULL;
@@ -510,6 +494,47 @@ void rrr_fifo_buffer_clear (
 	rrr_fifo_buffer_clear_with_callback(buffer, NULL, NULL);
 }
 
+// TODO : Use this in the search function
+int rrr_fifo_buffer_search_return_value_process (
+		unsigned char *do_keep,
+		unsigned char *do_give,
+		unsigned char *do_free,
+		unsigned char *do_stop,
+		int actions
+) {
+	int err = RRR_FIFO_OK;
+
+	*do_keep = 0;
+	*do_give = 0;
+	*do_free = 0;
+	*do_stop = 0;
+
+	if (actions == RRR_FIFO_SEARCH_KEEP) { // Just a 0
+		*do_keep = 1;
+		goto out;
+	}
+	if ((actions & RRR_FIFO_CALLBACK_ERR) != 0) {
+		err = RRR_FIFO_CALLBACK_ERR;
+		goto out;
+	}
+	if ((actions & RRR_FIFO_SEARCH_GIVE) != 0) {
+		*do_give = 1;
+		if ((actions & RRR_FIFO_SEARCH_FREE) != 0) {
+			*do_free = 1;
+		}
+	}
+	if ((actions & RRR_FIFO_SEARCH_STOP) != 0) {
+		*do_stop = 1;
+	}
+
+	if (*do_free == 0 && *do_stop == 0) {
+		RRR_BUG("BUG: Unknown return values to rrr_fifo_buffer_search_return_value_process\n");
+	}
+
+	out:
+	return err;
+}
+
 /*
  * Search entries and act according to the return value of the callback function. We
  * can delete entries or stop looping. See buffer.h . The callback function is expected
@@ -522,8 +547,10 @@ int rrr_fifo_buffer_search (
 		void *callback_data,
 		unsigned int wait_milliseconds
 ) {
-	__rrr_fifo_attempt_write_queue_merge(buffer);
-	rrr_fifo_wait_for_data(buffer, wait_milliseconds);
+	int combined_count = rrr_fifo_buffer_get_entry_count_combined(buffer);
+	if (combined_count == 0) {
+		rrr_fifo_wait_for_data(buffer, wait_milliseconds);
+	}
 
 	int err = 0;
 
@@ -538,7 +565,7 @@ int rrr_fifo_buffer_search (
 	struct rrr_fifo_buffer_entry *next;
 	struct rrr_fifo_buffer_entry *prev = NULL;
 	for (entry = buffer->gptr_first; entry != NULL; entry = next) {
-		RRR_DBG_4("Buffer %p search loop entry %p next %p prev %p\n", buffer, entry, entry->next, prev);
+		RRR_DBG_4("buffer %p search loop entry %p next %p prev %p\n", buffer, entry, entry->next, prev);
 		next = entry->next;
 
 		int did_something = 0;
@@ -568,8 +595,6 @@ int rrr_fifo_buffer_search (
 
 			cleared_entries++;
 
-//			RRR_DBG_4("Buffer %p free entry %p after GIVE command\n", buffer, entry);
-
 			// If we are not asked to free, zero out the pointer to stop it from being
 			// destroyed by entry destroy functions
 			if ((actions & RRR_FIFO_SEARCH_FREE) == 0) {
@@ -621,42 +646,44 @@ static int __rrr_fifo_buffer_write_callback_return_check (
 	*do_ordered_write = 0;
 	*do_drop = 0;
 
+	if (ret_to_check == 0) {
+		goto out;
+	}
+
+	if ((ret_to_check & RRR_FIFO_WRITE_ORDERED) == RRR_FIFO_WRITE_ORDERED) {
+		if ((ret_to_check & ~(RRR_FIFO_WRITE_AGAIN|RRR_FIFO_WRITE_ORDERED|RRR_FIFO_WRITE_DROP)) != 0) {
+			RRR_BUG("BUG: Callback return WRITE_ORDERED along with other illegal return values %i in __rrr_fifo_buffer_write_callback_return_check\n", ret_to_check);
+		}
+		*do_ordered_write = 1;
+	}
+
+	if ((ret_to_check & RRR_FIFO_WRITE_AGAIN) == RRR_FIFO_WRITE_AGAIN) {
+		if ((ret_to_check & ~(RRR_FIFO_WRITE_AGAIN|RRR_FIFO_WRITE_ORDERED|RRR_FIFO_WRITE_DROP)) != 0) {
+			RRR_BUG("BUG: Callback return WRITE_AGAIN along with other illegal return values %i in __rrr_fifo_buffer_write_callback_return_check\n", ret_to_check);
+		}
+		*write_again = 1;
+	}
+
+	if ((ret_to_check & RRR_FIFO_GLOBAL_ERR) == RRR_FIFO_GLOBAL_ERR) {
+		if ((ret_to_check & ~(RRR_FIFO_GLOBAL_ERR)) != 0) {
+			RRR_BUG("BUG: Callback returned GLOBAL_ERR along with return values %i in __rrr_fifo_buffer_write_callback_return_check\n", ret_to_check);
+		}
+		ret = 1;
+		goto out;
+	}
+
+	if ((ret_to_check & RRR_FIFO_WRITE_DROP) == RRR_FIFO_WRITE_DROP) {
+		if ((ret_to_check &= ~(RRR_FIFO_WRITE_DROP|RRR_FIFO_WRITE_AGAIN)) != 0) {
+			RRR_BUG("BUG: Callback returned WRITE_DROP along with return values %i in __rrr_fifo_buffer_write_callback_return_check\n", ret_to_check);
+		}
+		*do_drop = 1;
+		goto out;
+	}
+
+	ret_to_check &= ~(RRR_FIFO_WRITE_AGAIN|RRR_FIFO_WRITE_ORDERED|RRR_FIFO_WRITE_DROP);
+
 	if (ret_to_check != 0) {
-		if ((ret_to_check & RRR_FIFO_WRITE_ORDERED) == RRR_FIFO_WRITE_ORDERED) {
-			if ((ret_to_check & ~(RRR_FIFO_WRITE_AGAIN|RRR_FIFO_WRITE_ORDERED|RRR_FIFO_WRITE_DROP)) != 0) {
-				RRR_BUG("BUG: Callback return WRITE_ORDERED along with other illegal return values %i in __rrr_fifo_buffer_write_callback_return_check\n", ret_to_check);
-			}
-			*do_ordered_write = 1;
-		}
-
-		if ((ret_to_check & RRR_FIFO_WRITE_AGAIN) == RRR_FIFO_WRITE_AGAIN) {
-			if ((ret_to_check & ~(RRR_FIFO_WRITE_AGAIN|RRR_FIFO_WRITE_ORDERED|RRR_FIFO_WRITE_DROP)) != 0) {
-				RRR_BUG("BUG: Callback return WRITE_AGAIN along with other illegal return values %i in __rrr_fifo_buffer_write_callback_return_check\n", ret_to_check);
-			}
-			*write_again = 1;
-		}
-
-		if ((ret_to_check & RRR_FIFO_GLOBAL_ERR) == RRR_FIFO_GLOBAL_ERR) {
-			if ((ret_to_check & ~(RRR_FIFO_GLOBAL_ERR)) != 0) {
-				RRR_BUG("BUG: Callback returned GLOBAL_ERR along with return values %i in __rrr_fifo_buffer_write_callback_return_check\n", ret_to_check);
-			}
-			ret = 1;
-			goto out;
-		}
-
-		if ((ret_to_check & RRR_FIFO_WRITE_DROP) == RRR_FIFO_WRITE_DROP) {
-			if ((ret_to_check &= ~(RRR_FIFO_WRITE_DROP|RRR_FIFO_WRITE_AGAIN)) != 0) {
-				RRR_BUG("BUG: Callback returned WRITE_DROP along with return values %i in __rrr_fifo_buffer_write_callback_return_check\n", ret_to_check);
-			}
-			*do_drop = 1;
-			goto out;
-		}
-
-		ret_to_check &= ~(RRR_FIFO_WRITE_AGAIN|RRR_FIFO_WRITE_ORDERED|RRR_FIFO_WRITE_DROP);
-
-		if (ret_to_check != 0) {
-			RRR_BUG("Unknown return values %i from callback in __rrr_fifo_buffer_write_callback_return_check\n", ret_to_check);
-		}
+		RRR_BUG("Unknown return values %i from callback in __rrr_fifo_buffer_write_callback_return_check\n", ret_to_check);
 	}
 
 	out:
@@ -788,8 +815,10 @@ int rrr_fifo_buffer_search_and_replace (
 		unsigned int wait_milliseconds,
 		int call_again_after_looping
 ) {
-	__rrr_fifo_attempt_write_queue_merge(buffer);
-	rrr_fifo_wait_for_data(buffer, wait_milliseconds);
+	int combined_count = rrr_fifo_buffer_get_entry_count_combined(buffer);
+	if (combined_count == 0) {
+		rrr_fifo_wait_for_data(buffer, wait_milliseconds);
+	}
 
 	int ret = 0;
 
@@ -805,7 +834,7 @@ int rrr_fifo_buffer_search_and_replace (
 	struct rrr_fifo_buffer_entry *next;
 	struct rrr_fifo_buffer_entry *prev = NULL;
 	for (entry = buffer->gptr_first; entry != NULL; entry = next) {
-		RRR_DBG_4("Buffer %p search_and_replace loop entry %p data %p next %p prev %p\n",
+		RRR_DBG_4("buffer %p search_and_replace loop entry %p data %p next %p prev %p\n",
 				buffer, entry, entry->data, entry->next, prev);
 		next = entry->next;
 
@@ -870,8 +899,6 @@ int rrr_fifo_buffer_search_and_replace (
 				new_entries++;
 			}
 
-//			RRR_DBG_4("Buffer %p free entry %p after GIVE command\n", buffer, entry);
-
 			entry = prev;
 			did_something = 1;
 		}
@@ -909,83 +936,16 @@ int rrr_fifo_buffer_search_and_replace (
 	return ret;
 }
 
-/*
- * Delete entries with and order value < order_min. We assume the buffer is
- * already ordered by using fifo_buffer_write_ordered writes only.
- */
-int rrr_fifo_buffer_clear_order_lt (
-		struct rrr_fifo_buffer *buffer,
-		uint64_t order_min
-) {
-	rrr_fifo_write_lock(buffer);
-	pthread_cleanup_push(rrr_fifo_unlock_void, buffer);
-
-	__rrr_fifo_merge_write_queue_nolock(buffer);
-
-	int cleared_entries = 0;
-
-	struct rrr_fifo_buffer_entry *entry;
-	struct rrr_fifo_buffer_entry *clear_end = NULL;
-	for (entry = buffer->gptr_first; entry != NULL; entry = entry->next){
-		if (entry->order < order_min) {
-			// All entries up to here are to be cleared
-			clear_end = entry;
-			cleared_entries++;
-		}
-		else {
-			break;
-		}
-	}
-
-	if (clear_end) {
-		struct rrr_fifo_buffer_entry *clear_start = buffer->gptr_first;
-		struct rrr_fifo_buffer_entry *clear_stop = clear_end->next;
-		buffer->gptr_first = clear_end->next;
-
-		if (clear_end->next == NULL) {
-			// We are clearing the whole buffer
-			buffer->gptr_last = NULL;
-			buffer->gptr_first = NULL;
-		}
-
-		struct rrr_fifo_buffer_entry *next;
-		for (entry = clear_start; entry != clear_stop; entry = next) {
-			next = entry->next;
-
-			RRR_DBG_4 ("Buffer free entry %p in ordered clear with data %p order %" PRIu64 "\n", entry, entry->data, entry->order);
-
-			__rrr_fifo_buffer_entry_destroy_unlocked(buffer, entry);
-		}
-	}
-
-	RRR_FIFO_BUFFER_CONSISTENCY_CHECK();
-
-	__rrr_fifo_buffer_stats_add_deleted(buffer, cleared_entries);
-
-	pthread_mutex_lock(&buffer->ratelimit_mutex);
-	buffer->entry_count -= cleared_entries;
-	pthread_mutex_unlock(&buffer->ratelimit_mutex);
-
-	if (buffer->gptr_first != NULL) {
-		__rrr_fifo_buffer_set_data_available(buffer);
-	}
-
-	pthread_cleanup_pop(1);
-	return 0;
-}
-
-/*
- * This reading method holds a write lock for a minimum amount of time by
- * taking control of the start of the queue making it inaccessible to
- * others. The callback function must store the data pointer or free it.
- */
-int rrr_fifo_buffer_read_clear_forward (
+static int __rrr_fifo_buffer_read_clear_forward (
 		struct rrr_fifo_buffer *buffer,
 		int (*callback)(void *callback_data, char *data, unsigned long int size),
 		void *callback_data,
 		unsigned int wait_milliseconds
 ) {
-	rrr_fifo_wait_for_data(buffer, wait_milliseconds);
+	int combined_count = rrr_fifo_buffer_get_entry_count_combined(buffer);
+	if (combined_count == 0) {
+		rrr_fifo_wait_for_data(buffer, wait_milliseconds);
+	}
 
 	int ret = RRR_FIFO_OK;
 
@@ -1035,20 +995,26 @@ int rrr_fifo_buffer_read_clear_forward (
 
 		int ret_tmp = 0;
 
-		// Don't access entry pointers outside lock
 		{
+			// Don't access entry pointers outside lock
+			// Also, don't hold read lock while in callback
 			rrr_fifo_read_lock(buffer);
-			pthread_cleanup_push(rrr_fifo_unlock_void, buffer);
-
-			__rrr_fifo_buffer_entry_lock(current);
-			pthread_cleanup_push(__rrr_fifo_buffer_entry_unlock_void, current);
 
 			next = current->next;
 
-			ret_tmp = callback(callback_data, current->data, current->size);
+			unsigned long int size = current->size;
+			char *data = current->data;
 
-			pthread_cleanup_pop(1);
-			pthread_cleanup_pop(1);
+			rrr_fifo_unlock(buffer); 
+
+			{
+				__rrr_fifo_buffer_entry_lock(current);
+				pthread_cleanup_push(__rrr_fifo_buffer_entry_unlock_void, current);
+
+				ret_tmp = callback(callback_data, data, size);
+
+				pthread_cleanup_pop(1);
+			}
 		}
 
 		processed_entries++;
@@ -1071,7 +1037,15 @@ int rrr_fifo_buffer_read_clear_forward (
 			if ((ret_tmp & (RRR_FIFO_SEARCH_GIVE)) != 0) {
 				RRR_BUG("Bug: FIFO_SEARCH_GIVE returned to fifo_read_clear_forward, we always GIVE by default\n");
 			}
-			if ((ret_tmp & RRR_FIFO_SEARCH_STOP) != 0) {
+			if ((ret_tmp & RRR_FIFO_CALLBACK_ERR) != 0) {
+				// Callback will free the memory also on error, unless FIFO_SEARCH_FREE is specified
+				ret |= RRR_FIFO_CALLBACK_ERR;
+			}
+			if ((ret_tmp & RRR_FIFO_GLOBAL_ERR) != 0) {
+				// Callback will free the memory also on error, unless FIFO_SEARCH_FREE is specified
+				ret |= RRR_FIFO_GLOBAL_ERR;
+			}
+			if ((ret_tmp & (RRR_FIFO_SEARCH_STOP|RRR_FIFO_CALLBACK_ERR|RRR_FIFO_GLOBAL_ERR)) != 0) {
 				// Stop processing and put the rest back into the buffer
 				{
 					rrr_fifo_write_lock(buffer);
@@ -1097,14 +1071,6 @@ int rrr_fifo_buffer_read_clear_forward (
 
 				break;
 			}
-			if ((ret_tmp & RRR_FIFO_CALLBACK_ERR) != 0) {
-				// Callback will free the memory also on error, unless FIFO_SEARCH_FREE is specified
-				ret = RRR_FIFO_CALLBACK_ERR;
-			}
-			if ((ret_tmp & RRR_FIFO_GLOBAL_ERR) != 0) {
-				// Callback will free the memory also on error, unless FIFO_SEARCH_FREE is specified
-				ret = RRR_FIFO_GLOBAL_ERR;
-			}
 			ret_tmp &= ~(RRR_FIFO_SEARCH_GIVE|RRR_FIFO_SEARCH_FREE|RRR_FIFO_SEARCH_STOP|RRR_FIFO_CALLBACK_ERR|RRR_FIFO_GLOBAL_ERR);
 			if (ret_tmp != 0) {
 				RRR_BUG("Unknown flags %i returned to fifo_read_clear_forward\n", ret_tmp);
@@ -1130,13 +1096,49 @@ int rrr_fifo_buffer_read_clear_forward (
 
 #ifdef FIFO_DEBUG_COUNTER
 	if (fifo_verify_counter(buffer) != 0) {
-		RRR_BUG("Buffer size mismatch\n");
+		RRR_BUG("BUG: buffer size mismatch\n");
 	}
 #endif /* FIFO_DEBUG_COUNTER */
 
 	return ret;
 }
 
+/*
+ * This reading method holds a write lock for a minimum amount of time by
+ * taking control of the start of the queue making it inaccessible to
+ * others. The callback function must store the data pointer or free it.
+ * Reads at most RRR_FIFO_MAX_READS entries.
+ */
+int rrr_fifo_buffer_read_clear_forward (
+		struct rrr_fifo_buffer *buffer,
+		int (*callback)(void *callback_data, char *data, unsigned long int size),
+		void *callback_data,
+		unsigned int wait_milliseconds
+) {
+	return __rrr_fifo_buffer_read_clear_forward(buffer, callback, callback_data, wait_milliseconds);
+}
+/*
+ * Same as rrr_fifo_buffer_read_clear_forward(), but reads all entries.
+ */
+int rrr_fifo_buffer_read_clear_forward_all (
+		struct rrr_fifo_buffer *buffer,
+		int (*callback)(void *callback_data, char *data, unsigned long int size),
+		void *callback_data
+) {
+	int ret = RRR_FIFO_OK;
+	int entry_count = 0;
+
+	do {
+		ret = __rrr_fifo_buffer_read_clear_forward(buffer, callback, callback_data, 0);
+		
+		pthread_mutex_lock(&buffer->ratelimit_mutex);
+		entry_count = buffer->entry_count;
+		pthread_mutex_unlock(&buffer->ratelimit_mutex);
+	} while (ret == 0 && entry_count > 0);
+
+	return ret;
+}
+
 /*
  * This reading method blocks writers but allow other readers to traverse at the
  * same time. The callback function must not free the data or store it's pointer.
@@ -1148,7 +1150,11 @@ int rrr_fifo_buffer_read (
 		void *callback_data,
 		unsigned int wait_milliseconds
 ) {
-	rrr_fifo_wait_for_data(buffer, wait_milliseconds);
+	int combined_count = rrr_fifo_buffer_get_entry_count_combined(buffer);
+	if (combined_count == 0) {
+		rrr_fifo_wait_for_data(buffer, wait_milliseconds);
+	}
+
 	__rrr_fifo_attempt_write_queue_merge(buffer);
 
 	int ret = RRR_FIFO_OK;
@@ -1214,7 +1220,11 @@ int rrr_fifo_buffer_read_minimum (
 		uint64_t minimum_order,
 		unsigned int wait_milliseconds
 ) {
-	rrr_fifo_wait_for_data(buffer, wait_milliseconds);
+	int combined_count = rrr_fifo_buffer_get_entry_count_combined(buffer);
+	if (combined_count == 0) {
+		rrr_fifo_wait_for_data(buffer, wait_milliseconds);
+	}
+
 	__rrr_fifo_attempt_write_queue_merge(buffer);
 
 	int res = 0;
@@ -1271,6 +1281,10 @@ static void __rrr_fifo_buffer_do_ratelimit(struct rrr_fifo_buffer *buffer) {
 		return;
 	}
 
+#ifdef RRR_FIFO_BUFFER_RATELIMIT_DEBUG
+	uint64_t ratelimit_in = rrr_time_get_64();
+#endif
+
 	pthread_mutex_lock(&buffer->ratelimit_mutex);
 
 	struct rrr_fifo_buffer_ratelimit *ratelimit = &buffer->ratelimit;
@@ -1319,12 +1333,6 @@ static void __rrr_fifo_buffer_do_ratelimit(struct rrr_fifo_buffer *buffer) {
 		}
 		pthread_mutex_lock(&buffer->ratelimit_mutex);
 
-/*		printf ("Spin time %lu rounds %llu usleep %lu\n",
-				time_diff,
-				spin_time_orig,
-				do_usleep
-		);*/
-
 		long long int current_spins_per_us = spin_time_orig / time_diff;
 
 		if (ratelimit->spins_per_us == 0) {
@@ -1334,10 +1342,16 @@ static void __rrr_fifo_buffer_do_ratelimit(struct rrr_fifo_buffer *buffer) {
 			// Give little weight to the new value when updating
 			ratelimit->spins_per_us = (ratelimit->spins_per_us * 9 + current_spins_per_us) / 10;
 		}
-//		VL_DEBUG_MSG_1("spintime %llu spins per us %llu\n", time_diff, buffer->spins_per_us);
 	}
 
 	pthread_mutex_unlock(&buffer->ratelimit_mutex);
+
+#ifdef RRR_FIFO_BUFFER_RATELIMIT_DEBUG
+	uint64_t time = rrr_time_get_64() - ratelimit_in;
+	if (time > 0) {
+		printf("Ratelimit %p: %" PRIu64 "\tc: %i\n", buffer, time, buffer->entry_count);
+	}
+#endif
 }
 
 static void __rrr_fifo_buffer_update_ratelimit(struct rrr_fifo_buffer *buffer) {
@@ -1384,7 +1398,7 @@ static void __rrr_fifo_buffer_update_ratelimit(struct rrr_fifo_buffer *buffer) {
 
 	unsigned long long int spintime_us = (ratelimit->sleep_spin_time / (ratelimit->spins_per_us + 1));
 
-	RRR_DBG_4("Buffer %p read/write balance %f spins %llu (%llu us) spins/us %llu entries %i (do sleep = %i)\n",
+	RRR_DBG_4("buffer %p read/write balance %f spins %llu (%llu us) spins/us %llu entries %i (do sleep = %i)\n",
 			buffer,
 			ratelimit->read_write_balance,
 			ratelimit->sleep_spin_time,
@@ -1448,9 +1462,10 @@ int rrr_fifo_buffer_write (
 		int do_free_entry = 0;
 
 		rrr_fifo_write_lock(buffer);
-		pthread_cleanup_push(rrr_fifo_unlock_void, buffer);
+		ret = __rrr_fifo_buffer_entry_new_unlocked(&entry);
+		rrr_fifo_unlock(buffer);
 
-		if ((__rrr_fifo_buffer_entry_new_unlocked(&entry)) != 0) {
+		if (ret != 0) {
 			RRR_MSG_0("Could not allocate entry in rrr_fifo_buffer_write\n");
 			ret = 1;
 			goto loop_out_no_entry_free;
@@ -1465,6 +1480,9 @@ int rrr_fifo_buffer_write (
 		ret = callback(&entry->data, &entry->size, &order, callback_arg);
 		pthread_cleanup_pop(1);
 
+		rrr_fifo_write_lock(buffer);
+		pthread_cleanup_push(rrr_fifo_unlock_void, buffer);
+
 		int do_ordered_write = 0;
 		int do_drop = 0;
 
@@ -1502,11 +1520,10 @@ int rrr_fifo_buffer_write (
 		loop_out_drop:
 			do_free_entry = 1;
 		loop_out_no_drop:
+			pthread_cleanup_pop(1);
 			pthread_cleanup_pop(do_free_entry);
 		loop_out_no_entry_free:
-			pthread_cleanup_pop(1);
-
-		__rrr_fifo_buffer_do_ratelimit(buffer);
+			__rrr_fifo_buffer_do_ratelimit(buffer);
 	} while (write_again);
 
 	if (entry_count_before != 0 || entry_count_after != 0) {
@@ -1514,8 +1531,6 @@ int rrr_fifo_buffer_write (
 				buffer, entry_count_before, entry_count_after);
 	}
 
-//	VL_DEBUG_MSG_4 ("New buffer entry %p data %p\n", entry, entry->data);
-
 	return ret;
 }
 
