@@ -26,7 +26,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "cmdline_defines.h"
 
-#include "../../../config.h"
 #include "../util/linked_list.h"
 
 struct cmd_arg_value {
@@ -116,10 +115,9 @@ int cmd_exists (
 		const char *key,
 		cmd_arg_count index
 );
-int cmd_iterate_subvalues (
+int cmd_iterate_subvalues_if_exists (
 		struct cmd_data *data,
 		const char *key,
-		cmd_arg_count req_index,
 		int (*callback)(const char *value, void *arg),
 		void *callback_arg
 );
