@@ -2,7 +2,7 @@
 
 Read Route Record
 
-Copyright (C) 2020 Atle Solbakken atle@goliathdns.no
+Copyright (C) 2020-2021 Atle Solbakken atle@goliathdns.no
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -24,8 +24,9 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <sys/types.h>
 
-#include "../messages/msg_addr.h"
 #include "cmodule_defines.h"
+#include "../instances.h"
+#include "../event/event.h"
 
 struct rrr_instance_runtime_data;
 struct rrr_stats_instance;
@@ -34,19 +35,25 @@ struct rrr_msg_msg;
 struct rrr_msg_addr;
 struct rrr_cmodule;
 
+extern struct rrr_instance_event_functions rrr_cmodule_helper_event_functions;
+
+const struct rrr_cmodule_config_data *rrr_cmodule_helper_config_data_get (
+		struct rrr_instance_runtime_data *thread_data
+);
 void rrr_cmodule_helper_loop (
 		struct rrr_instance_runtime_data *thread_data,
-		struct rrr_stats_instance *stats,
-		struct rrr_poll_collection *poll,
-		pid_t fork_pid
+		unsigned int periodic_interval_us
 );
 int rrr_cmodule_helper_parse_config (
 		struct rrr_instance_runtime_data *thread_data,
 		const char *config_prefix,
 		const char *config_suffix
 );
-int rrr_cmodule_helper_start_worker_fork (
-		pid_t *handle_pid,
+void rrr_cmodule_helper_mmap_channel_event_function_set (
+		struct rrr_instance_runtime_data *thread_data,
+		int (*function)(RRR_EVENT_FUNCTION_ARGS)
+);
+int rrr_cmodule_helper_worker_forks_start (
 		struct rrr_instance_runtime_data *thread_data,
 		int (*init_wrapper_callback)(RRR_CMODULE_INIT_WRAPPER_CALLBACK_ARGS),
 		void *init_wrapper_callback_arg,
@@ -55,21 +62,25 @@ int rrr_cmodule_helper_start_worker_fork (
 		int (*process_callback) (RRR_CMODULE_PROCESS_CALLBACK_ARGS),
 		void *process_callback_arg
 );
-void rrr_cmodule_helper_get_mmap_channel_to_fork_stats (
+int rrr_cmodule_helper_worker_custom_fork_start (
+		struct rrr_instance_runtime_data *thread_data,
+		unsigned int tick_interval_us,
+		int (*init_wrapper_callback)(RRR_CMODULE_INIT_WRAPPER_CALLBACK_ARGS),
+		void *init_wrapper_callback_arg,
+		int (*custom_tick_callback)(RRR_CMODULE_CUSTOM_TICK_CALLBACK_ARGS),
+		void *custom_tick_callback_arg
+);
+void rrr_cmodule_helper_get_mmap_channel_to_forks_stats (
 		unsigned long long int *read_starvation_counter,
 		unsigned long long int *write_full_counter,
 		unsigned long long int *write_retry_counter,
-		unsigned long long int *deferred_queue_entries,
-		struct rrr_cmodule *cmodule,
-		pid_t pid
+		struct rrr_cmodule *cmodule
 );
 void rrr_cmodule_helper_get_mmap_channel_to_parent_stats (
 		unsigned long long int *read_starvation_counter,
 		unsigned long long int *write_full_counter,
 		unsigned long long int *write_retry_counter,
-		unsigned long long int *deferred_queue_entries,
-		struct rrr_cmodule *cmodule,
-		pid_t pid
+		struct rrr_cmodule *cmodule
 );
 
-#endif /* RRR_CMODULE_COMMON_H */
+#endif /* RRR_CMODULE_HELPER_H */
