@@ -28,7 +28,9 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "log.h"
 #include "fixed_point.h"
 #include "rrr_types.h"
+#include "allocator.h"
 #include "util/rrr_endian.h"
+#include "util/gnu.h"
 
 static const double decimal_fractions_base2[24] = {
 		1.0/2.0,
@@ -57,7 +59,10 @@ static const double decimal_fractions_base2[24] = {
 		1.0/16777216.0
 };
 
-int rrr_fixp_ldouble_to_fixp (rrr_fixp *target, long double source) {
+int rrr_fixp_ldouble_to_fixp (
+		rrr_fixp *target,
+		long double source
+) {
 	long double integer = 0;
 	long double fraction = modfl(source, &integer);
 
@@ -97,7 +102,10 @@ int rrr_fixp_ldouble_to_fixp (rrr_fixp *target, long double source) {
 	return 0;
 }
 
-int rrr_fixp_to_ldouble (long double *target, rrr_fixp source) {
+int rrr_fixp_to_ldouble (
+		long double *target,
+		rrr_fixp source
+) {
 	long double result = 0;
 	uint64_t sign = ((uint64_t) source) >> 63;
 	source &= ~(sign<<63);
@@ -123,7 +131,11 @@ int rrr_fixp_to_ldouble (long double *target, rrr_fixp source) {
 	return 0;
 }
 
-int rrr_fixp_to_str_16 (char *target, ssize_t target_size, rrr_fixp source) {
+int rrr_fixp_to_str_16 (
+		char *target,
+		ssize_t target_size,
+		rrr_fixp source
+) {
 	unsigned char buf[8];
 
 	source = rrr_htobe64(source);
@@ -154,7 +166,11 @@ int rrr_fixp_to_str_16 (char *target, ssize_t target_size, rrr_fixp source) {
 	return 0;
 }
 
-int rrr_fixp_to_str_double (char *target, ssize_t target_size, rrr_fixp source) {
+int rrr_fixp_to_str_double (
+		char *target,
+		ssize_t target_size,
+		rrr_fixp source
+) {
 	char buf[512];
 	long double intermediate = 0;
 
@@ -164,17 +180,50 @@ int rrr_fixp_to_str_double (char *target, ssize_t target_size, rrr_fixp source)
 
 	int bytes = snprintf(buf, 511, "%.10Lf", intermediate);
 
+	if (bytes <= 0) {
+		return 1;
+	}
+
 	if (bytes > 511 || bytes > target_size - 1) {
 		return 1;
 	}
 
 	buf[bytes] = '\0';
 
-	memcpy(target, buf, strlen(buf));
+	memcpy(target, buf, bytes + 1);
 
 	return 0;
 }
 
+int rrr_fixp_to_new_str_double (
+		char **target,
+		rrr_fixp fixp
+) {
+	int ret = 0;
+
+	*target = NULL;
+
+	char *buf = NULL;
+
+	long double intermediate = 0;
+	if (rrr_fixp_to_ldouble(&intermediate, fixp) != 0) {
+		ret = 1;
+		goto out;
+	}
+
+	if (rrr_asprintf(&buf, "%.10Lf", intermediate) <= 0) {
+		ret = 0;
+		goto out;
+	}
+
+	*target = buf;
+	buf = NULL;
+
+	out:
+	RRR_FREE_IF_NOT_NULL(buf);
+	return ret;
+}
+
 static long double __rrr_fixp_convert_char (char c) {
 	if (c >= '0' && c <= '9') {
 		c -= '0';
@@ -292,7 +341,12 @@ static int __rrr_fixp_str_preliminary_parse (
 	return 0;
 }
 
-int rrr_fixp_str_to_fixp (rrr_fixp *target, const char *str, ssize_t str_length, const char **endptr) {
+int rrr_fixp_str_to_fixp (
+		rrr_fixp *target,
+		const char *str,
+		ssize_t str_length,
+		const char **endptr
+) {
 	*target = 0;
 
 	int ret = 0;
