@@ -28,10 +28,12 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <signal.h>
 #include <sys/wait.h>
 
+// To prevent global locks being held while forking, don't include
+// any frameworks using global locks
+
 #include "fork.h"
 #include "log.h"
-#include "rrr_strerror.h"
-#include "common.h"
+#include "signal_defines.h"
 #include "util/posix.h"
 
 // Many forks may to lock the handle lock during shutdown. The shutdown
@@ -57,31 +59,34 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 
 static void __rrr_fork_handler_lock (
-	struct rrr_fork_handler *handler
+		struct rrr_fork_handler *handler
 ) {
-//	printf("%i %p trylock\n", getpid(), handler);
 	// Make sure that we actually wake up and take the lock while 
 	// other users sleep between unlock/lock
-	while (pthread_mutex_trylock (&handler->lock) != 0) {
-		rrr_posix_usleep (RRR_FORK_TRYLOCK_LOOP_SLEEP_MS * 1000);
-	}
-//	printf("%i %p locked\n", getpid(), handler);
+// XXX : This seems not to be needed anymore
+//	while (pthread_mutex_trylock (&handler->lock) != 0) {
+//		rrr_posix_usleep (RRR_FORK_TRYLOCK_LOOP_SLEEP_MS * 1000);
+//	}
+	pthread_mutex_lock (&handler->lock);
 }
 
-static void __rrr_fork_handler_unlock (struct rrr_fork_handler *handler) {
-//	printf("%i %p unlock\n", getpid(), handler);
+static void __rrr_fork_handler_unlock (
+		struct rrr_fork_handler *handler
+) {
 	pthread_mutex_unlock (&handler->lock);
 }
 
-int rrr_fork_handler_new (struct rrr_fork_handler **result) {
+int rrr_fork_handler_new (
+		struct rrr_fork_handler **result
+) {
 	int ret = 0;
 
 	struct rrr_fork_handler *handler = NULL;
 
 	*result = NULL;
 
-	if ((handler = rrr_posix_mmap(RRR_FORK_HANDLER_ALLOCATION_SIZE)) == MAP_FAILED) {
-		RRR_MSG_0("Could not allocate memory in rrr_fork_handler_new: %s\n", rrr_strerror(errno));
+	if ((handler = rrr_posix_mmap(RRR_FORK_HANDLER_ALLOCATION_SIZE, 1 /* Is shared */)) == MAP_FAILED) {
+		RRR_MSG_0("Could not allocate memory in rrr_fork_handler_new\n");
 		ret = 1;
 		goto out;
 	}
@@ -110,22 +115,31 @@ int rrr_fork_handler_new (struct rrr_fork_handler **result) {
 		return ret;
 }
 
-static void __rrr_fork_handler_free (struct rrr_fork_handler *handler) {
+static void __rrr_fork_handler_free (
+		struct rrr_fork_handler *handler
+) {
 	munmap(handler, RRR_FORK_HANDLER_ALLOCATION_SIZE);
 }
 
-static int __rrr_fork_clear (struct rrr_fork *fork) {
+static int __rrr_fork_clear (
+		struct rrr_fork *fork
+) {
 	fork->exit_notify_arg = NULL;
 	fork->exit_notify = NULL;
 	fork->pid = 0;
 	fork->parent_pid = 0;
 	fork->was_waited_for = 0;
+
 	return 0;
 }
 
-void rrr_fork_handler_destroy (struct rrr_fork_handler *handler) {
+void rrr_fork_handler_destroy (
+		struct rrr_fork_handler *handler
+) {
 	RRR_FORK_HANDLER_VERIFY_SELF();
+
 	__rrr_fork_handler_lock(handler);
+
 	RRR_LL_ITERATE_BEGIN(handler, struct rrr_fork);
 		if (node->pid > 0) {
 			RRR_MSG_0("Warning: Child fork pid %i had not yet exited or exit notifications was not set while destroying fork handler.\n", node->pid);
@@ -133,9 +147,12 @@ void rrr_fork_handler_destroy (struct rrr_fork_handler *handler) {
 				RRR_MSG_0("Warning: Child fork pid %i had not been waited for while destroying fork handler, is now possibly a zombie.\n", node->pid);
 			}
 		}
+
 		RRR_LL_ITERATE_SET_DESTROY();
 	RRR_LL_ITERATE_END_CHECK_DESTROY_NO_REMOVE(__rrr_fork_clear(node));
+
 	__rrr_fork_handler_unlock(handler);
+
 	pthread_mutex_destroy(&handler->lock);
 	__rrr_fork_handler_free(handler);
 }
@@ -157,14 +174,18 @@ int rrr_fork_signal_handler (int s, void *arg) {
 	return RRR_SIGNAL_NOT_HANDLED;
 }
 
-static int __rrr_fork_set_waited_for (struct rrr_fork *fork) {
+static int __rrr_fork_set_waited_for (
+		struct rrr_fork *fork
+) {
 	// The parent must call the notify function
 	fork->was_waited_for = 1;
 
 	return 0;
 }
 
-static int __rrr_fork_notify_and_clear (struct rrr_fork *fork) {
+static int __rrr_fork_notify_and_clear (
+		struct rrr_fork *fork
+) {
 	RRR_DBG_1("Notification to exit handlers for exited pid %i in parent pid %i\n",
 			fork->pid, getpid());
 
@@ -181,19 +202,25 @@ static int __rrr_fork_waitpid (pid_t pid, int *status, int options) {
 		RRR_DBG_1("=== WAIT PID %i ========================================================================================\n", ret);
 		if (WIFSIGNALED(*status)) {
 			int signal = WTERMSIG(*status);
-			RRR_DBG_4("Fork %i was terminated by signal %i\n", ret, signal);
+			RRR_DBG_1("Fork %i was terminated by signal %i\n", ret, signal);
 		}
 		if (WIFEXITED(*status)) {
 			int child_status = WEXITSTATUS(*status);
-			RRR_DBG_4("Fork %i has exited with status %i\n", ret, child_status);
+			RRR_DBG_1("Fork %i has exited with status %i\n", ret, child_status);
 		}
 	}
 	return ret;
 }
 
-static void __rrr_fork_wait_loop (int *active_forks_found, struct rrr_fork_handler *handler, int max_rounds) {
+static void __rrr_fork_wait_loop (
+		int *active_forks_found,
+		struct rrr_fork_handler *handler,
+		int max_rounds
+) {
 	*active_forks_found = 0;
 
+	pid_t self = getpid();
+
 	do {
 		if (max_rounds-- <= 0) {
 			RRR_MSG_0("Timeout reached while waiting for forks to exit\n");
@@ -202,17 +229,18 @@ static void __rrr_fork_wait_loop (int *active_forks_found, struct rrr_fork_handl
 		*active_forks_found = 0;
 		RRR_LL_ITERATE_BEGIN(handler, struct rrr_fork);
 			if (node->pid > 0 && node->was_waited_for == 0) {
-				RRR_DBG_4("After signalling, checking wait for pid %i has exited is %i\n", node->pid, node->was_waited_for);
+				RRR_DBG_1("Checking wait for pid %i self is %i\n", node->pid, self);
+
 				pid_t pid;
-				int status;
+				int status = 0;
 
 				if ((pid = __rrr_fork_waitpid(node->pid, &status, WNOHANG)) == node->pid) {
-					RRR_DBG_4("Wait pid %i ok\n", node->pid);
+					RRR_DBG_1("Wait pid %i ok\n", node->pid);
 					RRR_LL_ITERATE_SET_DESTROY();
 				}
 				else if (pid == -1 && errno == ECHILD) {
-					RRR_DBG_4("Error from waitpid on pid %i in parent %i status %i after signalling: %s. Not exited yet? Might be a child of a child (whos parent has not exited).\n",
-							node->pid, getpid(), status, rrr_strerror(errno));
+					RRR_DBG_1("Error from waitpid on pid %i in parent %i status %i after signalling errno is %i. Not exited yet? Might be a child of a child (whos parent has not exited).\n",
+							node->pid, getpid(), status, errno);
 				}
 
 				*active_forks_found = 1;
@@ -225,10 +253,21 @@ static void __rrr_fork_wait_loop (int *active_forks_found, struct rrr_fork_handl
 	} while (*active_forks_found != 0);
 }
 
+void rrr_fork_send_sigusr1_to_pid (
+		pid_t pid
+) {
+	RRR_DBG_1("SIGUSR1 to fork %i\n", pid);
+	kill(pid, SIGUSR1);
+}
+
 // Call from main() only
-void rrr_fork_send_sigusr1_and_wait (struct rrr_fork_handler *handler) {
+void rrr_fork_send_sigusr1_and_wait (
+		struct rrr_fork_handler *handler
+) {
 	RRR_FORK_HANDLER_VERIFY_SELF();
 
+	pid_t self = getpid();
+
 	__rrr_fork_handler_lock(handler);
 
 	// Signal handlers must be disabled before we do this
@@ -238,7 +277,7 @@ void rrr_fork_send_sigusr1_and_wait (struct rrr_fork_handler *handler) {
 		RRR_DBG_1("Sending SIGUSR1 to all forks and waiting in pid %i\n", getpid());
 
 		RRR_LL_ITERATE_BEGIN(handler, struct rrr_fork);
-			if (node->pid > 0) {
+			if (node->pid > 0 && node->pid != self) {
 				RRR_DBG_1("SIGUSR1 to fork %i\n", node->pid);
 				kill(node->pid, SIGUSR1);
 			}
@@ -249,13 +288,13 @@ void rrr_fork_send_sigusr1_and_wait (struct rrr_fork_handler *handler) {
 	 *		}*/
 		RRR_LL_ITERATE_END();
 
-		__rrr_fork_wait_loop(&active_forks_found, handler, 5); // 5 rounds = ~0.5 seconds
+		__rrr_fork_wait_loop(&active_forks_found, handler, 20); // 20 rounds = ~2 seconds
 	}
 
 	// Try SIGKILL
 	if (RRR_LL_COUNT(handler) > 0) {
 		RRR_LL_ITERATE_BEGIN(handler, struct rrr_fork);
-			if (node->pid > 0 && node->was_waited_for == 0) {
+			if (node->pid > 0 && node->was_waited_for == 0 && node->pid != self) {
 				RRR_DBG_1("SIGKILL to fork %i\n", node->pid);
 				kill(node->pid, SIGKILL);
 			}
@@ -271,7 +310,10 @@ void rrr_fork_send_sigusr1_and_wait (struct rrr_fork_handler *handler) {
 	__rrr_fork_handler_unlock (handler);
 }
 
-void rrr_fork_handle_sigchld_and_notify_if_needed  (struct rrr_fork_handler *handler, int force_wait_all) {
+void rrr_fork_handle_sigchld_and_notify_if_needed (
+		struct rrr_fork_handler *handler,
+		int force_wait_all
+) {
 	pid_t self = getpid();
 
 	// We cannot lock this because it's written to within signal context
@@ -280,10 +322,10 @@ void rrr_fork_handle_sigchld_and_notify_if_needed  (struct rrr_fork_handler *han
 		rrr_fork_handler_signal_pending = 0;
 
 		if (force_wait_all) {
-			RRR_DBG_4("Fork force wait all, self pid is %i\n", self);
+			RRR_DBG_1("Fork force wait all, self pid is %i\n", self);
 		}
 		else {
-			RRR_DBG_4("Fork handle SIGCHLD, self pid is %i\n", self);
+			RRR_DBG_1("Fork handle SIGCHLD, self pid is %i\n", self);
 		}
 
 		int pid_count = 0;
@@ -296,13 +338,13 @@ void rrr_fork_handle_sigchld_and_notify_if_needed  (struct rrr_fork_handler *han
 				RRR_BUG("BUG: Too many forks in rrr_fork_handle_sigchld_and_notify_if_needed\n");
 			}
 			waited_for_pids[pid_count++] = pid_tmp;
-			RRR_DBG_4("Fork %i exited with status %i\n", pid_tmp, status);
+			RRR_DBG_1("Fork %i exited with status %i\n", pid_tmp, status);
 		}
 
 		RRR_LL_ITERATE_BEGIN(handler, struct rrr_fork);
 			int was_waited_for = 0;
 
-			if (node->pid <= 0) {
+			if (node->pid <= 0 || node->pid == self) {
 				RRR_LL_ITERATE_NEXT();
 			}
 			else if (node->was_waited_for != 0) {
@@ -311,12 +353,12 @@ void rrr_fork_handle_sigchld_and_notify_if_needed  (struct rrr_fork_handler *han
 			else {
 				if (force_wait_all) {
 					int status;
-					RRR_DBG_4("Fork late wait for pid %i self is %i\n", node->pid, self);
+					RRR_DBG_1("Fork late wait for pid %i self is %i\n", node->pid, self);
 					if (__rrr_fork_waitpid(node->pid, &status, WNOHANG) > 0) {
 						was_waited_for = 1;
 					}
 					else {
-						RRR_DBG_4("Fork %i had not yet exited or we are not the parent when late-waiting\n", node->pid);
+						RRR_DBG_1("Fork %i had not yet exited or we are not the parent when late-waiting\n", node->pid);
 					}
 				}
 				else {
@@ -337,7 +379,9 @@ void rrr_fork_handle_sigchld_and_notify_if_needed  (struct rrr_fork_handler *han
 	}
 }
 
-static struct rrr_fork *__rrr_fork_allocate_unlocked (struct rrr_fork_handler *handler) {
+static struct rrr_fork *__rrr_fork_allocate_unlocked (
+		struct rrr_fork_handler *handler
+) {
 	struct rrr_fork *result = NULL;
 
 	RRR_LL_ITERATE_BEGIN(handler, struct rrr_fork);
@@ -357,6 +401,7 @@ pid_t rrr_fork (
 ) {
 	pid_t ret = 0;
 
+	// XXX : This seems not to be needed anymore
 	/*
 	 * XXX : For some reason the perl5 module might sometimes hang during
 	 *       initialization when it forks out the worker. It will then hang
@@ -365,24 +410,27 @@ pid_t rrr_fork (
 	 *       if when spin on trylock like this (also for some reason).
 	 */
 	__rrr_fork_handler_lock(handler);
-
 	struct rrr_fork *result = __rrr_fork_allocate_unlocked (handler);
+	__rrr_fork_handler_unlock (handler);
+
 	if (result == NULL) {
 		RRR_MSG_0("No available fork slot while forking in rrr_fork\n");
 		ret = -1;
-		goto out_unlock;
+		goto out;
 	}
 
 	ret = fork();
 
 	if (ret < 0) {
-		RRR_MSG_0("Error while forking in rrr_fork: %s\n", rrr_strerror(errno));
+		// Don't use rrr_strerror while other forks might be forking
+		RRR_MSG_0("Error while forking in rrr_fork errno %i\n", errno);
 		ret = -1;
-		goto out_unlock;
+		goto out;
 	}
 	else if (ret == 0) {
 		// Child code
 		// Only parent unlocks. This is a PSHARED lock.
+		// Don't print debug message, may mess with log hooks
 		goto out;
 	}
 
@@ -390,13 +438,15 @@ pid_t rrr_fork (
 
 	RRR_DBG_1("=== FORK PID %i ========================================================================================\n", ret);
 
+	__rrr_fork_handler_lock(handler);
+
 	result->parent_pid = getpid();
 	result->pid = ret;
 	result->exit_notify = exit_notify;
 	result->exit_notify_arg = exit_notify_arg;
 
-	out_unlock:
-		__rrr_fork_handler_unlock (handler);
+	__rrr_fork_handler_unlock (handler);
+
 	out:
 		return ret;
 }
