@@ -2,7 +2,7 @@
 
 Read Route Record
 
-Copyright (C) 2020 Atle Solbakken atle@goliathdns.no
+Copyright (C) 2021 Atle Solbakken atle@goliathdns.no
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -64,4 +64,16 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #	define umask(x)			RRR_INTERCEPT_H_UNSAFE_LIBARY_FUNCTION_UMASK
 #endif
 
+#ifndef RRR_INTERCEPT_ALLOW_GETTID
+#	define gettid(void)		RRR_INTERCEPT_H_UNSAFE_LIBRARY_FUNCTION_GETTID
+#endif
+/*
+#define malloc(a)      RRR_INTERCEPT_H_UNSAFE_LIBRARY_FUNCTION_MALLOC
+#define strdup(a)      RRR_INTERCEPT_H_UNSAFE_LIBRARY_FUNCTION_STRDUP
+#define calloc(a,b)    RRR_INTERCEPT_H_UNSAFE_LIBRARY_FUNCTION_CALLOC
+#define realloc(a,b)   RRR_INTERCEPT_H_UNSAFE_LIBRARY_FUNCTION_REALLOC
+#define free(a)        RRR_INTERCEPT_H_UNSAFE_LIBRARY_FUNCTION_FREE
+*/
+#define asprintf(...)  rrr_asprintf(__VA_ARGS__)
+
 #endif /* RRR_INTERCEPT_H */
