@@ -2,7 +2,7 @@
 
 Read Route Record
 
-Copyright (C) 2019 Atle Solbakken atle@goliathdns.no
+Copyright (C) 2019-2021 Atle Solbakken atle@goliathdns.no
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -46,6 +46,12 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #define RRR_MAP_ITERATE_IS_FIRST()									\
 	RRR_LL_ITERATE_IS_FIRST()
 
+#define RRR_MAP_ITERATE_IS_LAST()									\
+	RRR_LL_ITERATE_IS_LAST()
+
+#define RRR_MAP_ITERATE_BREAK() \
+	RRR_LL_ITERATE_BREAK()
+
 #define RRR_MAP_COUNT(map)											\
 	RRR_LL_COUNT(map)
 
@@ -58,6 +64,9 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #define RRR_MAP_ITERATOR_NEXT(iterator) \
 	rrr_map_iterator_next(iterator)
 
+#define RRR_MAP_MERGE_AND_CLEAR_SOURCE_HEAD(target,source) \
+	RRR_LL_MERGE_AND_CLEAR_SOURCE_HEAD(target,source)
+
 // Make sure this is EQUAL to rrr_linked_list_node. Different
 // pointer types are OK.
 struct rrr_map_item {
@@ -77,7 +86,9 @@ struct rrr_map_iterator {
 	struct rrr_map_item *cur;
 };
 
-static inline struct rrr_map_item *rrr_map_iterator_next (struct rrr_map_iterator *iterator) {
+static inline struct rrr_map_item *rrr_map_iterator_next (
+		struct rrr_map_iterator *iterator
+) {
 	if (iterator->cur == NULL) {
 		if (iterator->rpos == 0) {
 			iterator->cur = iterator->source->ptr_first;
@@ -91,15 +102,60 @@ static inline struct rrr_map_item *rrr_map_iterator_next (struct rrr_map_iterato
 	return iterator->cur;
 }
 
-void rrr_map_item_destroy (struct rrr_map_item *item);
-void rrr_map_clear (struct rrr_map *map);
-int rrr_map_item_new (struct rrr_map_item **target, ssize_t field_size);
-int rrr_map_item_add (struct rrr_map *map, struct rrr_map_item *item);
-int rrr_map_item_add_new (struct rrr_map *map, const char *tag, const char *value);
-int rrr_map_parse_pair (const char *input, struct rrr_map *target, const char *delimeter);
+void rrr_map_item_destroy (
+		struct rrr_map_item *item
+);
+void rrr_map_clear (
+		struct rrr_map *map
+);
+int rrr_map_item_new (
+		struct rrr_map_item **target,
+		ssize_t field_size
+);
+int rrr_map_item_add (
+		struct rrr_map *map,
+		struct rrr_map_item *item
+);
+int rrr_map_item_replace_new (
+		struct rrr_map *map,
+		const char *tag,
+		const char *value
+);
+int rrr_map_item_replace_new_with_callback (
+		struct rrr_map *map,
+		const char *tag,
+		const char *value,
+		int (*callback_confirm)(void *arg),
+		void *callback_arg
+);
+int rrr_map_item_replace_new_with_callback (
+		struct rrr_map *map,
+		const char *tag,
+		const char *value,
+		int (*callback_confirm)(void *arg),
+		void *callback_arg
+);
+int rrr_map_item_add_new (
+		struct rrr_map *map,
+		const char *tag,
+		const char *value
+);
+int rrr_map_item_prepend_new (
+		struct rrr_map *map,
+		const char *tag,
+		const char *value
+);
+int rrr_map_parse_pair (
+		const char *input,
+		struct rrr_map *target,
+		const char *delimeter
+);
 int rrr_map_parse_pair_arrow (const char *input, void *arg);
 int rrr_map_parse_pair_equal (const char *input, void *arg);
 int rrr_map_parse_tag_only (const char *input, void *arg);
-const char *rrr_map_get_value (const struct rrr_map *map, const char *tag);
+const char *rrr_map_get_value (
+		const struct rrr_map *map,
+		const char *tag
+);
 
 #endif /* RRR_MAP_H */
