@@ -26,13 +26,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <sys/socket.h>
 
 struct rrr_msg_holder;
-struct rrr_mqtt_topic_token;
 
-int rrr_msg_holder_util_message_topic_match (
-		int *does_match,
-		const struct rrr_msg_holder *entry,
-		const struct rrr_mqtt_topic_token *filter_first_token
-);
 int rrr_msg_holder_util_new_with_empty_message (
 		struct rrr_msg_holder **result,
 		ssize_t message_data_length,
