@@ -2,7 +2,7 @@
 
 Read Route Record
 
-Copyright (C) 2020 Atle Solbakken atle@goliathdns.no
+Copyright (C) 2020-2021 Atle Solbakken atle@goliathdns.no
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -33,28 +33,46 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "mmap_channel.h"
 #include "rrr_mmap.h"
 #include "random.h"
+#include "event/event.h"
+#include "event/event_functions.h"
 #include "util/rrr_time.h"
 #include "util/posix.h"
 
 // Messages larger than this limit are transferred using SHM
 #define RRR_MMAP_CHANNEL_SHM_LIMIT 1024
 #define RRR_MMAP_CHANNEL_SHM_MIN_ALLOC_SIZE 4096
-/* Currently not used
-int rrr_mmap_channel_write_is_possible (struct rrr_mmap_channel *target) {
-	int possible = 1;
 
+struct rrr_mmap_channel_block {
+	pthread_mutex_t block_lock;
+	size_t size_capacity;
+	size_t size_data; // If set to 0, block is free
+	int shmid;
+	void *ptr_shm_or_mmap;
+};
+
+struct rrr_mmap_channel {
+	struct rrr_mmap *mmap;
+	pthread_mutex_t index_lock;
+	int entry_count;
+	int wpos;
+	int rpos;
+	struct rrr_mmap_channel_block blocks[RRR_MMAP_CHANNEL_SLOTS];
+	char name[64];
+
+	unsigned long long int read_starvation_counter;
+	unsigned long long int write_full_counter;
+};
+
+int rrr_mmap_channel_count (
+		struct rrr_mmap_channel *target
+) {
+	int count;
 	pthread_mutex_lock(&target->index_lock);
-
-	struct rrr_mmap_channel_block *block = &(target->blocks[target->wpos]);
-	if (block->size_data != 0) {
-		possible = 0;
-	}
-
+	count = target->entry_count;
 	pthread_mutex_unlock(&target->index_lock);
-
-	return possible;
+	return count;
 }
-*/
+
 static int __rrr_mmap_channel_block_free (
 		struct rrr_mmap_channel *target,
 		struct rrr_mmap_channel_block *block
@@ -94,18 +112,6 @@ static int __rrr_mmap_channel_block_free (
 	return 0;
 }
 
-void rrr_mmap_channel_writer_free_unused_mmap_blocks (struct rrr_mmap_channel *target) {
-	pthread_mutex_lock(&target->index_lock);
-
-	for (int i = 0; i != RRR_MMAP_CHANNEL_SLOTS; i++) {
-		if (target->blocks[i].size_data == 0 && target->blocks[i].shmid == 0 && target->blocks[i].ptr_shm_or_mmap != NULL) {
-			__rrr_mmap_channel_block_free(target, &target->blocks[i]);
-		}
-	}
-
-	pthread_mutex_unlock(&target->index_lock);
-}
-
 static int __rrr_mmap_channel_allocate (
 		struct rrr_mmap_channel *target,
 		struct rrr_mmap_channel_block *block,
@@ -113,9 +119,19 @@ static int __rrr_mmap_channel_allocate (
 ) {
 	int ret = 0;
 
-	if (block->size_capacity >= data_size) {
-		goto out;
+	// To reduce the chance of hitting the operating system limit on the total number of
+	// shared memory blocks, free the allocation if shm is not needed for this write.
+	if (block->shmid != 0 && block->ptr_shm_or_mmap != NULL) {
+		if (data_size >= RRR_MMAP_CHANNEL_SHM_MIN_ALLOC_SIZE && data_size <= block->size_capacity) {
+			goto out;
+		}
 	}
+	else {
+		if (data_size <= block->size_capacity) {
+			goto out;
+		}
+	}
+
 	if ((ret = __rrr_mmap_channel_block_free(target, block)) != 0) {
 		goto out;
 	}
@@ -129,7 +145,6 @@ static int __rrr_mmap_channel_allocate (
 		int shmid = 0;
 		do {
 			new_key = rrr_rand();
-//			printf("allocate shmget key %i pos %i size %lu\n", new_key, block_pos, data_size);
 			if ((shmid = shmget(new_key, data_size, IPC_CREAT|IPC_EXCL|0600)) == -1) {
 				if (errno == EEXIST) {
 					// OK, try another key
@@ -159,7 +174,6 @@ static int __rrr_mmap_channel_allocate (
 	}
 	else {
 		if ((block->ptr_shm_or_mmap = rrr_mmap_allocate(target->mmap, data_size)) == NULL) {
-//			RRR_MSG_0("Could not allocate mmap memory in __rrr_mmap_channel_allocate \n");
 			ret = 1;
 			goto out;
 		}
@@ -171,57 +185,20 @@ static int __rrr_mmap_channel_allocate (
 	return ret;
 }
 
-static int __rrr_mmap_channel_cond_wait (
-		pthread_mutex_t *mutex,
-		pthread_cond_t *cond,
-		unsigned int timeout_us
-) {
-	int ret = 0;
-
-	if (timeout_us == 0) {
-		ret = RRR_MMAP_CHANNEL_FULL_OR_EMPTY;
-		goto out;
-	}
-
-	pthread_mutex_lock(mutex);
-
-	struct timespec time;
-	rrr_time_gettimeofday_timespec(&time, timeout_us);
-
-	if ((ret = pthread_cond_timedwait (
-			cond,
-			mutex,
-			&time
-	)) != 0) {
-		switch (ret) {
-			case ETIMEDOUT:
-				ret = RRR_MMAP_CHANNEL_FULL_OR_EMPTY;
-				break;
-			default:
-				RRR_MSG_0("Error while waiting on condition in __rrr_mmap_channel_cond_wait: %i\n", ret);
-				ret = RRR_MMAP_CHANNEL_ERROR;
-				break;
-		};
-		goto out_unlock;
-	}
-
-	out_unlock:
-		pthread_mutex_unlock(mutex);
-	out:
-	return ret;
-}
-
 int rrr_mmap_channel_write_using_callback (
 		struct rrr_mmap_channel *target,
+		struct rrr_event_queue *queue_notify,
 		size_t data_size,
+		int wait_attempts_max,
 		unsigned int full_wait_time_us,
 		int (*callback)(void *target, void *arg),
-		void *callback_arg
+		void *callback_arg,
+		int (*check_cancel_callback)(void *arg),
+		void *check_cancel_callback_arg
 ) {
 	int ret = RRR_MMAP_CHANNEL_OK;
 
 	int do_unlock_block = 0;
-	int wait_attempts_max = 4;
 
 	struct rrr_mmap_channel_block *block = NULL;
 
@@ -236,24 +213,14 @@ int rrr_mmap_channel_write_using_callback (
 		}
 
 		if (full_wait_time_us == 0 || wait_attempts_max-- == 0) {
-//			printf("mmap channel %p %s full wait time %u us attempts %i\n", target, target->name, full_wait_time_us, wait_attempts_max);
 			pthread_mutex_lock(&target->index_lock);
 			target->write_full_counter++;
 			pthread_mutex_unlock(&target->index_lock);
 			ret = RRR_MMAP_CHANNEL_FULL;
 			goto out_unlock;
 		}
-//		uint64_t time_start = rrr_time_get_64();
-//		printf("mmap channel %p %s full wait %u us\n", target, target->name, full_wait_time_us);
-		if ((ret = __rrr_mmap_channel_cond_wait(
-				&target->index_lock,
-				&target->full_cond,
-				full_wait_time_us
-		)) != 0) {
-//			printf("mmap channel %p %s full wait timeout result %i\n", target, target->name, ret);
-			goto out_unlock;
-		}
-//		printf("mmap channel %p %s full wait done in %" PRIu64 "\n", target, target->name, rrr_time_get_64() - time_start);
+
+		rrr_posix_usleep(full_wait_time_us);
 
 	attempt_block_lock:
 		pthread_mutex_lock(&target->index_lock);
@@ -286,22 +253,29 @@ int rrr_mmap_channel_write_using_callback (
 
 	block->size_data = data_size;
 
-	RRR_DBG_4("mmap channel %p %s wr blk %i size %li\n", target, target->name, target->wpos, data_size);
+	RRR_MMAP_DBG("mmap channel %p %s wr blk %i size %li\n", target, target->name, target->wpos, data_size);
 
 	pthread_mutex_unlock(&block->block_lock);
 	do_unlock_block = 0;
 
 	pthread_mutex_lock(&target->index_lock);
+	target->entry_count++;
 	target->wpos++;
 	if (target->wpos == RRR_MMAP_CHANNEL_SLOTS) {
 		target->wpos = 0;
 	}
 	pthread_mutex_unlock(&target->index_lock);
 
-	if ((ret = pthread_cond_signal(&target->empty_cond)) != 0) {
-		RRR_MSG_0("Error while signalling empty condition in rrr_mmap_channel_write_using_callback: %i\n", ret);
-		ret = RRR_MMAP_CHANNEL_ERROR;
-		goto out_unlock;
+	if (queue_notify != NULL) {
+		if ((ret = rrr_event_pass (
+				queue_notify,
+				RRR_EVENT_FUNCTION_MMAP_CHANNEL_DATA_AVAILABLE,
+				1,
+				check_cancel_callback,
+				check_cancel_callback_arg
+		)) != 0) {
+			goto out_unlock;
+		}
 	}
 
 	out_unlock:
@@ -325,9 +299,13 @@ static int __rrr_mmap_channel_write_callback (void *target_ptr, void *arg) {
 
 int rrr_mmap_channel_write (
 		struct rrr_mmap_channel *target,
+		struct rrr_event_queue *queue_notify,
 		const void *data,
 		size_t data_size,
-		unsigned int full_wait_time_us
+		unsigned int full_wait_time_us,
+		int retries_max,
+		int (*check_cancel_callback)(void *arg),
+		void *check_cancel_callback_arg
 ) {
 	struct rrr_mmap_channel_write_callback_arg callback_data = {
 			data,
@@ -335,73 +313,59 @@ int rrr_mmap_channel_write (
 	};
 	return rrr_mmap_channel_write_using_callback (
 			target,
+			queue_notify,
 			data_size,
 			full_wait_time_us,
+			retries_max,
 			__rrr_mmap_channel_write_callback,
-			&callback_data
+			&callback_data,
+			check_cancel_callback,
+			check_cancel_callback_arg
 	);
 }
 
 int rrr_mmap_channel_read_with_callback (
+		int *read_count,
 		struct rrr_mmap_channel *source,
-		unsigned int empty_wait_time_us,
 		int (*callback)(const void *data, size_t data_size, void *arg),
 		void *callback_arg
 ) {
 	int ret = RRR_MMAP_CHANNEL_OK;
 
-	int do_rpos_increment = 1;
+	*read_count = 0;
 
+	int do_rpos_increment = 1;
 	int do_unlock_block = 0;
-	int wait_attempts_max = 4;
 
 	struct rrr_mmap_channel_block *block = NULL;
 
-	goto attempt_block_lock;
+	pthread_mutex_lock(&source->index_lock);
 
-	attempt_read_wait:
-		// We MUST NOT hold index lock and block lock simultaneously. cond_wait will lock index.
-		if (do_unlock_block) {
-			pthread_mutex_unlock(&block->block_lock);
-			do_unlock_block = 0;
-		}
+	block = &(source->blocks[source->rpos]);
+	int entry_count = source->entry_count;
 
-		if (empty_wait_time_us == 0 || wait_attempts_max-- == 0) {
-			pthread_mutex_lock(&source->index_lock);
-			source->read_starvation_counter++;
-			pthread_mutex_unlock(&source->index_lock);
-			ret = RRR_MMAP_CHANNEL_EMPTY;
-			goto out_unlock;
-		}
-//		uint64_t time_start = rrr_time_get_64();
-//		printf("mmap channel %p %s empty wait %u us\n", source, source->name, empty_wait_time_us);
-		if ((ret = __rrr_mmap_channel_cond_wait (
-				&source->index_lock,
-				&source->empty_cond,
-				empty_wait_time_us
-		)) != 0) {
-//			printf("mmap channel %p %s empty wait timeout result %i in %" PRIu64 "\n", source, source->name, ret, rrr_time_get_64() - time_start);
-			goto out_unlock;
-		}
-//		printf("mmap channel %p %s empty wait done in %" PRIu64 "\n", source, source->name, rrr_time_get_64() - time_start);
+	pthread_mutex_unlock(&source->index_lock);
 
-	attempt_block_lock:
-		pthread_mutex_lock(&source->index_lock);
-		block = &(source->blocks[source->rpos]);
-		pthread_mutex_unlock(&source->index_lock);
-
-		if (pthread_mutex_trylock(&block->block_lock) != 0) {
-			goto attempt_read_wait;
-		}
-		do_unlock_block = 1;
-
-	if (block->size_data == 0) {
-		goto attempt_read_wait;
+	if (entry_count == 0) {
+		goto out_unlock;
 	}
 
-	RRR_DBG_4("mmap channel %p %s rd blk %i size %li\n", source, source->name, source->rpos, block->size_data);
+	if (pthread_mutex_trylock(&block->block_lock) != 0) {
+		pthread_mutex_lock(&source->index_lock);
+		source->read_starvation_counter++;
+		pthread_mutex_unlock(&source->index_lock);
+		ret = RRR_MMAP_CHANNEL_EMPTY;
+		goto out_unlock;
+	}
+
+	do_unlock_block = 1;
+
+	if (block->size_data == 0) {
+		goto out_unlock;
+	}
+
+	RRR_MMAP_DBG("mmap channel %p %s rd blk %i size %li\n", source, source->name, source->rpos, block->size_data);
 
-//	uint64_t start_time = rrr_time_get_64();
 	if (block->shmid != 0) {
 		const char *data_pointer = NULL;
 
@@ -430,21 +394,16 @@ int rrr_mmap_channel_read_with_callback (
 			do_rpos_increment = 0;
 		}
 	}
-//	printf("read callback time %p: %" PRIu64 "\n", source, rrr_time_get_64() - start_time);
 
 	out_rpos_increment:
 	if (do_rpos_increment) {
+		*read_count = 1;
 		block->size_data = 0;
 		pthread_mutex_unlock(&block->block_lock);
 		do_unlock_block = 0;
 
-		if ((ret = pthread_cond_signal(&source->full_cond)) != 0) {
-			RRR_MSG_0("Error while signalling full condition in __rrr_mmap_channel_read_with_callback: %i\n", ret);
-			ret = RRR_MMAP_CHANNEL_ERROR;
-			goto out_unlock;
-		}
-
 		pthread_mutex_lock(&source->index_lock);
+		source->entry_count--;
 		source->rpos++;
 		if (source->rpos == RRR_MMAP_CHANNEL_SLOTS) {
 			source->rpos = 0;
@@ -452,8 +411,6 @@ int rrr_mmap_channel_read_with_callback (
 		pthread_mutex_unlock(&source->index_lock);
 	}
 
-//	printf ("mmap channel read from mmap %p to local %p size_data %lu\n", block->ptr, result, *target_size);
-
 	out_unlock:
 	if (do_unlock_block) {
 		pthread_mutex_unlock(&block->block_lock);
@@ -461,62 +418,6 @@ int rrr_mmap_channel_read_with_callback (
 	return ret;
 }
 
-/*
- * Not currently used
-struct rrr_mmap_channel_read_callback_data {
-	void *data;
-	size_t data_size;
-};
-
-static int __rrr_mmap_channel_read_callback (const void *data, size_t data_size, void *arg) {
-	struct rrr_mmap_channel_read_callback_data *callback_data = arg;
-
-	int ret = 0;
-
-	if ((callback_data->data = malloc(data_size)) == NULL) {
-		RRR_MSG_0("Could not allocate memory in __rrr_mmap_channel_read_callback\n");
-		ret = 1;
-		goto out;
-	}
-
-	memcpy(callback_data->data, data, data_size);
-	callback_data->data_size = data_size;
-
-	out:
-	return ret;
-}
-*/
-/*
- * Not currently used
-int rrr_mmap_channel_read (
-		void **target,
-		size_t *target_size,
-		unsigned int empty_wait_time_us,
-		struct rrr_mmap_channel *source
-) {
-	struct rrr_mmap_channel_read_callback_data callback_data = {0};
-
-	int ret = 0;
-
-	*target = NULL;
-	*target_size = 0;
-
-	if ((ret = __rrr_mmap_channel_read_with_callback (
-			source,
-			empty_wait_time_us,
-			__rrr_mmap_channel_read_callback,
-			&callback_data
-	)) != 0) {
-		return ret;
-	}
-
-	*target = callback_data.data;
-	*target_size = callback_data.data_size;
-
-	return ret;
-}
-*/
-
 void rrr_mmap_channel_bubblesort_pointers (struct rrr_mmap_channel *target, int *was_sorted) {
 	*was_sorted = 1;
 
@@ -537,7 +438,6 @@ void rrr_mmap_channel_bubblesort_pointers (struct rrr_mmap_channel *target, int
 					block_j->ptr_shm_or_mmap != NULL &&
 					block_i->ptr_shm_or_mmap > block_j->ptr_shm_or_mmap
 				) {
-//					printf ("swap i and j (%p > %p)\n", block_i->ptr_shm_or_mmap, block_j->ptr_shm_or_mmap);
 					// The data structure here makes sure we don't forget to update
 					// this function when the struct changes.
 					struct rrr_mmap_channel_block tmp = {
@@ -568,21 +468,33 @@ void rrr_mmap_channel_bubblesort_pointers (struct rrr_mmap_channel *target, int
 }
 
 void rrr_mmap_channel_destroy (struct rrr_mmap_channel *target) {
-	pthread_mutex_destroy(&target->index_lock);
-	pthread_cond_destroy(&target->empty_cond);
-	pthread_cond_destroy(&target->full_cond);
+	if (pthread_mutex_trylock(&target->index_lock) != 0) {
+		RRR_MSG_0("Warning: Not destroying index lock of mmap channel %s, it's possibly still being used\n",
+				target->name);
+	}
+	else {
+		pthread_mutex_unlock(&target->index_lock);
+		pthread_mutex_destroy(&target->index_lock);
+	}
 
 	int msg_count = 0;
 	for (int i = 0; i != RRR_MMAP_CHANNEL_SLOTS; i++) {
 		if (target->blocks[i].ptr_shm_or_mmap != NULL) {
 			if (++msg_count == 1) {
-				RRR_MSG_0("Warning: Pointer was still present in block in rrr_mmap_channel_destroy\n");
+				RRR_MSG_1("Note: Pointer was still present in block in rrr_mmap_channel_destroy, fork might not have exited yet or has been killed before cleanup.\n");
 			}
 		}
-		pthread_mutex_destroy(&target->blocks[i].block_lock);
+		if (pthread_mutex_trylock(&target->blocks[i].block_lock) != 0) {
+			RRR_MSG_0("Warning: Not destroying block lock %i of mmap channel %s, it's possibly still being used\n",
+					i, target->name);
+		}
+		else {
+			pthread_mutex_unlock(&target->blocks[i].block_lock);
+			pthread_mutex_destroy(&target->blocks[i].block_lock);
+		}
 	}
 	if (msg_count > 1) {
-		RRR_MSG_0("Last message duplicated %i times\n", msg_count - 1);
+		RRR_MSG_1("Note: Last message duplicated %i times\n", msg_count - 1);
 	}
 
 	rrr_mmap_free(target->mmap, target);
@@ -633,18 +545,6 @@ int rrr_mmap_channel_new (struct rrr_mmap_channel **target, struct rrr_mmap *mma
 		}
 	}
 
-	if ((ret = rrr_posix_cond_init(&result->empty_cond, RRR_POSIX_MUTEX_IS_PSHARED)) != 0) {
-		RRR_MSG_0("Could not initialize empty condition mutex in rrr_mmap_channel_new: %i\n", ret);
-		ret = 1;
-		goto out_destroy_mutexes;
-	}
-
-	if ((ret = rrr_posix_cond_init(&result->full_cond, RRR_POSIX_MUTEX_IS_PSHARED)) != 0) {
-		RRR_MSG_0("Could not initialize full condition mutex in rrr_mmap_channel_new: %i\n", ret);
-		ret = 1;
-		goto out_destroy_empty_cond;
-	}
-
     strncpy(result->name, name, sizeof(result->name));
     result->name[sizeof(result->name) - 1] = '\0';
 
@@ -653,11 +553,8 @@ int rrr_mmap_channel_new (struct rrr_mmap_channel **target, struct rrr_mmap *mma
 	*target = result;
 	result = NULL;
 
-	// Remember to destroy mutex attributes
 	goto out;
 
-	out_destroy_empty_cond:
-		pthread_cond_destroy(&result->empty_cond);
 	out_destroy_mutexes:
 		// Be careful with the counters, we should only destroy initialized locks
 		for (mutex_i = mutex_i - 1; mutex_i >= 0; mutex_i--) {
