@@ -20,7 +20,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <poll.h>
-#include <pthread.h>
 #include <sys/ioctl.h>
 #include <arpa/inet.h>
 #include <unistd.h>
@@ -28,6 +27,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <string.h>
 
 #include "../log.h"
+#include "../allocator.h"
 
 #include "mqtt_common.h"
 #include "mqtt_connection.h"
@@ -42,6 +42,9 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../rrr_strerror.h"
 #include "../util/macro_utils.h"
 #include "../util/rrr_time.h"
+#include "../util/posix.h"
+
+#define RRR_MQTT_CONN_SEND_CHUNK_LIMIT_FACTOR 0.9
 
 static int __rrr_mqtt_connection_call_event_handler (struct rrr_mqtt_conn *connection, int event, int no_repeat, void *arg) {
 	int ret = RRR_MQTT_OK;
@@ -68,6 +71,18 @@ static int __rrr_mqtt_connection_call_event_handler (struct rrr_mqtt_conn *conne
 #define CALL_EVENT_HANDLER_NO_REPEAT(event)	\
 		__rrr_mqtt_connection_call_event_handler(connection, event, 1, NULL)
 
+int rrr_mqtt_conn_set_client_id (
+		struct rrr_mqtt_conn *connection,
+		const char *id
+) {
+	RRR_FREE_IF_NOT_NULL(connection->client_id);
+	if ((connection->client_id = rrr_strdup(id)) == NULL) {
+		RRR_MSG_0("Could not allocate memory in rrr_mqtt_conn_update_client_id\n");
+		return 1;
+	}
+	return 0;
+}
+
 int rrr_mqtt_conn_update_state (
 		struct rrr_mqtt_conn *connection,
 		struct rrr_mqtt_p *packet,
@@ -146,7 +161,7 @@ int rrr_mqtt_conn_iterator_ctx_send_disconnect (
 
 	// Check if CONNECT is not yet received
 	if (connection->protocol_version == NULL) {
-		goto out_nolock;
+		goto out_no_decref;
 	}
 
 	struct rrr_mqtt_p_disconnect *disconnect = (struct rrr_mqtt_p_disconnect *) rrr_mqtt_p_allocate (
@@ -156,16 +171,14 @@ int rrr_mqtt_conn_iterator_ctx_send_disconnect (
 	if (disconnect == NULL) {
 		RRR_MSG_0("Could not allocate DISCONNECT packet in rrr_mqtt_conn_iterator_ctx_send_disconnect\n");
 		ret = RRR_MQTT_INTERNAL_ERROR;
-		goto out_nolock;
+		goto out_no_decref;
 	}
 
-	RRR_MQTT_P_LOCK(disconnect);
-
 	disconnect->reason_v5 = connection->disconnect_reason_v5_;
 
 	// If a CONNACK is sent, we must not sent DISCONNECT packet
 	if (RRR_MQTT_CONN_STATE_SEND_ANY_IS_ALLOWED(connection)) {
-		if ((ret = rrr_mqtt_conn_iterator_ctx_send_packet (
+		if ((ret = rrr_mqtt_conn_iterator_ctx_send_packet_urgent (
 				handle,
 				(struct rrr_mqtt_p *) disconnect
 		)) != RRR_MQTT_OK) {
@@ -185,10 +198,9 @@ int rrr_mqtt_conn_iterator_ctx_send_disconnect (
 	}
 
 	out:
-	RRR_MQTT_P_UNLOCK(disconnect);
 	RRR_MQTT_P_DECREF(disconnect);
 
-	out_nolock:
+	out_no_decref:
 	// Force state transition even when sending disconnect packet fails
 	if (!RRR_MQTT_CONN_STATE_IS_CLOSED_OR_CLOSE_WAIT(connection)) {
 		RRR_DBG_1 ("Sending disconnect packet failed, force state transition to CLOSE_WAIT\n");
@@ -230,7 +242,7 @@ static void __rrr_mqtt_connection_destroy (struct rrr_mqtt_conn *connection) {
 	RRR_MQTT_P_DECREF_IF_NOT_NULL(connection->will_publish);
 	__rrr_mqtt_connection_will_properties_destroy(&connection->will_properties);
 
-	free(connection);
+	rrr_free(connection);
 }
 
 static void __rrr_mqtt_connection_destroy_void (void *arg) {
@@ -249,7 +261,7 @@ static int __rrr_mqtt_conn_new (
 	*connection = NULL;
 	struct rrr_mqtt_conn *res = NULL;
 
-	res = malloc(sizeof(*res));
+	res = rrr_allocate(sizeof(*res));
 	if (res == NULL) {
 		RRR_MSG_0("Could not allocate memory in rrr_mqtt_connection_new\n");
 		ret = RRR_MQTT_INTERNAL_ERROR;
@@ -294,7 +306,7 @@ static int __rrr_mqtt_conn_new (
 	goto out;
 
 	out_free:
-		free(res);
+		rrr_free(res);
 
 	out:
 		return ret;
@@ -418,7 +430,7 @@ int rrr_mqtt_conn_set_data_from_connect_and_connack (
 
 	if (username != NULL && *username != '\0') {
 		RRR_FREE_IF_NOT_NULL(connection->username);
-		if ((connection->username = strdup(username)) == NULL) {
+		if ((connection->username = rrr_strdup(username)) == NULL) {
 			RRR_MSG_0("Could not allocate memory for username in rrr_mqtt_conn_iterator_ctx_set_data_from_connect\n");
 			ret = RRR_MQTT_INTERNAL_ERROR;
 		}
@@ -484,25 +496,19 @@ int rrr_mqtt_conn_set_will_data_from_connect (
 			RRR_MQTT_PROPERTY_USER_PROPERTY
 	};
 
-	RRR_MQTT_P_LOCK_IN(publish);
-		RRR_MQTT_P_PUBLISH_SET_FLAG_QOS(publish, RRR_MQTT_P_CONNECT_GET_FLAG_WILL_QOS(connect));
-		RRR_MQTT_P_PUBLISH_SET_FLAG_RETAIN(publish, RRR_MQTT_P_CONNECT_GET_FLAG_WILL_RETAIN(connect));
+	RRR_MQTT_P_PUBLISH_SET_FLAG_QOS(publish, RRR_MQTT_P_CONNECT_GET_FLAG_WILL_QOS(connect));
+	RRR_MQTT_P_PUBLISH_SET_FLAG_RETAIN(publish, RRR_MQTT_P_CONNECT_GET_FLAG_WILL_RETAIN(connect));
 
-		publish->will_delay_interval = connection->will_properties.will_delay_interval;
+	publish->will_delay_interval = connection->will_properties.will_delay_interval;
 
-		if (rrr_mqtt_property_collection_add_selected_from_collection (
-				&publish->properties,
-				&connect->will_properties,
-				will_property_list,
-				sizeof(will_property_list)/sizeof(*will_property_list)
-		) != 0) {
-			RRR_MSG_0("Error while copying will properties to publish in rrr_mqtt_conn_set_will_data_from_connect\n");
-			ret = 1;
-			RRR_MQTT_P_LOCK_BREAK();
-		}
-	RRR_MQTT_P_LOCK_OUT(publish);
-
-	if (ret != 0) {
+	if (rrr_mqtt_property_collection_add_selected_from_collection (
+			&publish->properties,
+			&connect->will_properties,
+			will_property_list,
+			sizeof(will_property_list)/sizeof(*will_property_list)
+	) != 0) {
+		RRR_MSG_0("Error while copying will properties to publish in rrr_mqtt_conn_set_will_data_from_connect\n");
+		ret = 1;
 		goto out;
 	}
 
@@ -514,28 +520,30 @@ int rrr_mqtt_conn_set_will_data_from_connect (
 	return ret;
 }
 
-int rrr_mqtt_conn_housekeeping (
-		struct rrr_mqtt_conn *connection,
-		void *rrr_mqtt_conn_iterator_ctx_housekeeping_callback_data
+int rrr_mqtt_conn_iterator_ctx_housekeeping (
+		struct rrr_net_transport_handle *handle,
+		int (*exceeded_keep_alive_callback)(struct rrr_net_transport_handle *handle, void *arg),
+		void *callback_arg
 ) {
-	int ret = RRR_MQTT_OK;
+	RRR_MQTT_DEFINE_CONN_FROM_HANDLE_AND_CHECK;
 
-	struct rrr_mqtt_conn_iterator_ctx_housekeeping_callback_data *callback_data = rrr_mqtt_conn_iterator_ctx_housekeeping_callback_data;
+	int ret = RRR_MQTT_OK;
 
 	if (connection->keep_alive > 0) {
 		uint64_t limit_ping = (double) connection->keep_alive;
 		uint64_t limit = (double) connection->keep_alive * 1.5;
+
 		limit_ping *= 1000000;
 		limit *= 1000000;
-		if (connection->last_write_time + limit < rrr_time_get_64() || connection->last_read_time + limit < rrr_time_get_64()) {
+		if (connection->last_read_time + limit < rrr_time_get_64()) {
 			RRR_DBG_1("Keep-alive exceeded for connection\n");
 			ret = RRR_MQTT_SOFT_ERROR;
 			goto out;
 		}
-		else if (callback_data->exceeded_keep_alive_callback != NULL &&
+		else if (exceeded_keep_alive_callback != NULL &&
 				(connection->last_read_time + limit_ping < rrr_time_get_64() ||
 				connection->last_write_time + limit_ping < rrr_time_get_64()) &&
-				(ret = callback_data->exceeded_keep_alive_callback(connection, callback_data->callback_arg)) != RRR_MQTT_OK
+				(ret = exceeded_keep_alive_callback(handle, callback_arg)) != RRR_MQTT_OK
 		) {
 			RRR_MSG_0("Error from callback in rrr_mqtt_conn_iterator_ctx_housekeeping after exceeded keep-alive\n");
 			goto out;
@@ -569,39 +577,35 @@ void rrr_mqtt_conn_accept_and_connect_callback (
 		goto out;
 	}
 
-//	new_connection->transport = handle->transport;
-	new_connection->transport_handle = handle->handle;
+	rrr_net_transport_ctx_handle_application_data_bind(handle, new_connection, __rrr_mqtt_connection_destroy_void);
+	rrr_net_transport_ctx_handle_pre_destroy_function_set(handle, __rrr_mqtt_connection_in_iterator_disconnect);
 
-	handle->application_private_ptr = new_connection;
-	handle->application_ptr_destroy = __rrr_mqtt_connection_destroy_void;
-	handle->application_ptr_iterator_pre_destroy = __rrr_mqtt_connection_in_iterator_disconnect;
-
-	callback_data->transport_handle = handle->handle;
+	new_connection->transport_handle = callback_data->transport_handle = RRR_NET_TRANSPORT_CTX_HANDLE(handle);
 
 	out:
 	return;
 }
 
-int rrr_mqtt_conn_iterator_ctx_check_alive_callback (
-		struct rrr_net_transport_handle *handle,
-		void *rrr_mqtt_conn_check_alive_callback_data
+int rrr_mqtt_conn_iterator_ctx_check_alive (
+		int *alive,
+		int *send_allowed,
+		struct rrr_net_transport_handle *handle
 ) {
 	RRR_MQTT_DEFINE_CONN_FROM_HANDLE_AND_CHECK;
 
 	int ret = RRR_MQTT_OK;
 
-	struct rrr_mqtt_conn_check_alive_callback_data *callback_data = rrr_mqtt_conn_check_alive_callback_data;
-	callback_data->alive = 1;
+	*alive = 1;
 
 	if (RRR_MQTT_CONN_STATE_IS_CLOSED_OR_CLOSE_WAIT(connection) ||
 		RRR_MQTT_CONN_STATE_IS_CLOSED(connection) ||
 		connection->session == NULL
 	) {
-		callback_data->alive = 0;
+		*alive = 0;
 	}
 
 	if (RRR_MQTT_CONN_STATE_SEND_ANY_IS_ALLOWED(connection)) {
-		callback_data->send_allowed = 1;
+		*send_allowed = 1;
 	}
 
 	return ret;
@@ -674,7 +678,7 @@ static int __rrr_mqtt_conn_read_get_target_size (
 	int ret = RRR_SOCKET_READ_INCOMPLETE;
 
 	struct rrr_mqtt_conn_read_callback_data *callback_data = arg;
-	struct rrr_mqtt_conn *connection = callback_data->handle->application_private_ptr;
+	struct rrr_mqtt_conn *connection = RRR_NET_TRANSPORT_CTX_PRIVATE_PTR(callback_data->handle);
 
 //	printf ("get target size in %p wpos %li target size %li buf size %li\n",
 //			read_session, read_session->rx_buf_wpos, read_session->target_size, read_session->rx_buf_size);
@@ -685,6 +689,7 @@ static int __rrr_mqtt_conn_read_get_target_size (
 				RRR_MQTT_CONN_SET_DISCONNECT_REASON_IF_ZERO(connection, RRR_MQTT_P_5_REASON_MALFORMED_PACKET);
 			}
 			ret &= ~(RRR_SOCKET_READ_INCOMPLETE);
+			RRR_MSG_0("Returned error from __rrr_mqtt_conn_parse: %i\n", ret);
 			goto out;
 		}
 		// Don't got out, fixed header might be done
@@ -710,7 +715,7 @@ static int __rrr_mqtt_conn_read_complete_callback (
 	struct rrr_mqtt_p *packet = NULL;
 
 	struct rrr_mqtt_conn_read_callback_data *callback_data = arg;
-	struct rrr_mqtt_conn *connection = callback_data->handle->application_private_ptr;
+	struct rrr_mqtt_conn *connection = RRR_NET_TRANSPORT_CTX_PRIVATE_PTR(callback_data->handle);
 
 //	printf ("read_complete %p wpos %li target size %li buf size %li\n",
 //			read_session, read_session->rx_buf_wpos, read_session->target_size, read_session->rx_buf_size);
@@ -773,7 +778,6 @@ static int __rrr_mqtt_conn_read_complete_callback (
 			packet,
 			callback_data->handler_callback_arg
 	)) != 0) {
-		RRR_MSG_0("Error from handler callback in __rrr_mqtt_conn_read_complete_callback, return was %i\n", ret);
 		goto out;
 	}
 
@@ -806,63 +810,64 @@ int rrr_mqtt_conn_iterator_ctx_read (
 			handler_callback_arg
 	};
 
-	int consecutive_nothing_happened = 0;
-	int read_loops = 0;
-
-	// TODO : Make this better
-	// Do this 60 times as we send 50 packets at a time (10 more)
-	for (int i = 0; i < read_per_round_max; i++) {
-		uint64_t prev_bytes_read = handle->bytes_read_total;
-
+	do {
 		if ((ret = rrr_net_transport_ctx_read_message (
 				handle,
 				2, // Read two times this round
 				2, // Read only two bytes the first time
 				read_step_max_size,
 				0, // No max read size
+				100 * 1000, // 100 ms ratelimit interval
+				(1 * 1024 * 1024) / 20, // 1/20 MB
 				__rrr_mqtt_conn_read_get_target_size,
 				&callback_data,
 				__rrr_mqtt_conn_read_complete_callback,
 				&callback_data
 		)) != 0) {
+			if (ret == RRR_NET_TRANSPORT_READ_RATELIMIT) {
+				ret = RRR_SOCKET_READ_INCOMPLETE;
+			}
+			else if (ret != RRR_SOCKET_READ_INCOMPLETE && connection->disconnect_reason_v5_ == 0) {
+				connection->disconnect_reason_v5_ = RRR_MQTT_P_5_REASON_UNSPECIFIED_ERROR;
+			}
 			goto out;
 		}
-
-		if (prev_bytes_read == handle->bytes_read_total && ++consecutive_nothing_happened > 5) {
-			// Nothing was read
-			break;
-		}
-		read_loops++;
-	}
-
-	if (read_loops > 0) {
-//		printf("Read loops: %i\n", read_loops);
-	}
+	} while (--read_per_round_max && ret == 0);
 
 	out:
-	if (ret != 0) {
-		if (connection->disconnect_reason_v5_ == 0) {
-			connection->disconnect_reason_v5_ = RRR_MQTT_P_5_REASON_UNSPECIFIED_ERROR;
-		}
-	}
 	return ret;
 }
 
-static int __rrr_mqtt_conn_iterator_ctx_write (
+static int __rrr_mqtt_conn_iterator_ctx_send_push (
 		struct rrr_net_transport_handle *handle,
-		const char *data,
+		void **data,
 		ssize_t data_size
 ) {
 	RRR_MQTT_DEFINE_CONN_FROM_HANDLE_AND_CHECK;
 
 	int ret = 0;
 
-	if ((ret = rrr_net_transport_ctx_send_nonblock(handle, data, data_size)) != 0) {
-		if (ret == RRR_NET_TRANSPORT_SEND_INCOMPLETE) {
-			ret = RRR_MQTT_INCOMPLETE;
-			goto out;
-		}
-		RRR_MSG_0("Error while sending packet in __rrr_mqtt_conn_iterator_ctx_write\n");
+	if ((ret = rrr_net_transport_ctx_send_push(handle, data, data_size)) != 0) {
+		RRR_MSG_0("Error while pushing packet to send queue in __rrr_mqtt_conn_iterator_ctx_write\n");
+		ret = RRR_MQTT_SOFT_ERROR;
+		goto out;
+	}
+
+	out:
+	return ret;
+}
+
+static int __rrr_mqtt_conn_iterator_ctx_send_push_urgent (
+		struct rrr_net_transport_handle *handle,
+		void **data,
+		ssize_t data_size
+) {
+	RRR_MQTT_DEFINE_CONN_FROM_HANDLE_AND_CHECK;
+
+	int ret = 0;
+
+	if ((ret = rrr_net_transport_ctx_send_push_urgent(handle, data, data_size)) != 0) {
+		RRR_MSG_0("Error while sending packet in __rrr_mqtt_conn_iterator_ctx_send_urgent\n");
 		ret = RRR_MQTT_SOFT_ERROR;
 		goto out;
 	}
@@ -896,9 +901,11 @@ int __rrr_mqtt_connection_create_variable_int (
 	return RRR_MQTT_OK;
 }
 
-int rrr_mqtt_conn_iterator_ctx_send_packet (
+static int __rrr_mqtt_conn_iterator_ctx_send_packet (
+		int *do_stop,
 		struct rrr_net_transport_handle *handle,
-		struct rrr_mqtt_p *packet
+		struct rrr_mqtt_p *packet,
+		int urgent
 ) {
 	RRR_MQTT_DEFINE_CONN_FROM_HANDLE_AND_CHECK;
 
@@ -908,6 +915,11 @@ int rrr_mqtt_conn_iterator_ctx_send_packet (
 	struct rrr_mqtt_p_payload *payload = NULL;
 	char *network_data = NULL;
 	ssize_t network_size = 0;
+	void *send_data = NULL;
+
+	if (!RRR_MQTT_CONN_STATE_SEND_ANY_IS_ALLOWED(connection) && RRR_MQTT_P_GET_TYPE(packet) == RRR_MQTT_P_TYPE_PUBLISH) {
+		RRR_BUG("BUG: Tried to send PUBLISH while not allowed in __rrr_mqtt_conn_iterator_ctx_send_packet\n");
+	}
 
 	// Packets which originate from other hosts might have different protocol
 	// version.
@@ -932,7 +944,7 @@ int rrr_mqtt_conn_iterator_ctx_send_packet (
 		}
 
 		if (network_size == 0) {
-			free(network_data);
+			rrr_free(network_data);
 			network_data = NULL;
 		}
 
@@ -969,7 +981,6 @@ int rrr_mqtt_conn_iterator_ctx_send_packet (
 	ssize_t payload_length = 0;
 	payload = packet->payload;
 	if (payload != NULL) {
-		RRR_MQTT_P_LOCK(packet->payload);
 		payload_length = packet->payload->length;
 	}
 
@@ -985,36 +996,34 @@ int rrr_mqtt_conn_iterator_ctx_send_packet (
 
 	ssize_t total_size = 1 + variable_int_length + packet->assembled_data_size + payload_length;
 
-	RRR_DBG_3("Sending packet %p of type %s flen: 1, vlen: %li, alen: %li, plen: %li, total: %li, id: %u\n",
+	RRR_DBG_3("Sending packet %p of type %s flen: 1, vlen: %li, alen: %li, plen: %li, total: %li, id: %u, urgent: %i\n",
 			packet,
 			RRR_MQTT_P_GET_TYPE_NAME(packet),
 			variable_int_length,
 			packet->assembled_data_size,
 			payload_length,
 			total_size,
-			RRR_MQTT_P_GET_IDENTIFIER(packet)
+			RRR_MQTT_P_GET_IDENTIFIER(packet),
+			urgent
 	);
 
 	__rrr_mqtt_connection_update_last_write_time(connection);
 
-	if ((ret = __rrr_mqtt_conn_iterator_ctx_write (handle, (char*) &header, sizeof(header.type) + variable_int_length)) != 0) {
-		if (ret == RRR_MQTT_INCOMPLETE) {
-			RRR_DBG_3("Note: Connection busy while sending fixed header in rrr_mqtt_conn_iterator_ctx_send_packet\n");
-			ret = RRR_MQTT_OK;
-			goto out;
-		}
-
-		RRR_MSG_0("Error while sending fixed header in rrr_mqtt_conn_iterator_ctx_send_packet\n");
+	const size_t send_size = sizeof(header.type) + variable_int_length + packet->assembled_data_size + (payload != NULL ? payload->length : 0);
+	if ((send_data = rrr_allocate(send_size)) == NULL) {
+		RRR_MSG_0("Failed to allocate send data in rrr_mqtt_conn_iterator_ctx_send_packet\n");
+		ret = 1;
 		goto out;
 	}
 
+	void *send_data_pos = send_data;
+
+	memcpy(send_data_pos, (char*) &header, sizeof(header.type) + variable_int_length);
+	send_data_pos += sizeof(header.type) + variable_int_length;
+
 	if (packet->assembled_data_size > 0) {
-		if ((ret = __rrr_mqtt_conn_iterator_ctx_write (handle, packet->_assembled_data, packet->assembled_data_size)) != 0) {
-			// TODO : Recover from this?
-			RRR_MSG_0("Error: Error while sending assembled data in rrr_mqtt_conn_iterator_ctx_send_packet. Fixed data was already sent, cannot recover from this. Return was %i.\n", ret);
-			ret = RRR_MQTT_SOFT_ERROR;
-			goto out;
-		}
+		memcpy(send_data_pos, packet->_assembled_data, packet->assembled_data_size);
+		send_data_pos += packet->assembled_data_size;
 	}
 	else if (payload != NULL) {
 		RRR_BUG("Payload was present without variable header in rrr_mqtt_conn_iterator_ctx_send_packet\n");
@@ -1024,10 +1033,21 @@ int rrr_mqtt_conn_iterator_ctx_send_packet (
 		if (payload_length == 0) {
 			RRR_BUG("Payload size was 0 but payload pointer was not NULL in rrr_mqtt_conn_iterator_ctx_send_packet\n");
 		}
-		if ((ret = __rrr_mqtt_conn_iterator_ctx_write (handle, payload->payload_start, payload->length)) != 0) {
-			RRR_MSG_0("Error while sending payload data in rrr_mqtt_conn_iterator_ctx_send_packet\n");
-			goto out;
-		}
+		memcpy(send_data_pos, payload->payload_start, payload->length);
+	}
+
+	int (*send_method)(
+			struct rrr_net_transport_handle *handle,
+			void **data,
+			ssize_t data_size
+	) = (urgent
+		? __rrr_mqtt_conn_iterator_ctx_send_push_urgent
+		: __rrr_mqtt_conn_iterator_ctx_send_push
+	);
+
+	if ((ret = send_method (handle, &send_data, send_size)) != 0) {
+		RRR_MSG_0("Error while pushing data in rrr_mqtt_conn_iterator_ctx_send_packet\n");
+		goto out;
 	}
 
 	ret = rrr_mqtt_conn_update_state (
@@ -1042,10 +1062,29 @@ int rrr_mqtt_conn_iterator_ctx_send_packet (
 
 	packet->last_attempt = rrr_time_get_64();
 
-	out:
-	if (payload != NULL) {
-		RRR_MQTT_P_UNLOCK(packet->payload);
+	if (!urgent && rrr_net_transport_ctx_send_waiting_chunk_limit_factor(handle) >= RRR_MQTT_CONN_SEND_CHUNK_LIMIT_FACTOR) {
+		*do_stop = 1;
 	}
+
+	out:
+	RRR_FREE_IF_NOT_NULL(send_data);
 	RRR_FREE_IF_NOT_NULL(network_data);
 	return ret | ret_destroy;
 }
+
+int rrr_mqtt_conn_iterator_ctx_send_packet (
+		int *do_stop,
+		struct rrr_net_transport_handle *handle,
+		struct rrr_mqtt_p *packet
+) {
+	return __rrr_mqtt_conn_iterator_ctx_send_packet (do_stop, handle, packet, 0 /* Not urgent */);
+}
+
+int rrr_mqtt_conn_iterator_ctx_send_packet_urgent (
+		struct rrr_net_transport_handle *handle,
+		struct rrr_mqtt_p *packet
+) {
+	int do_stop_dummy = 0;
+
+	return __rrr_mqtt_conn_iterator_ctx_send_packet (&do_stop_dummy, handle, packet, 1 /* Urgent */);
+}
