@@ -23,17 +23,24 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <poll.h>
 #include <string.h>
 #include <errno.h>
+#include <limits.h>
+#include <stdlib.h>
 
 #define RRR_NET_TRANSPORT_H_ENABLE_INTERNALS
 
+#include "../log.h"
+#include "../allocator.h"
+
 #include "net_transport_libressl.h"
 #include "net_transport_tls_common.h"
+#include "net_transport_struct.h"
 #include "net_transport.h"
 
-#include "../log.h"
 #include "../rrr_strerror.h"
 #include "../util/macro_utils.h"
+#include "../util/posix.h"
 #include "../ip/ip.h"
+#include "../ip/ip_util.h"
 #include "../ip/ip_accept_data.h"
 
 static void __rrr_net_transport_libressl_destroy (
@@ -44,7 +51,6 @@ static void __rrr_net_transport_libressl_destroy (
 	if (tls->config != NULL) {
 		tls_config_free(tls->config);
 	}
-
 	rrr_net_transport_tls_common_destroy(tls);
 }
 
@@ -64,7 +70,9 @@ static void __rrr_net_transport_libressl_data_destroy (
 		rrr_ip_close(&data->ip_data);
 	}
 
-	free(data);
+	RRR_FREE_IF_NOT_NULL(data->alpn_selected_proto);
+
+	rrr_free(data);
 }
 
 static int __rrr_net_transport_libressl_data_new (
@@ -74,7 +82,7 @@ static int __rrr_net_transport_libressl_data_new (
 
 	*result = NULL;
 
-	struct rrr_net_transport_tls_data *data = malloc(sizeof(*data));
+	struct rrr_net_transport_tls_data *data = rrr_allocate(sizeof(*data));
 	if (data == NULL) {
 		RRR_MSG_0("Could not allocate memory in __rrr_net_transport_libressl_data_new\n");
 		ret = 1;
@@ -88,14 +96,47 @@ static int __rrr_net_transport_libressl_data_new (
 	out:
 	return ret;
 }
+/*
+static int __rrr_net_transport_libressl_handshake_perform (
+		struct tls *ctx
+) {
+	int ret = 0;
 
+	int handshake_max_retry = 1000;
+
+	while (--handshake_max_retry) {
+		ret = tls_handshake(ctx);
+		if (ret == TLS_WANT_POLLIN || ret == TLS_WANT_POLLOUT) {
+			// OK, retry
+		}
+		else if (ret != 0) {
+			RRR_MSG_0("Error during TLS handshake: %s\n", tls_error(ctx));
+			ret = RRR_READ_SOFT_ERROR;
+			goto out;
+		}
+		else {
+			break;
+		}
+		rrr_posix_usleep(100);
+	}
+
+	if (handshake_max_retry == 0) {
+		RRR_MSG_0("TLS handshake timeout\n");
+		ret = RRR_READ_SOFT_ERROR;
+		goto out;
+	}
+
+	out:
+	return ret;
+}
+*/
 struct rrr_net_transport_libressl_connect_callback_data {
 	struct rrr_net_transport_tls *tls;
 	struct rrr_ip_accept_data *accept_data;
 	const char *server_name;
 };
 
-int __rrr_net_transport_libressl_connect_callback (
+static int __rrr_net_transport_libressl_connect_callback (
 		RRR_NET_TRANSPORT_BIND_AND_LISTEN_CALLBACK_ARGS
 ) {
 	struct rrr_net_transport_libressl_connect_callback_data *callback_data = arg;
@@ -107,11 +148,10 @@ int __rrr_net_transport_libressl_connect_callback (
 	struct rrr_net_transport_tls_data *data = NULL;
 
 	*submodule_private_ptr = NULL;
-	*submodule_private_fd = 0;
+	*submodule_fd = 0;
 
 	if ((ret = __rrr_net_transport_libressl_data_new(&data)) != 0) {
 		RRR_MSG_0("Could not create TLS data in __rrr_net_transport_libressl_connect_callback\n");
-		ret = 1;
 		goto out;
 	}
 
@@ -138,8 +178,12 @@ int __rrr_net_transport_libressl_connect_callback (
 		goto out_destroy_data;
 	}
 
+	data->sockaddr = callback_data->accept_data->addr;
+	data->socklen = callback_data->accept_data->len;
+	data->ip_data = callback_data->accept_data->ip_data;
+
 	*submodule_private_ptr = data;
-	*submodule_private_fd = callback_data->accept_data->ip_data.fd;
+	*submodule_fd = callback_data->accept_data->ip_data.fd;
 
 	goto out;
 	out_config_error:
@@ -165,7 +209,7 @@ static int __rrr_net_transport_libressl_connect (
 
 	struct rrr_ip_accept_data *accept_data = NULL;
 
-	if (rrr_ip_network_connect_tcp_ipv4_or_ipv6(&accept_data, port, host, NULL) != 0) {
+	if (rrr_ip_network_connect_tcp_ipv4_or_ipv6(&accept_data, port, host) != 0) {
 		RRR_DBG_1("Could not create TLS connection to %s:%u\n", host, port);
 		ret = RRR_NET_TRANSPORT_READ_SOFT_ERROR;
 		goto out;
@@ -190,8 +234,7 @@ static int __rrr_net_transport_libressl_connect (
 			__rrr_net_transport_libressl_connect_callback,
 			&callback_data
 	)) != 0) {
-		RRR_MSG_0("Could not get handle in __rrr_net_transport_libressl_accept return was %i\n", ret);
-		ret = 1;
+		RRR_MSG_0("Could not get handle in __rrr_net_transport_libressl_connect return was %i\n", ret);
 		goto out_destroy_ip;
 	}
 
@@ -210,6 +253,7 @@ static int __rrr_net_transport_libressl_connect (
 struct rrr_net_transport_libressl_bind_and_listen_callback_data {
 	struct rrr_net_transport_tls *tls;
 	unsigned int port;
+	int do_ipv6;
 };
 
 static int __rrr_net_transport_libressl_bind_and_listen_callback (
@@ -224,7 +268,7 @@ static int __rrr_net_transport_libressl_bind_and_listen_callback (
 	struct rrr_net_transport_tls_data *data = NULL;
 
 	*submodule_private_ptr = NULL;
-	*submodule_private_fd = 0;
+	*submodule_fd = 0;
 
 	if ((ret = __rrr_net_transport_libressl_data_new(&data)) != 0) {
 		RRR_MSG_0("Could not create TLS data in __rrr_net_transport_libressl_bind_and_listen_callback\n");
@@ -234,8 +278,8 @@ static int __rrr_net_transport_libressl_bind_and_listen_callback (
 
 	data->ip_data.port = callback_data->port;
 
-	if (rrr_ip_network_start_tcp_ipv4_and_ipv6 (&data->ip_data, 10) != 0) {
-		RRR_MSG_0("Could not start IP listening in __rrr_net_transport_libressl_bind_and_listen_callback\n");
+	if (rrr_ip_network_start_tcp (&data->ip_data, 10, callback_data->do_ipv6) != 0) {
+		RRR_DBG_1("Note: Could not start IP listening in __rrr_net_transport_libressl_bind_and_listen_callback\n");
 		ret = 1;
 		goto out_destroy_data;
 	}
@@ -253,7 +297,7 @@ static int __rrr_net_transport_libressl_bind_and_listen_callback (
 	}
 
 	*submodule_private_ptr = data;
-	*submodule_private_fd = data->ip_data.fd;
+	*submodule_fd = data->ip_data.fd;
 
 	goto out;
 	out_config_error:
@@ -280,7 +324,8 @@ static int __rrr_net_transport_libressl_bind_and_listen (
 
 	struct rrr_net_transport_libressl_bind_and_listen_callback_data callback_data = {
 			tls,
-			port
+			port,
+			do_ipv6
 	};
 
 	int new_handle;
@@ -291,11 +336,11 @@ static int __rrr_net_transport_libressl_bind_and_listen (
 			__rrr_net_transport_libressl_bind_and_listen_callback,
 			&callback_data
 	)) != 0) {
-		RRR_MSG_0("Could not get handle in __rrr_net_transport_libressl_bind_and_listen return was %i\n", ret);
-		ret = 1;
 		goto out;
 	}
 
+	RRR_DBG_7("LibreSSL listening started on port %u transport handle %p/%i\n", port, transport, new_handle);
+
 	ret = callback (
 			transport,
 			new_handle,
@@ -322,20 +367,15 @@ int __rrr_net_transport_libressl_accept_callback (
 	int ret = 0;
 
 	*submodule_private_ptr = NULL;
-	*submodule_private_fd = 0;
+	*submodule_fd = 0;
 
 	struct rrr_net_transport_tls_data *new_data = NULL;
 
 	if ((ret = __rrr_net_transport_libressl_data_new(&new_data)) != 0) {
 		RRR_MSG_0("Could not allocate memory for SSL data in __rrr_net_transport_libressl_accept_callback\n");
-		ret = 1;
 		goto out;
 	}
 
-	new_data->sockaddr = callback_data->accept_data->addr;
-	new_data->socklen = callback_data->accept_data->len;
-	new_data->ip_data = callback_data->accept_data->ip_data;
-
 	if (tls_accept_socket (
 			callback_data->ctx,
 			&new_data->ctx,
@@ -346,8 +386,12 @@ int __rrr_net_transport_libressl_accept_callback (
 		goto out_destroy_data;
 	}
 
+	new_data->sockaddr = callback_data->accept_data->addr;
+	new_data->socklen = callback_data->accept_data->len;
+	new_data->ip_data = callback_data->accept_data->ip_data;
+
 	*submodule_private_ptr = new_data;
-	*submodule_private_fd = callback_data->accept_data->ip_data.fd;
+	*submodule_fd = callback_data->accept_data->ip_data.fd;
 
 	goto out;
 	out_destroy_data:
@@ -364,6 +408,8 @@ int __rrr_net_transport_libressl_accept (
 
 	int ret = 0;
 
+	*did_accept = 0;
+
 	struct rrr_ip_accept_data *accept_data = NULL;
 
 	if ((ret = rrr_ip_accept(&accept_data, &data->ip_data, "net_transport_tls", 0)) != 0) {
@@ -391,10 +437,16 @@ int __rrr_net_transport_libressl_accept (
 			&callback_data
 	)) != 0) {
 		RRR_MSG_0("Could not get handle in __rrr_net_transport_libressl_accept return was %i\n", ret);
-		ret = 1;
 		goto out_destroy_ip;
 	}
 
+	{
+		char buf[128];
+		rrr_ip_to_str(buf, sizeof(buf), (const struct sockaddr *) &accept_data->addr, accept_data->len);
+		RRR_DBG_7("LibreSSL accepted connection on port %u from %s transport handle %p/%i\n",
+				data->ip_data.port, buf, listen_handle->transport, new_handle);
+	}
+
 	ret = callback (
 			listen_handle->transport,
 			new_handle,
@@ -405,6 +457,8 @@ int __rrr_net_transport_libressl_accept (
 			callback_arg
 	);
 
+	*did_accept = 1;
+
 	goto out;
 	out_destroy_ip:
 		rrr_ip_close(&accept_data->ip_data);
@@ -418,6 +472,32 @@ static int __rrr_net_transport_libressl_close (struct rrr_net_transport_handle *
 	return 0;
 }
 
+static int __rrr_net_transport_libressl_read_raw (
+		char *buf,
+		ssize_t *read_bytes,
+		struct rrr_net_transport_tls_data *tls_data,
+		ssize_t read_step_max_size
+) {
+	int ret = RRR_READ_OK;
+
+	ssize_t result = tls_read(tls_data->ctx, buf, read_step_max_size);
+	if (result == 0) {
+		ret = RRR_READ_EOF;
+	}
+	else if (result < 0) {
+		if (result == TLS_WANT_POLLIN || result == TLS_WANT_POLLOUT) {
+			goto out;
+		}
+
+		RRR_MSG_0("Error while reading in __rrr_net_transport_libressl_read_raw: %s\n", tls_error(tls_data->ctx));
+		ret = RRR_READ_SOFT_ERROR;
+	}
+
+	out:
+	*read_bytes = (result >= 0 ? result : 0);
+	return ret;
+}
+
 static int __rrr_net_transport_libressl_read_read (
 		char *buf,
 		ssize_t *read_bytes,
@@ -427,26 +507,11 @@ static int __rrr_net_transport_libressl_read_read (
 	struct rrr_net_transport_read_callback_data *callback_data = private_arg;
 	struct rrr_net_transport_tls_data *tls_data = callback_data->handle->submodule_private_ptr;
 
-	int ret = RRR_READ_OK;
-
-	ssize_t result = tls_read(tls_data->ctx, buf, read_step_max_size);
-	if (result < 0) {
-		if (result == TLS_WANT_POLLIN || result == TLS_WANT_POLLOUT) {
-			goto out;
-		}
-
-		RRR_MSG_0("Error while reading in __rrr_net_transport_libressl_read_read: %s\n", tls_error(tls_data->ctx));
-		ret = RRR_READ_SOFT_ERROR;
-		goto out;
-	}
-
-	out:
-	*read_bytes = (result >= 0 ? result : 0);
-	return ret;
+	return __rrr_net_transport_libressl_read_raw(buf, read_bytes, tls_data, read_step_max_size);
 }
 
 static int __rrr_net_transport_libressl_read_message (
-		RRR_NET_TRANSPORT_READ_ARGS
+		RRR_NET_TRANSPORT_READ_MESSAGE_ARGS
 ) {
 	int ret = 0;
 
@@ -467,6 +532,9 @@ static int __rrr_net_transport_libressl_read_message (
 				read_step_initial,
 				read_step_max_size,
 				read_max_size,
+				RRR_LL_FIRST(&handle->read_sessions),
+				ratelimit_interval_us,
+				ratelimit_max_bytes,
 				rrr_net_transport_tls_common_read_get_target_size,
 				rrr_net_transport_tls_common_read_complete_callback,
 				__rrr_net_transport_libressl_read_read,
@@ -481,7 +549,7 @@ static int __rrr_net_transport_libressl_read_message (
 		if (ret == RRR_NET_TRANSPORT_READ_INCOMPLETE) {
 			continue;
 		}
-		else if (ret == RRR_NET_TRANSPORT_READ_OK) {
+		else if (ret == RRR_NET_TRANSPORT_READ_OK || ret == RRR_NET_TRANSPORT_READ_RATELIMIT) {
 			break;
 		}
 		else {
@@ -494,6 +562,31 @@ static int __rrr_net_transport_libressl_read_message (
 	return ret;
 }
 
+static int __rrr_net_transport_libressl_read (
+		RRR_NET_TRANSPORT_READ_ARGS
+) {
+	int ret = RRR_NET_TRANSPORT_READ_OK;
+
+	if (buf_size > SSIZE_MAX) {
+		RRR_MSG_0("Buffer size too large in __rrr_net_transport_libressl_read\n");
+		ret = RRR_NET_TRANSPORT_READ_HARD_ERROR;
+		goto out;
+	}
+
+	ssize_t bytes_read_s = 0;
+
+	ret = __rrr_net_transport_libressl_read_raw(buf, &bytes_read_s, handle->submodule_private_ptr, buf_size);
+
+	if (bytes_read_s < 0) {
+		RRR_BUG("BUG: Negative bytes read value in __rrr_net_transport_libressl_read\n");
+	}
+
+	*bytes_read = bytes_read_s;
+
+	out:
+	return ret;
+}
+
 static int __rrr_net_transport_libressl_send (
 	uint64_t *sent_bytes,
 	struct rrr_net_transport_handle *handle,
@@ -504,22 +597,24 @@ static int __rrr_net_transport_libressl_send (
 
 	*sent_bytes = 0;
 
-	int ret = RRR_NET_TRANSPORT_SEND_SOFT_ERROR;
+	int ret = RRR_NET_TRANSPORT_SEND_INCOMPLETE;
 
-	int retries = 10;
+	int retries = 1000;
 	ssize_t size_remaining = size;
 	struct pollfd pfd = {0};
 
-	pfd.fd = handle->submodule_private_fd;
+	pfd.fd = handle->submodule_fd;
 	pfd.events = POLLIN|POLLOUT;
 	while (size_remaining > 0 && --retries > 0) {
 		int ret_tmp = poll(&pfd, 1, 0);
 		if (ret_tmp == -1) {
-			RRR_MSG_1("Poll failed for TLS fd %i while writing: %s\n", pfd.fd, rrr_strerror(errno));
+			RRR_DBG_7("Poll failed for TLS fd %i while writing: %s\n", pfd.fd, rrr_strerror(errno));
+			ret = RRR_NET_TRANSPORT_SEND_HARD_ERROR;
 			goto out;
 		}
 		else if ((pfd.revents & (POLLERR|POLLNVAL))) {
-			RRR_MSG_1("Bad file descriptor for TLS fd %i while writing\n", pfd.fd);
+			RRR_DBG_7("Bad file descriptor for TLS fd %i while writing, maybe remote has closed the connection\n", pfd.fd);
+			ret = RRR_NET_TRANSPORT_SEND_HARD_ERROR;
 			goto out;
 		}
 		else if ((pfd.revents & (pfd.events|POLLHUP))) {
@@ -533,7 +628,8 @@ static int __rrr_net_transport_libressl_send (
 				pfd.events = POLLOUT;
 			}
 			else if (bytes == -1) {
-				RRR_MSG_1("Error while writing to TLS fd %i: %s\n", pfd.fd, tls_error(tls_data->ctx));
+				RRR_DBG_7("Error while writing to TLS fd %i: %s\n", pfd.fd, tls_error(tls_data->ctx));
+				ret = RRR_NET_TRANSPORT_SEND_HARD_ERROR;
 				goto out;
 			}
 			else {
@@ -547,6 +643,8 @@ static int __rrr_net_transport_libressl_send (
 				break;
 			}
 		}
+		pthread_testcancel();
+		rrr_posix_usleep(1); // Schedule
 	}
 
 	out:
@@ -556,7 +654,36 @@ static int __rrr_net_transport_libressl_send (
 static int __rrr_net_transport_libressl_poll (
 		struct rrr_net_transport_handle *handle
 ) {
-	return rrr_socket_check_alive (handle->submodule_private_fd);
+	return rrr_socket_check_alive (handle->submodule_fd);
+}
+
+static int __rrr_net_transport_libressl_handshake (
+		struct rrr_net_transport_handle *handle
+) {
+	struct rrr_net_transport_tls_data *tls_data = handle->submodule_private_ptr;
+
+	int ret_tmp;
+	if ((ret_tmp = tls_handshake(tls_data->ctx)) != 0) {
+		return (ret_tmp == TLS_WANT_POLLIN || ret_tmp == TLS_WANT_POLLOUT
+			? RRR_NET_TRANSPORT_SEND_INCOMPLETE :
+			RRR_NET_TRANSPORT_SEND_SOFT_ERROR
+		);
+	}
+
+	return RRR_NET_TRANSPORT_SEND_OK;
+}
+
+static int __rrr_net_transport_libressl_is_tls (void) {
+	return 1;
+}
+
+static void __rrr_net_transport_libressl_selected_proto_get (
+		const char **proto,
+		struct rrr_net_transport_handle *handle
+) {
+	struct rrr_net_transport_tls_data *tls_data = handle->submodule_private_ptr;
+
+	*proto = tls_conn_alpn_selected(tls_data->ctx);
 }
 
 static const struct rrr_net_transport_methods libressl_methods = {
@@ -566,8 +693,12 @@ static const struct rrr_net_transport_methods libressl_methods = {
 	__rrr_net_transport_libressl_accept,
 	__rrr_net_transport_libressl_close,
 	__rrr_net_transport_libressl_read_message,
+	__rrr_net_transport_libressl_read,
 	__rrr_net_transport_libressl_send,
-	__rrr_net_transport_libressl_poll
+	__rrr_net_transport_libressl_poll,
+	__rrr_net_transport_libressl_handshake,
+	__rrr_net_transport_libressl_is_tls,
+	__rrr_net_transport_libressl_selected_proto_get
 };
 
 int rrr_net_transport_libressl_new (
@@ -576,13 +707,22 @@ int rrr_net_transport_libressl_new (
 		const char *certificate_file,
 		const char *private_key_file,
 		const char *ca_file,
-		const char *ca_path
+		const char *ca_path,
+		const char *alpn_protos,
+		unsigned int alpn_protos_length
 ) {
 	int ret = 0;
 
 	const char *err_str = NULL;
+	char alpn_protos_tmp[256];
 
-	if ((ret = rrr_net_transport_tls_common_new(target, flags, certificate_file, private_key_file, ca_file, ca_path)) != 0) {
+	alpn_protos_tmp[0] = '\0';
+
+	if (alpn_protos != NULL && alpn_protos_length > 0) {
+		rrr_net_transport_tls_common_alpn_protos_to_str_comma_separated((unsigned char *) alpn_protos_tmp, sizeof(alpn_protos_tmp), (unsigned const char *) alpn_protos, alpn_protos_length);
+	}
+
+	if ((ret = rrr_net_transport_tls_common_new(target, flags, certificate_file, private_key_file, ca_file, ca_path, alpn_protos, alpn_protos_length)) != 0) {
 		goto out;
 	}
 
@@ -597,6 +737,10 @@ int rrr_net_transport_libressl_new (
 
 	tls_config_insecure_noverifyname(tls->config);
 
+	if (flags & RRR_NET_TRANSPORT_F_TLS_NO_CERT_VERIFY) {
+		tls_config_insecure_noverifycert(tls->config);
+	}
+
 	if (certificate_file != NULL && *certificate_file != '\0' && tls_config_set_cert_file(tls->config, certificate_file) < 0) {
 		goto out_config_error;
 	}
@@ -627,6 +771,12 @@ int rrr_net_transport_libressl_new (
 		goto out_config_error;
 	}
 
+	if (strlen(alpn_protos_tmp) > 0) {
+		if (tls_config_set_alpn(tls->config, alpn_protos_tmp) < 0) {
+			goto out_config_error;
+		}
+	}
+
 	goto out;
 	out_config_error:
 		ret = 1;
