@@ -30,17 +30,21 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #define RRR_NET_TRANSPORT_H_ENABLE_INTERNALS
 
+#include "../log.h"
+#include "../allocator.h"
+
 #include "net_transport.h"
+#include "net_transport_struct.h"
 #include "net_transport_openssl.h"
 #include "net_transport_tls_common.h"
 
-#include "../log.h"
 #include "../socket/rrr_socket.h"
 #include "../rrr_openssl.h"
 #include "../rrr_strerror.h"
 #include "../read.h"
 #include "../read_constants.h"
 #include "../ip/ip.h"
+#include "../ip/ip_util.h"
 #include "../ip/ip_accept_data.h"
 #include "../util/gnu.h"
 #include "../util/macro_utils.h"
@@ -62,8 +66,8 @@ static void __rrr_net_transport_openssl_ssl_data_destroy (struct rrr_net_transpo
 		if (ssl_data->ip_data.fd != 0) {
 			rrr_ip_close(&ssl_data->ip_data);
 		}
-
-		free(ssl_data);
+		RRR_FREE_IF_NOT_NULL(ssl_data->alpn_selected_proto);
+		rrr_free(ssl_data);
 	}
 }
 
@@ -110,7 +114,7 @@ static int __rrr_net_transport_openssl_verify_always_ok (X509_STORE_CTX *x509, v
 struct rrr_net_transport_tls_data *__rrr_net_transport_openssl_ssl_data_new (void) {
 	struct rrr_net_transport_tls_data *ssl_data = NULL;
 
-	if ((ssl_data = malloc(sizeof(*ssl_data))) == NULL) {
+	if ((ssl_data = rrr_allocate(sizeof(*ssl_data))) == NULL) {
 		RRR_MSG_0("Could not allocate memory for SSL data in __rrr_net_transport_ssl_data_new \n");
 		return NULL;
 	}
@@ -119,6 +123,84 @@ struct rrr_net_transport_tls_data *__rrr_net_transport_openssl_ssl_data_new (voi
 	return ssl_data;
 }
 
+static int __rrr_net_transport_openssl_alpn_select_cb (
+		SSL *s,
+		const unsigned char **out,
+		unsigned char *outlen,
+		const unsigned char *in,
+		unsigned int inlen,
+		void *arg
+) {
+	struct rrr_net_transport_tls_alpn *alpn = arg;
+
+	(void)(s);
+
+	int ret = SSL_TLSEXT_ERR_NOACK;
+
+	*out = NULL;
+	*outlen = 0;
+
+	// -2 for comma and \0
+	if (alpn->length > 256 - 2|| inlen > 256 - 2) {
+		RRR_MSG_1("Error: Large ALPN proto vectors (%u and %u) in __rrr_net_transport_openssl_alpn_select_cb\n", alpn->length, inlen);
+		ret = SSL_TLSEXT_ERR_ALERT_FATAL;
+		goto out;
+	}
+
+	if (RRR_DEBUGLEVEL_3) {
+		unsigned char server_protocols_tmp[256];
+		unsigned char client_protocols_tmp[256];
+		rrr_net_transport_tls_common_alpn_protos_to_str_comma_separated(server_protocols_tmp, sizeof(server_protocols_tmp), (unsigned char *) alpn->protos, alpn->length);
+		rrr_net_transport_tls_common_alpn_protos_to_str_comma_separated(client_protocols_tmp, sizeof(client_protocols_tmp), in, inlen);
+		RRR_DBG_3("TLS ALPN server protocols: '%s' client protocols: '%s'\n", server_protocols_tmp, client_protocols_tmp);
+	}
+
+	if (alpn->length == 0 || inlen == 0) {
+		goto out;
+	}
+
+	// Strategy : Pick the first protocol from the server list which is also in the client list
+	int server_index = 0;
+	for (unsigned int i = 0; i < alpn->length;/* increment at loop end */) {
+		const char *i_text = alpn->protos + i + 1;
+		unsigned char i_text_length = alpn->protos[i];
+
+		if (i + i_text_length >= alpn->length) {
+			RRR_BUG("BUG: Invalid size in self-created ALPN vector in __rrr_net_transport_openssl_alpn_select_cb\n");
+		}
+
+		for (unsigned int j = 0; j < inlen;/* increment at loop end */) {
+			const unsigned char *j_text = in + j + 1;
+			unsigned char j_text_length = in[j];
+
+			if (j + j_text_length >= inlen) {
+				RRR_MSG_0("Error: Invalid size in vector from input in __rrr_net_transport_openssl_alpn_select_cb\n");
+				ret = SSL_TLSEXT_ERR_ALERT_FATAL;
+				goto out;
+			}
+
+			if (i_text_length == j_text_length && memcmp(i_text, j_text, i_text_length) == 0) {
+				*out = (const unsigned char *) alpn->protos + i + 1;
+				*outlen = i_text_length;
+				RRR_DBG_3("TLS ALPN selected protocol at server position %i\n", server_index);
+				ret = SSL_TLSEXT_ERR_OK;
+				goto out;
+			}
+
+			j += j_text_length + 1;
+		}
+
+		server_index++;
+		i += i_text_length + 1;
+	}
+
+	RRR_DBG_3("TLS ALPN no protocol selected\n");
+
+	out:
+	return ret;
+}
+
+// Memory in *alpn must be permanently available during connection lifetime
 static int __rrr_net_transport_openssl_new_ctx (
 		SSL_CTX **target,
 		const SSL_METHOD *method,
@@ -126,7 +208,8 @@ static int __rrr_net_transport_openssl_new_ctx (
 		const char *certificate_file,
 		const char *private_key_file,
 		const char *ca_file,
-		const char *ca_path
+		const char *ca_path,
+		struct rrr_net_transport_tls_alpn *alpn
 ) {
 	int ret = 0;
 
@@ -137,11 +220,11 @@ static int __rrr_net_transport_openssl_new_ctx (
 	if (((certificate_file == NULL || *certificate_file == '\0') && (private_key_file != NULL && *private_key_file != '\0')) ||
 		((private_key_file == NULL || *private_key_file == '\0') && (certificate_file != NULL && *certificate_file != '\0'))
 	) {
-		RRR_BUG("BUG: Certificate file and private key file must both be either set or unset in __rrr_net_transport_tls_new_ctx\n");
+		RRR_BUG("BUG: Certificate file and private key file must both be either set or unset in __rrr_net_transport_openssl_new_ctx\n");
 	}
 
 	if ((ctx = SSL_CTX_new(method)) == NULL) {
-		RRR_SSL_ERR("Could not get SSL CTX in __rrr_net_transport_tls_new_ctx ");
+		RRR_SSL_ERR("Could not get SSL CTX in __rrr_net_transport_openssl_new_ctx ");
 		ret = 1;
 		goto out;
 	}
@@ -155,7 +238,7 @@ static int __rrr_net_transport_openssl_new_ctx (
 	SSL_CTX_set_options(ctx, SSL_OP_NO_SSLv2 | SSL_OP_NO_SSLv3 | SSL_OP_NO_TLSv1_1 | SSL_OP_NO_COMPRESSION);
 
 	unsigned int min_version = TLS1_2_VERSION;
-	if ((flags & RRR_NET_TRANSPORT_F_MIN_VERSION_TLS_1_1) != 0) {
+	if ((flags & RRR_NET_TRANSPORT_F_TLS_VERSION_MIN_1_1) != 0) {
 		min_version = TLS1_1_VERSION;
 	}
 
@@ -199,6 +282,19 @@ static int __rrr_net_transport_openssl_new_ctx (
 		}
 	}
 
+	if (alpn != NULL && alpn->protos != NULL) {
+		// For client
+		// Note: Returns 0 on success as opposed to other OpenSSL functions
+		if (SSL_CTX_set_alpn_protos(ctx, (unsigned const char *) alpn->protos, alpn->length) != 0) {
+			RRR_SSL_ERR("SSL_CTX_set_alpn_protos failed");
+			ret = 1;
+			goto out_destroy;
+		}
+
+		// For server
+		SSL_CTX_set_alpn_select_cb(ctx, __rrr_net_transport_openssl_alpn_select_cb, alpn);
+	}
+
 	*target = ctx;
 
 	goto out;
@@ -251,6 +347,67 @@ const char *__rrr_net_transport_openssl_ssl_version_to_str (
 	};
 	return result;
 }
+/*
+static int __rrr_net_transport_openssl_handshake_perform (
+		struct rrr_net_transport_tls_data *ssl_data,
+		SSL *ssl
+) {
+	int ret = 0;
+
+	int handshake_retry_max = 1000;
+	handshake_retry:
+	if ((ret = SSL_do_handshake(ssl)) != 1) {
+		if (ret < 0) {
+			if (--handshake_retry_max > 0 && (BIO_should_retry(ssl_data->web) || SSL_want_read(ssl) || SSL_want_write(ssl))) {
+				rrr_posix_usleep(1000); // 1 ms
+				goto handshake_retry;
+			}
+			if (handshake_retry_max == 0) {
+				RRR_MSG_0("TLS handshake timeout in __rrr_net_transport_openssl_handshake_perform\n");
+			}
+			else {
+				RRR_SSL_ERR("Could not perform handshake in __rrr_net_transport_openssl_handshake_perform");
+			}
+		}
+		else {
+			RRR_MSG_3("TLS handshake aborted\n");
+		}
+		ret = RRR_READ_SOFT_ERROR;
+		goto out;
+	}
+	else {
+		ret = 0;
+	}
+
+	out:
+	return ret;
+}
+*/
+static int __rrr_net_transport_openssl_alpn_selected_proto_save (
+		struct rrr_net_transport_tls_data *ssl_data,
+		SSL *ssl
+) {
+	int ret = 0;
+
+	const unsigned char *alpn_proto = NULL;
+	unsigned int alpn_proto_length = 0;
+
+	SSL_get0_alpn_selected(ssl, &alpn_proto, &alpn_proto_length);
+
+	if (alpn_proto != NULL && alpn_proto_length > 0) {
+		unsigned int str_size = alpn_proto_length + 1;
+		if ((ssl_data->alpn_selected_proto = rrr_allocate(str_size)) == NULL) {
+			RRR_MSG_0("Could not allocate memory for ALPN protocol name in __rrr_net_transport_openssl_alpn_selected_proto_save\n");
+			ret = 1;
+			goto out;
+		}
+		memcpy(ssl_data->alpn_selected_proto, alpn_proto, alpn_proto_length);
+		ssl_data->alpn_selected_proto[alpn_proto_length] = '\0';
+	}
+
+	out:
+	return ret;
+}
 
 int __rrr_net_transport_openssl_connect_callback (
 		RRR_NET_TRANSPORT_BIND_AND_LISTEN_CALLBACK_ARGS
@@ -262,15 +419,11 @@ int __rrr_net_transport_openssl_connect_callback (
 
 	struct rrr_net_transport_tls_data *ssl_data = NULL;
 	if ((ssl_data = __rrr_net_transport_openssl_ssl_data_new()) == NULL) {
-		RRR_MSG_0("Could not allocate memory for SSL data in __rrr_net_transport_tls_connect\n");
+		RRR_MSG_0("Could not allocate memory for SSL data in __rrr_net_transport_openssl_connect_callback\n");
 		ret = 1;
 		goto out_final;
 	}
 
-	ssl_data->sockaddr = callback_data->accept_data->addr;
-	ssl_data->socklen = callback_data->accept_data->len;
-	ssl_data->ip_data = callback_data->accept_data->ip_data;
-
 	if (__rrr_net_transport_openssl_new_ctx (
 			&ssl_data->ctx,
 			tls->ssl_client_method,
@@ -278,15 +431,16 @@ int __rrr_net_transport_openssl_connect_callback (
 			tls->certificate_file,
 			tls->private_key_file,
 			tls->ca_file,
-			tls->ca_path
+			tls->ca_path,
+			&tls->alpn
 	) != 0) {
-		RRR_SSL_ERR("Could not get SSL CTX in __rrr_net_transport_tls_connect");
+		RRR_SSL_ERR("Could not get SSL CTX in __rrr_net_transport_openssl_connect_callback");
 		ret = 1;
 		goto out_destroy_ssl_data;
 	}
 
 	if ((ssl_data->web = BIO_new_ssl(ssl_data->ctx, 1)) == NULL) {
-		RRR_SSL_ERR("Could not get BIO in __rrr_net_transport_tls_connect");
+		RRR_SSL_ERR("Could not get BIO in __rrr_net_transport_openssl_connect_callback");
 		ret = 1;
 		goto out_destroy_ssl_data;
 	}
@@ -294,8 +448,8 @@ int __rrr_net_transport_openssl_connect_callback (
 	SSL *ssl = NULL;
 	BIO_get_ssl(ssl_data->web, &ssl);
 
-	if (SSL_set_fd(ssl, ssl_data->ip_data.fd) != 1) {
-		RRR_SSL_ERR("Could not set FD for SSL in __rrr_net_transport_tls_accept\n");
+	if (SSL_set_fd(ssl, callback_data->accept_data->ip_data.fd) != 1) {
+		RRR_SSL_ERR("Could not set FD for SSL in __rrr_net_transport_openssl_connect_callback\n");
 		ret = 1;
 		goto out_destroy_ssl_data;
 	}
@@ -306,14 +460,12 @@ int __rrr_net_transport_openssl_connect_callback (
 		goto out_destroy_ssl_data;
 	}
 
-	if ((ret = SSL_set_max_proto_version(ssl, TLS1_3_VERSION)) != 1) {
+	if (SSL_set_max_proto_version(ssl, TLS1_3_VERSION) != 1) {
 		RRR_SSL_ERR("Could set SSL protocol version");
 		ret = 1;
 		goto out_destroy_ssl_data;
 	}
 
-	ret = 0;
-
 	// Not used for TLSv1.3
 	//const char* const PREFERRED_CIPHERS = "HIGH:!aNULL:!kRSA:!PSK:!SRP:!MD5:!RC4";
 	if (SSL_set_cipher_list(ssl, "DEFAULT") != 1) {
@@ -337,45 +489,21 @@ int __rrr_net_transport_openssl_connect_callback (
 	// Set non-blocking I/O
 	BIO_set_nbio(ssl_data->web, 1); // Always returns 1
 
-	retry_handshake:
-	if (BIO_do_handshake(ssl_data->web) != 1) {
-		if (BIO_should_retry(ssl_data->web)) {
-			rrr_posix_usleep(1000);
-			goto retry_handshake;
-		}
-		RRR_SSL_ERR("Could not do TLS handshake");
-		ret = 1;
-		goto out_destroy_ssl_data;
-	}
-
-	X509 *cert = SSL_get_peer_certificate(ssl);
-	if (cert != NULL) {
-		X509_free(cert);
-	}
-	else {
-		RRR_MSG_0("No certificate received in TLS handshake with %s:%u\n",
-				callback_data->host, callback_data->port);
-		ret = 1;
-		goto out_destroy_ssl_data;
-	}
-
-	long verify_result = 0;
-	if ((verify_result = SSL_get_verify_result(ssl)) != X509_V_OK) {
-		RRR_MSG_0("Certificate verification failed for %s:%u with reason %li\n",
-				callback_data->host, callback_data->port, verify_result);
-		ret = 1;
-		goto out_destroy_ssl_data;
-	}
-
-	// TODO : Hostname verification
+	SSL_set_connect_state(ssl);
 
 	*submodule_private_ptr = ssl_data;
-	*submodule_private_fd = 0;
+	*submodule_fd = callback_data->accept_data->ip_data.fd;
+
+	// Set this data, including FD at the end. Caller will try to close the FD
+	// upon errors from this function, and we wish to avoid double close() as
+	// the FD will attempted to be closed by the destroy function below.
+	ssl_data->sockaddr = callback_data->accept_data->addr;
+	ssl_data->socklen = callback_data->accept_data->len;
+	ssl_data->ip_data = callback_data->accept_data->ip_data;
 
 	goto out_final;
 	out_destroy_ssl_data:
 		__rrr_net_transport_openssl_ssl_data_destroy(ssl_data);
-		// ssl_data_destroy calls ip_close, skip doing it again
 	out_final:
 		return ret;
 }
@@ -391,15 +519,15 @@ static int __rrr_net_transport_openssl_connect (
 	struct rrr_ip_accept_data *accept_data = NULL;
 
 	if (*socklen < sizeof(accept_data->addr)) {
-		RRR_BUG("BUG: socklen too small in __rrr_net_transport_tls_connect\n");
+		RRR_BUG("BUG: socklen too small in __rrr_net_transport_openssl_connect\n");
 	}
 
 	*handle = 0;
 
 	int ret = 0;
 
-	if (rrr_ip_network_connect_tcp_ipv4_or_ipv6(&accept_data, port, host, NULL) != 0) {
-		RRR_DBG_1("Could not create TLS connection to %s:%u\n", host, port);
+	if (rrr_ip_network_connect_tcp_ipv4_or_ipv6(&accept_data, port, host) != 0) {
+		RRR_DBG_3("Could not create TCP connection to %s:%u for TLS usage\n", host, port);
 		ret = RRR_NET_TRANSPORT_READ_SOFT_ERROR;
 		goto out;
 	}
@@ -419,7 +547,7 @@ static int __rrr_net_transport_openssl_connect (
 			__rrr_net_transport_openssl_connect_callback,
 			&callback_data
 	)) != 0) {
-		RRR_MSG_0("Could not get handle in __rrr_net_transport_tls_connect return was %i\n", ret);
+		RRR_MSG_0("Could not get handle in __rrr_net_transport_openssl_connect return was %i\n", ret);
 		ret = 1;
 		goto out_destroy_ip;
 	}
@@ -440,6 +568,7 @@ static int __rrr_net_transport_openssl_connect (
 struct rrr_net_transport_openssl_bind_and_listen_callback_data {
 	struct rrr_net_transport_tls *tls;
 	unsigned int port;
+	int do_ipv6;
 };
 
 static int __rrr_net_transport_openssl_bind_and_listen_callback (
@@ -453,15 +582,15 @@ static int __rrr_net_transport_openssl_bind_and_listen_callback (
 	struct rrr_net_transport_tls_data *ssl_data = NULL;
 
 	if ((ssl_data = __rrr_net_transport_openssl_ssl_data_new()) == NULL) {
-		RRR_MSG_0("Could not allocate memory for SSL data in __rrr_net_transport_tls_bind_and_listen\n");
+		RRR_MSG_0("Could not allocate memory for SSL data in __rrr_net_transport_openssl_bind_and_listen_callback\n");
 		ret = 1;
 		goto out;
 	}
 
 	ssl_data->ip_data.port = callback_data->port;
 
-	if (rrr_ip_network_start_tcp_ipv4_and_ipv6 (&ssl_data->ip_data, 10) != 0) {
-		RRR_MSG_0("Could not start IP listening in __rrr_net_transport_tls_bind_and_listen\n");
+	if (rrr_ip_network_start_tcp (&ssl_data->ip_data, 10, callback_data->do_ipv6) != 0) {
+		RRR_DBG_1("Note: Could not start IP listening in __rrr_net_transport_openssl_bind_and_listen_callback\n");
 		ret = 1;
 		goto out_free_ssl_data;
 	}
@@ -473,15 +602,16 @@ static int __rrr_net_transport_openssl_bind_and_listen_callback (
 			tls->certificate_file,
 			tls->private_key_file,
 			tls->ca_file,
-			tls->ca_path
+			tls->ca_path,
+			&tls->alpn
 	) != 0) {
-		RRR_SSL_ERR("Could not get SSL CTX in __rrr_net_transport_tls_bind_and_listen");
+		RRR_SSL_ERR("Could not get SSL CTX in __rrr_net_transport_openssl_bind_and_listen_callback");
 		ret = 1;
 		goto out_destroy_ip;
 	}
 
 	*submodule_private_ptr = ssl_data;
-	*submodule_private_fd = 0;
+	*submodule_fd = ssl_data->ip_data.fd;
 
 	goto out;
 //	out_destroy_ctx:
@@ -509,7 +639,8 @@ static int __rrr_net_transport_openssl_bind_and_listen (
 
 	struct rrr_net_transport_openssl_bind_and_listen_callback_data callback_data = {
 			tls,
-			port
+			port,
+			do_ipv6
 	};
 
 	int new_handle;
@@ -520,11 +651,11 @@ static int __rrr_net_transport_openssl_bind_and_listen (
 			__rrr_net_transport_openssl_bind_and_listen_callback,
 			&callback_data
 	)) != 0) {
-		RRR_MSG_0("Could not get handle in __rrr_net_transport_openssl_bind_and_listen return was %i\n", ret);
-		ret = 1;
 		goto out;
 	}
 
+	RRR_DBG_7("OpenSSL listening started on port %u transport handle %p/%i\n", port, transport, new_handle);
+
 	ret = callback (
 			transport,
 			new_handle,
@@ -553,15 +684,11 @@ static int __rrr_net_transport_openssl_accept_callback (
 	struct rrr_net_transport_tls_data *ssl_data = NULL;
 
 	if ((ssl_data = __rrr_net_transport_openssl_ssl_data_new()) == NULL) {
-		RRR_MSG_0("Could not allocate memory for SSL data in __rrr_net_transport_tls_accept\n");
+		RRR_MSG_0("Could not allocate memory for SSL data in __rrr_net_transport_openssl_accept_callback\n");
 		ret = 1;
 		goto out;
 	}
 
-	ssl_data->sockaddr = callback_data->accept_data->addr;
-	ssl_data->socklen = callback_data->accept_data->len;
-	ssl_data->ip_data = callback_data->accept_data->ip_data;
-
 	if (__rrr_net_transport_openssl_new_ctx (
 			&ssl_data->ctx,
 			tls->ssl_server_method,
@@ -569,15 +696,16 @@ static int __rrr_net_transport_openssl_accept_callback (
 			tls->certificate_file,
 			tls->private_key_file,
 			tls->ca_file,
-			tls->ca_path
+			tls->ca_path,
+			&tls->alpn
 	) != 0) {
-		RRR_SSL_ERR("Could not get SSL CTX in __rrr_net_transport_tls_accept\n");
+		RRR_SSL_ERR("Could not get SSL CTX in __rrr_net_transport_openssl_accept_callback");
 		ret = 1;
 		goto out_destroy_ssl_data;
 	}
 
 	if ((ssl_data->web = BIO_new_ssl(ssl_data->ctx, 0)) == NULL) {
-		RRR_SSL_ERR("Could not allocate BIO in __rrr_net_transport_tls_accept\n");
+		RRR_SSL_ERR("Could not allocate BIO in __rrr_net_transport_openssl_accept_callback");
 		ret = 1;
 		goto out_destroy_ssl_data;
 	}
@@ -585,16 +713,25 @@ static int __rrr_net_transport_openssl_accept_callback (
 	SSL *ssl;
 	BIO_get_ssl(ssl_data->web, &ssl);
 
-	if (SSL_set_fd(ssl, ssl_data->ip_data.fd) != 1) {
-		RRR_SSL_ERR("Could not set FD for SSL in __rrr_net_transport_tls_accept\n");
+	if (SSL_set_fd(ssl, callback_data->accept_data->ip_data.fd) != 1) {
+		RRR_SSL_ERR("Could not set FD for SSL in __rrr_net_transport_openssl_accept_callback");
 		ret = 1;
 		goto out_destroy_ssl_data;
 	}
 
 	BIO_set_nbio(ssl_data->web, 1);
 
+	SSL_set_accept_state(ssl);
+
+	// Set this data, including FD at the end. Caller will try to close the FD
+	// upon errors from this function, and we wish to avoid double close() as
+	// the FD will attempted to be closed by the destroy function below.
+	ssl_data->sockaddr = callback_data->accept_data->addr;
+	ssl_data->socklen = callback_data->accept_data->len;
+	ssl_data->ip_data = callback_data->accept_data->ip_data;
+
 	*submodule_private_ptr = ssl_data;
-	*submodule_private_fd = 0;
+	*submodule_fd = ssl_data->ip_data.fd;
 
 	goto out;
 	out_destroy_ssl_data:
@@ -611,6 +748,8 @@ int __rrr_net_transport_openssl_accept (
 
 	int ret = 0;
 
+	*did_accept = 0;
+
 	struct rrr_net_transport_tls_data *listen_ssl_data = listen_handle->submodule_private_ptr;
 
 	if ((ret = rrr_ip_accept(&accept_data, &listen_ssl_data->ip_data, "net_transport_tls", 0)) != 0) {
@@ -636,12 +775,18 @@ int __rrr_net_transport_openssl_accept (
 			__rrr_net_transport_openssl_accept_callback,
 			&callback_data
 	)) != 0) {
-		RRR_MSG_0("Could not get handle in __rrr_net_transport_tls_accept return was %i\n", ret);
-		ret = 1;
+		RRR_MSG_0("Could not get handle in __rrr_net_transport_openssl_accept return was %i\n", ret);
 		goto out_destroy_ip;
 	}
 
-	ret = callback(
+	{
+		char buf[128];
+		rrr_ip_to_str(buf, sizeof(buf), (const struct sockaddr *) &accept_data->addr, accept_data->len);
+		RRR_DBG_7("OpenSSL accepted connection on port %u from %s transport handle %p/%i\n",
+				listen_ssl_data->ip_data.port, buf, listen_handle->transport, new_handle);
+	}
+
+	ret = callback (
 			listen_handle->transport,
 			new_handle,
 			(struct sockaddr *) &accept_data->addr,
@@ -651,6 +796,8 @@ int __rrr_net_transport_openssl_accept (
 			callback_arg
 	);
 
+	*did_accept = 1;
+
 	goto out;
 
 	out_destroy_ip:
@@ -660,29 +807,28 @@ int __rrr_net_transport_openssl_accept (
 		return ret;
 }
 
-static int __rrr_net_transport_tls_read_read (
+static int __rrr_net_transport_openssl_read_raw (
 		char *buf,
 		ssize_t *read_bytes,
-		ssize_t read_step_max_size,
-		void *private_arg
+		struct rrr_net_transport_tls_data *ssl_data,
+		ssize_t read_step_max_size
 ) {
-	struct rrr_net_transport_read_callback_data *callback_data = private_arg;
-	struct rrr_net_transport_tls_data *ssl_data = callback_data->handle->submodule_private_ptr;
-
 	int ret = RRR_READ_OK;
 
 	ssize_t result = BIO_read(ssl_data->web, buf, read_step_max_size);
-	if (result < 0) {
+	if (result <= 0) {
 		if (BIO_should_retry(ssl_data->web) == 0) {
 //			int reason = BIO_get_retry_reason(ssl_data->web);
-			RRR_SSL_ERR("Error while reading from TLS connection");
+			RRR_SSL_DBG_3("Error while reading from TLS connection, possible close of connection");
 			// Possible close of connection
-			ret = RRR_READ_SOFT_ERROR;
+			ret = RRR_READ_EOF;
 			goto out;
 		}
+		ret = rrr_socket_check_alive(BIO_get_fd(ssl_data->web, NULL));
+		goto out;
 	}
 	else if (ERR_peek_error() != 0) {
-		RRR_SSL_ERR("Error while reading in __rrr_net_transport_tls_read_read");
+		RRR_SSL_ERR("Error while reading in __rrr_net_transport_openssl_read_raw");
 		return RRR_READ_SOFT_ERROR;
 	}
 
@@ -692,8 +838,20 @@ static int __rrr_net_transport_tls_read_read (
 	return ret;
 }
 
+static int __rrr_net_transport_openssl_read_read (
+		char *buf,
+		ssize_t *read_bytes,
+		ssize_t read_step_max_size,
+		void *private_arg
+) {
+	struct rrr_net_transport_read_callback_data *callback_data = private_arg;
+	struct rrr_net_transport_tls_data *ssl_data = callback_data->handle->submodule_private_ptr;
+
+	return __rrr_net_transport_openssl_read_raw (buf, read_bytes, ssl_data, read_step_max_size);
+}
+
 static int __rrr_net_transport_openssl_read_message (
-		RRR_NET_TRANSPORT_READ_ARGS
+		RRR_NET_TRANSPORT_READ_MESSAGE_ARGS
 ) {
 	int ret = 0;
 
@@ -714,9 +872,12 @@ static int __rrr_net_transport_openssl_read_message (
 				read_step_initial,
 				read_step_max_size,
 				read_max_size,
+				RRR_LL_FIRST(&handle->read_sessions),
+				ratelimit_interval_us,
+				ratelimit_max_bytes,
 				rrr_net_transport_tls_common_read_get_target_size,
 				rrr_net_transport_tls_common_read_complete_callback,
-				__rrr_net_transport_tls_read_read,
+				__rrr_net_transport_openssl_read_read,
 				rrr_net_transport_tls_common_read_get_read_session_with_overshoot,
 				rrr_net_transport_tls_common_read_get_read_session,
 				rrr_net_transport_tls_common_read_remove_read_session,
@@ -728,11 +889,15 @@ static int __rrr_net_transport_openssl_read_message (
 		if (ret == RRR_NET_TRANSPORT_READ_INCOMPLETE) {
 			continue;
 		}
-		else if (ret == RRR_NET_TRANSPORT_READ_OK) {
+		else if ( ret == RRR_NET_TRANSPORT_READ_OK ||
+		          ret == RRR_NET_TRANSPORT_READ_RATELIMIT ||
+			  ret == RRR_NET_TRANSPORT_READ_READ_EOF ||
+			  ret == RRR_NET_TRANSPORT_READ_SOFT_ERROR
+		) {
 			break;
 		}
 		else {
-			RRR_MSG_0("Error %i while reading from remote in __rrr_net_transport_tls_read_message\n", ret);
+			RRR_MSG_0("Error %i while reading from remote in __rrr_net_transport_openssl_read_message\n", ret);
 			goto out;
 		}
 	}
@@ -741,6 +906,31 @@ static int __rrr_net_transport_openssl_read_message (
 	return ret;
 }
 
+static int __rrr_net_transport_openssl_read (
+		RRR_NET_TRANSPORT_READ_ARGS
+) {
+	int ret = RRR_NET_TRANSPORT_READ_OK;
+
+	if (buf_size > SSIZE_MAX) {
+		RRR_MSG_0("Buffer size too large in __rrr_net_transport_openssl_read\n");
+		ret = RRR_NET_TRANSPORT_READ_HARD_ERROR;
+		goto out;
+	}
+
+	ssize_t bytes_read_s = 0;
+
+	ret = __rrr_net_transport_openssl_read_raw(buf, &bytes_read_s, handle->submodule_private_ptr, buf_size);
+
+	if (bytes_read_s < 0) {
+		RRR_BUG("BUG: Negative bytes read value in __rrr_net_transport_openssl_read\n");
+	}
+
+	*bytes_read = bytes_read_s;
+
+	out:
+	return ret;
+}
+
 static int __rrr_net_transport_openssl_send (
 	uint64_t *sent_bytes,
 	struct rrr_net_transport_handle *handle,
@@ -751,18 +941,27 @@ static int __rrr_net_transport_openssl_send (
 
 	*sent_bytes = 0;
 
-	if (BIO_write(ssl_data->web, data, size) <= 0) {
+	int sent_bytes_tmp;
+	if ((sent_bytes_tmp = BIO_write(ssl_data->web, data, size)) <= 0) {
 		if (BIO_should_retry(ssl_data->web)) {
-			return 0;
+			return RRR_NET_TRANSPORT_SEND_INCOMPLETE;
 		}
-		RRR_MSG_0("Write failure in __rrr_net_transport_tls_send\n");
-		return 1;
+		return RRR_NET_TRANSPORT_SEND_HARD_ERROR;
 	}
 	else {
-		*sent_bytes = (size > 0 ? size : 0);
+		*sent_bytes = sent_bytes_tmp;
 	}
 
-	return 0;
+	return RRR_NET_TRANSPORT_SEND_OK;
+}
+
+static void __rrr_net_transport_openssl_selected_proto_get (
+		const char **proto,
+		struct rrr_net_transport_handle *handle
+) {
+	struct rrr_net_transport_tls_data *ssl_data = handle->submodule_private_ptr;
+
+	*proto = ssl_data->alpn_selected_proto;
 }
 
 static int __rrr_net_transport_openssl_poll (
@@ -775,7 +974,66 @@ static int __rrr_net_transport_openssl_poll (
 		return RRR_NET_TRANSPORT_READ_SOFT_ERROR;
 	}
 
-	return rrr_socket_check_alive (fd);
+	if (rrr_socket_check_alive (fd) != 0) {
+		return RRR_READ_EOF;
+	}
+
+	return RRR_READ_OK;
+}
+
+static int __rrr_net_transport_openssl_handshake (
+		struct rrr_net_transport_handle *handle
+) {
+	struct rrr_net_transport_tls_data *ssl_data = handle->submodule_private_ptr;
+
+	SSL *ssl;
+	BIO_get_ssl(ssl_data->web, &ssl);
+
+	int ret_tmp;
+	if ((ret_tmp = SSL_do_handshake(ssl)) != 1) {
+		if (BIO_should_retry(ssl_data->web) || SSL_want_read(ssl) || SSL_want_write(ssl)) {
+			return RRR_NET_TRANSPORT_SEND_INCOMPLETE;
+		}
+		if (ret_tmp < 0) {
+			RRR_MSG_0("Fatal error during handshake: %i\n", SSL_get_error(ssl, ret_tmp));
+		}
+		else {
+			RRR_SSL_ERR("Handshake failure");
+		}
+		return RRR_NET_TRANSPORT_SEND_SOFT_ERROR;
+	}
+
+	if (!SSL_is_server(ssl)) {
+		// TODO : Hostname verification
+
+		X509 *cert = SSL_get_peer_certificate(ssl);
+		if (cert != NULL) {
+			X509_free(cert);
+		}
+		else {
+			RRR_MSG_0("No certificate received in TLS handshake fd %i\n",
+					handle->submodule_fd);
+			return RRR_NET_TRANSPORT_SEND_SOFT_ERROR;
+		}
+	}
+
+	long verify_result = 0;
+	if ((verify_result = SSL_get_verify_result(ssl)) != X509_V_OK) {
+		RRR_MSG_0("Certificate verification failed for fd %i with reason %li\n",
+				handle->submodule_fd, verify_result);
+		return RRR_NET_TRANSPORT_SEND_SOFT_ERROR;
+	}
+
+	if (__rrr_net_transport_openssl_alpn_selected_proto_save (ssl_data, ssl)) {
+		RRR_MSG_0("Failed to save ALPN selected protocol\n");
+		return RRR_NET_TRANSPORT_SEND_SOFT_ERROR;
+	}
+
+	return RRR_NET_TRANSPORT_SEND_OK;
+}
+
+static int __rrr_net_transport_openssl_is_tls (void) {
+	return 1;
 }
 
 static const struct rrr_net_transport_methods tls_methods = {
@@ -785,8 +1043,12 @@ static const struct rrr_net_transport_methods tls_methods = {
 	__rrr_net_transport_openssl_accept,
 	__rrr_net_transport_openssl_ssl_data_close,
 	__rrr_net_transport_openssl_read_message,
+	__rrr_net_transport_openssl_read,
 	__rrr_net_transport_openssl_send,
-	__rrr_net_transport_openssl_poll
+	__rrr_net_transport_openssl_poll,
+	__rrr_net_transport_openssl_handshake,
+	__rrr_net_transport_openssl_is_tls,
+	__rrr_net_transport_openssl_selected_proto_get
 };
 
 int rrr_net_transport_openssl_new (
@@ -795,9 +1057,11 @@ int rrr_net_transport_openssl_new (
 		const char *certificate_file,
 		const char *private_key_file,
 		const char *ca_file,
-		const char *ca_path
+		const char *ca_path,
+		const char *alpn_protos,
+		unsigned int alpn_protos_length
 ) {
-	if ((rrr_net_transport_tls_common_new(target, flags, certificate_file, private_key_file, ca_file, ca_path)) != 0) {
+	if ((rrr_net_transport_tls_common_new(target, flags, certificate_file, private_key_file, ca_file, ca_path, alpn_protos, alpn_protos_length)) != 0) {
 		return 1;
 	}
 
