@@ -2,7 +2,7 @@
 
 Read Route Record
 
-Copyright (C) 2020 Atle Solbakken atle@goliathdns.no
+Copyright (C) 2020-2021 Atle Solbakken atle@goliathdns.no
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -23,40 +23,74 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <stdlib.h>
 #include <string.h>
 #include <poll.h>
-
+#include <limits.h>
 
 #define RRR_NET_TRANSPORT_H_ENABLE_INTERNALS
 
+#include "../log.h"
+#include "../allocator.h"
+
 #include "net_transport_plain.h"
 
-#include "../log.h"
 #include "../socket/rrr_socket.h"
 #include "../socket/rrr_socket_read.h"
 #include "../read.h"
 #include "../ip/ip.h"
+#include "../ip/ip_util.h"
 #include "../ip/ip_accept_data.h"
 #include "../util/macro_utils.h"
 
-static int __rrr_net_transport_plain_close (struct rrr_net_transport_handle *handle) {
-	if (rrr_socket_close(handle->submodule_private_fd) != 0) {
-		RRR_MSG_0("Warning: Error from rrr_socket_close in __rrr_net_transport_plain_close\n");
+static void __rrr_net_transport_plain_data_destroy (struct rrr_net_transport_plain_data *data) {
+	RRR_FREE_IF_NOT_NULL(data);
+}
+
+static int __rrr_net_transport_plain_data_new (struct rrr_net_transport_plain_data **result) {
+	*result = NULL;
+
+	struct rrr_net_transport_plain_data *data = rrr_allocate(sizeof(*data));
+	if (data == NULL) {
+		RRR_MSG_0("Could not allocate memory in __rrr_net_transport_plain_data_new\n");
 		return 1;
 	}
+
+	memset(data, '\0', sizeof(*data));
+
+	*result = data;
+
+	return 0;
+}
+
+static int __rrr_net_transport_plain_close (struct rrr_net_transport_handle *handle) {
+	if (rrr_socket_close(handle->submodule_fd) != 0) {
+		RRR_MSG_0("Warning: Error from rrr_socket_close in __rrr_net_transport_plain_close\n");
+	}
+	__rrr_net_transport_plain_data_destroy (handle->submodule_private_ptr);
 	return 0;
 }
 
 static void __rrr_net_transport_plain_destroy (struct rrr_net_transport *transport) {
 	struct rrr_net_transport_plain *plain = (struct rrr_net_transport_plain *) transport;
-	free(plain);
+	rrr_free(plain);
 }
 
+struct rrr_net_transport_plain_allocate_and_add_callback_data {
+	const struct rrr_ip_data *ip_data;
+};
+
 static int __rrr_net_transport_plain_handle_allocate_and_add_callback (
 		RRR_NET_TRANSPORT_BIND_AND_LISTEN_CALLBACK_ARGS
 ) {
-	int fd = *((int*) arg);
+	struct rrr_net_transport_plain_allocate_and_add_callback_data *callback_data = arg;
 
-	*submodule_private_ptr = 0;
-	*submodule_private_fd = fd;
+	struct rrr_net_transport_plain_data *data = NULL;
+	if (__rrr_net_transport_plain_data_new(&data) != 0) {
+		return 1;
+	}
+
+	data->ip_data = *(callback_data->ip_data);
+
+	*submodule_private_ptr = data;
+	*submodule_fd = callback_data->ip_data->fd;
 
 	return 0;
 }
@@ -76,19 +110,23 @@ static int __rrr_net_transport_plain_connect (
 		RRR_BUG("BUG: socklen too small in __rrr_net_transport_plain_connect\n");
 	}
 
-	if (rrr_ip_network_connect_tcp_ipv4_or_ipv6(&accept_data, port, host, NULL) != 0) {
+	if (rrr_ip_network_connect_tcp_ipv4_or_ipv6(&accept_data, port, host) != 0) {
 		RRR_DBG_1("Could not connect to server '%s' port '%u'\n", host, port);
 		ret = RRR_NET_TRANSPORT_READ_SOFT_ERROR;
 		goto out;
 	}
 
+	struct rrr_net_transport_plain_allocate_and_add_callback_data callback_data = {
+			&accept_data->ip_data
+	};
+
 	int new_handle = 0;
 	if ((ret = rrr_net_transport_handle_allocate_and_add (
 			&new_handle,
 			transport,
 			RRR_NET_TRANSPORT_SOCKET_MODE_CONNECTION,
 			__rrr_net_transport_plain_handle_allocate_and_add_callback,
-			&accept_data->ip_data.fd
+			&callback_data
 	)) != 0) {
 		RRR_MSG_0("Could not register handle in __rrr_net_transport_plain_connect\n");
 		ret = 1;
@@ -129,7 +167,7 @@ static int __rrr_net_transport_plain_read_complete_callback (
 }
 
 static int __rrr_net_transport_plain_read_message (
-		RRR_NET_TRANSPORT_READ_ARGS
+		RRR_NET_TRANSPORT_READ_MESSAGE_ARGS
 ) {
 	int ret = 0;
 
@@ -148,11 +186,13 @@ static int __rrr_net_transport_plain_read_message (
 		ret = rrr_socket_read_message_default (
 				&bytes_read_tmp,
 				&handle->read_sessions,
-				handle->submodule_private_fd,
+				handle->submodule_fd,
 				read_step_initial,
 				read_step_max_size,
 				read_max_size,
-				RRR_SOCKET_READ_METHOD_RECV | RRR_SOCKET_READ_USE_TIMEOUT | RRR_SOCKET_READ_CHECK_POLLHUP | RRR_SOCKET_READ_CHECK_EOF,
+				RRR_SOCKET_READ_METHOD_RECV | RRR_SOCKET_READ_CHECK_POLLHUP | RRR_SOCKET_READ_CHECK_EOF,
+				ratelimit_interval_us,
+				ratelimit_max_bytes,
 				__rrr_net_transport_plain_read_get_target_size_callback,
 				&callback_data,
 				__rrr_net_transport_plain_read_complete_callback,
@@ -160,7 +200,7 @@ static int __rrr_net_transport_plain_read_message (
 		);
 		*bytes_read += bytes_read_tmp;
 
-		if (ret == RRR_SOCKET_OK) {
+		if (ret == RRR_SOCKET_OK || ret == RRR_READ_RATELIMIT) {
 			// TODO : Check for persistent connection/more results which might be
 			//		  stored in read session overshoot buffer
 			goto out;
@@ -170,7 +210,7 @@ static int __rrr_net_transport_plain_read_message (
 				RRR_MSG_0("Hard error while reading from remote in __rrr_net_transport_plain_read_message\n");
 			}
 			else {
-				RRR_DBG_1("Read returned %i while reading from remote in __rrr_net_transport_plain_read_message\n", ret);
+				RRR_DBG_7("Read returned %i while reading from remote in __rrr_net_transport_plain_read_message\n", ret);
 			}
 			goto out;
 		}
@@ -180,6 +220,41 @@ static int __rrr_net_transport_plain_read_message (
 	return ret;
 }
 
+static int __rrr_net_transport_plain_read (
+		RRR_NET_TRANSPORT_READ_ARGS
+) {
+	int ret = RRR_NET_TRANSPORT_READ_OK;
+
+	if (buf_size > SSIZE_MAX) {
+		RRR_MSG_0("Buffer size too large in __rrr_net_transport_plain_read\n");
+		ret = RRR_NET_TRANSPORT_READ_HARD_ERROR;
+		goto out;
+	}
+
+	ssize_t bytes_read_s = 0;
+
+	ret = rrr_socket_read (
+			buf,
+			&bytes_read_s,
+			handle->submodule_fd,
+			buf_size,
+			NULL,
+			NULL,
+			RRR_SOCKET_READ_METHOD_RECV | RRR_SOCKET_READ_CHECK_POLLHUP | RRR_SOCKET_READ_CHECK_EOF
+	);
+
+	ret &= ~(RRR_SOCKET_READ_INCOMPLETE);
+
+	if (bytes_read_s < 0) {
+		RRR_BUG("BUG: Negative bytes read value in __rrr_net_transport_libressl_read\n");
+	}
+
+	*bytes_read = bytes_read_s;
+
+	out:
+	return ret;
+}
+
 static int __rrr_net_transport_plain_send (
 	uint64_t *written_bytes,
 	struct rrr_net_transport_handle *handle,
@@ -192,13 +267,10 @@ static int __rrr_net_transport_plain_send (
 
 	ssize_t written_bytes_tmp = 0;
 
-	if ((ret = rrr_socket_send_nonblock(&written_bytes_tmp, handle->submodule_private_fd, data, size)) != 0) {
-		goto out;
-	}
+	ret = rrr_socket_send_nonblock_check_retry(&written_bytes_tmp, handle->submodule_fd, data, size);
 
 	*written_bytes += (written_bytes_tmp > 0 ? written_bytes_tmp : 0);
 
-	out:
 	return ret;
 }
 
@@ -213,22 +285,27 @@ int __rrr_net_transport_plain_bind_and_listen (
 
 	ip_data.port = port;
 
-	if ((ret = rrr_ip_network_start_tcp_ipv4_and_ipv6(&ip_data, 10)) != 0) {
+	if ((ret = rrr_ip_network_start_tcp(&ip_data, 10, do_ipv6)) != 0) {
 		goto out;
 	}
 
+	struct rrr_net_transport_plain_allocate_and_add_callback_data callback_data = {
+			&ip_data
+	};
+
 	int new_handle = 0;
 	if ((ret = rrr_net_transport_handle_allocate_and_add (
 			&new_handle,
 			transport,
 			RRR_NET_TRANSPORT_SOCKET_MODE_LISTEN,
 			__rrr_net_transport_plain_handle_allocate_and_add_callback ,
-			&ip_data.fd
+			&callback_data
 	)) != 0) {
-		RRR_MSG_0("Could not add handle in __rrr_net_transport_plain_bind_and_listen\n");
 		goto out_destroy_ip;
 	}
 
+	RRR_DBG_7("Plain listening started on port %u transport handle %p/%i\n", port, transport, new_handle);
+
 	ret = callback(transport, new_handle, callback_final, callback_final_arg, callback_arg);
 
 	goto out;
@@ -245,9 +322,11 @@ int __rrr_net_transport_plain_accept (
 
 	int ret = 0;
 
+	*did_accept = 0;
+
 	struct rrr_ip_data ip_data = {0};
 
-	ip_data.fd = listen_handle->submodule_private_fd;
+	ip_data.fd = listen_handle->submodule_fd;
 
 	if ((ret = rrr_ip_accept(&accept_data, &ip_data, "net_transport_plain", 0)) != 0) {
 		RRR_MSG_0("Error while accepting connection in plain server\n");
@@ -259,19 +338,32 @@ int __rrr_net_transport_plain_accept (
 		goto out;
 	}
 
+	struct rrr_net_transport_plain_allocate_and_add_callback_data callback_data = {
+			&accept_data->ip_data
+	};
+
 	int new_handle = 0;
 	if ((ret = rrr_net_transport_handle_allocate_and_add (
 			&new_handle,
 			listen_handle->transport,
 			RRR_NET_TRANSPORT_SOCKET_MODE_CONNECTION,
 			__rrr_net_transport_plain_handle_allocate_and_add_callback,
-			&accept_data->ip_data.fd
+			&callback_data
 	)) != 0) {
 		RRR_MSG_0("Could not get handle in __rrr_net_transport_plain_accept return was %i\n", ret);
 		ret = 1;
 		goto out_destroy_ip;
 	}
 
+	const struct rrr_net_transport_plain_data *listen_plain_data = listen_handle->submodule_private_ptr;
+
+	{
+		char buf[128];
+		rrr_ip_to_str(buf, sizeof(buf), (const struct sockaddr *) &accept_data->addr, accept_data->len);
+		RRR_DBG_7("Plain transport accepted connection on port %u from %s transport handle %p/%i\n",
+				listen_plain_data->ip_data.port, buf, listen_handle->transport, new_handle);
+	}
+
 	ret = callback (
 			listen_handle->transport,
 			new_handle,
@@ -282,6 +374,8 @@ int __rrr_net_transport_plain_accept (
 			callback_arg
 	);
 
+	*did_accept = 1;
+
 	goto out;
 	out_destroy_ip:
 		rrr_ip_close(&accept_data->ip_data);
@@ -293,7 +387,26 @@ int __rrr_net_transport_plain_accept (
 static int __rrr_net_transport_plain_poll (
 		struct rrr_net_transport_handle *handle
 ) {
-	return rrr_socket_check_alive (handle->submodule_private_fd);
+	return rrr_socket_check_alive (handle->submodule_fd);
+}
+
+static int __rrr_net_transport_plain_handshake (
+		struct rrr_net_transport_handle *handle
+) {
+	(void)(handle);
+	return RRR_NET_TRANSPORT_SEND_OK;
+}
+
+static int __rrr_net_transport_plain_is_tls (void) {
+	return 0;
+}
+
+static void __rrr_net_transport_plain_selected_proto_get (
+		const char **proto,
+		struct rrr_net_transport_handle *handle
+) {
+	(void)(handle);
+	*proto = NULL;
 }
 
 static const struct rrr_net_transport_methods plain_methods = {
@@ -303,8 +416,12 @@ static const struct rrr_net_transport_methods plain_methods = {
 	__rrr_net_transport_plain_accept,
 	__rrr_net_transport_plain_close,
 	__rrr_net_transport_plain_read_message,
+	__rrr_net_transport_plain_read,
 	__rrr_net_transport_plain_send,
-	__rrr_net_transport_plain_poll
+	__rrr_net_transport_plain_poll,
+	__rrr_net_transport_plain_handshake,
+	__rrr_net_transport_plain_is_tls,
+	__rrr_net_transport_plain_selected_proto_get
 };
 
 int rrr_net_transport_plain_new (struct rrr_net_transport_plain **target) {
@@ -314,7 +431,7 @@ int rrr_net_transport_plain_new (struct rrr_net_transport_plain **target) {
 
 	int ret = 0;
 
-	if ((result = malloc(sizeof(*result))) == NULL) {
+	if ((result = rrr_allocate(sizeof(*result))) == NULL) {
 		RRR_MSG_0("Could not allocate memory in rrr_net_transport_plain_new\n");
 		ret = 1;
 		goto out;
