@@ -24,6 +24,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "log.h"
 #include "parse.h"
+#include "allocator.h"
 #include "util/macro_utils.h"
 
 void rrr_parse_pos_init (
@@ -38,7 +39,9 @@ void rrr_parse_pos_init (
 	target->line_begin_pos = 0;
 }
 
-void rrr_parse_ignore_space_and_tab (struct rrr_parse_pos *pos) {
+void rrr_parse_ignore_space_and_tab (
+		struct rrr_parse_pos *pos
+) {
 	if (pos->pos >= pos->size) {
 		return;
 	}
@@ -55,7 +58,9 @@ void rrr_parse_ignore_space_and_tab (struct rrr_parse_pos *pos) {
 	}
 }
 
-void rrr_parse_ignore_spaces_and_increment_line (struct rrr_parse_pos *pos) {
+void rrr_parse_ignore_spaces_and_increment_line (
+		struct rrr_parse_pos *pos
+) {
 	if (pos->pos >= pos->size) {
 		return;
 	}
@@ -91,7 +96,9 @@ void rrr_parse_ignore_spaces_and_increment_line (struct rrr_parse_pos *pos) {
 	}
 }
 
-void rrr_parse_comment (struct rrr_parse_pos *pos) {
+void rrr_parse_comment (
+		struct rrr_parse_pos *pos
+) {
 	if (pos->pos >= pos->size) {
 		return;
 	}
@@ -296,7 +303,7 @@ void rrr_parse_non_newline (
 	*end = pos->pos - 1;
 }
 
-int rrr_parse_extract_string (
+int rrr_parse_str_extract (
 		char **target,
 		struct rrr_parse_pos *pos,
 		const int begin,
@@ -308,7 +315,7 @@ int rrr_parse_extract_string (
 		RRR_BUG("BUG: length was 0 in __rrr_config_extract_string\n");
 	}
 
-	char *bytes = malloc(length + 1);
+	char *bytes = rrr_allocate(length + 1);
 
 	if (bytes == NULL) {
 		RRR_MSG_0("Could not allocate memory in __rrr_config_extract_string\n");
@@ -346,7 +353,7 @@ int rrr_parse_str_split (
 		goto do_callback;
 	}
 
-	if ((tmp = strdup(str)) == NULL) {
+	if ((tmp = rrr_strdup(str)) == NULL) {
 		RRR_MSG_0("Could not allocate memory in rrr_parse_str_split\n");
 		ret = 1;
 		goto out;
@@ -356,7 +363,6 @@ int rrr_parse_str_split (
 	const char *element = tmp;
 	int zero_found = 0;
 	while (!zero_found) {
-//		printf ("Split pos %s\n", pos);
 
 		if (elements_count == elements_max) {
 			RRR_MSG_0("Too many elements while splitting string (more than %lu)\n", elements_max);
@@ -399,12 +405,12 @@ int rrr_parse_str_extract_until (
 	const char *end = strchr(str, end_char);
 
 	if (end == NULL) {
-		return 1;
+		return 0;
 	}
 
 	size_t length = end - pos;
 
-	char *match = malloc(length + 1);
+	char *match = rrr_allocate(length + 1);
 	if (match == NULL) {
 		RRR_MSG_0("Could not allocate memory in rrr_parse_extract_until\n");
 		return 1;
@@ -438,3 +444,17 @@ void rrr_parse_str_strip_newlines (
 		}
 	}
 }
+
+void rrr_parse_str_trim (
+		char *str
+) {
+	size_t length = strlen(str);
+	for (char *pos = str + length -1; pos >= str; pos--) {
+		if (*pos == ' ' || *pos == '\r' || *pos == '\n' || *pos == '\t') {
+			*pos = '\0';
+		}
+		else {
+			break;
+		}
+	}
+}
