@@ -40,6 +40,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 	((c) == '\r' || (c) == '\n')
 #define RRR_PARSE_MATCH_C_COMMAS(c)		\
 	((c) == ',' || (c) == ';')
+#define RRR_PARSE_MATCH_C_DASH(c)		\
+	((c) == '-')
 #define RRR_PARSE_MATCH_C_LETTER(c) 	\
 	(	((c) >= 'a' && (c) <= 'z') ||	\
 		((c) >= 'A' && (c) <= 'Z') ||	\
@@ -53,9 +55,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #define RRR_PARSE_MATCH_C_NULL(c) 		\
 	((c) == '\0')
 
-// XXX : Removed - for compatibility with condition expressions
-// || ((c) == '-')
-
 struct rrr_parse_pos {
 	const char *data;
 	int pos;
@@ -69,9 +68,15 @@ void rrr_parse_pos_init (
 		const char *data,
 		int size
 );
-void rrr_parse_ignore_space_and_tab (struct rrr_parse_pos *pos);
-void rrr_parse_ignore_spaces_and_increment_line (struct rrr_parse_pos *pos);
-void rrr_parse_comment (struct rrr_parse_pos *pos);
+void rrr_parse_ignore_space_and_tab (
+		struct rrr_parse_pos *pos
+);
+void rrr_parse_ignore_spaces_and_increment_line (
+		struct rrr_parse_pos *pos
+);
+void rrr_parse_comment (
+		struct rrr_parse_pos *pos
+);
 int rrr_parse_match_word_case (
 		struct rrr_parse_pos *pos,
 		const char *word
@@ -100,7 +105,7 @@ void rrr_parse_non_newline (
 		int *start,
 		int *end
 );
-int rrr_parse_extract_string (
+int rrr_parse_str_extract (
 		char **target,
 		struct rrr_parse_pos *pos,
 		const int begin,
@@ -122,5 +127,8 @@ int rrr_parse_str_extract_until (
 void rrr_parse_str_strip_newlines (
 		char *str
 );
+void rrr_parse_str_trim (
+		char *str
+);
 
 #endif /* RRR_PARSE_H */
