@@ -35,6 +35,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "parse.h"
 #include "log.h"
 #include "passwd.h"
+#include "allocator.h"
 #include "socket/rrr_socket.h"
 #include "rrr_strerror.h"
 #include "util/macro_utils.h"
@@ -52,7 +53,7 @@ static void __rrr_passwd_permission_destroy (
 		struct rrr_passwd_permission *permission
 ) {
 	RRR_FREE_IF_NOT_NULL(permission->permission);
-	free(permission);
+	rrr_free(permission);
 }
 
 void rrr_passwd_permission_collection_clear (
@@ -69,7 +70,7 @@ static int __rrr_passwd_permission_new (
 
 	*target = NULL;
 
-	struct rrr_passwd_permission *permission = malloc(sizeof(*permission));
+	struct rrr_passwd_permission *permission = rrr_allocate(sizeof(*permission));
 
 	if (permission == NULL) {
 		RRR_MSG_0("Could not allocate memory for permission in __rrr_passwd_permission_new\n");
@@ -79,7 +80,7 @@ static int __rrr_passwd_permission_new (
 
 	memset(permission, '\0', sizeof(*permission));
 
-	if ((permission->permission = strdup(permission_str)) == NULL) {
+	if ((permission->permission = rrr_strdup(permission_str)) == NULL) {
 		RRR_MSG_0("Could not allocate memory for permission string in  __rrr_passwd_permission_new\n");
 		ret = 1;
 		goto out_free;
@@ -90,7 +91,7 @@ static int __rrr_passwd_permission_new (
 
 	goto out;
 	out_free:
-		free(permission);
+		rrr_free(permission);
 	out:
 	return ret;
 }
@@ -331,7 +332,10 @@ static int __rrr_passwd_check_callback (
 	return ret;
 }
 
-int rrr_passwd_check (const char *hash, const char *password) {
+int rrr_passwd_check (
+		const char *hash,
+		const char *password
+) {
 	struct passwd_check_callback_data callback_data = { password };
 
 	if (*password == '\0') {
@@ -347,18 +351,19 @@ int rrr_passwd_check (const char *hash, const char *password) {
 	return rrr_parse_str_split(hash, '$', 4, __rrr_passwd_check_callback, &callback_data);
 }
 
-int rrr_passwd_encrypt (char **result, const char *password) {
+int rrr_passwd_encrypt (
+		char **result,
+		const char *password
+) {
 	unsigned char *base64_tmp = NULL;
 	size_t base64_tmp_length = 0;
 
-	unsigned char tmp[RRR_PASSWD_HASH_MAX_LENGTH + 1];
-
 	int ret = 0;
 
 	*result = NULL;
 
 	// Must be more than 256 to hold OpenSSL error strings
-	char *final = malloc(RRR_PASSWD_HASH_MAX_LENGTH + 1);
+	char *final = rrr_allocate(RRR_PASSWD_HASH_MAX_LENGTH + 1);
 	if (final == NULL) {
 		RRR_MSG_0("Could not allocate memory in rrr_passwd_encrypt\n");
 		ret = 1;
@@ -369,6 +374,8 @@ int rrr_passwd_encrypt (char **result, const char *password) {
 
 #ifdef RRR_WITH_OPENSSL_CRYPT
 	unsigned char salt[RRR_PASSWD_SALT_BYTES];
+	unsigned char tmp[RRR_PASSWD_HASH_MAX_LENGTH + 1];
+
 	if (RAND_bytes(salt, RRR_PASSWD_SALT_BYTES) != 1) {
 		ERR_error_string_n(ERR_get_error(), (char *) tmp, RRR_PASSWD_HASH_MAX_LENGTH + 1);
 		RRR_MSG_0("Could not generate salt in rrr_passwd_encrypt: %s\n", tmp);
@@ -527,7 +534,7 @@ int rrr_passwd_iterate_lines (
 		}
 
 		RRR_FREE_IF_NOT_NULL(line_tmp);
-		if (rrr_parse_extract_string(&line_tmp, &parse_pos, line_start, (line_end - line_start) + 1) != 0) {
+		if (rrr_parse_str_extract(&line_tmp, &parse_pos, line_start, (line_end - line_start) + 1) != 0) {
 			RRR_MSG_0("Could not allocate memory for line in process()\n");
 			ret = 1;
 			goto out;
@@ -624,8 +631,6 @@ static int __rrr_passwd_authenticate_callback (
 	return ret;
 }
 
-// TODO : Move to separate daemon
-
 int rrr_passwd_authenticate (
 		const char *filename,
 		const char *username,
@@ -652,6 +657,12 @@ int rrr_passwd_authenticate (
 		goto out;
 	}
 
+	if (passwd_file_size == 0) {
+		RRR_DBG_1("Password file '%s' was empty, access denied\n", filename);
+		ret = 1;
+		goto out;
+	}
+
 	struct rrr_passwd_authenticate_callback_data callback_data = {
 			username,
 			password,
@@ -797,7 +808,7 @@ int rrr_passwd_read_password_from_terminal (
 		}
 	}
 
-	*result = strdup(buf);
+	*result = rrr_strdup(buf);
 	if (*result == NULL) {
 		RRR_MSG_0("Could not allocate memory in read_password_stdin\n");
 		ret = 1;
@@ -839,7 +850,7 @@ int rrr_passwd_read_password_from_stdin (
 
 	buf[bytes] = '\0';
 
-	*result = strdup(buf);
+	*result = rrr_strdup(buf);
 	if (*result == NULL) {
 		RRR_MSG_0("Could not allocate memory in rrr_passwd_read_password_from_stdin\n");
 		ret = 1;
