@@ -28,6 +28,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "../util/posix.h"
 #include "../log.h"
+#include "../allocator.h"
 #include "perl5.h"
 #include "perl5_types.h"
 #include "perl5_hv_macros.h"
@@ -145,7 +146,6 @@ static PerlInterpreter *__rrr_perl5_construct(void) {
 	out_unlock:
 	__rrr_perl5_init_unlock();
 
-	out:
 	return ret;
 }
 
@@ -219,20 +219,20 @@ void rrr_perl5_destroy_ctx (struct rrr_perl5_ctx *ctx) {
 	}
 	__rrr_perl5_destruct(ctx->interpreter);
 	__rrr_perl5_remove_ctx(ctx);
-	free(ctx);
+	rrr_free(ctx);
 }
 
 int rrr_perl5_new_ctx (
 		struct rrr_perl5_ctx **target,
 		void *private_data,
-		int (*send_message) (struct rrr_msg_msg *message, const struct rrr_msg_addr *message_addr, void *private_data),
+		int (*send_message) (const struct rrr_msg_msg *message, const struct rrr_msg_addr *message_addr, void *private_data),
 		char *(*get_setting) (const char *key, void *private_data),
 		int (*set_setting) (const char *key, const char *value, void *private_data)
 ) {
 	int ret = 0;
 	struct rrr_perl5_ctx *ctx = NULL;
 
-	ctx = malloc(sizeof(*ctx));
+	ctx = rrr_allocate(sizeof(*ctx));
 	if (ctx == NULL) {
 		RRR_MSG_0("Could not allocate memory in rrr_perl5_new_ctx\n");
 		ret = 1;
@@ -420,7 +420,7 @@ int rrr_perl5_call_blessed_hvref (struct rrr_perl5_ctx *ctx, const char *sub, co
 		}} while (0)
 
 struct rrr_perl5_message_hv *rrr_perl5_allocate_message_hv_with_hv (struct rrr_perl5_ctx *ctx, HV *hv) {
-    struct rrr_perl5_message_hv *message_hv = malloc(sizeof(*message_hv));
+    struct rrr_perl5_message_hv *message_hv = rrr_allocate(sizeof(*message_hv));
     if (message_hv == NULL) {
     	RRR_MSG_0("Could not allocate memory in rrr_perl5_message_allocate_hv\n");
     	return NULL;
@@ -437,7 +437,7 @@ struct rrr_perl5_message_hv *__rrr_perl5_allocate_message_hv (struct rrr_perl5_c
 
     HV *hv = NULL;
 
-    struct rrr_perl5_message_hv *message_hv = malloc(sizeof(*message_hv));
+    struct rrr_perl5_message_hv *message_hv = rrr_allocate(sizeof(*message_hv));
     if (message_hv == NULL) {
     	RRR_MSG_0("Could not allocate memory in rrr_perl5_message_allocate_hv\n");
     	goto out;
@@ -525,7 +525,7 @@ void rrr_perl5_destruct_settings_hv (
 	source->allocated_entries = 0;
 	source->used_entries = 0;
 
-	free(source);
+	rrr_free(source);
 }
 
 int rrr_perl5_settings_to_hv (
@@ -538,7 +538,7 @@ int rrr_perl5_settings_to_hv (
 	PerlInterpreter *my_perl = ctx->interpreter;
     PERL_SET_CONTEXT(my_perl);
 
-	struct rrr_perl5_settings_hv *settings_hv = malloc(sizeof(*settings_hv));
+	struct rrr_perl5_settings_hv *settings_hv = rrr_allocate(sizeof(*settings_hv));
 	if (settings_hv == NULL) {
 		RRR_MSG_0("Could not allocate memory in rrr_perl5_config_to_hv\n");
 		ret = 1;
@@ -548,18 +548,6 @@ int rrr_perl5_settings_to_hv (
 
 	settings_hv->hv = newHV();
 
-/* TODO:	We don't actually fill up the HV but instead force the user to utilize the
- * 			get()-method so that we can update was_used-parameter, maybe delete the following
-
-	struct rrr_perl5_settings_to_hv_callback_args callback_args = {
-			ctx, settings_hv
-	};
-	ret = rrr_settings_iterate(source, __rrr_perl5_settings_to_hv_callback, &callback_args);
-	if (ret != 0) {
-		RRR_MSG_0("Error while converting instance settings to hv in perl5\n");
-		goto out;
-	}*/
-
 	*target = settings_hv;
 
 	out:
@@ -582,10 +570,10 @@ void rrr_perl5_destruct_message_hv (
 
 	SV_DEC_UNLESS_NULL(source->hv);
 
-	free(source);
+	rrr_free(source);
 }
 
-static int __rrr_perl5_hv_to_message_extract_array (
+static int __rrr_perl5_hv_to_message_process_array (
 		struct rrr_msg_msg **target,
 		struct rrr_perl5_ctx *ctx,
 		struct rrr_perl5_message_hv *source
@@ -599,22 +587,22 @@ static int __rrr_perl5_hv_to_message_extract_array (
 	struct rrr_array array_tmp = {0};
 	HV *hv = source->hv;
 
-	// New style array handling
 	RRR_PERL5_DEFINE_AND_FETCH_ARRAY_PTR_FROM_HV(hv);
 	if (array != NULL) {
 		if (rrr_array_append_from (
 				&array_tmp,
 				array
 		) != 0) {
-			RRR_MSG_0("Failed to clone array values in __rrr_perl5_hv_to_message_extract_array\n");
+			RRR_MSG_0("Failed to clone array values in __rrr_perl5_hv_to_message_process_array\n");
 			ret = 1;
 			goto out;
 		}
 	}
 
-	// av_len returns -1 when array is empty
-	if (/*av_len(array_values_final) < 0 && */RRR_LL_COUNT(&array_tmp) == 0) {
-		// No array values
+	if (RRR_LL_COUNT(&array_tmp) == 0) {
+		// No array values, make sure class does not have array
+		// class set in case it had this from earlier.
+		MSG_SET_CLASS(*target, MSG_CLASS_DATA);
 		ret = 0;
 		goto out;
 	}
@@ -626,20 +614,20 @@ static int __rrr_perl5_hv_to_message_extract_array (
 			MSG_TOPIC_PTR(*target),
 			MSG_TOPIC_LENGTH(*target)
 	) != 0) {
-		RRR_MSG_0("Could not create new array message in _rrr_perl5_hv_to_message_extract_array\n");
+		RRR_MSG_0("Could not create new array message in _rrr_perl5_hv_to_message_process_array\n");
 		ret = 1;
 		goto out;
 	}
 
 	message_tmp->timestamp = (*target)->timestamp;
 
-	free (*target);
+	rrr_free (*target);
 	*target = message_tmp;
 	message_tmp = NULL;
 
 	out:
 	if (message_tmp != NULL) {
-		free(message_tmp);
+		rrr_free(message_tmp);
 	}
 	rrr_array_clear(&array_tmp);
 	return ret;
@@ -751,7 +739,7 @@ int rrr_perl5_hv_to_message (
 	}
 
 	if (MSG_TOTAL_SIZE(target) > old_total_len) {
-		struct rrr_msg_msg *new_message = realloc(target, MSG_TOTAL_SIZE(target));
+		struct rrr_msg_msg *new_message = rrr_reallocate_group(target, old_total_len, MSG_TOTAL_SIZE(target), RRR_ALLOCATOR_GROUP_MSG);
 		if (new_message == NULL) {
 			RRR_MSG_0("Could not re-allocate memory in rrr_perl5_hv_to_message\n");
 			ret = 1;
@@ -770,8 +758,8 @@ int rrr_perl5_hv_to_message (
 	memcpy (MSG_TOPIC_PTR(target), topic_str, new_topic_len);
 	memcpy (MSG_DATA_PTR(target), data_str, new_data_len);
 
-	// This function will re-allocate the message and erase data if array values are set in the perl5 script
-	if (__rrr_perl5_hv_to_message_extract_array(&target, ctx, source)) {
+	// This function will re-allocate the message and erase data if array values are set in the perl5 script.
+	if (__rrr_perl5_hv_to_message_process_array(&target, ctx, source)) {
 		RRR_MSG_0("Error while converting HV to RRR message in rrr_perl5_hv_to_message\n");
 		ret = 1;
 		goto out;
@@ -827,7 +815,10 @@ static int __rrr_perl5_message_to_hv (
 	RRR_PERL5_DEFINE_AND_FETCH_FROM_HV(rrr_array_ptr, hv);
 
 	SvFLAGS(rrr_array_ptr) &= ~(SVf_PROTECT|SVf_READONLY);
-	sv_setiv(rrr_array_ptr, (IV) array);
+	sv_setiv(rrr_array_ptr, (intptr_t) array);
+	if (SvIV(rrr_array_ptr) != (intptr_t) array) {
+		RRR_BUG("BUG: RRR array pointer storage failure in __rrr_perl5_message_to_hv, possibly a problem on this particular architecture\n");
+	}
 	SvFLAGS(rrr_array_ptr) |= SVf_PROTECT|SVf_READONLY;
 
     SvUTF8_off(ip_addr);
@@ -850,7 +841,8 @@ static int __rrr_perl5_message_to_hv (
     // New style array handling
     rrr_array_clear(array);
     if (MSG_IS_ARRAY(message)) {
-		if (rrr_array_message_append_to_collection(array, message) != 0) {
+	    	uint16_t array_version_dummy;
+		if (rrr_array_message_append_to_collection(&array_version_dummy, array, message) != 0) {
 			RRR_MSG_0("Could not convert message to array collection in __rrr_perl5_message_to_hv\n");
 			ret = 1;
 			goto out;
