@@ -27,6 +27,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <perl.h>
 
 #include "../log.h"
+#include "../allocator.h"
 #include "perl5.h"
 #include "perl5_types.h"
 #include "perl5_xsub.h"
@@ -44,38 +45,34 @@ unsigned int rrr_perl5_message_send (HV *hv) {
 	PerlInterpreter *my_perl = PERL_GET_CONTEXT;
 	struct rrr_perl5_ctx *ctx = rrr_perl5_find_ctx (my_perl);
 	struct rrr_perl5_message_hv *message_new_hv = NULL;
+	struct rrr_msg_msg *message_new = NULL;
 
 	int ret = 0;
 
 	if ((message_new_hv = rrr_perl5_allocate_message_hv_with_hv (ctx, hv)) == NULL) { // Will incref
 		RRR_MSG_0("Could not allocate message hv in rrr_perl5_message_send\n");
 		ret = 1;
-		goto out_final;
+		goto out;
 	}
 
 	struct rrr_msg_addr addr_msg;
-	struct rrr_msg_msg *message_new = NULL;
 	if (rrr_msg_msg_new_empty(&message_new, MSG_TYPE_MSG, MSG_CLASS_DATA, rrr_time_get_64(), 0, 0) != 0) {
 		RRR_MSG_0("Could not allocate new message in rrr_perl5_message_send\n");
 		ret = 1;
-		goto out_free_message_hv;
+		goto out;
 	}
 	if (rrr_perl5_hv_to_message(&message_new, &addr_msg, ctx, message_new_hv) != 0) {
 		ret = 1;
-		goto out_free_message_new;
+		goto out;
 	}
 
-	// Takes ownership of memory of message (but not address message)
 	ctx->send_message(message_new, &addr_msg, ctx->private_data);
-	message_new = NULL;
 
-	// Always destroy message hv
-	goto out_free_message_hv;
-	out_free_message_new:
+	out:
+		if (message_new_hv != NULL) {
+			rrr_perl5_destruct_message_hv(ctx, message_new_hv);
+		}
 		RRR_FREE_IF_NOT_NULL(message_new);
-	out_free_message_hv:
-		rrr_perl5_destruct_message_hv(ctx, message_new_hv);
-	out_final:
 		return ret;
 }
 
@@ -430,6 +427,7 @@ AV *rrr_perl5_message_ip_get (HV *hv) {
 	av_push(result, newSVuv(result_port));
 
 	out:
+	(void)(ret); // Ignore dummy ret value
 	return result;
 }
 
@@ -471,6 +469,7 @@ SV *rrr_perl5_message_ip_get_protocol (HV *hv) {
 	}
 
 	out:
+	(void)(ret); // Ignore dummy ret value
 	return result;
 }
 
