@@ -27,6 +27,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "python3_module_common.h"
 #include "python3_config.h"
 #include "../log.h"
+#include "../allocator.h"
 #include "../settings.h"
 
 struct rrr_python3_config_data {
@@ -181,7 +182,9 @@ PyTypeObject rrr_python3_config_type = {
 	    .tp_basicsize	= sizeof(struct rrr_python3_config_data),
 		.tp_itemsize	= 0,
 	    .tp_dealloc		= (destructor) rrr_python3_config_f_dealloc,
+#ifdef RRR_PYTHON3_HAS_PTYPEOBJECT_TP_PRINT
 	    .tp_print		= NULL,
+#endif
 	    .tp_getattr		= NULL,
 	    .tp_setattr		= NULL,
 	    .tp_as_async	= NULL,
