@@ -35,9 +35,10 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "python3_module.h"
 #include "python3_module_common.h"
 #include "python3_socket.h"
-#include "python3_vl_message.h"
+#include "python3_message.h"
 
 #include "../log.h"
+#include "../allocator.h"
 #include "../settings.h"
 #include "../read.h"
 #include "../socket/rrr_socket.h"
@@ -88,13 +89,13 @@ static PyObject *rrr_python3_socket_f_send (PyObject *self, PyObject *arg) {
 	const struct rrr_msg_msg *message_orig = NULL;
 	struct rrr_msg_msg *message = NULL;
 
-	if (!rrr_python3_rrr_msg_msg_check(arg)) {
+	if (!rrr_python3_rrr_message_check(arg)) {
 		RRR_MSG_0("Received unknown object type in python3 socket send\n");
 		ret = 1;
 		goto out;
 	}
 
-	message_orig = rrr_python3_rrr_msg_msg_get_message (&message_addr, arg);
+	message_orig = rrr_python3_rrr_message_get_message (&message_addr, arg);
 
 	message = rrr_msg_msg_duplicate(message_orig);
 	if (message == NULL) {
@@ -136,7 +137,9 @@ PyTypeObject rrr_python3_socket_type = {
 	    .tp_basicsize	= sizeof(struct rrr_python3_socket_data),
 		.tp_itemsize	= 0,
 	    .tp_dealloc		= (destructor) rrr_python3_socket_f_dealloc,
+#ifdef RRR_PYTHON3_HAS_PTYPEOBJECT_TP_PRINT
 	    .tp_print		= NULL,
+#endif
 	    .tp_getattr		= NULL,
 	    .tp_setattr		= NULL,
 	    .tp_as_async	= NULL,
@@ -226,8 +229,7 @@ int rrr_python3_socket_send (
 
 	pthread_mutex_lock(&socket_data->send_lock);
 
-	// Always frees message
-	if ((ret = rrr_cmodule_ext_send_message_to_parent(
+	if ((ret = rrr_cmodule_ext_send_message_to_parent (
 			socket_data->worker,
 			message,
 			message_addr
@@ -237,5 +239,6 @@ int rrr_python3_socket_send (
 	}
 
 	pthread_mutex_unlock(&socket_data->send_lock);
+	rrr_free(message);
 	return ret;
 }
