@@ -2,7 +2,7 @@
 
 Read Route Record
 
-Copyright (C) 2019-2020 Atle Solbakken atle@goliathdns.no
+Copyright (C) 2019-2021 Atle Solbakken atle@goliathdns.no
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "log.h"
 #include "read.h"
 #include "read_constants.h"
+#include "allocator.h"
 #include "messages/msg_msg.h"
 #include "messages/msg_addr.h"
 #include "messages/msg_log.h"
@@ -41,7 +42,7 @@ struct rrr_read_session *rrr_read_session_new (
 		struct sockaddr *src_addr,
 		socklen_t src_addr_len
 ) {
-	struct rrr_read_session *read_session = malloc(sizeof(*read_session));
+	struct rrr_read_session *read_session = rrr_allocate(sizeof(*read_session));
 	if (read_session == NULL) {
 		RRR_MSG_0("Could not allocate memory in __rrr_socket_read_session_new\n");
 		return NULL;
@@ -62,8 +63,8 @@ struct rrr_read_session *rrr_read_session_new (
 int rrr_read_session_cleanup (
 		struct rrr_read_session *read_session
 ) {
-	RRR_FREE_IF_NOT_NULL(read_session->rx_buf_ptr);
-	RRR_FREE_IF_NOT_NULL(read_session->rx_overshoot);
+	RRR_ALLOCATOR_FREE_IF_NOT_NULL(read_session->rx_buf_ptr);
+	RRR_ALLOCATOR_FREE_IF_NOT_NULL(read_session->rx_overshoot);
 	return 0;
 }
 
@@ -71,7 +72,7 @@ int rrr_read_session_destroy (
 		struct rrr_read_session *read_session
 ) {
 	rrr_read_session_cleanup(read_session);
-	free(read_session);
+	rrr_free(read_session);
 	return 0;
 }
 
@@ -90,7 +91,6 @@ void rrr_read_session_collection_clear (
 struct rrr_read_session *rrr_read_session_collection_get_session_with_overshoot (
 		struct rrr_read_session_collection *collection
 ) {
-
 	RRR_LL_ITERATE_BEGIN(collection,struct rrr_read_session);
 		if (node->rx_overshoot != NULL) {
 			return node;
@@ -99,13 +99,27 @@ struct rrr_read_session *rrr_read_session_collection_get_session_with_overshoot
 	return NULL;
 }
 
+int rrr_read_session_collection_has_unprocessed_data (
+		const struct rrr_read_session_collection *collection
+) {
+	RRR_LL_ITERATE_BEGIN(collection,struct rrr_read_session);
+		if (node->rx_overshoot != NULL || (node->rx_buf_wpos > 0 && node->rx_buf_ptr != NULL)) {
+			return 1;
+		}
+	RRR_LL_ITERATE_END();
+	return 0;
+}
+
 struct rrr_read_session *rrr_read_session_collection_maintain_and_find_or_create (
+		int *is_new,
 		struct rrr_read_session_collection *collection,
 		struct sockaddr *src_addr,
 		socklen_t src_addr_len
 ) {
 	struct rrr_read_session *res = NULL;
 
+	*is_new = 0;
+
 	uint64_t time_now = rrr_time_get_64();
 	uint64_t time_limit = time_now - RRR_READ_COLLECTION_CLIENT_TIMEOUT_S * 1000 * 1000;
 
@@ -113,7 +127,9 @@ struct rrr_read_session *rrr_read_session_collection_maintain_and_find_or_create
 		if (node->last_read_time < time_limit) {
 			RRR_LL_ITERATE_SET_DESTROY();
 		}
-		else if (memcmp(src_addr, &node->src_addr, sizeof(*src_addr)) == 0) {
+		else if (	node->src_addr_len == src_addr_len &&
+				memcmp(src_addr, &node->src_addr, src_addr_len) == 0
+		) {
 			if (res != NULL) {
 				RRR_BUG("Two equal src_addr in rrr_socket_read_session_collection_maintain_and_find\n");
 			}
@@ -129,6 +145,8 @@ struct rrr_read_session *rrr_read_session_collection_maintain_and_find_or_create
 		}
 
 		RRR_LL_UNSHIFT(collection,res);
+
+		*is_new = 1;
 	}
 
 	out:
@@ -152,37 +170,41 @@ int rrr_read_message_using_callbacks (
 		ssize_t read_step_initial,
 		ssize_t read_step_max_size,
 		ssize_t read_max_size,
-		int									 (*function_get_target_size) (
-													struct rrr_read_session *read_session,
-													void *private_arg
-											 ),
-		int									 (*function_complete_callback) (
-													struct rrr_read_session *read_session,
-													void *private_arg
-											 ),
-		int									 (*function_read) (
-													char *buf,
-													ssize_t *read_bytes,
-													ssize_t read_step_max_size,
-													void *private_arg
-	 	 	 	 	 	 	 	 	 	 	 ),
-		struct rrr_read_session				*(*function_get_read_session_with_overshoot) (
-													void *private_arg
-											 ),
-		struct rrr_read_session				*(*function_get_read_session) (
-													void *private_arg
-											 ),
-		void								 (*function_read_session_remove) (
-													struct rrr_read_session *read_session,
-													void *private_arg
-										 	 ),
-		int									 (*function_get_socket_options) (
-													struct rrr_read_session *read_session,
-													void *private_arg
-											 ),
+		struct rrr_read_session *read_session_ratelimit,
+		uint64_t ratelimit_interval_us,
+		ssize_t ratelimit_max_bytes,
+		int (*function_get_target_size) (
+				struct rrr_read_session *read_session,
+				void *private_arg
+		),
+		int (*function_complete_callback) (
+				struct rrr_read_session *read_session,
+				void *private_arg
+		),
+		int (*function_read) (
+				char *buf,
+				ssize_t *read_bytes,
+				ssize_t read_step_max_size,
+				void *private_arg
+		),
+		struct rrr_read_session*(*function_get_read_session_with_overshoot) (
+				void *private_arg
+		),
+		struct rrr_read_session*(*function_get_read_session) (
+				void *private_arg
+		),
+		void (*function_read_session_remove) (
+				struct rrr_read_session *read_session,
+				void *private_arg
+		),
+		int (*function_get_socket_options) (
+				struct rrr_read_session *read_session,
+				void *private_arg
+		),
 		void *functions_callback_arg
 ) {
 	int ret = RRR_READ_OK;
+	int ret_from_read = RRR_READ_OK;
 
 	*bytes_read = 0;
 
@@ -195,8 +217,30 @@ int rrr_read_message_using_callbacks (
 		goto process_overshoot;
 	}
 
+	/* Check ratelimit. It is not possible to distinguish different read sessions when ratelimiting e.g.
+	 * when there are multiple read sessions for an UDP socket. */
+	if (read_session_ratelimit != NULL) {
+		if (ratelimit_max_bytes > 0) {
+			const uint64_t time_now = rrr_time_get_64();
+			if (time_now - read_session_ratelimit->ratelimit_time > ratelimit_interval_us) {
+				read_session_ratelimit->ratelimit_time = time_now;
+				read_session_ratelimit->ratelimit_bytes = 0;
+			}
+			else if (read_session_ratelimit->ratelimit_bytes > ratelimit_max_bytes) {
+				RRR_DBG_7("Read ratelimited %lli > %lli within %" PRIu64 " us\n",
+					(long long int) read_session_ratelimit->ratelimit_bytes, (long long int) ratelimit_max_bytes, ratelimit_interval_us);
+				ret = RRR_READ_RATELIMIT;
+				goto out;
+			}
+		}
+		else {
+			read_session_ratelimit->ratelimit_bytes = 0;
+			read_session_ratelimit->ratelimit_time = 0;
+		}
+	}
+
 	/* Read */
-	int ret_from_read = ret = function_read (buf, &bytes, read_step_max_size, functions_callback_arg);
+	ret_from_read = ret = function_read (buf, &bytes, read_step_max_size, functions_callback_arg);
 
 	// We don't quit on soft error yet, downstream must be able to retrieve the correct read session to
 	// handle errors, which might include to remove the read_session from the collection
@@ -261,9 +305,11 @@ int rrr_read_message_using_callbacks (
 		}
 	}
 
+	read_session->ratelimit_bytes += bytes;
 	read_session->eof_ok_now = 0;
 
 	process_overshoot:
+
 	if (read_session->rx_buf_ptr == NULL) {
 		if (read_session->rx_overshoot != NULL) {
 			read_session->rx_buf_ptr = read_session->rx_overshoot;
@@ -274,7 +320,7 @@ int rrr_read_message_using_callbacks (
 			read_session->rx_overshoot_size = 0;
 		}
 		else {
-			read_session->rx_buf_ptr = malloc(bytes > read_step_max_size ? bytes : read_step_max_size);
+			read_session->rx_buf_ptr = rrr_allocate_group(bytes > read_step_max_size ? bytes : read_step_max_size, RRR_ALLOCATOR_GROUP_MSG);
 			if (read_session->rx_buf_ptr == NULL) {
 				RRR_MSG_0("Could not allocate memory in rrr_socket_read_message\n");
 				ret = RRR_READ_HARD_ERROR;
@@ -296,7 +342,7 @@ int rrr_read_message_using_callbacks (
 		*bytes_read = bytes;
 		if (bytes + read_session->rx_buf_wpos > read_session->rx_buf_size) {
 			ssize_t new_size = read_session->rx_buf_size + (bytes > read_step_max_size ? bytes : read_step_max_size);
-			char *new_buf = realloc(read_session->rx_buf_ptr, new_size);
+			char *new_buf = rrr_reallocate_group(read_session->rx_buf_ptr, read_session->rx_buf_size, new_size, RRR_ALLOCATOR_GROUP_MSG);
 			if (new_buf == NULL) {
 				RRR_MSG_0("Could not re-allocate memory in rrr_read_message_using_callbacks\n");
 				ret = RRR_READ_HARD_ERROR;
@@ -333,7 +379,8 @@ int rrr_read_message_using_callbacks (
 		// may change the read complete method. This function may called multiple times if it does
 		// not return OK the first time. In that case, we will read more data repeatedly time until
 		// OK is returned.
-		if ((ret = function_get_target_size(read_session, functions_callback_arg)) != RRR_READ_OK) {
+		ret = function_get_target_size(read_session, functions_callback_arg);
+		if (ret != RRR_READ_OK && ret != RRR_READ_INCOMPLETE) {
 			goto out;
 		}
 
@@ -345,7 +392,7 @@ int rrr_read_message_using_callbacks (
 
 			RRR_DBG_7("Aligning buffer, skipping %li bytes while reading from socket\n", read_session->rx_buf_skip);
 
-			char *new_buf = malloc(read_session->rx_buf_size);
+			char *new_buf = rrr_allocate_group(read_session->rx_buf_size, RRR_ALLOCATOR_GROUP_MSG);
 			if (new_buf == NULL) {
 				RRR_MSG_0("Could not allocate memory while aligning buffer in rrr_read_message_using_callbacks\n");
 				ret = RRR_READ_HARD_ERROR;
@@ -353,10 +400,20 @@ int rrr_read_message_using_callbacks (
 			}
 			memcpy(new_buf, read_session->rx_buf_ptr + read_session->rx_buf_skip, read_session->rx_buf_wpos - read_session->rx_buf_skip);
 
-			free(read_session->rx_buf_ptr);
+			// Put new buffer into overshoot so that it is picked up again
+			// in the next read loop
+			read_session->rx_overshoot = new_buf;
+			read_session->rx_overshoot_size = read_session->rx_buf_wpos - read_session->rx_buf_skip;
 
-			read_session->rx_buf_ptr = new_buf;
-			read_session->rx_buf_wpos -= read_session->rx_buf_skip;
+			rrr_free(read_session->rx_buf_ptr);
+			read_session->rx_buf_ptr = NULL;
+			read_session->rx_buf_skip = 0;
+
+			ret = RRR_READ_INCOMPLETE;
+		}
+
+		if (ret != RRR_READ_OK) {
+			goto out;
 		}
 
 		if (read_session->target_size == 0 &&
@@ -376,7 +433,7 @@ int rrr_read_message_using_callbacks (
 		read_session->rx_overshoot_size = read_session->rx_buf_wpos - read_session->target_size;
 		read_session->rx_buf_wpos -= read_session->rx_overshoot_size;
 
-		read_session->rx_overshoot = malloc(read_session->rx_overshoot_size);
+		read_session->rx_overshoot = rrr_allocate_group(read_session->rx_overshoot_size, RRR_ALLOCATOR_GROUP_MSG);
 		if (read_session->rx_overshoot == NULL) {
 			RRR_MSG_0("Could not allocate memory for overshoot in rrr_read_message_using_callbacks\n");
 			ret = RRR_READ_HARD_ERROR;
@@ -395,8 +452,10 @@ int rrr_read_message_using_callbacks (
 				goto out;
 			}
 
-			RRR_FREE_IF_NOT_NULL(read_session->rx_buf_ptr);
+			RRR_ALLOCATOR_FREE_IF_NOT_NULL(read_session->rx_buf_ptr);
 			read_session->read_complete = 0;
+			read_session->target_size = 0;
+			read_session->read_complete_method = 0;
 		}
 	}
 	else if (read_session->read_complete_method == RRR_READ_COMPLETE_METHOD_ZERO_BYTES_READ ||
@@ -416,106 +475,6 @@ int rrr_read_message_using_callbacks (
 	return ret;
 }
 
-int rrr_read_common_receive_message_raw_callback (
-		void **data,
-		ssize_t data_size,
-		struct rrr_read_common_receive_message_callback_data *callback_data
-) {
-	struct rrr_msg *msg = *data;
-
-	int ret = 0;
-
-	// Header CRC32 is checked when reading the data from remote and getting size
-	if (rrr_msg_head_to_host_and_verify(msg, data_size) != 0) {
-		RRR_MSG_0("Message was invalid in rrr_socket_common_receive_message_raw_callback\n");
-		ret = RRR_READ_SOFT_ERROR;
-		goto out;
-	}
-
-	if (rrr_msg_check_data_checksum_and_length(msg, data_size) != 0) {
-		RRR_MSG_0 ("Message checksum was invalid in rrr_socket_common_receive_message_raw_callback\n");
-		ret = RRR_READ_SOFT_ERROR;
-		goto out;
-	}
-
-	if (RRR_MSG_IS_RRR_MESSAGE(msg)) {
-		if (callback_data->callback_msg == NULL) {
-			RRR_MSG_0("Received an rrr_msg_msg in rrr_read_common_receive_message_raw_callback but no callback is defined for this type\n");
-			ret = RRR_READ_SOFT_ERROR;
-			goto out;
-		}
-
-		struct rrr_msg_msg *message = (struct rrr_msg_msg *) msg;
-		if (rrr_msg_msg_to_host_and_verify(message, data_size) != 0) {
-			RRR_MSG_0("Message verification failed in read_message_raw_callback (size: %u<>%u)\n",
-					MSG_TOTAL_SIZE(message), message->msg_size);
-			ret = RRR_READ_SOFT_ERROR;
-			goto out;
-		}
-
-		ret = callback_data->callback_msg((struct rrr_msg_msg **) data, callback_data->callback_arg);
-	}
-	else if (RRR_MSG_IS_RRR_MESSAGE_ADDR(msg)) {
-		if (callback_data->callback_addr_msg == NULL) {
-			RRR_MSG_0("Received an rrr_msg_addr in rrr_read_common_receive_message_raw_callback but no callback is defined for this type\n");
-			ret = RRR_READ_SOFT_ERROR;
-			goto out;
-		}
-
-		struct rrr_msg_addr *message = (struct rrr_msg_addr *) msg;
-		if (rrr_msg_addr_to_host(message) != 0) {
-			RRR_MSG_0("Invalid data in received address message in rrr_read_common_receive_message_raw_callback\n");
-			ret = RRR_READ_SOFT_ERROR;
-			goto out;
-		}
-
-		ret = callback_data->callback_addr_msg(message, callback_data->callback_arg);
-	}
-	else if (RRR_MSG_IS_RRR_MESSAGE_LOG(msg)) {
-		if (callback_data->callback_log_msg == NULL) {
-			RRR_MSG_0("Received an rrr_msg_msg_log in rrr_read_common_receive_message_raw_callback but no callback is defined for this type\n");
-			ret = RRR_READ_SOFT_ERROR;
-			goto out;
-		}
-
-		struct rrr_msg_log *message = (struct rrr_msg_log *) msg;
-		if (rrr_msg_msg_log_to_host(message) != 0) {
-			RRR_MSG_0("Invalid data in received log message in rrr_read_common_receive_message_raw_callback\n");
-			ret = RRR_READ_SOFT_ERROR;
-			goto out;
-		}
-
-		ret = callback_data->callback_log_msg(message, callback_data->callback_arg);
-	}
-	else {
-		RRR_MSG_0("Received a socket message of unknown type %u in rrr_read_common_receive_message_raw_callback\n",
-				msg->msg_type);
-		ret = RRR_READ_SOFT_ERROR;
-		goto out;
-	}
-
-	out:
-	return ret;
-
-}
-
-int rrr_read_common_receive_message_callback (
-		struct rrr_read_session *read_session,
-		void *arg
-) {
-	int ret = 0;
-
-	if ((ret = rrr_read_common_receive_message_raw_callback((void **) &read_session->rx_buf_ptr, read_session->rx_buf_wpos, arg)) != 0) {
-		// Returns soft error if message is invalid, might also return
-		// other errors from final callback function
-		goto out;
-	}
-
-	out:
-	RRR_FREE_IF_NOT_NULL(read_session->rx_buf_ptr);
-	return ret;
-}
-
 int rrr_read_common_get_session_target_length_from_message_and_checksum_raw (
 		ssize_t *result,
 		void *data,
@@ -603,7 +562,6 @@ int rrr_read_common_get_session_target_length_from_array_tree (
 //	printf ("Array wpos: %li\n", wpos);
 
 	ssize_t import_length = 0;
-	ssize_t skipped_bytes = 0;
 
 	while (wpos > 0) {
 		if (pos_max != NULL && pos > pos_max) {
@@ -629,15 +587,10 @@ int rrr_read_common_get_session_target_length_from_array_tree (
 			break;
 		}
 		else {
-			if (ret == RRR_TYPE_PARSE_SOFT_ERR) {
-				if (data->do_byte_by_byte_sync != 0) {
-					skipped_bytes++;
-					pos++;
-					wpos--;
-					ret = RRR_READ_OK;
-				}
+			if (ret == RRR_TYPE_PARSE_SOFT_ERR && data->do_byte_by_byte_sync != 0) {
+				read_session->rx_buf_skip += 1;
+				return RRR_READ_INCOMPLETE;
 			}
-
 			return ret;
 		}
 	}
@@ -649,9 +602,6 @@ int rrr_read_common_get_session_target_length_from_array_tree (
 	// Raw size to read for socket framework
 	read_session->target_size = import_length;
 
-	// Read position for array framework
-	read_session->rx_buf_skip = skipped_bytes;
-
 	if (read_session->target_size == read_session->rx_buf_wpos) {
 		read_session->eof_ok_now = 1;
 	}
