@@ -24,6 +24,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <string.h>
 
 #include "log.h"
+#include "allocator.h"
 
 #include "socket/rrr_socket.h"
 #include "settings.h"
@@ -31,30 +32,27 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "util/macro_utils.h"
 #include "util/posix.h"
 
-struct rrr_msg *rrr_setting_safe_cast (struct rrr_setting_packed *setting) {
-	struct rrr_msg *ret = (struct rrr_msg *) setting;
-	ret->msg_type = RRR_MSG_TYPE_SETTING;
-	ret->msg_size = sizeof(*setting);
-	ret->msg_value = 0;
-	return ret;
-}
-
-void rrr_settings_list_destroy (struct rrr_settings_list *list) {
+static void __rrr_settings_list_destroy (
+		struct rrr_settings_list *list
+) {
 	if (list->data != NULL) {
-		free(list->data);
+		rrr_free(list->data);
 	}
 	if (list->list != NULL) {
-		free(list->list);
+		rrr_free(list->list);
 	}
-	free(list);
+	rrr_free(list);
 }
 
-int __rrr_settings_init(struct rrr_instance_settings *target, const int count) {
+static int __rrr_settings_init (
+		struct rrr_instance_settings *target,
+		const int count
+) {
 	int ret = 0;
 
 	memset(target, '\0', sizeof(*target));
 
-	target->settings = malloc(sizeof(*(target->settings)) * count);
+	target->settings = rrr_allocate(sizeof(*(target->settings)) * count);
 	if (target->settings == NULL) {
 		RRR_MSG_0("Could not allocate memory in __rrr_settings_init\n");
 		ret = 1;
@@ -73,13 +71,15 @@ int __rrr_settings_init(struct rrr_instance_settings *target, const int count) {
 
 	goto out;
 	out_free:
-		free(target->settings);
+		rrr_free(target->settings);
 	out:
 		return ret;
 }
 
-struct rrr_instance_settings *rrr_settings_new(const int count) {
-	struct rrr_instance_settings *ret = malloc(sizeof(*ret));
+struct rrr_instance_settings *rrr_settings_new (
+		const int count
+) {
+	struct rrr_instance_settings *ret = rrr_allocate(sizeof(*ret));
 
 	if (ret == NULL) {
 		RRR_MSG_0("Could not allocate memory for module settings structure");
@@ -87,18 +87,22 @@ struct rrr_instance_settings *rrr_settings_new(const int count) {
 	}
 
 	if (__rrr_settings_init(ret, count) != 0) {
-		free(ret);
+		rrr_free(ret);
 		return NULL;
 	}
 
 	return ret;
 }
 
-void __rrr_settings_destroy_setting(struct rrr_setting *setting) {
-	free(setting->data);
+static void __rrr_settings_destroy_setting (
+		struct rrr_setting *setting
+) {
+	rrr_free(setting->data);
 }
 
-void rrr_settings_destroy(struct rrr_instance_settings *target) {
+void rrr_settings_destroy (
+		struct rrr_instance_settings *target
+) {
 	pthread_mutex_lock(&target->mutex);
 
 	if (target->initialized != 1) {
@@ -116,27 +120,32 @@ void rrr_settings_destroy(struct rrr_instance_settings *target) {
 	pthread_mutex_unlock(&target->mutex);
 	pthread_mutex_destroy(&target->mutex);
 
-	free(target->settings);
-	free(target);
+	rrr_free(target->settings);
+	rrr_free(target);
 }
 
-void __rrr_settings_lock(struct rrr_instance_settings *settings) {
+static void __rrr_settings_lock (
+		struct rrr_instance_settings *settings
+) {
 	if (settings->initialized != 1) {
 		RRR_BUG("BUG: Tried to lock destroyed settings structure\n");
 	}
-//	RRR_DBG_4 ("Settings %p lock\n", settings);
 	pthread_mutex_lock(&settings->mutex);
 }
 
-void __rrr_settings_unlock(struct rrr_instance_settings *settings) {
+static void __rrr_settings_unlock (
+		struct rrr_instance_settings *settings
+) {
 	if (settings->initialized != 1) {
 		RRR_BUG("BUG: Tried to unlock destroyed settings structure\n");
 	}
-//	RRR_DBG_4 ("Settings %p unlock\n", settings);
 	pthread_mutex_unlock(&settings->mutex);
 }
 
-struct rrr_setting *__rrr_settings_find_setting_nolock (struct rrr_instance_settings *source, const char *name) {
+static struct rrr_setting *__rrr_settings_find_setting_nolock (
+		struct rrr_instance_settings *source,
+		const char *name
+) {
 	for (unsigned int i = 0; i < source->settings_count; i++) {
 		struct rrr_setting *test = &source->settings[i];
 
@@ -149,7 +158,11 @@ struct rrr_setting *__rrr_settings_find_setting_nolock (struct rrr_instance_sett
 	return NULL;
 }
 
-struct rrr_setting *__rrr_settings_reserve_nolock (struct rrr_instance_settings *target, const char *name, int return_existing) {
+static struct rrr_setting *__rrr_settings_reserve_nolock (
+		struct rrr_instance_settings *target,
+		const char *name,
+		int return_existing
+) {
 	struct rrr_setting *ret = NULL;
 
 	if ((ret = __rrr_settings_find_setting_nolock(target, name)) != NULL) {
@@ -179,7 +192,10 @@ struct rrr_setting *__rrr_settings_reserve_nolock (struct rrr_instance_settings
 	return ret;
 }
 
-int __rrr_settings_set_setting_name(struct rrr_setting *setting, const char *name) {
+static int __rrr_settings_set_setting_name (
+		struct rrr_setting *setting,
+		const char *name
+) {
 	if (strlen(name) + 1 > RRR_SETTINGS_MAX_NAME_SIZE) {
 		RRR_MSG_0("Settings name %s was longer than maximum %d\n", name, RRR_SETTINGS_MAX_NAME_SIZE);
 		return 1;
@@ -190,7 +206,7 @@ int __rrr_settings_set_setting_name(struct rrr_setting *setting, const char *nam
 	return 0;
 }
 
-int __rrr_settings_add_raw (
+static int __rrr_settings_add_raw (
 		struct rrr_instance_settings *target,
 		const char *name,
 		const void *old_data,
@@ -200,7 +216,7 @@ int __rrr_settings_add_raw (
 ) {
 	int ret = 0;
 
-	void *new_data = malloc(size);
+	void *new_data = rrr_allocate(size);
 
 	if (new_data == NULL) {
 		RRR_MSG_0("Could not allocate memory for setting struct\n");
@@ -241,7 +257,12 @@ int __rrr_settings_add_raw (
 	return ret;
 }
 
-int __rrr_settings_get_string_noconvert (char **target, struct rrr_instance_settings *source, const char *name, int silent_not_found) {
+static int __rrr_settings_get_string_noconvert (
+		char **target,
+		struct rrr_instance_settings *source,
+		const char *name,
+		int silent_not_found
+) {
 	int ret = 0;
 	*target = NULL;
 
@@ -273,7 +294,7 @@ int __rrr_settings_get_string_noconvert (char **target, struct rrr_instance_sett
 		RRR_BUG("BUG: Data string type was not null terminated in rrr_settings_get_string_noconvert\n");
 	}
 
-	char *string = malloc(setting->data_size);
+	char *string = rrr_allocate(setting->data_size);
 	if (string == NULL) {
 		RRR_MSG_0("Could not allocate memory in rrr_settings_get_string_noconvert\n");
 		ret = 1;
@@ -289,7 +310,10 @@ int __rrr_settings_get_string_noconvert (char **target, struct rrr_instance_sett
 	return ret;
 }
 
-int rrr_settings_exists (struct rrr_instance_settings *source, const char *name) {
+int rrr_settings_exists (
+		struct rrr_instance_settings *source,
+		const char *name
+) {
 	int ret = 0;
 
 	__rrr_settings_lock(source);
@@ -304,15 +328,25 @@ int rrr_settings_exists (struct rrr_instance_settings *source, const char *name)
 	return ret;
 }
 
-int rrr_settings_get_string_noconvert (char **target, struct rrr_instance_settings *source, const char *name) {
+int rrr_settings_get_string_noconvert (
+		char **target,
+		struct rrr_instance_settings *source,
+		const char *name
+) {
 	return __rrr_settings_get_string_noconvert(target, source, name, 0);
 }
-int rrr_settings_get_string_noconvert_silent (char **target, struct rrr_instance_settings *source, const char *name) {
+
+int rrr_settings_get_string_noconvert_silent (
+		char **target,
+		struct rrr_instance_settings *source,
+		const char *name
+) {
 	return __rrr_settings_get_string_noconvert(target, source, name, 1);
 }
 
-int __rrr_settings_traverse_split_commas (
-		struct rrr_instance_settings *source, const char *name,
+static int __rrr_settings_traverse_split_commas (
+		struct rrr_instance_settings *source,
+		const char *name,
 		int (*callback)(const char *value, void *arg), void *arg,
 		int silent_fail
 ) {
@@ -350,21 +384,22 @@ int __rrr_settings_traverse_split_commas (
 
 	out:
 	if (value != NULL) {
-		free(value);
+		rrr_free(value);
 	}
 	return ret;
 }
 
-
 int rrr_settings_traverse_split_commas (
-		struct rrr_instance_settings *source, const char *name,
+		struct rrr_instance_settings *source,
+		const char *name,
 		int (*callback)(const char *value, void *arg), void *arg
 ) {
 	return __rrr_settings_traverse_split_commas(source, name, callback, arg, 0);
 }
 
 int rrr_settings_traverse_split_commas_silent_fail (
-		struct rrr_instance_settings *source, const char *name,
+		struct rrr_instance_settings *source,
+		const char *name,
 		int (*callback)(const char *value, void *arg), void *arg
 ) {
 	return __rrr_settings_traverse_split_commas(source, name, callback, arg, 1);
@@ -380,7 +415,7 @@ int rrr_settings_split_commas_to_array (
 
 	*target_ptr = NULL;
 
-	struct rrr_settings_list *target = malloc(sizeof(*target));
+	struct rrr_settings_list *target = rrr_allocate(sizeof(*target));
 	if (target == NULL) {
 		RRR_MSG_0("Could not allocate memory in rrr_settings_split_commas_to_array\n");
 		ret = 1;
@@ -402,7 +437,7 @@ int rrr_settings_split_commas_to_array (
 
 	int length = strlen(value);
 
-	target->data = malloc(length + 1);
+	target->data = rrr_allocate(length + 1);
 	if (target->data == NULL) {
 		RRR_MSG_0("Could not allocate memory in rrr_settings_split_commas_to_array\n");
 		ret = 1;
@@ -416,7 +451,7 @@ int rrr_settings_split_commas_to_array (
 		}
 	}
 
-	target->list = malloc(elements * sizeof(char*));
+	target->list = rrr_allocate(elements * sizeof(char*));
 	if (target->list == NULL) {
 		RRR_MSG_0("Could not allocate memory in rrr_settings_split_commas_to_array\n");
 		ret = 1;
@@ -442,11 +477,11 @@ int rrr_settings_split_commas_to_array (
 
 	out:
 	if (value != NULL) {
-		free(value);
+		rrr_free(value);
 	}
 
 	if (ret != 0 && target != NULL) {
-		rrr_settings_list_destroy(target);
+		__rrr_settings_list_destroy(target);
 	}
 	else {
 		*target_ptr = target;
@@ -455,47 +490,66 @@ int rrr_settings_split_commas_to_array (
 	return ret;
 }
 
-int rrr_settings_replace_string (struct rrr_instance_settings *target, const char *name, const char *value) {
+int rrr_settings_replace_string (
+		struct rrr_instance_settings *target,
+		const char *name,
+		const char *value
+) {
 	const void *data = value;
 	int size = strlen(value) + 1;
 
 	return __rrr_settings_add_raw(target, name, data, size, RRR_SETTINGS_TYPE_STRING, 1);
 }
 
-int rrr_settings_add_string (struct rrr_instance_settings *target, const char *name, const char *value) {
+int rrr_settings_add_string (
+		struct rrr_instance_settings *target,
+		const char *name,
+		const char *value
+) {
 	const void *data = value;
 	int size = strlen(value) + 1;
 
 	return __rrr_settings_add_raw(target, name, data, size, RRR_SETTINGS_TYPE_STRING, 0);
 }
 
-int rrr_settings_replace_unsigned_integer (struct rrr_instance_settings *target, const char *name, rrr_setting_uint value) {
+int rrr_settings_replace_unsigned_integer (
+		struct rrr_instance_settings *target,
+		const char *name,
+		rrr_setting_uint value
+) {
 	const void *data = &value;
 	int size = sizeof(rrr_setting_uint);
 
 	return __rrr_settings_add_raw(target, name, data, size, RRR_SETTINGS_TYPE_UINT, 1);
 }
 
-int rrr_settings_add_unsigned_integer (struct rrr_instance_settings *target, const char *name, rrr_setting_uint value) {
+int rrr_settings_add_unsigned_integer (
+		struct rrr_instance_settings *target,
+		const char *name,
+		rrr_setting_uint value
+) {
 	const void *data = &value;
 	int size = sizeof(rrr_setting_uint);
 
 	return __rrr_settings_add_raw(target, name, data, size, RRR_SETTINGS_TYPE_UINT, 0);
 }
 
-int rrr_settings_setting_to_string_nolock (char **target, struct rrr_setting *setting) {
+int rrr_settings_setting_to_string_nolock (
+		char **target,
+		struct rrr_setting *setting
+) {
 	int ret = 0;
 	*target = NULL;
 
 	char *value;
 	if (setting->type == RRR_SETTINGS_TYPE_STRING) {
-		if ((value = malloc(setting->data_size)) == NULL) {
+		if ((value = rrr_allocate(setting->data_size)) == NULL) {
 			goto out_malloc_err;
 		}
 		sprintf(value, "%s", (char*) setting->data);
 	}
 	else if (setting->type == RRR_SETTINGS_TYPE_UINT) {
-		if ((value = malloc(RRR_SETTINGS_UINT_AS_TEXT_MAX)) == NULL) {
+		if ((value = rrr_allocate(RRR_SETTINGS_UINT_AS_TEXT_MAX)) == NULL) {
 			goto out_malloc_err;
 		}
 		sprintf(value, "%llu", *((unsigned long long *) setting->data));
@@ -513,7 +567,10 @@ int rrr_settings_setting_to_string_nolock (char **target, struct rrr_setting *se
 	return RRR_SETTING_ERROR;
 }
 
-int rrr_settings_setting_to_uint_nolock (rrr_setting_uint *target, struct rrr_setting *setting) {
+int rrr_settings_setting_to_uint_nolock (
+		rrr_setting_uint *target,
+		struct rrr_setting *setting
+) {
 	int ret = 0;
 	char *tmp_string = NULL;
 	*target = 0;
@@ -558,13 +615,17 @@ int rrr_settings_setting_to_uint_nolock (rrr_setting_uint *target, struct rrr_se
 
 	out:
 	if (tmp_string != NULL) {
-		free(tmp_string);
+		rrr_free(tmp_string);
 	}
 
 	return ret;
 }
 
-int rrr_settings_read_string (char **target, struct rrr_instance_settings *settings, const char *name) {
+int rrr_settings_read_string (
+		char **target,
+		struct rrr_instance_settings *settings,
+		const char *name
+) {
 	int ret = 0;
 	*target = NULL;
 
@@ -584,7 +645,11 @@ int rrr_settings_read_string (char **target, struct rrr_instance_settings *setti
 	return ret;
 }
 
-int rrr_settings_read_unsigned_integer (rrr_setting_uint *target, struct rrr_instance_settings *settings, const char *name) {
+int rrr_settings_read_unsigned_integer (
+		rrr_setting_uint *target,
+		struct rrr_instance_settings *settings,
+		const char *name
+) {
 	int ret = 0;
 	*target = 0;
 
@@ -604,7 +669,11 @@ int rrr_settings_read_unsigned_integer (rrr_setting_uint *target, struct rrr_ins
 	return ret;
 }
 
-int rrr_settings_check_yesno (int *result, struct rrr_instance_settings *settings, const char *name) {
+int rrr_settings_check_yesno (
+		int *result,
+		struct rrr_instance_settings *settings,
+		const char *name
+) {
 	*result = -1;
 	int ret = 0;
 
@@ -627,13 +696,15 @@ int rrr_settings_check_yesno (int *result, struct rrr_instance_settings *setting
 
 	out:
 	if (string != NULL) {
-		free(string);
+		rrr_free(string);
 	}
 
 	return ret;
 }
 
-int rrr_settings_check_all_used (struct rrr_instance_settings *settings) {
+int rrr_settings_check_all_used (
+		struct rrr_instance_settings *settings
+) {
 	int ret = 0;
 
 	__rrr_settings_lock(settings);
@@ -650,7 +721,9 @@ int rrr_settings_check_all_used (struct rrr_instance_settings *settings) {
 	return ret;
 }
 
-int rrr_settings_dump (struct rrr_instance_settings *settings) {
+int rrr_settings_dump (
+		struct rrr_instance_settings *settings
+) {
 	int ret = 0;
 
 	__rrr_settings_lock(settings);
@@ -718,7 +791,10 @@ struct rrr_settings_update_used_callback_data {
 	int did_update;
 };
 
-int __rrr_settings_update_used_callback (struct rrr_setting *settings, void *callback_args) {
+static int __rrr_settings_update_used_callback (
+		struct rrr_setting *settings,
+		void *callback_args
+) {
 	struct rrr_settings_update_used_callback_data *data = callback_args;
 
 	if (strcmp (settings->name, data->name) == 0) {
@@ -753,7 +829,7 @@ void rrr_settings_update_used (
 	}
 }
 
-int __rrr_setting_pack(struct rrr_setting_packed **target, struct rrr_setting *source) {
+static int __rrr_setting_pack(struct rrr_setting_packed **target, struct rrr_setting *source) {
 	int ret = 0;
 	struct rrr_setting_packed *result = NULL;
 
@@ -765,7 +841,7 @@ int __rrr_setting_pack(struct rrr_setting_packed **target, struct rrr_setting *s
 		goto out;
 	}
 
-	result = malloc(sizeof(*result));
+	result = rrr_allocate(sizeof(*result));
 	if (result == NULL) {
 		RRR_MSG_0("Could not allocate memory in  __rrr_setting_pack\n");
 		ret = 1;
@@ -817,7 +893,7 @@ int rrr_settings_iterate_packed (
 
 		ret = callback(setting_packed, callback_arg);
 
-		free(setting_packed);
+		rrr_free(setting_packed);
 
 		if (ret != 0) {
 			break;
@@ -830,20 +906,26 @@ int rrr_settings_iterate_packed (
 	return ret;
 }
 
-void rrr_settings_packed_to_host (struct rrr_setting_packed *setting_packed) {
+void rrr_settings_packed_to_host (
+		struct rrr_setting_packed *setting_packed
+) {
 	setting_packed->type = rrr_be32toh(setting_packed->type);
 	setting_packed->was_used = rrr_be32toh(setting_packed->was_used);
 	setting_packed->data_size = rrr_be32toh(setting_packed->data_size);
 }
 
-void rrr_settings_packed_prepare_for_network (struct rrr_setting_packed *message) {
+void rrr_settings_packed_prepare_for_network (
+		struct rrr_setting_packed *message
+) {
 	message->type = rrr_htobe32(message->type);
 	message->was_used = rrr_htobe32(message->was_used);
 	message->data_size = rrr_htobe32(message->data_size);
 }
 
 
-int rrr_settings_packed_validate (const struct rrr_setting_packed *setting) {
+int rrr_settings_packed_validate (
+		const struct rrr_setting_packed *setting
+) {
 	int ret = 0;
 
 	const char *end = setting->name + sizeof(setting->name) - 1;
