@@ -2,7 +2,7 @@
 
 Read Route Record
 
-Copyright (C) 2019-2020 Atle Solbakken atle@goliathdns.no
+Copyright (C) 2019-2021 Atle Solbakken atle@goliathdns.no
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -32,13 +32,20 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <unistd.h>
 #include <fcntl.h>
 #include <sys/types.h>
-#include <sys/poll.h>
+#include <poll.h>
 #include <sys/un.h>
 #include <sys/stat.h>
 
-#include "rrr_socket.h"
+#ifdef RRR_HAVE_EVENTFD
+#	include <sys/eventfd.h>
+#endif
 
 #include "../log.h"
+#include "../allocator.h"
+
+#include "rrr_socket.h"
+#include "rrr_socket_send_chunk.h"
+
 #include "../rrr_strerror.h"
 #include "../log.h"
 #include "../rrr_umask.h"
@@ -77,7 +84,9 @@ struct rrr_socket_private_data_collection {
 struct rrr_socket_holder {
 	RRR_LL_NODE(struct rrr_socket_holder);
 	char *creator;
-	char *filename;
+	char *filename_unlink;
+	char *filename_no_unlink;
+	struct rrr_socket_send_chunk_collection send_chunks;
 	struct rrr_socket_options options;
 	struct rrr_socket_private_data_collection private_data;
 };
@@ -93,7 +102,7 @@ void __rrr_socket_private_data_destroy (
 		struct rrr_socket_private_data *node
 ) {
 	RRR_FREE_IF_NOT_NULL(node->data);
-	free(node);
+	rrr_free(node);
 }
 
 void __rrr_socket_private_data_collection_clear (
@@ -109,7 +118,7 @@ int __rrr_socket_private_data_collection_allocate_and_push (
 ) {
 	int ret = 0;
 
-	struct rrr_socket_private_data *new_node = malloc(sizeof(*new_node));
+	struct rrr_socket_private_data *new_node = rrr_allocate(sizeof(*new_node));
 	if (new_node == NULL) {
 		RRR_MSG_0("Could not allocate memory in __rrr_socket_private_data_collection_allocate_and_push\n");
 		ret = 1;
@@ -118,7 +127,7 @@ int __rrr_socket_private_data_collection_allocate_and_push (
 
 	memset(new_node, '\0', sizeof(*new_node));
 
-	void *new_data = malloc(size);
+	void *new_data = rrr_allocate(size);
 	if (new_data == NULL) {
 		RRR_MSG_0("Could not allocate memory in __rrr_socket_private_data_collection_allocate_and_push\n");
 		ret = 1;
@@ -133,7 +142,7 @@ int __rrr_socket_private_data_collection_allocate_and_push (
 
 	goto out;
 	out_free_node:
-		free(new_node);
+		rrr_free(new_node);
 	out:
 		return ret;
 }
@@ -150,14 +159,16 @@ int __rrr_socket_holder_close_and_destroy(struct rrr_socket_holder *holder, int
 			}
 		}
 	}
-	if (no_unlink == 0 && holder->filename != NULL) {
-		RRR_DBG_7("socket pid %i filename %s unlink\n", getpid(), holder->filename);
-		unlink(holder->filename);
+	if (no_unlink == 0 && holder->filename_unlink != NULL) {
+		RRR_DBG_7("socket pid %i filename %s unlink\n", getpid(), holder->filename_unlink);
+		unlink(holder->filename_unlink);
 	}
 	__rrr_socket_private_data_collection_clear(&holder->private_data);
-	RRR_FREE_IF_NOT_NULL(holder->filename);
+	RRR_FREE_IF_NOT_NULL(holder->filename_unlink);
+	RRR_FREE_IF_NOT_NULL(holder->filename_no_unlink);
 	RRR_FREE_IF_NOT_NULL(holder->creator);
-	free(holder);
+	rrr_socket_send_chunk_collection_clear(&holder->send_chunks);
+	rrr_free(holder);
 
 	// Must always return 0 or linked list won' remove the node
 	return 0;
@@ -167,6 +178,7 @@ int __rrr_socket_holder_new (
 		struct rrr_socket_holder **holder,
 		const char *creator,
 		const char *filename,
+		int filename_unlink,
 		int fd,
 		int domain,
 		int type,
@@ -176,7 +188,7 @@ int __rrr_socket_holder_new (
 
 	*holder = NULL;
 
-	struct rrr_socket_holder *result = malloc(sizeof(*result));
+	struct rrr_socket_holder *result = rrr_allocate(sizeof(*result));
 	if (result == NULL) {
 		RRR_MSG_0("Could not allocate memory in __rrr_socket_holder_new\n");
 		ret = 1;
@@ -189,15 +201,21 @@ int __rrr_socket_holder_new (
 		RRR_BUG("Creator was NULL in __rrr_socket_holder_new\n");
 	}
 
-	result->creator = strdup(creator);
+	result->creator = rrr_strdup(creator);
 
 	if (filename != NULL) {
-		result->filename = strdup(filename);
-		if (result->filename == NULL) {
+		char *filename_tmp = rrr_strdup(filename);
+		if (filename_tmp == NULL) {
 			RRR_MSG_0("Could not allocate memory for filename in __rrr_socket_holder_new\n");
 			ret = 1;
 			goto out;
 		}
+		if (filename_unlink) {
+			result->filename_unlink = filename_tmp;
+		}
+		else {
+			result->filename_no_unlink = filename_tmp;
+		}
 	}
 
 	result->options.fd = fd;
@@ -213,6 +231,25 @@ int __rrr_socket_holder_new (
 	return ret;
 }
 
+int rrr_socket_with_filename_do (
+		int fd,
+		int (*callback)(const char *filename, void *arg),
+		void *callback_arg
+) {
+	RRR_LL_ITERATE_BEGIN(&socket_list, struct rrr_socket_holder);
+		if (node->options.fd == fd) {
+			if (node->filename_unlink) {
+				return callback(node->filename_unlink, callback_arg);
+			}
+			else {
+				return callback(node->filename_no_unlink, callback_arg);
+			}
+		}
+	RRR_LL_ITERATE_END();
+
+	return 1;
+}
+		
 int rrr_socket_get_filename_from_fd (
 		char **result,
 		int fd
@@ -225,14 +262,15 @@ int rrr_socket_get_filename_from_fd (
 
 	RRR_LL_ITERATE_BEGIN(&socket_list, struct rrr_socket_holder);
 		if (node->options.fd == fd) {
-			if (node->filename != NULL && *(node->filename) != '\0') {
-				char *filename = strdup(node->filename);
-				if (filename == NULL) {
+			const char *filename = (node->filename_unlink ? node->filename_unlink : node->filename_no_unlink);
+			if (filename != NULL && *(filename) != '\0') {
+				char *filename_new = rrr_strdup(filename);
+				if (filename_new == NULL) {
 					RRR_MSG_0("Could not allocate memory in rrr_socket_get_filename_from_fd\n");
 					ret = 1;
 					goto out;
 				}
-				*result = filename;
+				*result = filename_new;
 			}
 			ret = 0;
 			goto out;
@@ -243,6 +281,26 @@ int rrr_socket_get_filename_from_fd (
 	pthread_mutex_unlock(&socket_lock);
 	return ret;
 }
+		
+int rrr_socket_get_fd_from_filename (
+		const char *filename
+) {
+	pthread_mutex_lock(&socket_lock);
+
+	int ret = -1;
+
+	RRR_LL_ITERATE_BEGIN(&socket_list, struct rrr_socket_holder);
+		const char *filename_node = (node->filename_unlink ? node->filename_unlink : node->filename_no_unlink);
+		if (filename_node != NULL && strcmp(filename, filename_node) == 0) {
+			ret = node->options.fd;
+			goto out;
+		}
+	RRR_LL_ITERATE_END();
+
+	out:
+	pthread_mutex_unlock(&socket_lock);
+	return ret;
+}
 
 int rrr_socket_get_options_from_fd (
 		struct rrr_socket_options *target,
@@ -313,7 +371,8 @@ int rrr_socket_with_lock_do (
 
 static void __rrr_socket_dump_unlocked (void) {
 	RRR_LL_ITERATE_BEGIN(&socket_list,struct rrr_socket_holder);
-		RRR_DBG_7 ("fd %i pid %i creator %s filename %s\n", node->options.fd, getpid(), node->creator, node->filename);
+		const char *filename = (node->filename_unlink ? node->filename_unlink : node->filename_no_unlink);
+		RRR_DBG_7 ("fd %i pid %i creator %s filename %s%s\n", node->options.fd, getpid(), node->creator, filename, node->filename_unlink ? " (listen)" : "");
 	RRR_LL_ITERATE_END();
 	RRR_DBG_7("---\n");
 }
@@ -324,12 +383,13 @@ static int __rrr_socket_add_unlocked (
 		int type,
 		int protocol,
 		const char *creator,
-		const char *filename
+		const char *filename,
+		int filename_unlink
 ) {
 	int ret = 0;
 	struct rrr_socket_holder *holder = NULL;
 
-	if (__rrr_socket_holder_new(&holder, creator, filename, fd, domain, type, protocol) != 0) {
+	if (__rrr_socket_holder_new(&holder, creator, filename, filename_unlink, fd, domain, type, protocol) != 0) {
 		RRR_MSG_0("Could not create socket holder in __rrr_socket_add_unlocked\n");
 		ret = 1;
 		goto out;
@@ -369,7 +429,7 @@ int rrr_socket_accept (
 	socklen_t addr_len_orig = *addr_len;
 	fd_out = accept(fd_in, (struct sockaddr *) addr, addr_len);
 	if (fd_out != -1) {
-		__rrr_socket_add_unlocked(fd_out, options.domain, options.type, options.protocol, creator, NULL);
+		__rrr_socket_add_unlocked(fd_out, options.domain, options.type, options.protocol, creator, NULL, 0);
 	}
 	if (*addr_len > addr_len_orig) {
 		RRR_BUG("BUG: Given addr_len was to short in rrr_socket_accept\n");
@@ -379,11 +439,11 @@ int rrr_socket_accept (
 	if (fd_out != -1 && (options.type & SOCK_NONBLOCK) == SOCK_NONBLOCK) {
 		int flags = fcntl(fd_out, F_GETFL, 0);
 		if (flags == -1) {
-			RRR_MSG_0("Error while getting flags with fcntl for socket in rrr_socket_accept: %s\n", rrr_strerror(errno));
+			RRR_MSG_0("fd %i<-%i error while getting flags with fcntl for socket in rrr_socket_accept: %s\n", fd_in, fd_out, rrr_strerror(errno));
 			goto out_close;
 		}
 		if (fcntl(fd_out, F_SETFL, flags | O_NONBLOCK) == -1) {
-			RRR_MSG_0("Error while setting O_NONBLOCK on socket in rrr_socket_accept: %s\n", rrr_strerror(errno));
+			RRR_MSG_0("fd %i<-%i error while setting O_NONBLOCK on socket in rrr_socket_accept: %s\n", fd_in, fd_out, rrr_strerror(errno));
 			goto out_close;
 		}
 	}
@@ -405,7 +465,7 @@ int rrr_socket_mkstemp (
 	pthread_mutex_lock(&socket_lock);
 	fd = mkstemp(filename);
 	if (fd != -1) {
-		__rrr_socket_add_unlocked(fd, 0, 0, 0, creator, filename);
+		__rrr_socket_add_unlocked(fd, 0, 0, 0, creator, filename, 1);
 	}
 	pthread_mutex_unlock(&socket_lock);
 
@@ -427,7 +487,7 @@ int rrr_socket_bind_and_listen (
 		}
 	}
 	if (bind(fd, addr, addr_len) != 0) {
-		RRR_MSG_0("Could not bind to socket: %s\n",rrr_strerror(errno));
+		RRR_DBG_1("Note: Could not bind to socket: %s\n",rrr_strerror(errno));
 		return 1;
 	}
 	if (listen(fd, num_clients) != 0) {
@@ -448,7 +508,7 @@ static int __rrr_socket_open_nolock (
 	fd = open(filename, flags, mode);
 
 	if (fd != -1) {
-		__rrr_socket_add_unlocked(fd, 0, 0, 0, creator, (register_for_unlink ? filename : NULL));
+		__rrr_socket_add_unlocked(fd, 0, 0, 0, creator, (register_for_unlink ? filename : NULL), register_for_unlink);
 	}
 
 	RRR_DBG_7("rrr_socket_open fd %i pid %i filename %s creator %s flags %i\n", fd, getpid(), filename, creator, flags);
@@ -484,7 +544,7 @@ int rrr_socket_open_and_read_file (
 	*result_bytes = 0;
 
 	char *contents_tmp = NULL;
-	int fd = rrr_socket_open(filename, options, mode, "rrr_socket_open_and_read_full_file", 0);
+	int fd = rrr_socket_open(filename, options, mode, "rrr_socket_open_and_read_file", 0);
 
 	if (fd <= 0) {
 		RRR_MSG_0("Could not open file '%s' for reading: %s\n",
@@ -493,11 +553,11 @@ int rrr_socket_open_and_read_file (
 		goto out;
 	}
 
-	ssize_t bytes = ret = lseek(fd, 0, SEEK_END);
-	if (ret == 0) {
+	ssize_t bytes = lseek(fd, 0, SEEK_END);
+	if (bytes == 0) {
 		goto out;
 	}
-	else if (ret < 0) {
+	else if (bytes < 0) {
 		RRR_MSG_0("Could not seek to end of file '%s': %s\n",
 				filename, rrr_strerror(errno));
 		ret = 1;
@@ -511,21 +571,20 @@ int rrr_socket_open_and_read_file (
 		goto out;
 	}
 
-	if ((contents_tmp = malloc(bytes + 1)) == NULL) {
-		RRR_MSG_0("Could not allocate memory in rrr_socket_open_and_read_full_file\n");
+	if ((contents_tmp = rrr_allocate(bytes + 1)) == NULL) {
+		RRR_MSG_0("Could not allocate memory in rrr_socket_open_and_read_file\n");
 		ret = 1;
 		goto out;
 	}
 
-	if ((ret = read(fd, contents_tmp, bytes)) != bytes) {
-		RRR_MSG_0("Could not read all bytes from file '%s', return was %i: %s\n",
-				filename, ret, rrr_strerror(errno));
+	ssize_t bytes_read;
+	if ((bytes_read = read(fd, contents_tmp, bytes)) != bytes) {
+		RRR_MSG_0("Could not read all bytes from file '%s', return was %lli: %s\n",
+				filename, (long long int) bytes_read, rrr_strerror(errno));
 		ret = 1;
 		goto out;
 	}
 
-	ret = 0;
-
 	// Make sure we allocate bytes + 1 above
 	contents_tmp[bytes] = '\0';
 
@@ -542,19 +601,91 @@ int rrr_socket_open_and_read_file (
 
 }
 
+#ifdef RRR_HAVE_EVENTFD
+int rrr_socket_eventfd (
+		const char *creator
+) {
+	int fd = 0;
+
+	if ((fd = eventfd(0, EFD_NONBLOCK)) < 0) {
+		RRR_MSG_0("Failed to create eventfd in rrr_socket_eventfd: %s\n", rrr_strerror(errno));
+	}
+
+	if (fd != -1) {
+		pthread_mutex_lock(&socket_lock);
+		__rrr_socket_add_unlocked(fd, 0, 0, 0, creator, NULL, 0);
+		pthread_mutex_unlock(&socket_lock);
+	}
+
+	RRR_DBG_7("rrr_socket_eventfd fd %i pid %i\n", fd, getpid());
+
+	return fd;
+}
+#endif /* RRR_HAVE_EVENTFD */
+
+int rrr_socket_pipe (
+		int result[2],
+		const char *creator
+) {
+	int ret = 0;
+
+	int fds[2];
+
+	if (pipe(fds) != 0) {
+		RRR_MSG_0("Failed to create pipe in rrr_socket_pipe: %s\n", rrr_strerror(errno));
+		ret = RRR_SOCKET_HARD_ERROR;
+		goto out;
+	}
+
+	for (int i = 0; i < 2; i++) {
+		if (fcntl (fds[i], F_SETFL, fcntl(fds[i], F_GETFL) | O_NONBLOCK) != 0) {
+			RRR_MSG_0("fcntl() failed in rrr_socket_pipe: %s\n", rrr_strerror(errno));
+			ret = RRR_SOCKET_HARD_ERROR;
+			goto out_close;
+		}
+	}
+
+	pthread_mutex_lock(&socket_lock);
+	ret |= __rrr_socket_add_unlocked(fds[0], 0, 0, 0, creator, NULL, 0);
+	ret |= __rrr_socket_add_unlocked(fds[1], 0, 0, 0, creator, NULL, 0);
+	pthread_mutex_unlock(&socket_lock);
+
+	RRR_DBG_7("rrr_socket_pipe fd %i<-%i pid %i\n", fds[0], fds[1], getpid());
+
+	if (ret != 0) {
+		RRR_MSG_0("Failed to add sockets in rrr_socket_pipe\n");
+		goto out_destroy;
+	}
+
+	memcpy(result, fds, sizeof(fds));
+
+	goto out;
+	out_destroy:
+		rrr_socket_close(fds[0]);
+		rrr_socket_close(fds[1]);
+		goto out;
+	out_close:
+		close(fds[0]);
+		close(fds[1]);
+		goto out;
+	out:
+		return ret;
+}
+
 int rrr_socket (
 		int domain,
 		int type,
 		int protocol,
 		const char *creator,
-		const char *filename
+		const char *filename,
+		int register_for_unlink
 ) {
 	int fd = 0;
 	pthread_mutex_lock(&socket_lock);
 	fd = socket(domain, type, protocol);
 
 	if (fd != -1) {
-		__rrr_socket_add_unlocked(fd, domain, type, protocol, creator, filename);
+		__rrr_socket_add_unlocked(fd, domain, type, protocol, creator, filename, register_for_unlink);
 	}
 
 	RRR_DBG_7("rrr_socket fd %i pid %i filename %s\n", fd, getpid(), filename);
@@ -574,8 +705,6 @@ static int __rrr_socket_close (int fd, int ignore_unregistered, int no_unlink) {
 
 	int did_destroy = 0;
 
-	__rrr_socket_dump_unlocked();
-
 	RRR_LL_ITERATE_BEGIN(&socket_list,struct rrr_socket_holder);
 		if (node->options.fd == fd) {
 			RRR_LL_ITERATE_SET_DESTROY();
@@ -584,11 +713,11 @@ static int __rrr_socket_close (int fd, int ignore_unregistered, int no_unlink) {
 		}
 	RRR_LL_ITERATE_END_CHECK_DESTROY(&socket_list,__rrr_socket_holder_close_and_destroy(node, no_unlink));
 
-	__rrr_socket_dump_unlocked();
-
 	pthread_mutex_unlock(&socket_lock);
 
 	if (did_destroy != 1 && ignore_unregistered == 0) {
+		// NOTE ! If this warning appears, program must be fixed. In a possible race
+		//        condition, we might try to close an FD opened by somebody else.
 		RRR_MSG_0("Warning: Socket close of fd %i called but it was not registered. Attempting to close anyway.\n", fd);
 		int ret = close(fd);
 		if (ret != 0) {
@@ -614,37 +743,33 @@ int rrr_socket_close_ignore_unregistered (int fd) {
 	return __rrr_socket_close (fd, 1, 0);
 }
 
-static int __rrr_socket_close_all_except (int fd, int no_unlink) {
+int __rrr_socket_close_all_except_array (int *fds, size_t fd_count, int no_unlink) {
 	int ret = 0;
 
-	if (fd < 0) {
-		RRR_BUG("rrr_socket_close_all_except called with fd < 0: %i\n", fd);
-	}
-
-	RRR_DBG_7("rrr_socket_close_all_except fd %i pid %i no_unlink %i\n", fd, getpid(), no_unlink);
+	RRR_DBG_7("rrr_socket_close_all_except_array pid %i no_unlink %i\n", getpid(), no_unlink);
 
 	int count = 0;
-	int found = 0;
 
 	pthread_mutex_lock(&socket_lock);
 
 	RRR_LL_ITERATE_BEGIN(&socket_list,struct rrr_socket_holder);
-		if (node->options.fd != fd) {
+		int match = 0;
+		for (size_t i = 0; i < fd_count; i++) {
+			if (node->options.fd == fds[i]) {
+				match = 1;
+				break;
+			}
+		}
+		if (match) {
+			RRR_DBG_7("- Not closing %i, was in except list\n", node->options.fd);
+		}
+		else {
+			RRR_DBG_7("- Closing %i\n", node->options.fd);
 			RRR_LL_ITERATE_SET_DESTROY();
 			count++;
 		}
-		else {
-			if (found != 0) {
-				RRR_BUG("At least two equal FD %i in socket list in rrr_socket_close_all_except\n", fd);
-			}
-			found = 1;
-		}
 	RRR_LL_ITERATE_END_CHECK_DESTROY(&socket_list,__rrr_socket_holder_close_and_destroy(node, no_unlink));
 
-	if (found != 1 && fd != 0) {
-		RRR_MSG_0("Warning: rrr_socket_close_all_except called with unregistered FD %i. All sockets are now closed.\n", fd);
-	}
-
 	if (RRR_DEBUGLEVEL_7) {
 		__rrr_socket_dump_unlocked();
 	}
@@ -656,6 +781,10 @@ static int __rrr_socket_close_all_except (int fd, int no_unlink) {
 	return ret;
 }
 
+static int __rrr_socket_close_all_except (int fd, int no_unlink) {
+	return __rrr_socket_close_all_except_array(&fd, 1, no_unlink);
+}
+
 int rrr_socket_close_all_except (int fd) {
 	return __rrr_socket_close_all_except(fd, 0);
 }
@@ -672,6 +801,14 @@ int rrr_socket_close_all_no_unlink (void) {
 	return __rrr_socket_close_all_except(0, 1);
 }
 
+int rrr_socket_close_all_except_array (int *fds, size_t fd_count) {
+	return __rrr_socket_close_all_except_array (fds, fd_count, 0);
+}
+
+int rrr_socket_close_all_except_array_no_unlink (int *fds, size_t fd_count) {
+	return __rrr_socket_close_all_except_array (fds, fd_count, 1);
+}
+
 int rrr_socket_fifo_create (
 		int *fd_result,
 		const char *filename,
@@ -791,7 +928,7 @@ int rrr_socket_unix_create_bind_and_listen (
 	if (do_mkstemp != 0) {
 		fd = rrr_socket_mkstemp(filename_tmp, creator);
 		if (fd < 0) {
-			RRR_MSG_0("mkstemp failed in rrr_socket_unix_create_bind_and_listen: %s\n", rrr_strerror(errno));
+			RRR_MSG_0("mkstemp ('%s') failed in while creating unix socket: %s\n", filename_tmp, rrr_strerror(errno));
 			ret = 1;
 			goto out;
 		}
@@ -817,7 +954,7 @@ int rrr_socket_unix_create_bind_and_listen (
 	strcpy(addr.sun_path, filename_tmp);
 	addr.sun_family = AF_UNIX;
 
-	fd = rrr_socket(AF_UNIX, SOCK_STREAM | (nonblock != 0 ? SOCK_NONBLOCK : 0), 0, creator, filename_tmp);
+	fd = rrr_socket(AF_UNIX, SOCK_STREAM | (nonblock != 0 ? SOCK_NONBLOCK : 0), 0, creator, filename_tmp, 1);
 
 	if (fd < 0) {
 		RRR_MSG_0("Could not create socket in rrr_socket_unix_create_bind_and_listen\n");
@@ -840,7 +977,7 @@ int rrr_socket_unix_create_bind_and_listen (
 		goto out;
 	}
 
-	RRR_DBG_7("rrr_socket_unix_create_bind_and_listen complete fd %i file %s pid %i clients %i umask %i\n",
+	RRR_DBG_7("fd %i unix socket created file %s pid %i clients %i umask %i\n",
 			fd, addr.sun_path, getpid(), num_clients, RRR_SOCKET_UNIX_DEFAULT_UMASK);
 
 	*fd_result = fd;
@@ -853,6 +990,47 @@ int rrr_socket_unix_create_bind_and_listen (
 	return ret;
 }
 
+int rrr_socket_send_check (
+		int fd
+) {
+	int ret = RRR_SOCKET_OK;
+
+	struct pollfd pollfd = {
+		fd, POLLOUT, 0
+	};
+
+	if ((poll(&pollfd, 1, 0) == -1) || ((pollfd.revents & (POLLERR|POLLHUP)) != 0)) {
+		if ((pollfd.revents & (POLLHUP)) != 0) {
+			RRR_DBG_7("fd %i connection refused or closed in send check (POLLHUP)\n", fd);
+			ret = RRR_SOCKET_SOFT_ERROR;
+			goto out;
+		}
+		else if (errno == EINPROGRESS || errno == EAGAIN || errno == EWOULDBLOCK) {
+			ret = RRR_SOCKET_NOT_READY;
+			goto out;
+		}
+		else if (errno == ECONNREFUSED) {
+			RRR_DBG_7("fd %i connection refused in send check (ECONNREFUSED)\n", fd);
+			ret = RRR_SOCKET_SOFT_ERROR;
+			goto out;
+		}
+
+		RRR_MSG_0("fd %i error from poll() in send check: %s\n", fd, rrr_strerror(errno));
+		ret = RRR_SOCKET_SOFT_ERROR;
+		goto out;
+	}
+	else if ((pollfd.revents & POLLOUT) != 0) {
+		goto out;
+	}
+	else {
+		ret = RRR_SOCKET_SOFT_ERROR;
+		goto out;
+	}
+
+	out:
+	return ret;
+}
+
 static int __rrr_socket_send_check (
 		int fd
 ) {
@@ -880,7 +1058,7 @@ static int __rrr_socket_send_check (
 			goto out;
 		}
 
-		RRR_MSG_0("Error from poll() while connecting: %s\n", rrr_strerror(errno));
+		RRR_MSG_0("fd %i error from poll() while connecting: %s\n", fd, rrr_strerror(errno));
 		ret = RRR_SOCKET_HARD_ERROR;
 		goto out;
 	}
@@ -924,7 +1102,7 @@ int rrr_socket_connect_nonblock_postcheck_loop (
 
 int rrr_socket_connect_nonblock (
 		int fd,
-		struct sockaddr *addr,
+		const struct sockaddr *addr,
 		socklen_t addr_len
 ) {
 	int ret = 0;
@@ -933,17 +1111,18 @@ int rrr_socket_connect_nonblock (
 		goto out;
 	}
 	else if (errno == EINPROGRESS || errno == EAGAIN) {
+		RRR_DBG_7 ("fd %i connection in progress\n", fd);
 		ret = 0;
 		goto out;
 	}
 	else if (errno == ECONNREFUSED) {
-		RRR_MSG_0 ("Connection refused in rrr_socket_connect_nonblock\n");
+		RRR_DBG_7 ("fd %i connection refused\n", fd);
 		ret = RRR_SOCKET_SOFT_ERROR;
 		goto out;
 	}
 	else {
-		RRR_MSG_0("Error while connecting, address family was %u: %s\n",
-				addr->sa_family, rrr_strerror(errno));
+		RRR_MSG_0 ("fd %i error while connecting, address family was %u: %s\n",
+				fd, addr->sa_family, rrr_strerror(errno));
 		ret = 1;
 		goto out;
 	}
@@ -975,7 +1154,7 @@ int rrr_socket_unix_connect (
 	addr.sun_family = AF_UNIX;
 	strcpy(addr.sun_path, filename);
 
-	socket_fd = rrr_socket(AF_UNIX, SOCK_STREAM|(nonblock ? SOCK_NONBLOCK : 0), 0, creator, NULL);
+	socket_fd = rrr_socket(AF_UNIX, SOCK_STREAM|(nonblock ? SOCK_NONBLOCK : 0), 0, creator, filename, 0);
 	if (socket_fd < 0) {
 		RRR_MSG_0("Error while creating socket in rrr_socket_unix_create_and_connect: %s\n", rrr_strerror(errno));
 		ret = RRR_SOCKET_HARD_ERROR;
@@ -985,8 +1164,8 @@ int rrr_socket_unix_connect (
 	int connected = 0;
 	for (int i = 0; i < 10 && connected == 0; i++) {
 		if (rrr_socket_connect_nonblock(socket_fd, (struct sockaddr *) &addr, addr_len) != 0) {
-			RRR_MSG_0("Could not connect to socket %s try %i of %i: %s\n",
-					filename, i, 10, rrr_strerror(errno));
+			RRR_MSG_0("fd %i could not connect to socket %s try %i of %i: %s\n",
+					socket_fd, filename, i + 1, 10, rrr_strerror(errno));
 			rrr_posix_usleep(25000);
 		}
 		else {
@@ -1043,15 +1222,15 @@ int rrr_socket_sendto_nonblock (
 
 	retry:
 	if (--max_retries == 0) {
-		RRR_DBG_7("Max retries reached in rrr_socket_sendto_nonblock for socket %i\n", fd);
+		RRR_DBG_7("fd %i max retries reached in rrr_socket_sendto_nonblock\n", fd);
 		ret = RRR_SOCKET_SOFT_ERROR;
 		goto out;
 	}
 
-	RRR_DBG_7("Non-blocking send on fd %i starting, writing %li bytes (where of %li is complete) address length %u\n",
+	RRR_DBG_7("fd %i nonblock send loop starting, writing %li bytes (where of %li is complete) address length %u\n",
 			fd, size, done_bytes_total, addr_len);
 
-	if (addr == NULL) {
+	if (addr == NULL || addr_len == 0) {
 		done_bytes = send(fd, data + done_bytes_total, size - done_bytes_total, flags);
 	}
 	else {
@@ -1084,7 +1263,7 @@ int rrr_socket_sendto_nonblock (
 				goto retry;
 			}
 			else {
-				RRR_DBG_7("Error from send(to) function in rrr_socket_sendto_nonblock fd %i flags %i addr ptr %p addr len %i: %s\n",
+				RRR_DBG_7("fd %i error from sendto flags %i addr ptr %p addr len %i: %s\n",
 						fd,
 						flags,
 						addr,
@@ -1111,6 +1290,35 @@ int rrr_socket_sendto_nonblock (
 	return ret;
 }
 
+int rrr_socket_sendto_nonblock_check_retry (
+		ssize_t *written_bytes,
+		int fd,
+		const void *data,
+		ssize_t size,
+		const struct sockaddr *addr,
+		socklen_t addr_len
+) {
+	int err = 0;
+	int ret = rrr_socket_sendto_nonblock(&err, written_bytes, fd, data, size, addr, addr_len);
+
+	if (ret == RRR_SOCKET_SOFT_ERROR) {
+		if (err == EWOULDBLOCK || err == EAGAIN || err == EINPROGRESS) {
+			ret = RRR_SOCKET_WRITE_INCOMPLETE;
+		}
+	}
+
+	return ret;
+}
+
+int rrr_socket_send_nonblock_check_retry (
+		ssize_t *written_bytes,
+		int fd,
+		const void *data,
+		ssize_t size
+) {
+	return rrr_socket_sendto_nonblock_check_retry(written_bytes, fd, data, size, NULL, 0);
+}
+
 int rrr_socket_sendto_blocking (
 		int fd,
 		const void *data,
@@ -1124,11 +1332,10 @@ int rrr_socket_sendto_blocking (
 	ssize_t written_bytes_total = 0;
 
 	while (written_bytes_total < size) {
-		RRR_DBG_7("Blocking send on fd %i starting, writing %li bytes (where of %li is complete)\n",
+		RRR_DBG_7("fd %i blocking send loop writing %li bytes (where of %li is complete)\n",
 				fd, size, written_bytes_total);
-		int err;
-		if ((ret = rrr_socket_sendto_nonblock (
-				&err,
+
+		if ((ret = rrr_socket_sendto_nonblock_check_retry (
 				&written_bytes,
 				fd,
 				data + written_bytes_total,
@@ -1136,13 +1343,13 @@ int rrr_socket_sendto_blocking (
 				addr,
 				addr_len
 		)) != 0) {
-			if (ret != RRR_SOCKET_SOFT_ERROR) {
-				RRR_MSG_0("Error from sendto in rrr_socket_sendto_blocking\n");
+			if (ret != RRR_SOCKET_WRITE_INCOMPLETE) {
+				RRR_DBG_7("Error from sendto on fd %i in rrr_socket_sendto_blocking\n", fd);
 				goto out;
 			}
 		}
 		written_bytes_total += written_bytes;
-		RRR_DBG_7("Blocking send on fd %i, written bytes total is %li (this round was %li)\n",
+		RRR_DBG_7("fd %i blocking send loop written bytes total is %li (this round was %li)\n",
 				fd, written_bytes_total, written_bytes);
 	}
 
@@ -1185,21 +1392,38 @@ int rrr_socket_sendto_nonblock_fail_on_partial_write (
 	return ret;
 }
 
-int rrr_socket_check_alive (int fd) {
-	int ret = RRR_SOCKET_OK;
+int rrr_socket_send_blocking (
+		int fd,
+		void *data,
+		ssize_t size
+) {
+	return rrr_socket_sendto_blocking(fd, data, size, NULL, 0);
+}
 
+int rrr_socket_check_alive (int fd) {
 	struct pollfd pollfd = {0};
 
 	pollfd.fd = fd;
+	pollfd.events = POLLIN;
 
-	if ((ret = poll(&pollfd, 1, 10)) > 0) {
-		if (pollfd.revents & (POLLHUP|POLLERR|POLLNVAL)) {
-			ret = RRR_SOCKET_SOFT_ERROR;
+	int ret_tmp = poll(&pollfd, 1, 10);
+
+	if (ret_tmp < 0 || pollfd.revents & (POLLHUP|POLLERR|POLLNVAL)) {
+		RRR_DBG_7("fd %i recv poll error in check alive: %s revents: %i\n", fd, rrr_strerror(errno), pollfd.revents);
+		return RRR_SOCKET_SOFT_ERROR;
+	}
+	else if (ret_tmp > 0) {
+		char buf[1];
+		ret_tmp = recv(fd, buf, sizeof(buf), MSG_PEEK|MSG_DONTWAIT);
+		if (ret_tmp < 0) {
+			RRR_DBG_7("fd %i recv peek error in check alive: %s\n", fd, rrr_strerror(errno));
+			return RRR_SOCKET_SOFT_ERROR;
+		}
+		else if (ret_tmp == 0) {
+			RRR_DBG_7("fd %i recv EOF in check alive, connection closed\n", fd);
+			return RRR_READ_EOF;
 		}
 	}
-	else if (ret < 0) {
-		ret = RRR_SOCKET_SOFT_ERROR;
-	}
 
-	return ret;
+	return RRR_SOCKET_OK;
 }
