@@ -2,7 +2,7 @@
 
 Read Route Record
 
-Copyright (C) 2019-2020 Atle Solbakken atle@goliathdns.no
+Copyright (C) 2019-2021 Atle Solbakken atle@goliathdns.no
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -48,11 +48,18 @@ struct rrr_socket_options {
 	int protocol;
 };
 
-struct rrr_read_session;
+int rrr_socket_with_filename_do (
+		int fd,
+		int (*callback)(const char *filename, void *arg),
+		void *callback_arg
+);
 int rrr_socket_get_filename_from_fd (
 		char **result,
 		int fd
 );
+int rrr_socket_get_fd_from_filename (
+		const char *filename
+);
 int rrr_socket_get_options_from_fd (
 		struct rrr_socket_options *target,
 		int fd
@@ -97,12 +104,22 @@ int rrr_socket_open_and_read_file (
 		int options,
 		int mode
 );
+#ifdef RRR_HAVE_EVENTFD
+int rrr_socket_eventfd (
+		const char *creator
+);
+#endif
+int rrr_socket_pipe (
+		int result[2],
+		const char *creator
+);
 int rrr_socket (
 		int domain,
 		int type,
 		int protocol,
 		const char *creator,
-		const char *filename
+		const char *filename,
+		int register_for_unlink
 );
 int rrr_socket_close (int fd);
 int rrr_socket_close_no_unlink (int fd);
@@ -111,6 +128,8 @@ int rrr_socket_close_all_except (int fd);
 int rrr_socket_close_all_except_no_unlink (int fd);
 int rrr_socket_close_all (void);
 int rrr_socket_close_all_no_unlink (void);
+int rrr_socket_close_all_except_array (int *fds, size_t fd_count);
+int rrr_socket_close_all_except_array_no_unlink (int *fds, size_t fd_count);
 int rrr_socket_fifo_create (
 		int *fd_result,
 		const char *filename,
@@ -128,13 +147,16 @@ int rrr_socket_unix_create_bind_and_listen (
 		int do_mkstemp,
 		int do_unlink_if_exists
 );
+int rrr_socket_send_check (
+		int fd
+);
 int rrr_socket_connect_nonblock_postcheck_loop (
 		int fd,
 		uint64_t timeout_ms
 );
 int rrr_socket_connect_nonblock (
 		int fd,
-		struct sockaddr *addr,
+		const struct sockaddr *addr,
 		socklen_t addr_len
 );
 int rrr_socket_unix_connect (
@@ -152,6 +174,20 @@ int rrr_socket_sendto_nonblock (
 		const struct sockaddr *addr,
 		socklen_t addr_len
 );
+int rrr_socket_sendto_nonblock_check_retry (
+		ssize_t *written_bytes,
+		int fd,
+		const void *data,
+		ssize_t size,
+		const struct sockaddr *addr,
+		socklen_t addr_len
+);
+int rrr_socket_send_nonblock_check_retry (
+		ssize_t *written_bytes,
+		int fd,
+		const void *data,
+		ssize_t size
+);
 int rrr_socket_sendto_blocking (
 		int fd,
 		const void *data,
@@ -167,22 +203,11 @@ int rrr_socket_sendto_nonblock_fail_on_partial_write (
 		const struct sockaddr *sockaddr,
 		socklen_t addrlen
 );
-static inline int rrr_socket_send_nonblock (
-		ssize_t *written_bytes,
-		int fd,
-		const void *data,
-		ssize_t size
-) {
-	int err = 0;
-	return rrr_socket_sendto_nonblock(&err, written_bytes, fd, data, size, NULL, 0);
-}
-static inline int rrr_socket_send_blocking (
+int rrr_socket_send_blocking (
 		int fd,
 		void *data,
 		ssize_t size
-) {
-	return rrr_socket_sendto_blocking(fd, data, size, NULL, 0);
-}
+);
 int rrr_socket_check_alive (int fd);
 
 
