@@ -23,6 +23,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <string.h>
 
 #include "../log.h"
+#include "../allocator.h"
 
 #include "stats_tree.h"
 #include "stats_message.h"
@@ -38,7 +39,7 @@ static int __rrr_stats_tree_branch_new (
 
 	*target = NULL;
 
-	struct rrr_stats_tree_branch *result = malloc(sizeof(*result));
+	struct rrr_stats_tree_branch *result = rrr_allocate(sizeof(*result));
 	if (result == NULL) {
 		RRR_MSG_0("Could not allocate memory in __rrr_stats_tree_branch_new A\n");
 		ret = 1;
@@ -47,7 +48,7 @@ static int __rrr_stats_tree_branch_new (
 
 	memset(result, '\0', sizeof(*result));
 
-	result->name = strdup(name);
+	result->name = rrr_strdup(name);
 	if (name == NULL) {
 		RRR_MSG_0("Could not allocate memory in __rrr_stats_tree_branch_new B\n");
 		ret = 1;
@@ -62,7 +63,7 @@ static int __rrr_stats_tree_branch_new (
 //	out_free_name:
 //		free(result->name);
 	out_free_result:
-		free(result);
+		rrr_free(result);
 	out:
 		return ret;
 }
@@ -70,10 +71,10 @@ static int __rrr_stats_tree_branch_new (
 static int __rrr_stats_tree_branch_destroy (struct rrr_stats_tree_branch *branch) {
 	RRR_LL_DESTROY(branch, struct rrr_stats_tree_branch, __rrr_stats_tree_branch_destroy(node));
 	if (branch->value != NULL) {
-		rrr_stats_message_destroy(branch->value);
+		rrr_msg_stats_destroy(branch->value);
 	}
 	RRR_FREE_IF_NOT_NULL(branch->name);
-	free(branch);
+	rrr_free(branch);
 	return 0;
 }
 
@@ -133,19 +134,19 @@ static void __rrr_stats_get_first_path_level (
 static int __rrr_stats_tree_insert_or_update_branch (
 		struct rrr_stats_tree_branch *branch,
 		const char *path_position,
-		const struct rrr_stats_message *value
+		const struct rrr_msg_stats *value
 ) {
 	branch->last_seen = rrr_time_get_64();
 
 	// Last level (leaf)?
 	if (strlen(path_position) == 0) {
-		struct rrr_stats_message *new_value;
-		if (rrr_stats_message_duplicate(&new_value, value) != 0) {
+		struct rrr_msg_stats *new_value;
+		if (rrr_msg_stats_duplicate(&new_value, value) != 0) {
 			RRR_MSG_0("Could not duplicate message in __rrr_stats_tree_insert_or_update_branch n");
 			return RRR_STATS_TREE_HARD_ERROR;
 		}
 		if (branch->value != NULL) {
-			rrr_stats_message_destroy(branch->value);
+			rrr_msg_stats_destroy(branch->value);
 		}
 		branch->value = new_value;
 
@@ -176,7 +177,7 @@ static int __rrr_stats_tree_insert_or_update_branch (
 	return __rrr_stats_tree_insert_or_update_branch(new_branch, path_position, value);
 }
 
-int rrr_stats_tree_insert_or_update (struct rrr_stats_tree *tree, const struct rrr_stats_message *message) {
+int rrr_stats_tree_insert_or_update (struct rrr_stats_tree *tree, const struct rrr_msg_stats *message) {
 	if (strlen (message->path) < 2) {
 		RRR_MSG_0("Path of message was too short in rrr_stats_tree_insert_or_update (value was '%s')", message->path);
 		return RRR_STATS_TREE_SOFT_ERROR;
