@@ -25,11 +25,16 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "log.h"
 #include "string_builder.h"
+#include "type.h"
+#include "allocator.h"
 #include "util/gnu.h"
 #include "util/macro_utils.h"
 
-void rrr_string_builder_unchecked_append (struct rrr_string_builder *string_builder, const char *str) {
-	ssize_t length = strlen(str);
+void rrr_string_builder_unchecked_append (
+		struct rrr_string_builder *string_builder,
+		const char *str
+) {
+	rrr_biglength length = strlen(str);
 	memcpy(string_builder->buf + string_builder->wpos, str, length + 1);
 	string_builder->wpos += length;
 	if (string_builder->wpos + 1 > string_builder->size) {
@@ -37,30 +42,61 @@ void rrr_string_builder_unchecked_append (struct rrr_string_builder *string_buil
 	}
 }
 
-void rrr_string_builder_unchecked_append_raw (struct rrr_string_builder *string_builder, const char *buf, size_t buf_size) {
+static void __rrr_string_builder_unchecked_append_raw (
+		struct rrr_string_builder *string_builder,
+		const char *buf, rrr_biglength buf_size
+) {
 	memcpy(string_builder->buf + string_builder->wpos, buf, buf_size);
+	string_builder->wpos += buf_size;
+	string_builder->buf[string_builder->wpos] = '\0';
 }
 
-char *rrr_string_builder_buffer_takeover (struct rrr_string_builder *string_builder) {
+char *rrr_string_builder_buffer_takeover (
+		struct rrr_string_builder *string_builder
+) {
 	char *ret = string_builder->buf;
 	memset(string_builder, '\0', sizeof(*string_builder));
 	return ret;
 }
 
-void rrr_string_builder_clear (struct rrr_string_builder *string_builder) {
+void rrr_string_builder_clear (
+		struct rrr_string_builder *string_builder
+) {
 	RRR_FREE_IF_NOT_NULL(string_builder->buf);
 	string_builder->size = 0;
 	string_builder->wpos = 0;
 }
 
-ssize_t rrr_string_builder_length (struct rrr_string_builder *string_builder) {
+void rrr_string_builder_clear_void (
+		void *string_builder
+) {
+	rrr_string_builder_clear(string_builder);
+}
+
+const char *rrr_string_builder_buf (
+		const struct rrr_string_builder *string_builder
+) {
+	return string_builder->buf;
+}
+
+rrr_biglength rrr_string_builder_length (
+		const struct rrr_string_builder *string_builder
+) {
 	return (string_builder->buf == NULL ? 0 : string_builder->wpos);
 }
 
-int rrr_string_builder_new (struct rrr_string_builder **result) {
+rrr_biglength rrr_string_builder_size (
+		const struct rrr_string_builder *string_builder
+) {
+	return (string_builder->size);
+}
+
+int rrr_string_builder_new (
+		struct rrr_string_builder **result
+) {
 	*result = NULL;
 
-	struct rrr_string_builder *string_builder = malloc(sizeof(*string_builder));
+	struct rrr_string_builder *string_builder = rrr_allocate(sizeof(*string_builder));
 
 	if (string_builder == NULL) {
 		RRR_MSG_0("Could not allocate memory in rrr_string_builder_new\n");
@@ -74,24 +110,27 @@ int rrr_string_builder_new (struct rrr_string_builder **result) {
 	return 0;
 }
 
-void rrr_string_builder_destroy (struct rrr_string_builder *string_builder) {
+void rrr_string_builder_destroy (
+		struct rrr_string_builder *string_builder
+) {
 	rrr_string_builder_clear (string_builder);
-	free(string_builder);
+	rrr_free(string_builder);
 }
 
-void rrr_string_builder_destroy_void (void *ptr) {
+void rrr_string_builder_destroy_void (
+		void *ptr
+) {
 	rrr_string_builder_clear (ptr);
-	free(ptr);
+	rrr_free(ptr);
 }
 
-int rrr_string_builder_reserve (struct rrr_string_builder *string_builder, ssize_t bytes) {
-	if (bytes == 0) {
-		return 0;
-	}
-
-	if (string_builder->wpos + bytes + 1 > string_builder->size) {
-		ssize_t new_size = bytes + 1 + string_builder->size + 1024;
-		char *new_buf = realloc(string_builder->buf, new_size);
+int rrr_string_builder_reserve (
+		struct rrr_string_builder *string_builder,
+		rrr_biglength bytes
+) {
+	if (string_builder->wpos + bytes + 1 > string_builder->size || string_builder->buf == NULL) {
+		rrr_biglength new_size = bytes + 1 + string_builder->size + 1024;
+		char *new_buf = rrr_reallocate(string_builder->buf, string_builder->size, new_size);
 		if (new_buf == NULL) {
 			RRR_MSG_0("Could not allocate memory in rrr_string_builder_reserve\n");
 			return 1;
@@ -103,12 +142,50 @@ int rrr_string_builder_reserve (struct rrr_string_builder *string_builder, ssize
 	return 0;
 }
 
-int rrr_string_builder_append (struct rrr_string_builder *string_builder, const char *str) {
-	if (*str == '\0') {
+
+int rrr_string_builder_append_from (
+		struct rrr_string_builder *target,
+		const struct rrr_string_builder *source
+) {
+	int ret = 0;
+
+	if (source->wpos == 0) {
+		goto out;
+	}
+
+	if ((ret = rrr_string_builder_reserve(target, source->wpos)) != 0) {
+		goto out;
+	}
+
+	__rrr_string_builder_unchecked_append_raw (target, source->buf, source->wpos);
+
+	out:
+	return ret;
+}
+
+int rrr_string_builder_append_raw (
+		struct rrr_string_builder *target,
+		const char *str,
+		rrr_biglength length
+) {
+	if (rrr_string_builder_reserve(target, length) != 0) {
+		return 1;
+	}
+
+	__rrr_string_builder_unchecked_append_raw(target, str, length);
+
+	return 0;
+}
+
+int rrr_string_builder_append (
+		struct rrr_string_builder *string_builder,
+		const char *str
+) {
+	if (*str == '\0' && string_builder->buf != NULL) {
 		return 0;
 	}
 
-	ssize_t length = strlen(str);
+	rrr_biglength length = strlen(str);
 
 	if (rrr_string_builder_reserve(string_builder, length) != 0) {
 		return 1;
@@ -120,7 +197,11 @@ int rrr_string_builder_append (struct rrr_string_builder *string_builder, const
 	return 0;
 }
 
-int rrr_string_builder_append_format (struct rrr_string_builder *string_builder, const char *format, ...) {
+int rrr_string_builder_append_format (
+		struct rrr_string_builder *string_builder,
+		const char *format,
+		...
+) {
 	int ret = 0;
 
 	va_list args;
@@ -133,7 +214,7 @@ int rrr_string_builder_append_format (struct rrr_string_builder *string_builder,
 		goto out;
 	}
 
-	ssize_t length = strlen(tmp);
+	rrr_biglength length = strlen(tmp);
 	if (rrr_string_builder_reserve(string_builder, length) != 0) {
 		ret = 1;
 		goto out;
@@ -148,3 +229,10 @@ int rrr_string_builder_append_format (struct rrr_string_builder *string_builder,
 	return ret;
 }
 
+void rrr_string_builder_chop (
+		struct rrr_string_builder *string_builder
+) {
+	if (string_builder->wpos > 0) {
+		string_builder->wpos -= 1;
+	}
+}
