@@ -2,7 +2,7 @@
 
 Read Route Record
 
-Copyright (C) 2019-2020 Atle Solbakken atle@goliathdns.no
+Copyright (C) 2019-2021 Atle Solbakken atle@goliathdns.no
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -55,10 +55,12 @@ static const union type_system_endian {
 #define RRR_TYPE_MSG		9  // Type which holds an RRR message
 #define RRR_TYPE_FIXP		10 // Signed 64 type of which 24 bits are fraction given as string in base10 or base16
 #define RRR_TYPE_STR		11 // Dynamic length string quoted with "
+#define RRR_TYPE_HEX		RRR_TYPE_STR // Alias, used when converting blobs to ascii hex
 #define RRR_TYPE_NSEP		12 // Group of any byte not being a separator byte
 #define RRR_TYPE_STX		13 // STX or SOH, start of transmission or start of header
 #define RRR_TYPE_ERR		14 // Always produces soft error when being parsed, used to abort branched parsing
-#define RRR_TYPE_MAX		14
+#define RRR_TYPE_VAIN		15 // The useless type, indicates NULL or void. Will parse 0 bytes.
+#define RRR_TYPE_MAX		15
 
 #define RRR_TYPE_NAME_LE	"le"
 #define RRR_TYPE_NAME_BE	"be"
@@ -73,6 +75,10 @@ static const union type_system_endian {
 #define RRR_TYPE_NAME_NSEP	"nsep"
 #define RRR_TYPE_NAME_STX	"stx"
 #define RRR_TYPE_NAME_ERR	"err"
+#define RRR_TYPE_NAME_VAIN	"vain"
+
+// Alias for string
+#define RRR_TYPE_NAME_HEX	"hex"
 
 #define RRR_TYPE_MAX_LE		sizeof(rrr_type_le)
 #define RRR_TYPE_MAX_BE		sizeof(rrr_type_be)
@@ -87,21 +93,24 @@ static const union type_system_endian {
 #define RRR_TYPE_MAX_NSEP	0
 #define RRR_TYPE_MAX_STX	64
 #define RRR_TYPE_MAX_ERR	0
+#define RRR_TYPE_MAX_VAIN	0
 
 #define RRR_TYPE_IS_64(type) 	(														\
 			(type) == RRR_TYPE_LE || (type) == RRR_TYPE_BE || (type) == RRR_TYPE_H ||	\
 			(type) == RRR_TYPE_USTR || (type) == RRR_TYPE_ISTR							\
 		)
-#define RRR_TYPE_IS_BLOB(type)		((type) == RRR_TYPE_BLOB || (type) == RRR_TYPE_SEP || (type) == RRR_TYPE_MSG || (type) == RRR_TYPE_STR || (type) == RRR_TYPE_NSEP || (type) == RRR_TYPE_STX)
-#define RRR_TYPE_IS_FIXP(type)		((type) == RRR_TYPE_FIXP)
-#define RRR_TYPE_IS_MSG(type)		((type) == RRR_TYPE_MSG)
-#define RRR_TYPE_IS_STR(type)		((type) == RRR_TYPE_STR || (type) == RRR_TYPE_SEP || (type) == RRR_TYPE_NSEP || (type) == RRR_TYPE_STX)
-#define RRR_TYPE_IS_STR_EXCACT(type)((type) == RRR_TYPE_STR)
-#define RRR_TYPE_IS_SEP(type)		((type) == RRR_TYPE_SEP)
-#define RRR_TYPE_IS_NSEP(type)		((type) == RRR_TYPE_NSEP)
-#define RRR_TYPE_IS_STX(type)		((type) == RRR_TYPE_STX)
-#define RRR_TYPE_ALLOWS_SIGN(type)	((type) == RRR_TYPE_LE || (type) == RRR_TYPE_BE || (type) == RRR_TYPE_H)
-#define RRR_TYPE_OK(type)			((type) >= RRR_TYPE_MIN && (type) <= RRR_TYPE_MAX)
+#define RRR_TYPE_IS_BLOB(type)        ((type) == RRR_TYPE_BLOB || (type) == RRR_TYPE_SEP || (type) == RRR_TYPE_MSG || (type) == RRR_TYPE_STR || (type) == RRR_TYPE_NSEP || (type) == RRR_TYPE_STX)
+#define RRR_TYPE_IS_BLOB_EXCACT(type) ((type) == RRR_TYPE_BLOB)
+#define RRR_TYPE_IS_FIXP(type)        ((type) == RRR_TYPE_FIXP)
+#define RRR_TYPE_IS_MSG(type)         ((type) == RRR_TYPE_MSG)
+#define RRR_TYPE_IS_STR(type)         ((type) == RRR_TYPE_STR || (type) == RRR_TYPE_SEP || (type) == RRR_TYPE_NSEP || (type) == RRR_TYPE_STX)
+#define RRR_TYPE_IS_STR_EXCACT(type)  ((type) == RRR_TYPE_STR)
+#define RRR_TYPE_IS_SEP(type)         ((type) == RRR_TYPE_SEP)
+#define RRR_TYPE_IS_NSEP(type)        ((type) == RRR_TYPE_NSEP)
+#define RRR_TYPE_IS_STX(type)         ((type) == RRR_TYPE_STX)
+#define RRR_TYPE_IS_VAIN(type)        ((type) == RRR_TYPE_VAIN)
+#define RRR_TYPE_ALLOWS_SIGN(type)    ((type) == RRR_TYPE_LE || (type) == RRR_TYPE_BE || (type) == RRR_TYPE_H)
+#define RRR_TYPE_OK(type)             ((type) >= RRR_TYPE_MIN && (type) <= RRR_TYPE_MAX)
 
 #define RRR_TYPE_FLAG_SIGNED ((uint8_t) (1<<0))
 
@@ -127,51 +136,54 @@ static const union type_system_endian {
 #define RRR_TYPE_CHAR_IS_SEP_F(c) \
 	(c == 0 || (c >= 3 &&c <= 4))     // NULL, ETX, EOT
 
-#define RRR_TYPE_CHAR_IS_SEP(c) (		\
-		RRR_TYPE_CHAR_IS_SEP_A(c)||		\
-		RRR_TYPE_CHAR_IS_SEP_B(c)||		\
-		RRR_TYPE_CHAR_IS_SEP_C(c)||		\
-		RRR_TYPE_CHAR_IS_SEP_D(c)||		\
-		RRR_TYPE_CHAR_IS_SEP_E(c)||		\
-		RRR_TYPE_CHAR_IS_SEP_F(c)		\
-	)
+#define RRR_TYPE_CHAR_IS_SEP(c) (                              \
+        RRR_TYPE_CHAR_IS_SEP_A(c)||                            \
+        RRR_TYPE_CHAR_IS_SEP_B(c)||                            \
+        RRR_TYPE_CHAR_IS_SEP_C(c)||                            \
+        RRR_TYPE_CHAR_IS_SEP_D(c)||                            \
+        RRR_TYPE_CHAR_IS_SEP_E(c)||                            \
+        RRR_TYPE_CHAR_IS_SEP_F(c)                              \
+    )
 
-#define RRR_TYPE_GET_IMPORT_LENGTH_ARGS		\
-		rrr_length *import_length,		\
-		const struct rrr_type_value *node,	\
-		const char *buf,					\
-		rrr_length buf_size
+#define RRR_TYPE_GET_IMPORT_LENGTH_ARGS                        \
+        rrr_length *import_length,                             \
+        const struct rrr_type_value *node,                     \
+        const char *buf,                                       \
+        rrr_length buf_size
 
-#define RRR_TYPE_IMPORT_ARGS				\
-		struct rrr_type_value *node,		\
-		rrr_length *parsed_bytes,		\
-		const char *start,					\
-		const char *end
+#define RRR_TYPE_IMPORT_ARGS                                   \
+        struct rrr_type_value *node,                           \
+        rrr_length *parsed_bytes,                              \
+        const char *start,                                     \
+        const char *end
 
-#define RRR_TYPE_GET_EXPORT_LENGTH_ARGS		\
-		rrr_length *bytes,				\
-		const struct rrr_type_value *node
+#define RRR_TYPE_GET_EXPORT_LENGTH_ARGS                        \
+        rrr_length *bytes,                                     \
+        const struct rrr_type_value *node
 
-#define RRR_TYPE_EXPORT_ARGS				\
-		char *target,						\
-		rrr_length *written_bytes,		\
-		const struct rrr_type_value *node
+#define RRR_TYPE_EXPORT_ARGS                                   \
+        char *target,                                          \
+        rrr_length *written_bytes,                             \
+        const struct rrr_type_value *node
 
-#define RRR_TYPE_UNPACK_ARGS				\
-		struct rrr_type_value *node
+#define RRR_TYPE_UNPACK_ARGS                                   \
+        struct rrr_type_value *node
 
-#define RRR_TYPE_PACK_ARGS					\
-		char *target,						\
-		rrr_length *written_bytes,		\
-		uint8_t *new_type_id,				\
-		const struct rrr_type_value *node
+#define RRR_TYPE_PACK_ARGS                                     \
+        char *target,                                          \
+        rrr_length *written_bytes,                             \
+        uint8_t *new_type_id,                                  \
+        const struct rrr_type_value *node
 
-#define RRR_TYPE_TO_STR_ARGS				\
-		char **target,						\
-		const struct rrr_type_value *node
+#define RRR_TYPE_TO_STR_ARGS                                   \
+        char **target,                                         \
+        const struct rrr_type_value *node
 
-#define RRR_TYPE_TO_64_ARGS					\
-		const struct rrr_type_value *node
+#define RRR_TYPE_TO_64_ARGS                                    \
+        const struct rrr_type_value *node
+
+#define RRR_TYPE_TO_ULL_ARGS                                   \
+        const struct rrr_type_value *node
 
 struct rrr_type_value;
 
@@ -192,6 +204,7 @@ struct rrr_type_definition {
 
 	int (*to_str)(RRR_TYPE_TO_STR_ARGS);
 	uint64_t (*to_64)(RRR_TYPE_TO_64_ARGS);
+	unsigned long long (*to_ull)(RRR_TYPE_TO_ULL_ARGS);
 	const char *identifier;
 };
 
@@ -209,33 +222,74 @@ struct rrr_type_value {
 	char *data;
 };
 
-#define RRR_TYPE_DEFINE_EXTERN(name) \
+#define RRR_TYPE_VALUE_TMP_CREATE(name, data, tag, type, flags, tag_length, total_length, element_count)             \
+    const struct rrr_type_value name = {                                                                             \
+        NULL, NULL, type, flags, tag_length, 0, NULL, total_length, element_count, NULL, (char *) tag, (char *) data \
+    }
+
+#define RRR_TYPE_DECLARE_EXTERN(name) \
 	extern const struct rrr_type_definition RRR_PASTE(rrr_type_definition_,name)
 
-RRR_TYPE_DEFINE_EXTERN(be);
-RRR_TYPE_DEFINE_EXTERN(h);
-RRR_TYPE_DEFINE_EXTERN(le);
-RRR_TYPE_DEFINE_EXTERN(blob);
-RRR_TYPE_DEFINE_EXTERN(ustr);
-RRR_TYPE_DEFINE_EXTERN(istr);
-RRR_TYPE_DEFINE_EXTERN(sep);
-RRR_TYPE_DEFINE_EXTERN(msg);
-RRR_TYPE_DEFINE_EXTERN(fixp);
-RRR_TYPE_DEFINE_EXTERN(str);
-RRR_TYPE_DEFINE_EXTERN(nsep);
-RRR_TYPE_DEFINE_EXTERN(stx);
-RRR_TYPE_DEFINE_EXTERN(null);
+#define RRR_TYPE_RAW_FIELDS                            \
+	const char *data_start,                        \
+	const struct rrr_type_definition *type,        \
+	rrr_type_flags flags,                          \
+	rrr_length tag_length,                         \
+	rrr_length total_length,                       \
+	rrr_length element_count
 
-int rrr_type_import_ustr_raw (uint64_t *target, rrr_length *parsed_bytes, const char *start, const char *end);
-int rrr_type_import_istr_raw (int64_t *target, rrr_length *parsed_bytes, const char *start, const char *end);
+RRR_TYPE_DECLARE_EXTERN(be);
+RRR_TYPE_DECLARE_EXTERN(h);
+RRR_TYPE_DECLARE_EXTERN(le);
+RRR_TYPE_DECLARE_EXTERN(blob);
+RRR_TYPE_DECLARE_EXTERN(ustr);
+RRR_TYPE_DECLARE_EXTERN(istr);
+RRR_TYPE_DECLARE_EXTERN(sep);
+RRR_TYPE_DECLARE_EXTERN(msg);
+RRR_TYPE_DECLARE_EXTERN(fixp);
+RRR_TYPE_DECLARE_EXTERN(str);
+RRR_TYPE_DECLARE_EXTERN(nsep);
+RRR_TYPE_DECLARE_EXTERN(stx);
+RRR_TYPE_DECLARE_EXTERN(err);
+RRR_TYPE_DECLARE_EXTERN(vain);
+RRR_TYPE_DECLARE_EXTERN(null);
 
+#define RRR_TYPE_DEFINITION_BE     rrr_type_definition_be
+#define RRR_TYPE_DEFINITION_H      rrr_type_definition_h
+#define RRR_TYPE_DEFINITION_LE     rrr_type_definition_le
+#define RRR_TYPE_DEFINITION_BLOB   rrr_type_definition_blob
+#define RRR_TYPE_DEFINITION_USTR   rrr_type_definition_ustr
+#define RRR_TYPE_DEFINITION_ISTR   rrr_type_definition_istr
+#define RRR_TYPE_DEFINITION_SEP    rrr_type_definition_sep
+#define RRR_TYPE_DEFINITION_MSG    rrr_type_definition_msg
+#define RRR_TYPE_DEFINITION_FIXP   rrr_type_definition_fixp
+#define RRR_TYPE_DEFINITION_STR    rrr_type_definition_str
+#define RRR_TYPE_DEFINITION_HEX    RRR_TYPE_DEFINITION_STR
+#define RRR_TYPE_DEFINITION_NSEP   rrr_type_definition_nsep
+#define RRR_TYPE_DEFINITION_STX    rrr_type_definition_stx
+#define RRR_TYPE_DEFINITION_ERR    rrr_type_definition_err
+#define RRR_TYPE_DEFINITION_VAIN    rrr_type_definition_vain
+#define RRR_TYPE_DEFINITION_NULL   rrr_type_definition_null
+
+int rrr_type_import_ustr_raw (
+		uint64_t *target,
+		rrr_length *parsed_bytes,
+		const char *start,
+		const char *end
+);
+int rrr_type_import_istr_raw (
+		int64_t *target,
+		rrr_length *parsed_bytes,
+		const char *start,
+		const char *end
+);
 const struct rrr_type_definition *rrr_type_parse_from_string (
 		rrr_length *parsed_bytes,
 		const char *start,
 		const char *end
 );
 const struct rrr_type_definition *rrr_type_get_from_id (
-		uint8_t type_in
+		const uint8_t type_in
 );
 void rrr_type_value_destroy (
 		struct rrr_type_value *template
@@ -245,6 +299,11 @@ int rrr_type_value_set_tag (
 		const char *tag,
 		rrr_length tag_length
 );
+void rrr_type_value_set_data (
+		struct rrr_type_value *value,
+		char *data,
+		rrr_length data_length
+);
 int rrr_type_value_new (
 		struct rrr_type_value **result,
 		const struct rrr_type_definition *type,
@@ -257,6 +316,24 @@ int rrr_type_value_new (
 		const char *element_count_ref,
 		rrr_length stored_length
 );
+int rrr_type_value_new_simple (
+		struct rrr_type_value **result,
+		const struct rrr_type_definition *type,
+		rrr_type_flags flags,
+		rrr_length tag_length,
+		const char *tag
+);
+int rrr_type_new_vain (
+		struct rrr_type_value **target,
+		rrr_length tag_length,
+		const char *tag
+);
+int rrr_type_new_h (
+		struct rrr_type_value **target,
+		rrr_length tag_length,
+		const char *tag,
+		rrr_length element_count
+);
 int rrr_type_value_clone (
 		struct rrr_type_value **target,
 		const struct rrr_type_value *source,
@@ -280,5 +357,10 @@ int rrr_type_value_allocate_and_import_raw (
 		rrr_length import_length,
 		rrr_length element_count
 );
+int rrr_type_value_raw_with_tmp_do (
+		RRR_TYPE_RAW_FIELDS,
+		int (*callback)(const struct rrr_type_value *value, void *arg),
+		void *callback_arg
+);
 
 #endif /* RRR_TYPE_HEADER */
