@@ -2,7 +2,7 @@
 
 Read Route Record
 
-Copyright (C) 2019-2020 Atle Solbakken atle@goliathdns.no
+Copyright (C) 2019-2021 Atle Solbakken atle@goliathdns.no
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -25,23 +25,26 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <sys/socket.h>
 #include <sys/types.h>
 #include <netinet/in.h>
-#include <read.h>
 
-#include "log.h"
+#include "../log.h"
+#include "../allocator.h"
 #include "udpstream.h"
-#include "random.h"
-#include "socket/rrr_socket.h"
-#include "socket/rrr_socket_read.h"
-#include "util/macro_utils.h"
-#include "util/linked_list.h"
-#include "util/rrr_time.h"
-#include "util/crc32.h"
-#include "util/posix.h"
+#include "../read.h"
+#include "../random.h"
+#include "../socket/rrr_socket.h"
+#include "../socket/rrr_socket_read.h"
+#include "../util/macro_utils.h"
+#include "../util/linked_list.h"
+#include "../util/rrr_time.h"
+#include "../util/crc32.h"
+#include "../util/posix.h"
+#include "../string_builder.h"
+#include "../ip/ip_util.h"
 
 static int __rrr_udpstream_frame_destroy(struct rrr_udpstream_frame *frame) {
 	RRR_FREE_IF_NOT_NULL(frame->data);
 	RRR_FREE_IF_NOT_NULL(frame->source_addr);
-	free(frame);
+	rrr_free(frame);
 	return 0;
 }
 
@@ -54,7 +57,7 @@ static int __rrr_udpstream_frame_new_from_data (
 
 	*target = NULL;
 
-	struct rrr_udpstream_frame *res = malloc(sizeof(*res));
+	struct rrr_udpstream_frame *res = rrr_allocate(sizeof(*res));
 	if (res == NULL) {
 		RRR_MSG_0("Could not allocate memory in __rrr_udpstream_frame_new_from_data A\n");
 		ret = 1;
@@ -67,7 +70,7 @@ static int __rrr_udpstream_frame_new_from_data (
 	}
 
 	if (data_size > 0) {
-		res->data = malloc(data_size);
+		res->data = rrr_allocate(data_size);
 		if (res->data == NULL) {
 			RRR_MSG_0("Could not allocate memory in __rrr_udpstream_frame_new_from_data B\n");
 			ret = 1;
@@ -104,7 +107,7 @@ static int __rrr_udpstream_frame_new_from_packed (
 	}
 
 	if (addr_len > 0) {
-		if ((result->source_addr = malloc(addr_len)) == NULL) {
+		if ((result->source_addr = rrr_allocate(addr_len)) == NULL) {
 			RRR_MSG_0("Could not allocate memory for address in __rrr_udpstream_frame_new_from_packed\n");
 			ret = 1;
 			goto out;
@@ -148,14 +151,14 @@ static int __rrr_udpstream_stream_destroy(struct rrr_udpstream_stream *stream) {
 	__rrr_udpstream_frame_buffer_clear(&stream->receive_buffer);
 	__rrr_udpstream_frame_buffer_clear(&stream->send_buffer);
 	RRR_FREE_IF_NOT_NULL(stream->remote_addr);
-	free(stream);
+	rrr_free(stream);
 	return 0;
 }
 
 static int __rrr_udpstream_stream_new(struct rrr_udpstream_stream **target) {
 	*target = NULL;
 
-	struct rrr_udpstream_stream *res = malloc(sizeof(*res));
+	struct rrr_udpstream_stream *res = rrr_allocate(sizeof(*res));
 	if (res == NULL) {
 		RRR_MSG_0("Could not allocate memory in __rrr_udpstream_stream_new\n");
 		return 1;
@@ -229,20 +232,48 @@ void rrr_udpstream_set_flags (
 	pthread_mutex_unlock(&data->lock);
 }
 
+static void __rrr_udpstream_frame_packed_dump (
+		const struct rrr_udpstream_frame_packed *frame
+) {
+	struct rrr_string_builder string_builder = {0};
+	RRR_DBG ("-- UDP-stream packed frame size %lu\n", RRR_UDPSTREAM_FRAME_PACKED_TOTAL_SIZE(frame));
+	RRR_DBG ("Header CRC32 : %" PRIu32 "\n", RRR_UDPSTREAM_FRAME_PACKED_HEADER_CRC32(frame));
+	RRR_DBG ("Data CRC32   : %" PRIu32 "\n", RRR_UDPSTREAM_FRAME_PACKED_DATA_CRC32(frame));
+	RRR_DBG ("Total size   : %lu\n", RRR_UDPSTREAM_FRAME_PACKED_TOTAL_SIZE(frame));
+	RRR_DBG ("Data size    : %u\n", RRR_UDPSTREAM_FRAME_PACKED_DATA_SIZE(frame));
+	RRR_DBG ("Flags        : %u\n", RRR_UDPSTREAM_FRAME_FLAGS(frame));
+	RRR_DBG ("Type         : %u\n", RRR_UDPSTREAM_FRAME_TYPE(frame));
+	RRR_DBG ("Version      : %u\n", RRR_UDPSTREAM_FRAME_PACKED_VERSION(frame));
+	RRR_DBG ("Stream-ID    : %u\n", RRR_UDPSTREAM_FRAME_PACKED_STREAM_ID(frame));
+	RRR_DBG ("Frame-ID     : %u\n", RRR_UDPSTREAM_FRAME_PACKED_FRAME_ID(frame));
+
+	unsigned char *data = (unsigned char *) frame;
+
+	rrr_string_builder_append(&string_builder, "-- 0x");
+	for (size_t i = 0; i < RRR_UDPSTREAM_FRAME_PACKED_TOTAL_SIZE(frame); i++) {
+		if (i == sizeof(*frame)) {
+			rrr_string_builder_append(&string_builder, " -- ");
+		}
+		rrr_string_builder_append_format(&string_builder, "%02x", *(data + i));
+	}
+	rrr_string_builder_append(&string_builder, "\n------------\n");
+	RRR_DBG("%s", rrr_string_builder_buf(&string_builder));
+	rrr_string_builder_clear(&string_builder);
+}
+
 static int __rrr_udpstream_checksum_and_send_packed_frame (
 		struct rrr_udpstream *udpstream_data,
 		const struct sockaddr *addr,
 		socklen_t addrlen,
-		struct rrr_udpstream_frame_packed *frame,
+		const struct rrr_udpstream_frame_packed *frame,
 		void *data,
 		uint16_t data_size,
 		int copies
 ) {
 	int ret = 0;
 
-	// TODO : Move allocation to init function
 	if (udpstream_data->send_buffer == NULL) {
-		udpstream_data->send_buffer = malloc(RRR_UDPSTREAM_MESSAGE_SIZE_MAX);
+		udpstream_data->send_buffer = rrr_allocate(RRR_UDPSTREAM_MESSAGE_SIZE_MAX);
 		if (udpstream_data->send_buffer == NULL) {
 			RRR_MSG_0("Could not allocate send buffer in __rrr_udpstream_checksum_and_send_packed_frame\n");
 			ret = 1;
@@ -251,7 +282,7 @@ static int __rrr_udpstream_checksum_and_send_packed_frame (
 		udpstream_data->send_buffer_size = RRR_UDPSTREAM_MESSAGE_SIZE_MAX;
 	}
 
-	if (data_size > udpstream_data->send_buffer_size) {
+	if ((size_t) data_size + sizeof(*frame) - 1 > (size_t) udpstream_data->send_buffer_size) {
 		RRR_BUG("data size too big in __rrr_udpstream_checksum_and_send_packed_frame\n");
 	}
 
@@ -259,26 +290,47 @@ static int __rrr_udpstream_checksum_and_send_packed_frame (
 		RRR_BUG("addr was NULL in __rrr_udpstream_checksum_and_send_packed_frame\n");
 	}
 
+	struct rrr_udpstream_frame_packed *frame_new = udpstream_data->send_buffer;
+
+	*frame_new = *frame;
+
 	// A packed frame created locally has the payload stored separately
 	if (data_size > 0) {
-		frame->data_crc32 = rrr_htobe32(rrr_crc32buf((char *) data, data_size));
-		frame->data_size = rrr_htobe16(data_size);
+		frame_new->data_crc32 = rrr_htobe32(rrr_crc32buf((char *) data, data_size));
+		frame_new->data_size = rrr_htobe16(data_size);
+	}
+	else {
+		frame_new->data_crc32 = 0;
+		frame_new->data_size = 0;
 	}
 
-	char *crc32_start_pos = ((char *) frame) + sizeof(frame->header_crc32);
-	ssize_t crc32_size = sizeof(*frame) - sizeof(frame->header_crc32) - 1;
+	char *crc32_start_pos = ((char *) frame_new) + sizeof(frame_new->header_crc32);
+	ssize_t crc32_size = sizeof(*frame_new) - sizeof(frame_new->header_crc32) - 1;
 
-	frame->header_crc32 = rrr_htobe32(rrr_crc32buf(crc32_start_pos, crc32_size));
+	frame_new->header_crc32 = rrr_htobe32(rrr_crc32buf(crc32_start_pos, crc32_size));
 
-	RRR_DBG_3("UDP-stream TX packed crc32: %" PRIu32 " size: %u flags_type: %u connect_handle/frame_id/window_size: %u stream: %u\n",
-			frame->header_crc32, rrr_be16toh(frame->data_size), frame->flags_and_type, rrr_be32toh(frame->connect_handle), rrr_be16toh(frame->stream_id));
-
-	memcpy(udpstream_data->send_buffer, frame, sizeof(*frame) - 1);
 	if (data_size > 0) {
 		if (data == NULL) {
 			RRR_BUG("BUG: Data was NULL in __rrr_udpstream_checksum_and_send_packed_frame\n");
 		}
-		memcpy(udpstream_data->send_buffer + sizeof(*frame) - 1, data, data_size);
+		memcpy(frame_new->data, data, data_size);
+	}
+	else {
+		frame_new->data[0] = '\0';
+	}
+
+	RRR_DBG_3("UDP-stream TX %u-%u CS: %" PRIu32 "/%" PRIu32 " S: %llu F/T: %u CH/ID/WS: %u\n",
+			rrr_be16toh(frame_new->stream_id),
+			rrr_be32toh(frame_new->frame_id),
+			rrr_be32toh(frame_new->header_crc32),
+			rrr_be32toh(frame_new->data_crc32),
+			(unsigned long long int) rrr_be16toh(frame_new->data_size) + sizeof(*frame) - 1,
+			frame_new->flags_and_type,
+			rrr_be32toh(frame_new->connect_handle)
+	);
+
+	if (RRR_DEBUGLEVEL_6) {
+		__rrr_udpstream_frame_packed_dump(frame_new);
 	}
 
 	while (copies--) {
@@ -319,7 +371,7 @@ static struct rrr_udpstream_stream *__rrr_udpstream_create_and_add_stream (
 	}
 
 	new_stream->remote_addr_len = addr_len;
-	new_stream->remote_addr = malloc(new_stream->remote_addr_len);
+	new_stream->remote_addr = rrr_allocate(new_stream->remote_addr_len);
 	if (new_stream->remote_addr == NULL) {
 		RRR_MSG_0("Could not allocate memory for address in __rrr_udpstream_send_connect\n");
 		__rrr_udpstream_stream_destroy(new_stream);
@@ -392,16 +444,15 @@ static int __rrr_udpstream_send_reset (
 		const struct sockaddr *addr,
 		socklen_t socklen,
 		uint16_t stream_id,
-		uint32_t frame_id
+		uint32_t connect_handle
 ) {
 	struct rrr_udpstream_frame_packed frame = {0};
 
 	frame.flags_and_type = RRR_UDPSTREAM_FRAME_TYPE_RESET;
 	frame.stream_id = rrr_htobe16(stream_id);
 
-	// If frame ID is zero, remote should perform a hard reset which implies creating a
-	// new connection and sending any data in send buffers again
-	frame.frame_id = rrr_htobe32(frame_id);
+	// Reset with connect handle is sent if wish to clean any existing stream on remote
+	frame.connect_handle = rrr_htobe32(connect_handle);
 
 	return __rrr_udpstream_checksum_and_send_packed_frame(data, addr, socklen, &frame, NULL, 0, 3);
 }
@@ -443,7 +494,7 @@ static uint32_t __rrr_udpstream_allocate_connect_handle (
 	return 0;
 }
 
-static int __rrr_udpstream_send_connect (
+static int __rrr_udpstream_send_reset_and_connect (
 		uint32_t *connect_handle_result,
 		struct rrr_udpstream *data,
 		const struct sockaddr *addr,
@@ -469,65 +520,50 @@ static int __rrr_udpstream_send_connect (
 		}
 	}
 
-	struct rrr_udpstream_frame_packed frame = {0};
-
-	frame.flags_and_type = RRR_UDPSTREAM_FRAME_TYPE_CONNECT;
-	frame.connect_handle = rrr_htobe32(connect_handle);
-
-	struct rrr_udpstream_stream *stream = NULL;
-	if ((stream = __rrr_udpstream_create_and_add_stream(data, addr, addr_len)) == NULL) {
-		RRR_MSG_0("Could not add stream to collection in __rrr_udpstream_send_connect\n");
-		ret = 1;
-		goto out;
+	{
+		struct rrr_udpstream_frame_packed frame = {0};
+		frame.flags_and_type = RRR_UDPSTREAM_FRAME_TYPE_RESET;
+		frame.connect_handle = rrr_htobe32(connect_handle);
+		if (__rrr_udpstream_checksum_and_send_packed_frame(data, addr, addr_len, &frame, NULL, 0, 3) != 0) {
+			RRR_MSG_0("Could not send RST packet in __rrr_udpstream_send_connect\n");
+			ret = 1;
+			goto out;
+		}
 	}
 
-	stream->connect_handle = connect_handle;
+	{
+		struct rrr_udpstream_frame_packed frame = {0};
 
-	if (__rrr_udpstream_checksum_and_send_packed_frame(data, stream->remote_addr, stream->remote_addr_len, &frame, NULL, 0, 3) != 0) {
-		RRR_MSG_0("Could not send CONNECT packet in __rrr_udpstream_send_connect\n");
-		ret = 1;
-		goto out;
+		frame.flags_and_type = RRR_UDPSTREAM_FRAME_TYPE_CONNECT;
+		frame.connect_handle = rrr_htobe32(connect_handle);
+
+		struct rrr_udpstream_stream *stream = NULL;
+		if ((stream = __rrr_udpstream_create_and_add_stream(data, addr, addr_len)) == NULL) {
+			RRR_MSG_0("Could not add stream to collection in __rrr_udpstream_send_connect\n");
+			ret = 1;
+			goto out;
+		}
+
+		stream->connect_handle = connect_handle;
+
+		if (__rrr_udpstream_checksum_and_send_packed_frame(data, stream->remote_addr, stream->remote_addr_len, &frame, NULL, 0, 3) != 0) {
+			RRR_MSG_0("Could not send CONNECT packet in __rrr_udpstream_send_connect\n");
+			ret = 1;
+			goto out;
+		}
 	}
 
-	RRR_DBG_2("Sent CONNECT with handle %" PRIu32 "\n", connect_handle);
-
 	*connect_handle_result = connect_handle;
 
 	out:
 	return ret;
 }
 
-static void __rrr_udpstream_frame_packed_dump (
-		const struct rrr_udpstream_frame_packed *frame
-) {
-	RRR_DBG ("-- UDP-stream packed frame size %lu\n", RRR_UDPSTREAM_FRAME_PACKED_TOTAL_SIZE(frame));
-	RRR_DBG ("Header CRC32 : %" PRIu32 "\n", RRR_UDPSTREAM_FRAME_PACKED_HEADER_CRC32(frame));
-	RRR_DBG ("Data CRC32   : %" PRIu32 "\n", RRR_UDPSTREAM_FRAME_PACKED_DATA_CRC32(frame));
-	RRR_DBG ("Flags        : %u\n", RRR_UDPSTREAM_FRAME_FLAGS(frame));
-	RRR_DBG ("Type	        : %u\n", RRR_UDPSTREAM_FRAME_TYPE(frame));
-	RRR_DBG ("Version      : %u\n", RRR_UDPSTREAM_FRAME_PACKED_VERSION(frame));
-	RRR_DBG ("Stream-ID    : %u\n", RRR_UDPSTREAM_FRAME_PACKED_STREAM_ID(frame));
-	RRR_DBG ("Frame-ID     : %u\n", RRR_UDPSTREAM_FRAME_PACKED_FRAME_ID(frame));
-
-	RRR_DBG("-- 0x");
-	for (size_t i = 0; i < RRR_UDPSTREAM_FRAME_PACKED_TOTAL_SIZE(frame); i++) {
-		char c = ((char *)frame)[i];
-		if (c < 0x10) {
-			RRR_DBG("0");
-		}
-		RRR_DBG("%x", c);
-	}
-	RRR_DBG("\n------------\n");
-}
-
 static int __rrr_udpstream_frame_packed_validate (
 		const struct rrr_udpstream_frame_packed *frame
 ) {
 	uint32_t header_crc32 = RRR_UDPSTREAM_FRAME_PACKED_HEADER_CRC32(frame);
 
-//	printf ("packed crc32 receive: %" PRIu32 "\n", frame->header_crc32);
-//	printf ("packed crc32 receive bswapped: %" PRIu32 "\n", header_crc32);
-
 	char *crc32_start_pos = ((char *) frame) + sizeof(frame->header_crc32);
 	ssize_t crc32_size = sizeof(*frame) - sizeof(frame->header_crc32) - 1;
 
@@ -563,7 +599,6 @@ static int __rrr_udpstream_read_get_target_size (
 		goto out;
 	}
 
-//	printf ("Data size: %u\n", frame->data_size);
 	ssize_t total_size = RRR_UDPSTREAM_FRAME_PACKED_TOTAL_SIZE(frame);
 
 	if (RRR_UDPSTREAM_FRAME_PACKED_DATA_SIZE(frame) > RRR_UDPSTREAM_FRAME_DATA_SIZE_LIMIT) {
@@ -577,6 +612,7 @@ static int __rrr_udpstream_read_get_target_size (
 	out:
 	return ret;
 }
+
 static struct rrr_udpstream_stream *__rrr_udpstream_find_stream_by_connect_handle (
 		struct rrr_udpstream *data,
 		uint32_t connect_handle
@@ -629,7 +665,7 @@ static int __rrr_udpstream_update_stream_remote (
 	if (stream->remote_addr == NULL || stream->remote_addr_len != addr_len || memcmp(stream->remote_addr, addr, addr_len) != 0) {
 		if (stream->remote_addr_len != addr_len || stream->remote_addr == NULL) {
 			RRR_FREE_IF_NOT_NULL(stream->remote_addr);
-			if ((stream->remote_addr = malloc(sizeof(*(stream->remote_addr)))) == NULL) {
+			if ((stream->remote_addr = rrr_allocate(sizeof(*(stream->remote_addr)))) == NULL) {
 				RRR_MSG_0("Could not allocate memory in __rrr_udpstream_update_stream_remote\n");
 				ret = 1;
 				goto out;
@@ -673,19 +709,22 @@ static int __rrr_udpstream_handle_received_connect (
 
 		struct rrr_udpstream_stream *stream_test = __rrr_udpstream_find_stream_by_stream_id(data, frame->stream_id);
 		if (stream_test != NULL) {
-			RRR_DBG_2("Stream ID collision for connect with handle %u, connection must be closed\n", frame->connect_handle);
+			RRR_DBG_3("Stream ID collision for connect with handle %u, connection must be closed\n", frame->connect_handle);
 			__rrr_udpstream_stream_invalidate(stream);
 			goto out;
 		}
 
 		stream->stream_id = frame->stream_id;
 
-		RRR_DBG_2("Outbound UDP-stream connection established with stream id %u connect handle was %u\n",
+		RRR_DBG_3("Outbound UDP-stream connection established with stream id %u connect handle was %u\n",
 				stream->stream_id, frame->connect_handle);
 	}
-	else if (stream != NULL && stream->stream_id != 0) {
+	else if (stream != NULL && stream->stream_id != 0 && frame->stream_id != 0) {
 		// Already connected
-		RRR_DBG_2("Incoming UDP-stream duplicate CONNECT (response)\n");
+		RRR_DBG_3("Incoming UDP-stream duplicate CONNECT (response or old unknown stream) stream_id local %" PRIu32 " stream id remote %" PRIu32 "\n",
+				stream->stream_id,
+				frame->stream_id
+		);
 		goto out;
 	}
 	else {
@@ -713,7 +752,7 @@ static int __rrr_udpstream_handle_received_connect (
 
 		if (stream != NULL) {
 			// Already connected, send new response
-			RRR_DBG_2("Incoming UDP-stream duplicate CONNECT\n");
+			RRR_DBG_3("Incoming UDP-stream duplicate CONNECT\n");
 			stream_id = stream->stream_id;
 			goto send_response;
 		}
@@ -732,18 +771,19 @@ static int __rrr_udpstream_handle_received_connect (
 			stream->stream_id = stream_id;
 			stream->connect_handle = frame->connect_handle;
 
-			RRR_DBG_2("Incoming UDP-stream connection established with stream id %u connect handle %u\n",
+			RRR_DBG_3("Incoming UDP-stream connection established with stream id %u connect handle %u\n",
 					stream_id, stream->connect_handle);
 		}
 		else {
 			// This is not considered an error
-			RRR_DBG_2("Incoming UDP-stream connection rejected\n");
+			RRR_DBG_3("Incoming UDP-stream connection rejected\n");
 			goto send_response;
 		}
 
 		send_response:
-		RRR_DBG_2("Sending UDP-stream CONNECT response stream id %u connect handle %u address length %u\n",
+		RRR_DBG_3("Sending UDP-stream CONNECT response stream id %u connect handle %u address length %u\n",
 				stream_id, (stream != NULL ? stream->connect_handle : 0), addr_len);
+
 		if (__rrr_udpstream_send_connect_response(data, src_addr, addr_len, stream_id, frame->connect_handle) != 0) {
 			RRR_MSG_0("Could not send connect response in __rrr_udpstream_handle_received_connect\n");
 			ret = RRR_SOCKET_HARD_ERROR;
@@ -756,27 +796,18 @@ static int __rrr_udpstream_handle_received_connect (
 }
 
 static int __rrr_udpstream_handle_received_reset (
-		struct rrr_udpstream_stream *stream,
-		struct rrr_udpstream_frame *new_frame
+		struct rrr_udpstream_stream *stream
 ) {
 	int ret = 0;
 
-	RRR_DBG_3("UDP-stream RX RST %u\n", new_frame->stream_id);
+	RRR_DBG_3("UDP-stream RX RST connect handle %" PRIu32 " stream id %" PRIu32 "\n",
+			stream->connect_handle, stream->stream_id);
 
-	if (new_frame->frame_id == 0) {
-		RRR_DBG_3("Performing hard reset for stream ID %u\n", new_frame->stream_id);
+	stream->receive_buffer.frame_id_max = stream->receive_buffer.frame_id_counter;
+	stream->send_buffer.frame_id_max = stream->send_buffer.frame_id_counter;
+	stream->hard_reset_received = 1;
 
-		stream->receive_buffer.frame_id_max = stream->receive_buffer.frame_id_counter;
-		stream->send_buffer.frame_id_max = stream->send_buffer.frame_id_counter;
-		stream->hard_reset_received = 1;
-
-		goto out;
-	}
-	else {
-		// Soft reset, continue until received last frame id. Do not send any more frames.
-		stream->receive_buffer.frame_id_max = new_frame->frame_id;
-		stream->send_buffer.frame_id_max = stream->send_buffer.frame_id_counter;
-	}
+	goto out;
 
 	out:
 	return ret;
@@ -857,14 +888,6 @@ static int __rrr_udpstream_handle_received_frame (
 
 	struct rrr_udpstream_frame *new_frame = NULL;
 
-	RRR_DBG_3("UDP-stream RX %u-%u crc32: %" PRIu32 " S: %u F/T: %u CH: %u\n",
-			rrr_be16toh(frame->stream_id),
-			rrr_be32toh(frame->frame_id),
-			frame->header_crc32,
-			rrr_be16toh(frame->data_size),
-			frame->flags_and_type, rrr_be32toh(frame->connect_handle)
-	);
-
 	if (__rrr_udpstream_frame_new_from_packed(&new_frame, frame, src_addr, addr_len) != 0) {
 		RRR_MSG_0("Could not allocate internal frame in __rrr_udpstream_handle_received_frame\n");
 		ret = RRR_SOCKET_HARD_ERROR;
@@ -876,19 +899,24 @@ static int __rrr_udpstream_handle_received_frame (
 		goto out;
 	}
 
-	if (new_frame->stream_id == 0) {
-		RRR_DBG_2("Unknown packet with type/flags %u and zero stream id in __rrr_udpstream_handle_received_frame\n",
-				new_frame->flags_and_type);
+	struct rrr_udpstream_stream *stream = NULL;
+
+	if (new_frame->stream_id != 0) {
+		stream = __rrr_udpstream_find_stream_by_stream_id(data, new_frame->stream_id);
+	}
+	else if (new_frame->connect_handle != 0) {
+		stream = __rrr_udpstream_find_stream_by_connect_handle(data, new_frame->connect_handle);
+	}
+	else {
+		RRR_DBG_1("Received UDP-stream packet with zero stream ID and zero connect ID, dropping it\n");
 		goto out;
 	}
 
-	struct rrr_udpstream_stream *stream = __rrr_udpstream_find_stream_by_stream_id(data, new_frame->stream_id);
-
 	if (stream == NULL) {
 		// Check that unknown packet is not a reset, if not we would keep sending resets back and forward
-		if (!RRR_UDPSTREAM_FRAME_IS_RESET(frame)) {
-			RRR_DBG_2("Received UDP-stream packet with unknown stream ID %u, sending hard reset\n", new_frame->stream_id);
-			if (__rrr_udpstream_send_reset(data, src_addr, addr_len, new_frame->stream_id, 0) != 0) {
+		if (!RRR_UDPSTREAM_FRAME_IS_RESET(new_frame)) {
+			RRR_DBG_3("Received UDP-stream packet with unknown stream ID %u, sending hard reset\n", new_frame->stream_id);
+			if (__rrr_udpstream_send_reset(data, src_addr, addr_len, new_frame->stream_id, new_frame->connect_handle) != 0) {
 				RRR_MSG_0("Could not send UDP-stream hard reset in __rrr_udpstream_handle_received_frame\n");
 				ret = RRR_SOCKET_HARD_ERROR;
 				goto out;
@@ -911,7 +939,13 @@ static int __rrr_udpstream_handle_received_frame (
 	}
 
 	if (RRR_UDPSTREAM_FRAME_IS_RESET(new_frame)) {
-		ret = __rrr_udpstream_handle_received_reset(stream, new_frame);
+		ret = __rrr_udpstream_handle_received_reset(stream);
+		goto out;
+	}
+
+	if (new_frame->stream_id == 0) {
+		RRR_DBG_3("Unknown packet with type/flags %u and zero stream id in __rrr_udpstream_handle_received_frame\n",
+				new_frame->flags_and_type);
 		goto out;
 	}
 
@@ -942,25 +976,17 @@ static int __rrr_udpstream_handle_received_frame (
 	}
 
 	if (new_frame->frame_id == 0) {
-		RRR_DBG_2("Received data frame with flags %u and zero frame id in __rrr_udpstream_handle_received_frame_control\n",
+		RRR_DBG_3("Received data frame with flags %u and zero frame id in __rrr_udpstream_handle_received_frame_control\n",
 				new_frame->flags_and_type);
 		goto out;
 	}
-/*
-	VL_DEBUG_MSG_3("UDP-stream packet with data for stream ID %u frame id %u boundary %i flags %i type %i\n",
-			new_frame->stream_id,
-			new_frame->frame_id,
-			RRR_UDPSTREAM_FRAME_IS_BOUNDARY(new_frame),
-			RRR_UDPSTREAM_FRAME_FLAGS(new_frame),
-			RRR_UDPSTREAM_FRAME_TYPE(frame)
-	);
-*/
+
 	if (stream->receive_buffer.frame_id_counter == 0 && new_frame->frame_id != 1) {
 		// First frame must be ID 1, this to be able to filter out "old data" from lost streams
 		// which might be retained if we are offline for a bit. If this happens, we must also
 		// reset the whole stream and the sender must connect again and obtain a new ID.
 
-		RRR_DBG_2("udpstream id %u dropping frame ID %u as we expect first frame id 1\n",
+		RRR_DBG_3("udpstream id %u dropping frame ID %u as we expect first frame id 1\n",
 				new_frame->stream_id, new_frame->frame_id);
 
 		if (__rrr_udpstream_send_reset(data, src_addr, addr_len, new_frame->stream_id, 0) != 0) {
@@ -989,8 +1015,6 @@ static int __rrr_udpstream_handle_received_frame (
 			goto out;
 		}
 		if (new_frame->frame_id < node->frame_id) {
-//			VL_DEBUG_MSG_2("udpstream stream-id %u frame id %u insert in receive buffer before %u\n",
-//					stream->stream_id, new_frame->frame_id, node->frame_id);
 			RRR_LL_ITERATE_INSERT(&stream->receive_buffer, new_frame);
 			new_frame = NULL;
 			goto out;
@@ -1006,8 +1030,6 @@ static int __rrr_udpstream_handle_received_frame (
 	RRR_LL_ITERATE_END();
 
 	out_append:
-//		VL_DEBUG_MSG_2("udpstream stream-id %u frame id %u append to receive buffer\n",
-//				stream->stream_id, new_frame->frame_id);
 		RRR_LL_APPEND(&stream->receive_buffer, new_frame);
 		new_frame = NULL;
 
@@ -1037,6 +1059,20 @@ static int __rrr_udpstream_read_callback (
 
 	callback_data->receive_count++;
 
+	RRR_DBG_3("UDP-stream RX %u-%u CS: %" PRIu32 "/%" PRIu32 " S: %llu F/T: %u CH/ID/WS: %u\n",
+			rrr_be16toh(frame->stream_id),
+			rrr_be32toh(frame->frame_id),
+			rrr_be32toh(frame->header_crc32),
+			rrr_be32toh(frame->data_crc32),
+			(unsigned long long int) rrr_be16toh(frame->data_size) + sizeof(*frame) - 1,
+			frame->flags_and_type,
+			rrr_be32toh(frame->connect_handle)
+	);
+
+	if (RRR_DEBUGLEVEL_6) {
+		__rrr_udpstream_frame_packed_dump(frame);
+	}
+
 	if (read_session->rx_buf_wpos != (ssize_t) RRR_UDPSTREAM_FRAME_PACKED_TOTAL_SIZE(frame)) {
 		RRR_MSG_0("Size mismatch in __rrr_udpstream_read_callback, packet was invalid\n");
 		ret = RRR_SOCKET_SOFT_ERROR;
@@ -1046,7 +1082,10 @@ static int __rrr_udpstream_read_callback (
 	ssize_t data_size = RRR_UDPSTREAM_FRAME_PACKED_DATA_SIZE(frame);
 	if (data_size > 0) {
 		if (rrr_crc32cmp (frame->data, data_size, RRR_UDPSTREAM_FRAME_PACKED_DATA_CRC32(frame)) != 0) {
-			RRR_MSG_0("CRC32 mismatch for data in __rrr_udpstream_read_callback\n");
+			RRR_MSG_0("Data CRC32 mismatch for data in __rrr_udpstream_read_callback\n");
+			if (RRR_DEBUGLEVEL_2) {
+				__rrr_udpstream_frame_packed_dump(frame);
+			}
 			ret = RRR_SOCKET_SOFT_ERROR;
 			goto out;
 		}
@@ -1065,7 +1104,7 @@ static int __rrr_udpstream_read_callback (
 	}
 
 	out:
-	free(read_session->rx_buf_ptr);
+	rrr_free(read_session->rx_buf_ptr);
 	read_session->rx_buf_ptr = NULL;
 	return ret;
 }
@@ -1083,9 +1122,9 @@ struct ack_list {
 
 struct rrr_udpstream_process_receive_buffer_callback_data {
 	struct rrr_udpstream_stream *stream;
-	int (*validator_callback)(ssize_t *target_size, void *data, ssize_t data_size, void *arg);
+	int (*validator_callback)(RRR_UDPSTREAM_VALIDATOR_CALLBACK_ARGS);
 	void *callback_validator_arg;
-	int (*receive_callback)(void **joined_data, const struct rrr_udpstream_receive_data *receive_data, void *arg);
+	int (*receive_callback)(RRR_UDPSTREAM_FINAL_RECEIVE_CALLBACK_ARGS);
 	void *receive_callback_arg;
 
 	uint32_t accumulated_data_size;
@@ -1204,16 +1243,11 @@ int rrr_udpstream_default_allocator (
 static int __rrr_udpstream_process_receive_buffer (
 		struct rrr_udpstream *data,
 		struct rrr_udpstream_stream *stream,
-		int (*allocator_callback) (
-				uint32_t size,
-				int (*receive_callback)(void **joined_data, void *allocation_handle, void *udpstream_callback_arg),
-				void *udpstream_callback_arg,
-				void *arg
-		),
+		int (*allocator_callback) (RRR_UDPSTREAM_ALLOCATOR_CALLBACK_ARGS),
 		void *allocator_callback_arg,
-		int (*validator_callback)(ssize_t *target_size, void *data, ssize_t data_size, void *arg),
+		int (*validator_callback)(RRR_UDPSTREAM_VALIDATOR_CALLBACK_ARGS),
 		void *validator_callback_arg,
-		int (*final_callback)(void **joined_data, const struct rrr_udpstream_receive_data *receive_data, void *arg),
+		int (*final_callback)(RRR_UDPSTREAM_FINAL_RECEIVE_CALLBACK_ARGS),
 		void *final_callback_arg
 ) {
 	int ret = 0;
@@ -1294,7 +1328,7 @@ static int __rrr_udpstream_process_receive_buffer (
 
 		goto no_add_ack;
 		add_ack:
-			ack_node = malloc(sizeof(*ack_node));
+			ack_node = rrr_allocate(sizeof(*ack_node));
 			if (ack_node == NULL) {
 				RRR_MSG_0("Could not allocate ACK node in __rrr_udpstream_process_receive_buffer\n");
 				ret = 1;
@@ -1316,6 +1350,8 @@ static int __rrr_udpstream_process_receive_buffer (
 		const struct sockaddr *use_addr = stream->remote_addr;
 		socklen_t use_sockaddr_len = stream->remote_addr_len;
 
+		// XXX : Do we need this NULL check? All frames should have a remote address set
+
 		if (use_addr == NULL) {
 			use_addr = node->last_frame->source_addr;
 			use_sockaddr_len = node->last_frame->source_addr_len;
@@ -1353,7 +1389,7 @@ static int __rrr_udpstream_process_receive_buffer (
 	RRR_LL_ITERATE_BEGIN(&stream->receive_buffer, struct rrr_udpstream_frame);
 		if (node->frame_id > stream->receive_buffer.frame_id_prev_boundary_pos) {
 			if (node->frame_id - stream->receive_buffer.frame_id_prev_boundary_pos > 1 + accumulated_frame_count) {
-				RRR_DBG_2("UDP-stream stream-id %u frame-id %u hole in the buffer detected, cannot deliver frames yet\n",
+				RRR_DBG_3("UDP-stream stream-id %u frame-id %u hole in the buffer detected, cannot deliver frames yet\n",
 						stream->stream_id, node->frame_id);
 				// Hole in the buffer
 				RRR_LL_ITERATE_BREAK();
@@ -1363,7 +1399,7 @@ static int __rrr_udpstream_process_receive_buffer (
 			if (accumulated_data_size != 0) {
 				RRR_BUG("Data accumulation started with already delivered frames in __rrr_udpstream_process_receive_buffer\n");
 			}
-			RRR_DBG_2("udpstream stream-id %u frame-id %u set destroy as already delivered\n",
+			RRR_DBG_3("udpstream stream-id %u frame-id %u set destroy as already delivered\n",
 					stream->stream_id, node->frame_id);
 			// Already delivered
 			RRR_LL_ITERATE_SET_DESTROY();
@@ -1410,6 +1446,8 @@ static int __rrr_udpstream_process_receive_buffer (
 
 	if ((ret = allocator_callback (
 			accumulated_data_size,
+			stream->remote_addr,
+			stream->remote_addr_len,
 			__rrr_udpstream_process_receive_buffer_callback,
 			&callback_data,
 			allocator_callback_arg
@@ -1421,7 +1459,7 @@ static int __rrr_udpstream_process_receive_buffer (
 	goto deliver_again;
 
 	out:
-	RRR_LL_DESTROY(&ack_list, struct ack_list_node, free(node));
+	RRR_LL_DESTROY(&ack_list, struct ack_list_node, rrr_free(node));
 	return ret;
 }
 
@@ -1429,16 +1467,11 @@ static int __rrr_udpstream_process_receive_buffer (
 // memory in receive_data->data or free it, also upon errors
 int rrr_udpstream_do_process_receive_buffers (
 		struct rrr_udpstream *data,
-		int (*allocator_callback) (
-				uint32_t size,
-				int (*receive_callback)(void **joined_data, void *allocation_handle, void *udpstream_callback_arg),
-				void *udpstream_callback_arg,
-				void *arg
-		),
+		int (*allocator_callback)(RRR_UDPSTREAM_ALLOCATOR_CALLBACK_ARGS),
 		void *allocator_callback_arg,
-		int (*validator_callback)(ssize_t *target_size, void *data, ssize_t data_size, void *arg),
+		int (*validator_callback)(RRR_UDPSTREAM_VALIDATOR_CALLBACK_ARGS),
 		void *validator_callback_arg,
-		int (*receive_callback)(void **joined_data, const struct rrr_udpstream_receive_data *receive_data, void *arg),
+		int (*receive_callback)(RRR_UDPSTREAM_FINAL_RECEIVE_CALLBACK_ARGS),
 		void *receive_callback_arg
 ) {
 	int ret = 0;
@@ -1477,15 +1510,15 @@ static int __rrr_udpstream_maintain (
 		uint64_t diff = time_now - node->last_seen;
 
 		if (diff > RRR_UDPSTREAM_CONNECTION_TIMEOUT_MS * 1000 && node->invalidated == 0) {
-			RRR_DBG_2("UDP-stream connection with id %u timed out, invalidating\n", node->stream_id);
+			RRR_DBG_3("UDP-stream connection with id %u timed out, invalidating\n", node->stream_id);
 			__rrr_udpstream_stream_invalidate(node);
 		}
 		else if (diff > RRR_UDPSTREAM_CONNECTION_INVALID_TIMEOUT_MS * 1000) {
-			RRR_DBG_2("UDP-stream connection with id %u timed out after being invalid\n", node->stream_id);
+			RRR_DBG_3("UDP-stream connection with id %u timed out after being invalid\n", node->stream_id);
 			RRR_LL_ITERATE_SET_DESTROY();
 		}
 		else if (node->hard_reset_received != 0) {
-			RRR_DBG_2("UDP-stream connection with id %u removing after hard reset\n", node->stream_id);
+			RRR_DBG_3("UDP-stream connection with id %u removing after hard reset\n", node->stream_id);
 			RRR_LL_ITERATE_SET_DESTROY();
 		}
 	RRR_LL_ITERATE_END_CHECK_DESTROY(&data->streams, __rrr_udpstream_stream_destroy(node));
@@ -1495,7 +1528,7 @@ static int __rrr_udpstream_maintain (
 
 // Do all reading and store messages into the buffer. Control messages are
 // not buffered, but delivered directly to the application layer through
-// the receive_callback function.
+// the callback function.
 int rrr_udpstream_do_read_tasks (
 		struct rrr_udpstream *data,
 		int (*control_frame_listener)(uint32_t connect_handle, uint64_t application_data, void *arg),
@@ -1524,10 +1557,12 @@ int rrr_udpstream_do_read_tasks (
 				&bytes_read,
 				&data->read_sessions,
 				data->ip.fd,
-				1024,
-				1024,
+				RRR_UDPSTREAM_FRAME_DATA_SIZE_LIMIT + sizeof(struct rrr_udpstream_frame_packed) - 1,
+				RRR_UDPSTREAM_FRAME_DATA_SIZE_LIMIT + sizeof(struct rrr_udpstream_frame_packed) - 1,
 				0, // No maximum size
 				RRR_SOCKET_READ_METHOD_RECVFROM,
+				0, // No ratelimit interval
+				0, // No ratelimit max bytes	
 				__rrr_udpstream_read_get_target_size,
 				data,
 				__rrr_udpstream_read_callback,
@@ -1639,9 +1674,6 @@ static int __rrr_udpstream_send_loop (
 		}
 	RRR_LL_ITERATE_END();
 
-//	VL_DEBUG_MSG_2("udpstream stream-id %u missing ACK count %i window size %i\n",
-//			stream->stream_id, missing_ack_count, stream->window_size);
-
 	*sent_count_return = sent_count;
 
 	out:
@@ -1719,18 +1751,20 @@ int rrr_udpstream_connection_check (
 
 	struct rrr_udpstream_stream *stream = __rrr_udpstream_find_stream_by_connect_handle(data, connect_handle);
 	if (stream == NULL) {
-		ret = RRR_UDPSTREAM_UNKNOWN_CONNECT_ID;
+		ret = RRR_UDPSTREAM_SOFT_ERR;
 		goto out;
 	}
 
 	if (stream->stream_id == 0) {
-//		VL_DEBUG_MSG_2("Check connection stream id %u connect handle %u: Not ready\n",
-//			stream->stream_id, stream->connect_handle);
+		RRR_DBG_3("UDP-stream %u not ready yet\n", stream->stream_id);
 		ret = RRR_UDPSTREAM_NOT_READY;
+		goto out;
 	}
 
 	if (stream->send_buffer.frame_id_counter >= stream->send_buffer.frame_id_max) {
-		ret = RRR_UDPSTREAM_RESET;
+		RRR_DBG_3("UDP-stream %u IDs exhausted\n", stream->stream_id);
+		ret = RRR_UDPSTREAM_SOFT_ERR;
+		goto out;
 	}
 
 	out:
@@ -1753,7 +1787,7 @@ int rrr_udpstream_regulate_window_size (
 
 	struct rrr_udpstream_stream *stream = __rrr_udpstream_find_stream_by_connect_handle(udpstream_data, connect_handle);
 	if (stream == NULL) {
-		ret = RRR_UDPSTREAM_UNKNOWN_CONNECT_ID;
+		ret = RRR_UDPSTREAM_SOFT_ERR;
 		goto out;
 	}
 	if (stream->stream_id == 0) {
@@ -1782,7 +1816,7 @@ int rrr_udpstream_send_control_frame (
 
 	struct rrr_udpstream_stream *stream = __rrr_udpstream_find_stream_by_connect_handle(udpstream_data, connect_handle);
 	if (stream == NULL) {
-		ret = RRR_UDPSTREAM_UNKNOWN_CONNECT_ID;
+		ret = RRR_UDPSTREAM_SOFT_ERR;
 		goto out;
 	}
 
@@ -1822,7 +1856,7 @@ int rrr_udpstream_queue_outbound_data (
 
 	struct rrr_udpstream_stream *stream = __rrr_udpstream_find_stream_by_connect_handle(udpstream_data, connect_handle);
 	if (stream == NULL) {
-		ret = RRR_UDPSTREAM_UNKNOWN_CONNECT_ID;
+		ret = RRR_UDPSTREAM_SOFT_ERR;
 		goto out;
 	}
 
@@ -1832,22 +1866,16 @@ int rrr_udpstream_queue_outbound_data (
 	}
 
 	if (RRR_LL_COUNT(&stream->send_buffer) >= RRR_UDPSTREAM_BUFFER_LIMIT) {
-//		VL_DEBUG_MSG_3("Buffer is full with %i items for udpstream stream-id %u\n",
-//				RRR_LL_COUNT(&stream->send_buffer), stream->stream_id);
-		ret = RRR_UDPSTREAM_BUFFER_FULL;
+		ret = RRR_UDPSTREAM_NOT_READY;
 		goto out;
 	}
 
 	if (stream->send_buffer.frame_id_counter + ((data_size / RRR_UDPSTREAM_FRAME_DATA_SIZE_LIMIT) + 1) > stream->send_buffer.frame_id_max) {
-		RRR_DBG_2("UDP-stream frame IDs exhausted for stream-id %u\n", stream->stream_id);
-		ret = RRR_UDPSTREAM_IDS_EXHAUSTED;
+		RRR_DBG_3("UDP-stream frame IDs exhausted for stream-id %u\n", stream->stream_id);
+		ret = RRR_UDPSTREAM_NOT_READY;
 		goto out;
 	}
 
-//	if (stream->send_buffer.frame_id_counter > RRR_UDPSTREAM_FRAME_ID_MAX) {
-//		VL_BUG("Frame IDs exhausted in __rrr_udpstream_send_loop\n");
-//	}
-
 	const void *pos = data;
 	struct rrr_udpstream_frame *new_frame = NULL;
 	while (data_size > 0) {
@@ -1855,7 +1883,7 @@ int rrr_udpstream_queue_outbound_data (
 		new_frame = NULL;
 		if ((ret = __rrr_udpstream_frame_new_from_data(&new_frame, pos, chunk_size)) != 0) {
 			RRR_MSG_0("Could not allocate frame in rrr_udpstream_queue_outbound_data\n");
-			ret = RRR_UDPSTREAM_ERR;
+			ret = RRR_UDPSTREAM_HARD_ERR;
 			goto out;
 		}
 
@@ -1887,26 +1915,79 @@ void rrr_udpstream_close (
 	pthread_mutex_unlock(&data->lock);
 }
 
-int rrr_udpstream_bind (
+static int __rrr_udpstream_bind (
+		struct rrr_ip_data *ip_data,
+		unsigned int local_port,
+		int do_ipv6
+) {
+
+	ip_data->port = local_port;
+
+	if (rrr_ip_network_start_udp (ip_data, do_ipv6) != 0) {
+		return 1;
+	}
+
+	return 0;
+}
+
+int rrr_udpstream_bind_v6_priority (
 		struct rrr_udpstream *data,
 		unsigned int local_port
 ) {
 	int ret = 0;
 
+	if (data->ip.fd != 0) {
+		RRR_BUG("rrr_udpstream_bind called with non-zero fd, bind already complete\n");
+	}
+
 	pthread_mutex_lock(&data->lock);
 
+	int ret_4 = 0, ret_6 = 0;
+
+	ret_6 = __rrr_udpstream_bind(&data->ip, local_port, 1);
+	if (ret_6 != 0) {
+		data->ip.fd = 0;
+		ret_4 = __rrr_udpstream_bind(&data->ip, local_port, 0);
+	}
+
+	if (ret_4 != 0 && ret_6 != 0) {
+		RRR_MSG_0("Listening failed on both IPv4 and IPv6 in udpstream on port %u\n", local_port);
+		ret = 1;
+		goto out;
+	}
+
+	if (ret_6 == 0) {
+		RRR_DBG_1("udpstream bind on port %u IPv6 (possibly dual-stack)\n", local_port);
+	}
+	else {
+		RRR_DBG_1("udpstream bind on port %u IPv4\n", local_port);
+	}
+
+	out:
+	pthread_mutex_unlock(&data->lock);
+	return ret;
+}
+
+int rrr_udpstream_bind_v4_only (
+		struct rrr_udpstream *data,
+		unsigned int local_port
+) {
+	int ret = 0;
+
 	if (data->ip.fd != 0) {
 		RRR_BUG("rrr_udpstream_bind called with non-zero fd, bind already complete\n");
 	}
 
-	data->ip.port = local_port;
+	pthread_mutex_lock(&data->lock);
 
-	if (rrr_ip_network_start_udp_ipv4 (&data->ip) != 0) {
-		RRR_MSG_0("Could not start IP in rrr_udpstream_bind\n");
+	if (__rrr_udpstream_bind(&data->ip, local_port, 0) != 0) {
+		RRR_MSG_0("Listening failed on IPv4 in udpstream on port %u\n", local_port);
 		ret = 1;
 		goto out;
 	}
 
+	RRR_DBG_1("udpstream bind on port %u IPv4 only\n", local_port);
+
 	out:
 	pthread_mutex_unlock(&data->lock);
 	return ret;
@@ -1926,7 +2007,7 @@ int rrr_udpstream_connect_raw (
 		RRR_BUG("FD was 0 in rrr_udpstream_connect_raw, must bind first\n");
 	}
 
-	if ((ret = __rrr_udpstream_send_connect(connect_handle, data, addr, socklen)) != 0) {
+	if ((ret = __rrr_udpstream_send_reset_and_connect(connect_handle, data, addr, socklen)) != 0) {
 		RRR_MSG_0("Could not send connect packet in rrr_udpstream_connect_raw\n");
 		goto out;
 	}
@@ -1943,36 +2024,51 @@ int rrr_udpstream_connect (
 		const char *remote_port
 ) {
 	int ret = 0;
-	struct addrinfo *res = NULL;
 
 	if (data->ip.fd == 0) {
 		RRR_BUG("FD was 0 in rrr_udpstream_connect, must bind first\n");
 	}
 
 	struct addrinfo hints;
-	memset(&hints,0,sizeof(hints));
-	hints.ai_family = AF_INET;
+	struct addrinfo *result;
+
+	memset (&hints, '\0', sizeof(hints));
+	hints.ai_family = AF_UNSPEC;
 	hints.ai_socktype = SOCK_DGRAM;
 	hints.ai_protocol = IPPROTO_UDP;
 	hints.ai_flags = AI_ADDRCONFIG;
 
-	ret = getaddrinfo(remote_host, remote_port, &hints, &res);
-	if (ret != 0) {
-		RRR_MSG_0 ("Could not get address info of server %s port %s in rrr_udpstream_connect: %s\n",
-				remote_host, remote_port, gai_strerror(ret));
+	int s = getaddrinfo(remote_host, remote_port, &hints, &result);
+	if (s != 0) {
+		RRR_MSG_0("Failed to get address of '%s' in udpstream: %s\n", remote_host, gai_strerror(s));
 		ret = 1;
 		goto out;
 	}
 
-	if ((ret = rrr_udpstream_connect_raw(connect_handle, data, res->ai_addr, res->ai_addrlen)) != 0) {
-		RRR_MSG_0("Could not send connect packet in rrr_udpstream_connect\n");
-		goto out;
+	struct addrinfo *rp;
+	for (rp = result; rp != NULL; rp = rp->ai_next) {
+		if (RRR_DEBUGLEVEL_1) {
+			char buf[128];
+			rrr_ip_to_str(buf, sizeof(buf), result->ai_addr, result->ai_addrlen);
+			RRR_MSG_1("UDP-stream connection attempt to %s\n", buf);
+		}
+		if ((ret = rrr_udpstream_connect_raw(connect_handle, data, result->ai_addr, result->ai_addrlen)) != 0) {
+			RRR_DBG_1("UDP-stream failed to send connect packet, return was %i\n", ret);
+		}
+		else {
+			break;
+		}
 	}
 
-	out:
-	if (res != NULL) {
-		freeaddrinfo(res);
+	// Let last return value propagate
+
+	if (ret != 0) {
+		RRR_MSG_0("Could not send connect packet in udpstream, all address suggestions failed\n");
 	}
+
+	freeaddrinfo(result);
+
+	out:
 	pthread_mutex_unlock(&data->lock);
 	return ret;
 }
