@@ -2,7 +2,7 @@
 
 Read Route Record
 
-Copyright (C) 2019-2020 Atle Solbakken atle@goliathdns.no
+Copyright (C) 2019-2021 Atle Solbakken atle@goliathdns.no
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -23,19 +23,21 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <stdlib.h>
 #include <string.h>
 
-#include "log.h"
+#include "../log.h"
+#include "../allocator.h"
 #include "udpstream_asd.h"
-#include "buffer.h"
-#include "read.h"
-#include "ip/ip.h"
-#include "message_holder/message_holder.h"
-#include "message_holder/message_holder_struct.h"
-#include "socket/rrr_socket_constants.h"
-#include "messages/msg_checksum.h"
-#include "messages/msg_msg.h"
-#include "util/macro_utils.h"
-#include "util/rrr_time.h"
-#include "util/posix.h"
+#include "../buffer.h"
+#include "../read.h"
+#include "../ip/ip.h"
+#include "../message_holder/message_holder.h"
+#include "../message_holder/message_holder_struct.h"
+#include "../socket/rrr_socket_constants.h"
+#include "../messages/msg_checksum.h"
+#include "../messages/msg_msg.h"
+#include "../messages/msg.h"
+#include "../util/macro_utils.h"
+#include "../util/rrr_time.h"
+#include "../util/posix.h"
 
 static struct rrr_udpstream_asd_control_msg __rrr_udpstream_asd_control_msg_split (uint64_t application_data) {
 	struct rrr_udpstream_asd_control_msg result;
@@ -60,60 +62,9 @@ static int __rrr_udpstream_asd_queue_entry_destroy (
 	if (entry->message != NULL) {
 		rrr_msg_holder_decref(entry->message);
 	}
-	free(entry);
+	rrr_free(entry);
 	return 0;
 }
-/*
-static void __rrr_udpstream_asd_queue_remove_entry (
-		struct rrr_udpstream_asd_queue *queue,
-		uint32_t message_id
-) {
-	int iterations = 0;
-
-	if (RRR_LL_COUNT(queue) == 0) {
-		return;
-	}
-
-	if (message_id < RRR_LL_FIRST(queue)->message_id ||
-		message_id > RRR_LL_LAST(queue)->message_id
-	) {
-		return;
-	}
-
-	int64_t diff_to_last = RRR_LL_LAST(queue)->message_id - message_id;
-	int64_t diff_to_first = message_id - RRR_LL_FIRST(queue)->message_id;
-
-	RRR_LL_ITERATE_BEGIN_EITHER(queue, struct rrr_udpstream_asd_queue_entry, (diff_to_last < diff_to_first));
-		iterations++;
-//		printf ("cmp %" PRIu64 " vs %" PRIu64 "\n", boundary_id_combined, node->boundary_id_combined);
-		if (node->message_id == message_id) {
-			RRR_LL_ITERATE_SET_DESTROY();
-			RRR_LL_ITERATE_LAST();
-		}
-	RRR_LL_ITERATE_END_CHECK_DESTROY(queue, __rrr_udpstream_asd_queue_entry_destroy(node));
-
-//	printf ("iterations to remove: %i\n", iterations);
-}
-
-static struct message_holder *__rrr_udpstream_asd_queue_remove_entry_and_get_data (
-		struct rrr_udpstream_asd_queue *queue,
-		uint32_t message_id
-) {
-	struct message_holder *ret = NULL;
-
-	RRR_LL_ITERATE_BEGIN(queue, struct rrr_udpstream_asd_queue_entry);
-//		VL_DEBUG_MSG("cmp boundary %" PRIu64 " vs %" PRIu64 "\n", boundary_id_combined, node->boundary_id_combined);
-		if (node->message_id == message_id) {
-			ret = node->message;
-			node->message = NULL;
-			RRR_LL_ITERATE_SET_DESTROY();
-			RRR_LL_ITERATE_LAST();
-		}
-	RRR_LL_ITERATE_END_CHECK_DESTROY(queue, __rrr_udpstream_asd_queue_entry_destroy(node));
-
-	return ret;
-}
-*/
 
 static struct rrr_udpstream_asd_queue_entry *__rrr_udpstream_asd_queue_find_entry (
 		struct rrr_udpstream_asd_queue_new *queue,
@@ -170,7 +121,6 @@ int __rrr_udpstream_asd_queue_collection_iterate (
 	return ret;
 }
 
-// TODO : Create cache for entry count
 int __rrr_udpstream_asd_queue_collection_count_entries (
 		struct rrr_udpstream_asd_queue_collection *collection
 ) {
@@ -189,8 +139,6 @@ static void __rrr_udpstream_asd_queue_insert_ordered (
 		struct rrr_udpstream_asd_queue_entry *entry
 ) {
 	if (RRR_LL_LAST(queue) == NULL || RRR_LL_LAST(queue)->message_id < entry->message_id) {
-//		VL_DEBUG_MSG("queue append entry with ip buf entry %p boundary %" PRIu64 "\n",
-//				entry->entry, entry->boundary_id);
 		RRR_LL_APPEND(queue, entry);
 		return;
 	}
@@ -198,8 +146,6 @@ static void __rrr_udpstream_asd_queue_insert_ordered (
 	RRR_LL_ITERATE_BEGIN(queue, struct rrr_udpstream_asd_queue_entry);
 		if (entry->message_id < node->message_id) {
 			RRR_LL_ITERATE_INSERT(queue, entry);
-//			VL_DEBUG_MSG("queue insert entry with ip buf entry %p boundary %" PRIu64 " before %" PRIu64 "\n",
-//					entry->entry, entry->boundary_id, node->boundary_id);
 			entry = NULL;
 			RRR_LL_ITERATE_BREAK();
 		}
@@ -213,7 +159,6 @@ static void __rrr_udpstream_asd_queue_insert_ordered (
 	}
 }
 
-// message pointer set to NULL if memory gets new owner
 static int __rrr_udpstream_asd_queue_incref_and_insert_entry (
 		struct rrr_udpstream_asd_queue_new *queue,
 		struct rrr_msg_holder *ip_entry,
@@ -226,7 +171,7 @@ static int __rrr_udpstream_asd_queue_incref_and_insert_entry (
 		goto out;
 	}
 
-	if ((new_entry = malloc(sizeof(*new_entry))) == NULL) {
+	if ((new_entry = rrr_allocate(sizeof(*new_entry))) == NULL) {
 		RRR_MSG_0("Could not allocate memory in __rrr_udpstream_asd_queue_insert_entry_or_free\n");
 		ret = 1;
 		goto out;
@@ -251,7 +196,7 @@ static int __rrr_udpstream_asd_queue_incref_and_insert_entry (
 static int __rrr_udpstream_asd_queue_new (struct rrr_udpstream_asd_queue_new **target, uint32_t connect_handle) {
 	*target = NULL;
 
-	struct rrr_udpstream_asd_queue_new *queue = malloc(sizeof(*queue));
+	struct rrr_udpstream_asd_queue_new *queue = rrr_allocate(sizeof(*queue));
 
 	if (queue == NULL) {
 		RRR_MSG_0("Could not allocate memory in __rrr_udpstream_asd_queue_new\n");
@@ -267,7 +212,6 @@ static int __rrr_udpstream_asd_queue_new (struct rrr_udpstream_asd_queue_new **t
 	return 0;
 }
 
-// message pointer set to NULL if memory gets new owner
 static int __rrr_udpstream_asd_queue_collection_incref_and_insert_entry (
 		struct rrr_udpstream_asd_queue_collection *collection,
 		struct rrr_msg_holder *entry,
@@ -306,7 +250,7 @@ void __rrr_udpstream_asd_queue_clear(struct rrr_udpstream_asd_queue_new *queue)
 
 void __rrr_udpstream_asd_queue_destroy(struct rrr_udpstream_asd_queue_new *queue) {
 	__rrr_udpstream_asd_queue_clear(queue);
-	free(queue);
+	rrr_free(queue);
 }
 
 void __rrr_udpstream_asd_queue_collection_clear(struct rrr_udpstream_asd_queue_collection *collection) {
@@ -314,7 +258,7 @@ void __rrr_udpstream_asd_queue_collection_clear(struct rrr_udpstream_asd_queue_c
 }
 
 void __rrr_udpstream_asd_control_queue_clear(struct rrr_udpstream_asd_control_queue *queue) {
-	RRR_LL_DESTROY(queue, struct rrr_udpstream_asd_control_queue_entry, free(node));
+	RRR_LL_DESTROY(queue, struct rrr_udpstream_asd_control_queue_entry, rrr_free(node));
 }
 
 void rrr_udpstream_asd_destroy (
@@ -330,7 +274,7 @@ void rrr_udpstream_asd_destroy (
 	rrr_udpstream_clear(&session->udpstream);
 	RRR_FREE_IF_NOT_NULL(session->remote_host);
 	RRR_FREE_IF_NOT_NULL(session->remote_port);
-	free(session);
+	rrr_free(session);
 }
 
 int rrr_udpstream_asd_new (
@@ -340,13 +284,14 @@ int rrr_udpstream_asd_new (
 		const char *remote_port,
 		uint32_t client_id,
 		int accept_connections,
-		int disallow_ip_swap
+		int disallow_ip_swap,
+		int v4_only
 ) {
 	int ret = 0;
 
 	*target = NULL;
 
-	struct rrr_udpstream_asd *session = malloc(sizeof(*session));
+	struct rrr_udpstream_asd *session = rrr_allocate(sizeof(*session));
 	if (session == NULL) {
 		RRR_MSG_0("Could not allocate memory in rrr_udpstream_asd_new\n");
 		ret = 1;
@@ -355,7 +300,7 @@ int rrr_udpstream_asd_new (
 	memset(session, '\0', sizeof(*session));
 
 	if (remote_host != NULL && *remote_host != '\0') {
-		if ((session->remote_host = strdup(remote_host)) == NULL) {
+		if ((session->remote_host = rrr_strdup(remote_host)) == NULL) {
 			RRR_MSG_0("Could not allocate remote host string in rrr_udpstream_asd_new\n");
 			ret = 1;
 			goto out_free;
@@ -363,7 +308,7 @@ int rrr_udpstream_asd_new (
 	}
 
 	if (remote_port != NULL && *remote_port != '\0') {
-		if ((session->remote_port = strdup(remote_port)) == NULL) {
+		if ((session->remote_port = rrr_strdup(remote_port)) == NULL) {
 			RRR_MSG_0("Could not allocate remote port string in rrr_udpstream_asd_new\n");
 			ret = 1;
 			goto out_free_remote_host;
@@ -383,9 +328,17 @@ int rrr_udpstream_asd_new (
 		goto out_free_remote_port;
 	}
 
-	if ((ret = rrr_udpstream_bind(&session->udpstream, local_port)) != 0) {
-		RRR_MSG_0("Could not bind to local port %u in rrr_udpstream_asd_new\n", local_port);
-		goto out_clear_udpstream;
+	if (v4_only) {
+		if ((ret = rrr_udpstream_bind_v4_only(&session->udpstream, local_port)) != 0) {
+			RRR_MSG_0("Could not bind to local port %u with IPv4 only in rrr_udpstream_asd_new\n", local_port);
+			goto out_clear_udpstream;
+		}
+	}
+	else {
+		if ((ret = rrr_udpstream_bind_v6_priority(&session->udpstream, local_port)) != 0) {
+			RRR_MSG_0("Could not bind to local port %u with IPv6 priority in rrr_udpstream_asd_new\n", local_port);
+			goto out_clear_udpstream;
+		}
 	}
 
 	if (rrr_posix_mutex_init(&session->message_id_lock, 0) != 0) {
@@ -421,15 +374,59 @@ int rrr_udpstream_asd_new (
 	out_clear_udpstream:
 		rrr_udpstream_clear(&session->udpstream);
 	out_free_remote_port:
-		free(session->remote_port);
+		rrr_free(session->remote_port);
 	out_free_remote_host:
-		free(session->remote_host);
+		rrr_free(session->remote_host);
 	out_free:
-		free(session);
+		rrr_free(session);
 	out:
 		return ret;
 }
 
+static int __rrr_udpstream_asd_queue_control_frame (
+		struct rrr_udpstream_asd *session,
+		uint32_t connect_handle,
+		uint32_t message_id,
+		uint32_t ack_flags
+) {
+	int ret = 0;
+
+	struct rrr_udpstream_asd_control_queue_entry *entry = rrr_allocate(sizeof(*entry));
+	if (entry == NULL) {
+		RRR_MSG_0("Could not allocate memory in __rrr_udpstream_asd_queue_control_frame\n");
+		ret = 1;
+		goto out;
+	}
+
+	entry->connect_handle = connect_handle;
+	entry->ack_flags = ack_flags;
+	entry->message_id = message_id;
+
+	RRR_LL_APPEND(&session->control_send_queue, entry);
+
+	out:
+	return ret;
+}
+
+int __rrr_udpstream_asd_send_control_message (
+		struct rrr_udpstream_asd *session,
+		uint32_t flags,
+		uint32_t connect_handle,
+		uint32_t message_id
+) {
+	struct rrr_udpstream_asd_control_msg control_msg = {
+			flags,
+			message_id
+	};
+
+	uint64_t application_data = __rrr_udpstream_asd_control_msg_join(control_msg);
+
+	RRR_DBG_3("ASD TX %" PRIu32 ":%" PRIu32 " CTRL flags %" PRIu32 "\n",
+			connect_handle, message_id, flags);
+
+	return rrr_udpstream_send_control_frame(&session->udpstream, connect_handle, application_data);
+}
+
 static int __rrr_udpstream_asd_buffer_connect_if_needed (
 		struct rrr_udpstream_asd *session
 ) {
@@ -440,7 +437,15 @@ static int __rrr_udpstream_asd_buffer_connect_if_needed (
 	int udpstream_ret = rrr_udpstream_connection_check(&session->udpstream, session->connect_handle);
 	if (udpstream_ret == 0) {
 		session->connection_attempt_time = 0;
-		session->is_connected = 1;
+		if (session->is_connected == 0) {
+			RRR_DBG_3("ASD %" PRIu32 " ready\n", session->connect_handle);
+			RRR_DBG_3("ASD TX %" PRIu32 " RST\n", session->connect_handle);
+			if ((ret = __rrr_udpstream_asd_send_control_message(session, RRR_UDPSTREAM_ASD_ACK_FLAGS_RST, session->connect_handle, 0)) != 0) {
+				RRR_MSG_0("Could not queue reset frame in __rrr_udpstream_asd_buffer_connect_if_needed\n");
+				goto out;
+			}
+			session->is_connected = 1;
+		}
 		goto out;
 	}
 	else if (udpstream_ret == RRR_UDPSTREAM_NOT_READY) {
@@ -463,6 +468,10 @@ static int __rrr_udpstream_asd_buffer_connect_if_needed (
 		}
 
 		uint32_t connect_handle = session->connect_handle;
+
+		RRR_DBG_3("ASD %u CONNECT send queue count %i\n",
+				session->connect_handle, RRR_LL_COUNT(&session->send_queue));
+
 		if ((ret = rrr_udpstream_connect (
 				&connect_handle,
 				&session->udpstream,
@@ -483,33 +492,22 @@ static int __rrr_udpstream_asd_buffer_connect_if_needed (
 	}
 
 	out:
+	if (ret == 0 && !session->is_connected) {
+		ret = RRR_UDPSTREAM_ASD_NOT_READY;
+	}
 	pthread_mutex_unlock(&session->connect_lock);
 	return ret;
 }
 
-static int __rrr_udpstream_asd_queue_control_frame (
+void __rrr_udpstream_asd_release_queue_clear_by_handle (
 		struct rrr_udpstream_asd *session,
-		uint32_t connect_handle,
-		uint32_t message_id,
-		uint32_t ack_flags
+		uint32_t connect_handle
 ) {
-	int ret = 0;
-
-	struct rrr_udpstream_asd_control_queue_entry *entry = malloc(sizeof(*entry));
-	if (entry == NULL) {
-		RRR_MSG_0("Could not allocate memory in __rrr_udpstream_asd_queue_control_frame\n");
-		ret = 1;
-		goto out;
-	}
-
-	entry->connect_handle = connect_handle;
-	entry->ack_flags = ack_flags;
-	entry->message_id = message_id;
-
-	RRR_LL_APPEND(&session->control_send_queue, entry);
-
-	out:
-	return ret;
+	RRR_LL_ITERATE_BEGIN(&session->release_queues, struct rrr_udpstream_asd_queue_new);
+		if (node->source_connect_handle == connect_handle) {
+			__rrr_udpstream_asd_queue_clear(node);
+		}
+	RRR_LL_ITERATE_END();
 }
 
 static int __rrr_udpstream_asd_control_frame_listener (
@@ -530,8 +528,8 @@ static int __rrr_udpstream_asd_control_frame_listener (
 	uint32_t reply_ack_flags = 0;
 
 	if (control_msg.flags == RRR_UDPSTREAM_ASD_ACK_FLAGS_DACK) {
-		RRR_DBG_3("ASD RX DACK %" PRIu32 "\n",
-				control_msg.message_id);
+		RRR_DBG_3("ASD RX %" PRIu32 ":%" PRIu32 " DACK\n",
+				session->connect_handle, control_msg.message_id);
 
 		node = __rrr_udpstream_asd_queue_find_entry(
 				&session->send_queue,
@@ -541,30 +539,34 @@ static int __rrr_udpstream_asd_control_frame_listener (
 			node->ack_status_flags |= RRR_UDPSTREAM_ASD_ACK_FLAGS_DACK;
 		}
 
-		RRR_DBG_3("ASD TX RACK %" PRIu32 "\n",
-				control_msg.message_id);
+		RRR_DBG_3("ASD TX %" PRIu32 ":%" PRIu32 " RACK\n",
+				session->connect_handle, control_msg.message_id);
 		reply_ack_flags = RRR_UDPSTREAM_ASD_ACK_FLAGS_RACK;
 	}
 	else if (control_msg.flags == RRR_UDPSTREAM_ASD_ACK_FLAGS_RACK) {
-		RRR_DBG_3("ASD RX RACK %" PRIu32 "\n",
-				control_msg.message_id);
-
 		node = __rrr_udpstream_asd_queue_collection_find_entry(
 				&session->release_queues,
 				connect_handle,
 				control_msg.message_id
 		);
+
 		if (node != NULL) {
+			RRR_DBG_3("ASD RX %" PRIu32 ":%" PRIu32 " RACK\n",
+					session->connect_handle, control_msg.message_id);
 			node->ack_status_flags |= RRR_UDPSTREAM_ASD_ACK_FLAGS_RACK;
 		}
+		else {
+			RRR_DBG_3("ASD RX %" PRIu32 ":%" PRIu32 " RACK (unknown)\n",
+					session->connect_handle, control_msg.message_id);
+		}
 
-		RRR_DBG_3("ASD TX CACK %" PRIu32 "\n",
-				control_msg.message_id);
+		RRR_DBG_3("ASD TX %" PRIu32 ":%" PRIu32 " CACK\n",
+				session->connect_handle, control_msg.message_id);
 		reply_ack_flags = RRR_UDPSTREAM_ASD_ACK_FLAGS_CACK;
 	}
 	else if (control_msg.flags == RRR_UDPSTREAM_ASD_ACK_FLAGS_CACK) {
-		RRR_DBG_3("ASD RX CACK %" PRIu32 "\n",
-				control_msg.message_id);
+		RRR_DBG_3("ASD RX %" PRIu32 ":%" PRIu32 " CACK\n",
+				session->connect_handle, control_msg.message_id);
 
 		node = __rrr_udpstream_asd_queue_find_entry (
 				&session->send_queue,
@@ -574,9 +576,14 @@ static int __rrr_udpstream_asd_control_frame_listener (
 			node->ack_status_flags |= RRR_UDPSTREAM_ASD_ACK_FLAGS_CACK;
 		}
 	}
+	else if (control_msg.flags == RRR_UDPSTREAM_ASD_ACK_FLAGS_RST) {
+		RRR_DBG_3("ASD RX %" PRIu32 " RST\n",
+				connect_handle);
+		__rrr_udpstream_asd_release_queue_clear_by_handle(session, connect_handle);
+	}
 	else {
-		RRR_DBG_1("UDP-stream ASD received control frame with unknown ACK flags %u for stream-id %u\n",
-				control_msg.flags, control_msg.message_id);
+		RRR_DBG_3("ASD RX %" PRIu32 " ACK (unknown flags %" PRIu32 ")\n",
+				control_msg.message_id, control_msg.flags);
 	}
 
 	// We cannot reply with ACK messages immediately as we already are in locked UDP-stream
@@ -596,8 +603,6 @@ static int __rrr_udpstream_asd_control_frame_listener (
 	return ret;
 }
 
-// Queue a message for transmission.
-// ip_message is set to NULL if memory is managed by new buffer
 int rrr_udpstream_asd_queue_and_incref_message (
 		struct rrr_udpstream_asd *session,
 		struct rrr_msg_holder *ip_message
@@ -612,13 +617,8 @@ int rrr_udpstream_asd_queue_and_incref_message (
 	pthread_mutex_lock(&session->queue_lock);
 	pthread_mutex_lock(&session->message_id_lock);
 
-	RRR_DBG_3("ASD %u QUEUE\n", session->connect_handle);
-
-//	struct rrr_msg_msg *message = (*ip_message)->message;
-//	printf ("type and class: %u\n", message->type_and_class);
-
 	if (RRR_LL_COUNT(&session->send_queue) >= RRR_UDPSTREAM_ASD_BUFFER_MAX) {
-		ret = RRR_UDPSTREAM_ASD_BUFFER_FULL;
+		ret = RRR_UDPSTREAM_ASD_NOT_READY;
 		goto out;
 	}
 
@@ -627,6 +627,9 @@ int rrr_udpstream_asd_queue_and_incref_message (
 		id = ++(session->message_id_pos);
 	}
 
+	RRR_DBG_3("ASD Q %" PRIu32 ":%" PRIu32 "\n",
+		session->connect_handle, session->message_id_pos);
+
 	// Not very likely
 	if (__rrr_udpstream_asd_queue_find_entry(&session->send_queue, id) != NULL) {
 		RRR_BUG("IDs exhausted in rrr_udpstream_asd_queue_message\n");
@@ -671,18 +674,18 @@ int __rrr_udpstream_asd_send_message (
 			message_network_size,
 			node->message_id
 	)) != 0) {
-		if (ret == RRR_UDPSTREAM_BUFFER_FULL) {
-			ret = RRR_UDPSTREAM_ASD_BUFFER_FULL;
+		if (ret == RRR_UDPSTREAM_NOT_READY) {
+			ret = RRR_UDPSTREAM_ASD_NOT_READY;
 			goto out;
 		}
-		else if (ret == RRR_UDPSTREAM_NOT_READY || ret == RRR_UDPSTREAM_IDS_EXHAUSTED || ret == RRR_UDPSTREAM_UNKNOWN_CONNECT_ID) {
+		else if (ret == RRR_UDPSTREAM_NOT_READY || ret == RRR_UDPSTREAM_SOFT_ERR) {
 			ret = RRR_UDPSTREAM_ASD_NOT_READY;
 			goto out;
 		}
 		else {
 			RRR_MSG_0("Error while queuing message for sending in UDP-stream ASD handle %u\n",
 					session->connect_handle);
-			ret = RRR_UDPSTREAM_ASD_ERR;
+			ret = RRR_UDPSTREAM_ASD_HARD_ERR;
 			goto out;
 		}
 	}
@@ -692,22 +695,6 @@ int __rrr_udpstream_asd_send_message (
 	return ret;
 }
 
-int __rrr_udpstream_asd_send_control_message (
-		struct rrr_udpstream_asd *session,
-		uint32_t flags,
-		uint32_t connect_handle,
-		uint32_t message_id
-) {
-	struct rrr_udpstream_asd_control_msg control_msg = {
-			flags,
-			message_id
-	};
-
-	uint64_t application_data = __rrr_udpstream_asd_control_msg_join(control_msg);
-
-	return rrr_udpstream_send_control_frame(&session->udpstream, connect_handle, application_data);
-}
-
 static int __rrr_udpstream_asd_do_release_queue_send_tasks (
 		struct rrr_udpstream_asd_queue_new *queue,
 		void *private_arg
@@ -718,16 +705,14 @@ static int __rrr_udpstream_asd_do_release_queue_send_tasks (
 	int ret = 0;
 
 	RRR_LL_ITERATE_BEGIN(queue, struct rrr_udpstream_asd_queue_entry);
-		if ((node->ack_status_flags & RRR_UDPSTREAM_ASD_ACK_FLAGS_CACK) != 0) {
-			RRR_LL_ITERATE_SET_DESTROY();
-		}
-		else if (node->send_time == 0 || time_now - node->send_time > RRR_UDPSTREAM_ASD_RESEND_INTERVAL_MS * 1000) {
+		if (node->send_time == 0 || time_now - node->send_time > RRR_UDPSTREAM_ASD_RESEND_INTERVAL_MS * 1000) {
 			// Always update send time to prevent hardcore looping upon error conditions
 			node->send_time = time_now;
 
 			if ((node->ack_status_flags & RRR_UDPSTREAM_ASD_ACK_FLAGS_DACK) == 0 || (node->ack_status_flags & RRR_UDPSTREAM_ASD_ACK_FLAGS_RACK) == 0) {
 				// We have not sent delivery ACK or need to re-send it
-				RRR_DBG_3("ASD TX DACK %u\n", node->message_id);
+				RRR_DBG_3("ASD TX %" PRIu32 ":%" PRIu32 " DACK DUP\n",
+						session->connect_handle, node->message_id);
 				ret = __rrr_udpstream_asd_send_control_message (
 						session,
 						RRR_UDPSTREAM_ASD_ACK_FLAGS_DACK,
@@ -735,10 +720,11 @@ static int __rrr_udpstream_asd_do_release_queue_send_tasks (
 						node->message_id
 				);
 				node->ack_status_flags |= RRR_UDPSTREAM_ASD_ACK_FLAGS_DACK;
+				node->send_count++;
 			}
 
 			if (ret != 0) {
-				RRR_DBG_1("Error while sending message B in rrr_udpstream_asd_do_send_tasks return was %i\n", ret);
+				RRR_DBG_3("Error while sending message B in rrr_udpstream_asd_do_send_tasks return was %i\n", ret);
 				goto out;
 			}
 		}
@@ -763,16 +749,16 @@ static int __rrr_udpstream_asd_do_send_tasks (struct rrr_udpstream_asd *session)
 				node->connect_handle,
 				node->message_id
 		);
-		if (ret == RRR_UDPSTREAM_NOT_READY || ret == RRR_UDPSTREAM_UNKNOWN_CONNECT_ID) {
-			RRR_DBG_1("ASD control message deleted after failed send attempt (connect handle %u)\n", node->connect_handle);
+		if (ret == RRR_UDPSTREAM_NOT_READY || ret == RRR_UDPSTREAM_SOFT_ERR) {
+			RRR_DBG_3("ASD control message deleted after failed send attempt (connect handle %u)\n", node->connect_handle);
 		}
 		else if (ret != 0) {
-			RRR_DBG_1("Could not send ASD control message in rrr_udpstream_asd_do_send_tasks return was %i\n", ret);
+			RRR_DBG_3("Could not send ASD control message in rrr_udpstream_asd_do_send_tasks return was %i\n", ret);
 			ret = 1;
 			goto out;
 		}
 		RRR_LL_ITERATE_SET_DESTROY();
-	RRR_LL_ITERATE_END_CHECK_DESTROY(&session->control_send_queue, 0; free(node));
+	RRR_LL_ITERATE_END_CHECK_DESTROY(&session->control_send_queue, 0; rrr_free(node));
 
 	int buffer_was_full = 0;
 
@@ -784,22 +770,29 @@ static int __rrr_udpstream_asd_do_send_tasks (struct rrr_udpstream_asd *session)
 		else if (node->send_time == 0 || time_now - node->send_time > RRR_UDPSTREAM_ASD_RESEND_INTERVAL_MS * 1000) {
 			// Always update send time to prevent hardcore looping upon error conditions
 			node->send_time = time_now;
+			node->send_count++;
+
+			const char *dup = node->send_count > 1 ? " DUP" : "";
 
 			if (node->ack_status_flags == 0 || node->ack_status_flags == RRR_UDPSTREAM_ASD_ACK_FLAGS_MSG) {
 				// We are missing delivery ACK, re-send message
 				if (buffer_was_full == 0) {
-					RRR_DBG_3("ASD TX MSG %u\n", node->message_id);
+					RRR_DBG_3("ASD TX %" PRIu32 ":%" PRIu32 " MSG%s\n",
+						session->connect_handle, node->message_id, dup);
 					ret = __rrr_udpstream_asd_send_message(session, node);
-					node->ack_status_flags |= RRR_UDPSTREAM_ASD_ACK_FLAGS_MSG;
-					if (ret == RRR_UDPSTREAM_ASD_BUFFER_FULL) {
+					if (ret == RRR_UDPSTREAM_ASD_NOT_READY) {
 						RRR_DBG_3("Buffer full while sending message in rrr_udpstream_asd_do_send_tasks\n");
 						buffer_was_full = 1;
 						ret = 0;
 					}
+					else {
+						node->ack_status_flags |= RRR_UDPSTREAM_ASD_ACK_FLAGS_MSG;
+					}
 				}
 			}
 			else if ((node->ack_status_flags & RRR_UDPSTREAM_ASD_ACK_FLAGS_DACK) != 0) {
-				RRR_DBG_3("ASD TX RACK %u\n", node->message_id);
+				RRR_DBG_3("ASD TX %" PRIu32 ":%" PRIu32 " RACK%s\n",
+						session->connect_handle, node->message_id, dup);
 
 				// We are missing complete ACK, re-send release ACK
 				ret = __rrr_udpstream_asd_send_control_message(
@@ -815,7 +808,7 @@ static int __rrr_udpstream_asd_do_send_tasks (struct rrr_udpstream_asd *session)
 			}
 
 			if (ret != 0) {
-				RRR_DBG_1("Error while sending message A in rrr_udpstream_asd_do_send_tasks return was %i\n", ret);
+				RRR_DBG_3("Error while sending message A in rrr_udpstream_asd_do_send_tasks return was %i\n", ret);
 				goto out;
 			}
 		}
@@ -898,10 +891,12 @@ int rrr_udpstream_asd_default_allocator (
 }
 */
 
-static int __rrr_udpstream_asd_receive_messages_callback_final (struct rrr_msg_msg **message, void *arg) {
+static int __rrr_udpstream_asd_receive_messages_callback_final (struct rrr_msg_msg **message, void *arg1, void *arg2) {
+	(void)(arg2);
+
 	int ret = 0;
 
-	struct rrr_asd_receive_messages_callback_data *receive_data = arg;
+	struct rrr_asd_receive_messages_callback_data *receive_data = arg1;
 	struct rrr_udpstream_asd *session = receive_data->session;
 	struct rrr_msg_holder *entry = receive_data->udpstream_receive_data->allocation_handle;
 
@@ -909,7 +904,6 @@ static int __rrr_udpstream_asd_receive_messages_callback_final (struct rrr_msg_m
 	// The entry must be locked already at this location, allocator is responsible for ensuring that
 	// The allocator must unlock the entry after the callback chain is complete
 
-	// TODO : Make this a soft error?
 	if (receive_data->udpstream_receive_data->application_data > 0xffffffff) {
 		RRR_MSG_0("Application data/message ID out of range (%" PRIu64 ") in __rrr_udpstream_asd_receive_messages_callback_final connect handle %" PRIu32 ", message dropped\n",
 				receive_data->udpstream_receive_data->application_data,
@@ -919,8 +913,8 @@ static int __rrr_udpstream_asd_receive_messages_callback_final (struct rrr_msg_m
 		goto out;
 	}
 
-	RRR_DBG_3("ASD %u RECV timestamp %" PRIu64 "\n",
-			session->connect_handle, (*message)->timestamp);
+	RRR_DBG_3("ASD RX %" PRIu32 ":%" PRIu64 " MSG timestamp %" PRIu64 "\n",
+			session->connect_handle, receive_data->udpstream_receive_data->application_data, (*message)->timestamp);
 
 	if ((ret = __rrr_udpstream_asd_queue_collection_incref_and_insert_entry (
 			&session->release_queues,
@@ -944,9 +938,7 @@ static int __rrr_udpstream_asd_receive_messages_callback_final (struct rrr_msg_m
 }
 
 static int __rrr_udpstream_asd_receive_messages_callback (
-		void **joined_data,
-		const struct rrr_udpstream_receive_data *receive_data,
-		void *arg
+		RRR_UDPSTREAM_FINAL_RECEIVE_CALLBACK_ARGS
 ) {
 	struct rrr_asd_receive_messages_callback_data *callback_data = arg;
 
@@ -954,17 +946,24 @@ static int __rrr_udpstream_asd_receive_messages_callback (
 
 	callback_data->udpstream_receive_data = receive_data;
 
-	struct rrr_read_common_receive_message_callback_data socket_callback_data = {
+#if SSIZE_MAX > RRR_LENGTH_MAX
+	if ((rrr_slength) receive_data->data_size > (rrr_slength) RRR_LENGTH_MAX) {
+		RRR_MSG_0("Received message too big in __rrr_udpstream_asd_receive_messages_callback\n");
+		ret = RRR_UDPSTREAM_ASD_HARD_ERR;
+		goto out;
+	}
+#endif
+
+	if ((ret = rrr_msg_to_host_and_verify_with_callback (
+			(struct rrr_msg **) joined_data,
+			receive_data->data_size,
 			__rrr_udpstream_asd_receive_messages_callback_final,
 			NULL,
 			NULL,
-			callback_data
-	};
-
-	if ((ret = rrr_read_common_receive_message_raw_callback (
-			joined_data,
-			receive_data->data_size,
-			&socket_callback_data
+			NULL,
+			NULL,
+			callback_data,
+			NULL
 	)) != 0) {
 		if (ret == RRR_SOCKET_SOFT_ERROR) {
 			RRR_MSG_0("Invalid message received in __rrr_udpstream_asd_receive_messages_callback, application data was %" PRIu64 "\n",
@@ -986,12 +985,7 @@ static int __rrr_udpstream_asd_receive_messages_callback (
 static int __rrr_udpstream_asd_do_receive_tasks (
 		int *receive_count,
 		struct rrr_udpstream_asd *session,
-		int (*allocator_callback) (
-				uint32_t size,
-				int (*receive_callback)(void **joined_data, void *allocation_handle, void *udpstream_callback_arg),
-				void *udpstream_callback_arg,
-				void *arg
-		),
+		int (*allocator_callback)(RRR_UDPSTREAM_ALLOCATOR_CALLBACK_ARGS),
 		void *allocator_callback_arg
 ) {
 	int ret = 0;
@@ -1017,22 +1011,18 @@ static int __rrr_udpstream_asd_do_receive_tasks (
 		}
 	}
 	else {
-		RRR_DBG_1("UDP-stream ASD handle %u release queue is full\n", session->connect_handle);
+		RRR_DBG_3("UDP-stream ASD handle %u release queue is full\n", session->connect_handle);
 	}
 
 	*receive_count = receive_callback_data.count;
 
-	if (*receive_count > 0) {
-		RRR_DBG_2("ASD recv cnt %i\n",
-				*receive_count);
-	}
-
 	out:
 	return ret;
 }
 
 int __rrr_udpstream_asd_queue_deliver_messages (
 		int *delivered_count,
+		struct rrr_udpstream_asd *session,
 		struct rrr_udpstream_asd_queue_new *queue,
 		int (*receive_callback)(struct rrr_msg_holder *message, void *arg),
 		void *receive_callback_arg
@@ -1048,10 +1038,9 @@ int __rrr_udpstream_asd_queue_deliver_messages (
 			}
 
 			struct rrr_msg_holder *message = node->message;
-			node->message = NULL;
 
-			RRR_DBG_3("ASD DELIVER %u timestamp %" PRIu64 ", grace started\n",
-					node->message_id, node->send_time);
+			RRR_DBG_3("ASD D %u:%u MSG timestamp %" PRIu64 ", grace started\n",
+					session->connect_handle, node->message_id, node->send_time);
 
 			// Callback must ALWAYS unlock
 			rrr_msg_holder_lock(message);
@@ -1064,7 +1053,7 @@ int __rrr_udpstream_asd_queue_deliver_messages (
 			rrr_msg_holder_decref(message);
 			node->message = NULL;
 
-			delivered_count++;
+			(*delivered_count)++;
 
 			node->delivered_grace_counter = RRR_UDPSTREAM_ASD_DELIVERY_GRACE_COUNTER;
 		}
@@ -1092,8 +1081,6 @@ int __rrr_udpstream_asd_queue_update_delivery_grace (
 			// entry might be delivered to application again
 			if (node->delivered_grace_counter <= 0) {
 				RRR_LL_ITERATE_SET_DESTROY();
-	//				VL_DEBUG_MSG_3("UDP-stream ASD grace time ended for message %u\n",
-	//						node->message_id);
 			}
 		}
 	RRR_LL_ITERATE_END_CHECK_DESTROY(queue, __rrr_udpstream_asd_queue_entry_destroy(node));
@@ -1115,7 +1102,7 @@ int __rrr_udpstream_asd_queue_regulate_window_size (
 				RRR_UDPSTREAM_ASD_WINDOW_SIZE_REDUCTION_AMOUNT
 		)) != 0) {
 			// Don't produce fatal error here, let something else fail
-			RRR_DBG_1("Error while regulating window size in ASD while delivering messages, return from UDP-stream was %i\n", ret);
+			RRR_DBG_3("Error while regulating window size in ASD while delivering messages, return from UDP-stream was %i\n", ret);
 			ret = 0;
 		}
 	}
@@ -1137,6 +1124,7 @@ static int __rrr_udpstream_asd_deliver_and_maintain_queue (
 	// Deliver messages
 	if ((ret = __rrr_udpstream_asd_queue_deliver_messages (
 			&delivered_messages_count,
+			session,
 			queue,
 			receive_callback,
 			receive_callback_arg
@@ -1201,12 +1189,7 @@ int rrr_udpstream_asd_deliver_and_maintain_queues (
 int rrr_udpstream_asd_buffer_tick (
 		int *receive_count,
 		int *send_count,
-		int (*allocator_callback) (
-				uint32_t size,
-				int (*final_callback)(void **joined_data, void *allocation_handle, void *udpstream_callback_arg),
-				void *udpstream_callback_arg,
-				void *arg
-		),
+		int (*allocator_callback)(RRR_UDPSTREAM_ALLOCATOR_CALLBACK_ARGS),
 		void *allocator_callback_arg,
 		struct rrr_udpstream_asd *session
 ) {
@@ -1216,11 +1199,24 @@ int rrr_udpstream_asd_buffer_tick (
 
 	if ((ret = __rrr_udpstream_asd_buffer_connect_if_needed(session)) != 0) {
 		if (ret == RRR_UDPSTREAM_ASD_NOT_READY) {
-			// Connection not ready yet, this is normal
-			goto out_not_ready;
+			// Connection not ready yet, do read and send tasks
+		}
+		else {
+			RRR_MSG_0("Error from connect_if_needed in ASD connect handle %" PRIu32 "\n", session->connect_handle);
+			goto out;
+		}
+	}
+
+	if ((ret = __rrr_udpstream_asd_do_send_tasks(session)) != 0) {
+		if (ret == RRR_UDPSTREAM_NOT_READY) {
+			// Send buffer full
+		}
+		else {
+			RRR_MSG_0("Error %i from UDP-stream while queuing messages to send of UDP-stream ASD handle %u\n",
+					ret, session->connect_handle);
+			ret = RRR_UDPSTREAM_ASD_HARD_ERR;
+			goto out;
 		}
-		RRR_MSG_0("Error from connect_if_needed in ASD connect handle %" PRIu32 "\n", session->connect_handle);
-		goto out;
 	}
 
 	if ((ret = rrr_udpstream_do_read_tasks(&session->udpstream, __rrr_udpstream_asd_control_frame_listener, session)) != 0) {
@@ -1233,18 +1229,6 @@ int rrr_udpstream_asd_buffer_tick (
 		// Ignore soft error
 	}
 
-	if ((ret = __rrr_udpstream_asd_do_send_tasks(session)) != 0) {
-		if (ret == RRR_UDPSTREAM_NOT_READY) {
-			goto out_not_ready;
-		}
-		else {
-			RRR_MSG_0("Error from UDP-stream while queuing messages to send of UDP-stream ASD handle %u\n",
-					session->connect_handle);
-			ret = 1;
-		}
-		goto out;
-	}
-
 	if (rrr_udpstream_do_send_tasks(send_count, &session->udpstream) != 0) {
 		RRR_MSG_0("UDP-stream send tasks failed in send_packets ASD\n");
 		ret = 1;
@@ -1265,8 +1249,4 @@ int rrr_udpstream_asd_buffer_tick (
 
 	out:
 	return ret;
-
-	out_not_ready:
-	RRR_DBG_1("UDP-stream not ready yet for connect handle %" PRIu32 "\n", session->connect_handle);
-	return 0;
 }
