@@ -2,7 +2,7 @@
 
 Read Route Record
 
-Copyright (C) 2020 Atle Solbakken atle@goliathdns.no
+Copyright (C) 2020-2021 Atle Solbakken atle@goliathdns.no
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -24,11 +24,12 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <sys/types.h>
 
-#define RRR_POSIX_MUTEX_IS_RECURSIVE	(1<<0)
-#define RRR_POSIX_MUTEX_IS_PSHARED		(1<<1)
+#define RRR_POSIX_MUTEX_IS_RECURSIVE    (1<<0)
+#define RRR_POSIX_MUTEX_IS_PSHARED      (1<<1)
+#define RRR_POSIX_MUTEX_IS_ERRORCHECK   (1<<2)
 
 int rrr_posix_usleep(int useconds);
-void *rrr_posix_mmap (size_t size);
+void *rrr_posix_mmap (size_t size, int is_shared);
 int rrr_posix_strcasecmp (const char *a, const char *b);
 int rrr_posix_strncasecmp (const char *a, const char *b, size_t n);
 int rrr_posix_mutex_init (pthread_mutex_t *mutex, int flags);
