@@ -2,7 +2,7 @@
 
 Read Route Record
 
-Copyright (C) 2018-2019 Atle Solbakken atle@goliathdns.no
+Copyright (C) 2018-2021 Atle Solbakken atle@goliathdns.no
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -22,6 +22,13 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #ifndef RRR_MAIN_H
 #define RRR_MAIN_H
 
+#define RRR_ENV_DEBUGLEVEL				"RRR_DEBUGLEVEL"
+#define RRR_ENV_DEBUGLEVEL_ON_EXIT		"RRR_DEBUGLEVEL_ON_EXIT"
+#define RRR_ENV_NO_WATCHDOG_TIMERS		"RRR_NO_WATCHDOG_TIMERS"
+#define RRR_ENV_NO_THREAD_RESTART		"RRR_NO_THREAD_RESTART"
+#define RRR_ENV_LOGLEVEL_TRANSLATION	"RRR_LOGLEVEL_TRANSLATION"
+#define RRR_ENV_RUN_DIRECTORY	"RRR_RUN_DIRECTORY"
+
 #include "lib/cmdlineparser/cmdline_defines.h"
 
 struct cmd_data;
@@ -44,7 +51,7 @@ int rrr_main_create_and_start_threads (
 
 void rrr_main_threads_stop_and_destroy (struct rrr_thread_collection *collection);
 int rrr_main_parse_cmd_arguments_and_env(struct cmd_data *cmd, const char **env, cmd_conf config);
-int rrr_main_print_help_and_version (
+int rrr_main_print_banner_help_and_version (
 		struct cmd_data *cmd,
 		int argc_minimum
 );
