@@ -2,7 +2,7 @@
 
 Read Route Record
 
-Copyright (C) 2018-2020 Atle Solbakken atle@goliathdns.no
+Copyright (C) 2018-2021 Atle Solbakken atle@goliathdns.no
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -33,11 +33,13 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <poll.h>
 
 #include "../lib/log.h"
+#include "../lib/allocator.h"
 
 #include "../lib/mqtt/mqtt_broker.h"
 #include "../lib/mqtt/mqtt_common.h"
 #include "../lib/mqtt/mqtt_session_ram.h"
 #include "../lib/mqtt/mqtt_acl.h"
+#include "../lib/message_broker.h"
 #include "../lib/instance_config.h"
 #include "../lib/settings.h"
 #include "../lib/instances.h"
@@ -112,7 +114,6 @@ static int mqttbroker_data_init (
 	return ret;
 }
 
-// TODO : Provide more configuration arguments
 static int mqttbroker_parse_config (struct mqtt_broker_data *data, struct rrr_instance_config_data *config) {
 	int ret = 0;
 
@@ -161,7 +162,7 @@ static int mqttbroker_parse_config (struct mqtt_broker_data *data, struct rrr_in
 
 	if (data->permission_name == NULL || *(data->permission_name) == '\0') {
 		RRR_FREE_IF_NOT_NULL(data->permission_name);
-		if ((data->permission_name = strdup("mqtt")) == NULL) {
+		if ((data->permission_name = rrr_strdup("mqtt")) == NULL) {
 			RRR_MSG_0("Could not allocate memory for permission name in mqttbroker_parse_config\n");
 			ret = 1;
 			goto out;
@@ -181,7 +182,7 @@ static int mqttbroker_parse_config (struct mqtt_broker_data *data, struct rrr_in
 
 	RRR_INSTANCE_CONFIG_PARSE_OPTIONAL_YESNO("mqtt_broker_v31_disconnect_on_publish_deny", do_disconnect_on_v31_publish_deny, 0);
 
-	if ((rrr_net_transport_config_parse (
+	if ((ret = rrr_net_transport_config_parse (
 			&data->net_transport_config,
 			config,
 			"mqtt_broker",
@@ -270,6 +271,29 @@ static int mqttbroker_parse_acl (struct mqtt_broker_data *data) {
 	return ret;
 }
 
+static int mqttbroker_event_broker_data_available (RRR_EVENT_FUNCTION_ARGS) {
+	(void)(amount);
+	(void)(arg);
+
+	RRR_BUG("BUG: mqttbroker_event_broker_data_available was called\n");
+
+	return 0;
+}
+
+static int mqttbroker_event_periodic (RRR_EVENT_FUNCTION_PERIODIC_ARGS) {
+	struct rrr_thread *thread = arg;
+	struct rrr_instance_runtime_data *thread_data = thread->private_data;
+	struct mqtt_broker_data *data = thread_data->private_data;
+
+	if (rrr_thread_signal_encourage_stop_check_and_update_watchdog_timer(thread) != 0) {
+		return RRR_EVENT_EXIT;
+	}
+
+	mqttbroker_update_stats(data, INSTANCE_D_STATS(thread_data));
+
+	return 0;
+}
+
 static void *thread_entry_mqttbroker (struct rrr_thread *thread) {
 	struct rrr_instance_runtime_data *thread_data = thread->private_data;
 	struct mqtt_broker_data *data = thread_data->private_data = thread_data->private_memory;
@@ -285,9 +309,7 @@ static void *thread_entry_mqttbroker (struct rrr_thread *thread) {
 
 	pthread_cleanup_push(mqttbroker_data_cleanup, data);
 
-	rrr_thread_set_state(thread, RRR_THREAD_STATE_INITIALIZED);
-	rrr_thread_signal_wait(thread, RRR_THREAD_SIGNAL_START);
-	rrr_thread_set_state(thread, RRR_THREAD_STATE_RUNNING);
+	rrr_thread_start_condition_helper_nofork(thread);
 
 	if (mqttbroker_parse_config(data, thread_data->init_data.instance_config) != 0) {
 		RRR_MSG_0("Configuration parse failed for mqtt broker instance '%s'\n", INSTANCE_D_NAME(thread_data));
@@ -311,6 +333,7 @@ static void *thread_entry_mqttbroker (struct rrr_thread *thread) {
 	if (rrr_mqtt_broker_new (
 			&data->mqtt_broker_data,
 			&init_data,
+			INSTANCE_D_EVENTS(thread_data),
 			data->max_keep_alive,
 			data->password_file,
 			data->permission_name,
@@ -329,9 +352,6 @@ static void *thread_entry_mqttbroker (struct rrr_thread *thread) {
 
 	RRR_DBG_1 ("mqtt broker started thread %p\n", thread_data);
 
-	int listen_handle_plain = 0;
-	int listen_handle_tls = 0;
-
 	if (data->do_transport_plain) {
 		RRR_DBG_1("MQTT broker instance %s starting plain listening on port %" PRIrrrbl "\n",
 				INSTANCE_D_NAME(thread_data), data->server_port_plain);
@@ -347,11 +367,9 @@ static void *thread_entry_mqttbroker (struct rrr_thread *thread) {
 			RRR_NET_TRANSPORT_PLAIN
 		};
 
-		// In case transport type is set to BOTH, we must reset
 		if (rrr_mqtt_broker_listen_ipv4_and_ipv6 (
-				&listen_handle_plain,
 				data->mqtt_broker_data,
-				&net_transport_config_tmp,
+				&net_transport_config_tmp, // <-- Pass in *temporary* struct
 				data->server_port_plain
 		) != 0) {
 			RRR_MSG_0("Could not start plain network transport in mqtt broker instance %s\n",
@@ -359,18 +377,18 @@ static void *thread_entry_mqttbroker (struct rrr_thread *thread) {
 			goto out_destroy_broker;
 		}
 	}
+
 	if (data->do_transport_tls) {
 		RRR_DBG_1("MQTT broker instance %s starting TLS listening on port %" PRIrrrbl "\n",
 				INSTANCE_D_NAME(thread_data), data->server_port_tls);
 
-		// In case transport type is set to BOTH, we set it to TLS
 		struct rrr_net_transport_config net_transport_config_tmp = data->net_transport_config;
 
-		// Only change temporary struct
+		// In case transport type is set to BOTH, we set it to TLS. Do not modify the
+		// original struct, only this temporary one.
 		net_transport_config_tmp.transport_type = RRR_NET_TRANSPORT_TLS;
 
 		if (rrr_mqtt_broker_listen_ipv4_and_ipv6 (
-				&listen_handle_plain,
 				data->mqtt_broker_data,
 				&net_transport_config_tmp, // <-- Pass in *temporary* struct
 				data->server_port_tls
@@ -381,51 +399,12 @@ static void *thread_entry_mqttbroker (struct rrr_thread *thread) {
 		}
 	}
 
-	// DO NOT use signed, let it overflow
-	unsigned long int consecutive_nothing_happened = 0;
-
-	uint64_t prev_stats_time = rrr_time_get_64();
-	while (rrr_thread_check_encourage_stop(thread) != 1) {
-		uint64_t time_now = rrr_time_get_64();
-		rrr_thread_update_watchdog_time(thread);
-
-		int plain_something_happened = 0;
-		int tls_something_happened = 0;
-
-		if (listen_handle_plain) {
-			if (rrr_mqtt_broker_synchronized_tick(&plain_something_happened, data->mqtt_broker_data, listen_handle_plain) != 0) {
-				RRR_MSG_0("Error from MQTT broker while running plain transport tasks\n");
-				break;
-			}
-		}
-		if (listen_handle_tls) {
-			if (rrr_mqtt_broker_synchronized_tick(&tls_something_happened, data->mqtt_broker_data, listen_handle_tls) != 0) {
-				RRR_MSG_0("Error from MQTT broker while running TLS transport tasks\n");
-				break;
-			}
-		}
-
-		if (plain_something_happened + tls_something_happened == 0) {
-			consecutive_nothing_happened++;
-		}
-		else {
-			consecutive_nothing_happened = 0;
-		}
-
-		if (consecutive_nothing_happened > 5000) {
-//			printf("Broker long sleep %lu\n", consecutive_nothing_happened);
-			rrr_posix_usleep(50000); // 50 ms
-		}
-		if (consecutive_nothing_happened > 50) {
-//			printf("Broker short sleep %lu\n", consecutive_nothing_happened);
-			rrr_posix_usleep(2000); // 2ms
-		}
-
-		if (time_now > (prev_stats_time + RRR_MQTT_CLIENT_STATS_INTERVAL_MS * 1000)) {
-			mqttbroker_update_stats(data, INSTANCE_D_STATS(thread_data));
-			prev_stats_time = rrr_time_get_64();
-		}
-	}
+	rrr_event_dispatch (
+			INSTANCE_D_EVENTS(thread_data),
+			1 * 1000 * 1000,
+			mqttbroker_event_periodic,
+			thread
+	);
 
 	// If clients run on the same machine, we hope they close the connection first
 	// to await TCP timeout
@@ -449,6 +428,10 @@ static struct rrr_module_operations module_operations = {
 		NULL
 };
 
+struct rrr_instance_event_functions event_functions = {
+	mqttbroker_event_broker_data_available
+};
+
 static const char *module_name = "mqtt_broker";
 
 __attribute__((constructor)) void load(void) {
@@ -459,11 +442,9 @@ void init(struct rrr_instance_module_data *data) {
 	data->module_name = module_name;
 	data->type = RRR_MODULE_TYPE_NETWORK;
 	data->operations = module_operations;
-	data->dl_ptr = NULL;
-	data->start_priority = RRR_THREAD_START_PRIORITY_NETWORK;
+	data->event_functions = event_functions;
 }
 
 void unload(void) {
 	RRR_DBG_1 ("Destroy mqtt broker module\n");
 }
-
