@@ -30,6 +30,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <stdlib.h>
 
 #include "../lib/log.h"
+#include "../lib/allocator.h"
 
 #include "../lib/rrr_strerror.h"
 #include "../lib/poll_helper.h"
@@ -41,6 +42,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../lib/ip/ip.h"
 #include "../lib/cmodule/cmodule_helper.h"
 #include "../lib/cmodule/cmodule_main.h"
+#include "../lib/cmodule/cmodule_worker.h"
+#include "../lib/cmodule/cmodule_config_data.h"
 #include "../lib/stats/stats_instance.h"
 #include "../lib/util/macro_utils.h"
 
@@ -136,7 +139,7 @@ static int __cmodule_load (
 		struct cmodule_run_data *run_data,
 		struct cmodule_data *data
 ) {
-	struct rrr_cmodule_config_data *cmodule_config_data = &(INSTANCE_D_CMODULE(data->thread_data)->config_data);
+	const struct rrr_cmodule_config_data *cmodule_config_data = rrr_cmodule_helper_config_data_get(data->thread_data);
 
 	int ret = 1; // Default is NOT OK
 
@@ -239,7 +242,9 @@ static int cmodule_init_wrapper_callback (RRR_CMODULE_INIT_WRAPPER_CALLBACK_ARGS
 			configuration_callback,
 			&run_data,
 			process_callback,
-			&run_data
+			&run_data,
+			custom_tick_callback,
+			custom_tick_callback_arg
 	)) != 0) {
 		RRR_MSG_0("Error from worker loop in __rrr_cmodule_worker_loop_init_wrapper_default\n");
 		// Don't goto out, run cleanup functions
@@ -321,6 +326,43 @@ static int cmodule_process_callback (RRR_CMODULE_PROCESS_CALLBACK_ARGS) {
 	return ret;
 }
 
+struct cmodule_fork_callback_data {
+	struct rrr_instance_runtime_data *thread_data;
+};
+
+static int cmodule_fork (void *arg) {
+	struct cmodule_fork_callback_data *callback_data = arg;
+	struct rrr_instance_runtime_data *thread_data = callback_data->thread_data;
+	struct cmodule_data *data = thread_data->private_data;
+
+	int ret = 0;
+
+	if (cmodule_parse_config(data, thread_data->init_data.instance_config) != 0) {
+		ret = 1;
+		goto out;
+	}
+	if (rrr_cmodule_helper_parse_config(thread_data, "cmodule", "function") != 0) {
+		ret = 1;
+		goto out;
+	}
+
+	if (rrr_cmodule_helper_worker_forks_start (
+			thread_data,
+			cmodule_init_wrapper_callback,
+			data,
+			cmodule_configuration_callback,
+			NULL, // <-- in the init wrapper, this callback arg is set to child_data
+			cmodule_process_callback,
+			NULL  // <-- in the init wrapper, this callback is set to child_data
+	) != 0) {
+		RRR_MSG_0("Error while starting cmodule worker fork for instance %s\n", INSTANCE_D_NAME(thread_data));
+		ret = 1;
+		goto out;
+	}
+	out:
+	return ret;
+}
+
 static void *thread_entry_cmodule (struct rrr_thread *thread) {
 	struct rrr_instance_runtime_data *thread_data = thread->private_data;
 	struct cmodule_data *data = thread_data->private_data = thread_data->private_memory;
@@ -334,48 +376,25 @@ static void *thread_entry_cmodule (struct rrr_thread *thread) {
 
 	pthread_cleanup_push(cmodule_data_cleanup, data);
 
-	rrr_thread_set_state(thread, RRR_THREAD_STATE_INITIALIZED);
-	rrr_thread_signal_wait(thread, RRR_THREAD_SIGNAL_START);
-	rrr_thread_set_state(thread, RRR_THREAD_STATE_RUNNING);
+	struct cmodule_fork_callback_data fork_callback_data = {
+		thread_data
+	};
 
-	if (cmodule_parse_config(data, thread_data->init_data.instance_config) != 0) {
-		goto out_message;
-	}
-	if (rrr_cmodule_helper_parse_config(thread_data, "cmodule", "function") != 0) {
+	if (rrr_thread_start_condition_helper_fork(thread, cmodule_fork, &fork_callback_data) != 0) {
 		goto out_message;
 	}
 
-	pid_t fork_pid;
-
-	if (rrr_cmodule_helper_start_worker_fork (
-			&fork_pid,
-			thread_data,
-			cmodule_init_wrapper_callback,
-			data,
-			cmodule_configuration_callback,
-			NULL, // <-- in the init wrapper, this callback arg is set to child_data
-			cmodule_process_callback,
-			NULL  // <-- in the init wrapper, this callback is set to child_data
-	) != 0) {
-		RRR_MSG_0("Error while starting cmodule worker fork for instance %s\n", INSTANCE_D_NAME(thread_data));
-		goto out_message;
-	}
-
-	rrr_thread_set_state(thread, RRR_THREAD_STATE_RUNNING_FORKED);
-
-	RRR_DBG_1 ("cmodule instance %s started thread %p fork handler ptr %p\n",
-			INSTANCE_D_NAME(thread_data), thread_data, INSTANCE_D_CMODULE(thread_data)->fork_handler);
+	RRR_DBG_1 ("cmodule instance %s started thread %p\n",
+			INSTANCE_D_NAME(thread_data), thread_data);
 
 	rrr_cmodule_helper_loop (
 			thread_data,
-			INSTANCE_D_STATS(thread_data),
-			&thread_data->poll,
-			fork_pid
+			1 * 1000 * 1000 // 1 s
 	);
 
 	out_message:
-	RRR_DBG_1 ("cmodule instance %s stopping thread %p fork handler ptr %p\n",
-			INSTANCE_D_NAME(thread_data), thread_data, INSTANCE_D_CMODULE(thread_data)->fork_handler);
+	RRR_DBG_1 ("cmodule instance %s stopping thread %p\n",
+			INSTANCE_D_NAME(thread_data), thread_data);
 
 	pthread_cleanup_pop(1);
 	pthread_exit(0);
@@ -399,8 +418,7 @@ void init(struct rrr_instance_module_data *data) {
 	data->module_name = module_name;
 	data->type = RRR_MODULE_TYPE_FLEXIBLE;
 	data->operations = module_operations;
-	data->start_priority = RRR_THREAD_START_PRIORITY_FORK;
-	data->dl_ptr = NULL;
+	data->event_functions = rrr_cmodule_helper_event_functions;
 }
 
 void unload(void) {
