@@ -34,6 +34,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <netinet/in.h>
 
 #include "../lib/log.h"
+#include "../lib/allocator.h"
 
 #include "../lib/instance_config.h"
 #include "../lib/instances.h"
@@ -68,8 +69,10 @@ struct ipclient_data {
 	struct rrr_instance_runtime_data *thread_data;
 
 	uint32_t client_number;
-	int disallow_remote_ip_swap;
-	int listen;
+
+	int do_disallow_remote_ip_swap;
+	int do_ipv4_only;
+	int do_listen;
 
 	uint64_t total_poll_count;
 	uint64_t total_queued_count;
@@ -154,8 +157,9 @@ int parse_config (struct ipclient_data *data, struct rrr_instance_config_data *c
 		goto out;
 	}
 
-	RRR_INSTANCE_CONFIG_PARSE_OPTIONAL_YESNO("ipclient_disallow_remote_ip_swap", disallow_remote_ip_swap, 0);
-	RRR_INSTANCE_CONFIG_PARSE_OPTIONAL_YESNO("ipclient_listen", listen, 0);
+	RRR_INSTANCE_CONFIG_PARSE_OPTIONAL_YESNO("ipclient_disallow_remote_ip_swap", do_disallow_remote_ip_swap, 0);
+	RRR_INSTANCE_CONFIG_PARSE_OPTIONAL_YESNO("ipclient_listen", do_listen, 0);
+	RRR_INSTANCE_CONFIG_PARSE_OPTIONAL_YESNO("ipclient_ipv4_only", do_ipv4_only, 0);
 
 	// Reset any NOT_FOUND
 	ret = 0;
@@ -170,15 +174,11 @@ static int poll_callback (RRR_MODULE_POLL_CALLBACK_SIGNATURE) {
 
 	struct rrr_msg_msg *message = entry->message;
 
-	RRR_DBG_3 ("ipclient instance %s: Result from buffer timestamp %" PRIu64 "\n",
+	RRR_DBG_2 ("ipclient instance %s: Result from buffer timestamp %" PRIu64 "\n",
 			INSTANCE_D_NAME(thread_data), message->timestamp);
 
 	rrr_msg_holder_incref_while_locked(entry);
 	RRR_LL_APPEND(&private_data->send_queue_intermediate, entry);
-	RRR_LL_VERIFY_HEAD(&private_data->send_queue_intermediate);
-	RRR_LL_ITERATE_BEGIN(&private_data->send_queue_intermediate, struct rrr_msg_holder);
-		RRR_LL_VERIFY_NODE(&private_data->send_queue_intermediate);
-	RRR_LL_ITERATE_END();
 
 	rrr_msg_holder_unlock(entry);
 	return 0;
@@ -195,12 +195,14 @@ static int receive_messages_callback_final(struct rrr_msg_holder *entry, void *a
 
 	int ret = 0;
 
+	rrr_thread_watchdog_time_update(INSTANCE_D_THREAD(data->thread_data));
+
 	// The allocator function below ensures that the entries we receive here are not dirty,
 	// all writing to it was performed while the locks were held
 	if ((ret = rrr_message_broker_incref_and_write_entry_unsafe_no_unlock (
-			INSTANCE_D_BROKER(data->thread_data),
-			INSTANCE_D_HANDLE(data->thread_data),
-			entry
+			INSTANCE_D_BROKER_ARGS(data->thread_data),
+			entry,
+			INSTANCE_D_CANCEL_CHECK_ARGS(data->thread_data)
 	)) != 0) {
 		RRR_MSG_0("Error while writing to output buffer in ipclient instance %s\n",
 				INSTANCE_D_NAME(data->thread_data));
@@ -260,10 +262,7 @@ int queue_message_callback (struct rrr_msg_holder *entry, struct ipclient_data *
 	int ret = 0;
 
 	if ((ret = rrr_udpstream_asd_queue_and_incref_message(ipclient_data->udpstream_asd, entry)) != 0) {
-		if (ret == RRR_UDPSTREAM_ASD_BUFFER_FULL) {
-/*			RRR_DEBUG_MSG_2("ASD-buffer full for ipclient instance %s\n",
-					INSTANCE_D_NAME(ipclient_data->thread_data));*/
-			ret = 0;
+		if (ret == RRR_UDPSTREAM_ASD_NOT_READY) {
 			goto out;
 		}
 		else {
@@ -291,6 +290,7 @@ int queue_or_delete_messages(int *send_count, struct ipclient_data *data) {
 
 		if ((ret = data->queue_method(node, data)) != 0) {
 			rrr_msg_holder_unlock(node);
+			ret &= ~(RRR_UDPSTREAM_ASD_NOT_READY);
 			RRR_LL_ITERATE_BREAK();
 		}
 
@@ -300,7 +300,7 @@ int queue_or_delete_messages(int *send_count, struct ipclient_data *data) {
 	RRR_LL_ITERATE_END_CHECK_DESTROY(&data->send_queue_intermediate, 0; rrr_msg_holder_decref_while_locked_and_unlock(node));
 
 	if ((*send_count) > 0) {
-		RRR_DBG_3 ("ipclient instance %s queued %i packets for transmission\n",
+		RRR_DBG_2 ("ipclient instance %s queued %i packets for transmission\n",
 				INSTANCE_D_NAME(data->thread_data), (*send_count));
 	}
 
@@ -321,8 +321,9 @@ static int __ipclient_asd_reconnect (struct ipclient_data *data) {
 			data->ip_default_remote,
 			data->ip_default_remote_port,
 			data->client_number,
-			data->listen,
-			data->disallow_remote_ip_swap
+			data->do_listen,
+			data->do_disallow_remote_ip_swap,
+			data->do_ipv4_only
 	)) != 0) {
 		RRR_MSG_0("Could not initialize ASD in ipclient instance %s\n", INSTANCE_D_NAME(data->thread_data));
 		ret = 1;
@@ -336,7 +337,9 @@ static int __ipclient_asd_reconnect (struct ipclient_data *data) {
 struct ipclient_udpstream_allocator_callback_data {
 	struct ipclient_data *data;
 	uint32_t size;
-	int (*callback)(void **joined_data, void *allocation_handle, void *udpstream_callback_data);
+	const struct sockaddr *remote_addr;
+	socklen_t remote_addr_len;
+	int (*callback)(RRR_UDPSTREAM_RECEIVE_CALLBACK_ARGS);
 	void *udpstream_callback_data;
 };
 
@@ -352,14 +355,24 @@ static int ipclient_udpstream_allocator_intermediate (void *arg1, void *arg2) {
 	// Points to data inside entry, not to be freed except from when entry is destroyed
 	void *joined_data = NULL;
 
-	if (rrr_msg_holder_util_new_with_empty_message(&entry, callback_data->size, NULL, 0, 0) != 0) {
+	if (rrr_msg_holder_util_new_with_empty_message (
+			&entry,
+			callback_data->size,
+			callback_data->remote_addr,
+			callback_data->remote_addr_len,
+			RRR_IP_UDP
+	) != 0) {
 		RRR_MSG_0("Could not allocate entry in ipclient_udpstream_allocator_intermediate\n");
 		ret = 1;
-		goto out;
+		goto out_no_unlock;
 	}
 
 	rrr_msg_holder_lock(entry);
 
+	pthread_cleanup_push(rrr_msg_holder_decref_while_locked_and_unlock_void, entry);
+
+	// The innermost callback will set joined_data when it has successfully
+	// filled a message into the entry, we use this for bugchecking
 	joined_data = entry->message;
 
 	if ((ret = callback_data->callback(&joined_data, entry, callback_data->udpstream_callback_data)) != 0) {
@@ -377,30 +390,32 @@ static int ipclient_udpstream_allocator_intermediate (void *arg1, void *arg2) {
 		}
 		ret = RRR_FIFO_GLOBAL_ERR;
 	out:
-		rrr_msg_holder_decref_while_locked_and_unlock(entry);
+		pthread_cleanup_pop(1);
 		if (joined_data != NULL && ret == 0) {
 			RRR_BUG("Callback returned non-error but still did not set joined_data to NULL in ipclient_udpstream_allocator_intermediate\n");
 		}
+	out_no_unlock:
 		return ret;
 }
 
 static int ipclient_udpstream_allocator (
-		uint32_t size,
-		int (*callback)(void **joined_data, void *allocation_handle, void *udpstream_callback_data),
-		void *udpstream_callback_data,
-		void *arg
+		RRR_UDPSTREAM_ALLOCATOR_CALLBACK_ARGS
 ) {
 	struct ipclient_data *data = arg;
 
 	int ret = 0;
 
 	struct ipclient_udpstream_allocator_callback_data callback_data = {
-			data, size, callback, udpstream_callback_data
+			data,
+			size,
+			remote_addr,
+			remote_addr_len,
+			receive_callback,
+			udpstream_callback_arg
 	};
 
 	if ((ret = rrr_message_broker_with_ctx_and_buffer_lock_do (
-			INSTANCE_D_BROKER(data->thread_data),
-			INSTANCE_D_HANDLE(data->thread_data),
+			INSTANCE_D_BROKER_ARGS(data->thread_data),
 			ipclient_udpstream_allocator_intermediate,
 			&callback_data,
 			NULL
@@ -428,9 +443,7 @@ static void *thread_entry_ipclient (struct rrr_thread *thread) {
 	pthread_cleanup_push(data_cleanup, data);
 //	pthread_cleanup_push(rrr_thread_set_stopping, thread);
 
-	rrr_thread_set_state(thread, RRR_THREAD_STATE_INITIALIZED);
-	rrr_thread_signal_wait(thread, RRR_THREAD_SIGNAL_START);
-	rrr_thread_set_state(thread, RRR_THREAD_STATE_RUNNING);
+	rrr_thread_start_condition_helper_nofork(thread);
 
 	if (parse_config(data, thread_data->init_data.instance_config) != 0) {
 		RRR_MSG_0("Configuration parse failed for ipclient instance %s\n", thread_data->init_data.module->instance_name);
@@ -439,12 +452,12 @@ static void *thread_entry_ipclient (struct rrr_thread *thread) {
 
 	rrr_instance_config_check_all_settings_used(thread_data->init_data.instance_config);
 
-	int no_polling = rrr_poll_collection_count(&thread_data->poll) > 0 ? 0 : 1;
+	int no_polling = rrr_message_broker_senders_count(INSTANCE_D_BROKER_ARGS(thread_data)) > 0 ? 0 : 1;
 
 	RRR_DBG_1 ("ipclient instance %s started thread %p\n", INSTANCE_D_NAME(thread_data), thread_data);
 
 	network_restart:
-	RRR_DBG_2 ("ipclient instance %s restarting network\n", INSTANCE_D_NAME(thread_data));
+	RRR_DBG_1 ("ipclient instance %s restarting network\n", INSTANCE_D_NAME(thread_data));
 
 	// TODO : Does the following comment still apply?
 	//     Only close here and not when shutting down the thread (might cause
@@ -461,30 +474,26 @@ static void *thread_entry_ipclient (struct rrr_thread *thread) {
 	int queued_total = 0;
 	int send_total = 0;
 	int delivered_total = 0;
-	while (rrr_thread_check_encourage_stop(thread) != 1) {
-		rrr_thread_update_watchdog_time(thread);
+	while (rrr_thread_signal_encourage_stop_check(thread) == 0) {
+		rrr_thread_watchdog_time_update(thread);
 
 		time_now = rrr_time_get_64();
 
 		uint64_t poll_timeout = time_now + 100 * 1000; // 100ms
-		do {
-			if (rrr_poll_do_poll_delete (thread_data, &thread_data->poll, poll_callback, 25) != 0) {
+		while ( RRR_LL_COUNT(&data->send_queue_intermediate) < RRR_IPCLIENT_SEND_BUFFER_INTERMEDIATE_MAX &&
+		        rrr_time_get_64() < poll_timeout &&
+		        no_polling == 0
+		) {
+			uint16_t amount = 100;
+			if (rrr_poll_do_poll_delete (&amount, thread_data, poll_callback, 25) != 0) {
 				RRR_MSG_0("Error while polling in ipclient instance %s\n",
 						INSTANCE_D_NAME(thread_data));
 				break;
 			}
-		} while (RRR_LL_COUNT(&data->send_queue_intermediate) < RRR_IPCLIENT_SEND_BUFFER_INTERMEDIATE_MAX &&
-				rrr_time_get_64() < poll_timeout &&
-				no_polling == 0
-		);
-	//			RRR_DEBUG_MSG_2("ipclient instance %s receive buffer size %i\n",
-	//					INSTANCE_D_NAME(thread_data), send_buffer_size_after);
-
-//		RRR_DEBUG_MSG_2("ipclient instance %s receive\n",
-//				INSTANCE_D_NAME(thread_data));
+		}
 
 		int queue_count = 0;
-		rrr_thread_update_watchdog_time(thread);
+		rrr_thread_watchdog_time_update(thread);
 		if (queue_or_delete_messages(&queue_count, data) != 0) {
 			rrr_posix_usleep (10000); // 10 ms
 			goto network_restart;
@@ -509,13 +518,12 @@ static void *thread_entry_ipclient (struct rrr_thread *thread) {
 		receive_total += receive_count;
 
 		if (receive_count == 0 && send_count == 0) {
-			if (consecutive_zero_recv_and_send > 1000) {
+			if (consecutive_zero_recv_and_send > 1000 && RRR_LL_COUNT(&data->send_queue_intermediate) < RRR_IPCLIENT_SEND_BUFFER_INTERMEDIATE_MAX) {
 /*				RRR_DEBUG_MSG_2("ipclient instance %s long sleep send buffer %i\n",
 						INSTANCE_D_NAME(thread_data), fifo_buffer_get_entry_count(&data->send_queue_intermediate));*/
 				rrr_posix_usleep (100000); // 100 ms
 			}
 			else {
-				sched_yield();
 				if (consecutive_zero_recv_and_send++ > 10) {
 					rrr_posix_usleep(10);
 				}
@@ -603,8 +611,6 @@ void init(struct rrr_instance_module_data *data) {
 	data->module_name = module_name;
 	data->type = RRR_MODULE_TYPE_FLEXIBLE;
 	data->operations = module_operations;
-	data->dl_ptr = NULL;
-	data->start_priority = RRR_THREAD_START_PRIORITY_NETWORK;
 }
 
 void unload(void) {
