@@ -19,6 +19,9 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 */
 
+#undef __USE_GNU
+#include <stdio.h>
+
 // Allow u_int which being used when including Perl.h
 #undef __BSD_VISIBLE
 #define __BSD_VISIBLE 1
@@ -32,6 +35,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <stdlib.h>
 
 #include "../lib/log.h"
+#include "../lib/allocator.h"
 
 #include "../lib/instance_config.h"
 #include "../lib/instances.h"
@@ -51,7 +55,9 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../lib/messages/msg_log.h"
 #include "../lib/cmodule/cmodule_helper.h"
 #include "../lib/cmodule/cmodule_main.h"
+#include "../lib/cmodule/cmodule_worker.h"
 #include "../lib/cmodule/cmodule_ext.h"
+#include "../lib/cmodule/cmodule_config_data.h"
 #include "../lib/ip/ip.h"
 #include "../lib/message_holder/message_holder.h"
 #include "../lib/util/gnu.h"
@@ -86,15 +92,14 @@ struct perl5_child_data {
 };
 
 static int xsub_send_message (
-		struct rrr_msg_msg *message,
+		const struct rrr_msg_msg *message,
 		const struct rrr_msg_addr *message_addr,
 		void *private_data
 ) {
 	struct perl5_child_data *child_data = private_data;
 	int ret = 0;
 
-	// Always frees message
-	if ((ret = rrr_cmodule_ext_send_message_to_parent(
+	if ((ret = rrr_cmodule_ext_send_message_to_parent (
 			child_data->worker,
 			message,
 			message_addr
@@ -111,7 +116,7 @@ static int xsub_send_message (
 
 static char *xsub_get_setting(const char *key, void *private_data) {
 	struct perl5_child_data *perl5_child_data = private_data;
-	struct rrr_instance_settings *settings = perl5_child_data->worker->settings;
+	struct rrr_instance_settings *settings = rrr_cmodule_worker_get_settings(perl5_child_data->worker);
 
 	char *value = NULL;
 	if (rrr_settings_get_string_noconvert_silent(&value, settings, key)) {
@@ -125,7 +130,7 @@ static char *xsub_get_setting(const char *key, void *private_data) {
 
 static int xsub_set_setting(const char *key, const char *value, void *private_data) {
 	struct perl5_child_data *perl5_child_data = private_data;
-	struct rrr_instance_settings *settings = perl5_child_data->worker->settings;
+	struct rrr_instance_settings *settings = rrr_cmodule_worker_get_settings(perl5_child_data->worker);
 
 	int ret = rrr_settings_replace_string(settings, key, value);
 	if (ret != 0) {
@@ -272,9 +277,11 @@ static int perl5_init_wrapper_callback (RRR_CMODULE_INIT_WRAPPER_CALLBACK_ARGS)
 			configuration_callback,
 			&child_data,
 			process_callback,
-			&child_data
+			&child_data,
+			custom_tick_callback,
+			custom_tick_callback_arg
 	)) != 0) {
-		RRR_MSG_0("Error from worker loop in __rrr_cmodule_worker_loop_init_wrapper_default\n");
+		RRR_MSG_0("Error from worker loop in perl5_init_wrapper_callback\n");
 		// Don't goto out, run cleanup functions
 	}
 
@@ -294,17 +301,17 @@ static int perl5_configuration_callback (RRR_CMODULE_CONFIGURATION_CALLBACK_ARGS
 
 	struct perl5_child_data *child_data = private_arg;
 	struct perl5_data *data = child_data->parent_data;
-	struct rrr_cmodule_config_data *cmodule_config_data = &(INSTANCE_D_CMODULE(data->thread_data)->config_data);
+	const struct rrr_cmodule_config_data *cmodule_config_data = rrr_cmodule_helper_config_data_get(data->thread_data);
 
 	struct rrr_instance_settings *settings = data->thread_data->init_data.instance_config->settings;
 	struct rrr_perl5_settings_hv *settings_hv = NULL;
 
 	if (cmodule_config_data->config_function == NULL || *(cmodule_config_data->config_function) == '\0') {
-		RRR_DBG_2("Perl5 instance %s no configure sub defined in configuration\n", INSTANCE_D_NAME(data->thread_data));
+		RRR_DBG_1("Perl5 instance %s no configure sub defined in configuration\n", INSTANCE_D_NAME(data->thread_data));
 		goto out;
 	}
 
-	RRR_DBG_2("Perl5 configuring, sub is %s\n", cmodule_config_data->config_function);
+	RRR_DBG_1("Perl5 configuring, sub is %s\n", cmodule_config_data->config_function);
 
 	if (rrr_perl5_settings_to_hv(&settings_hv, child_data->ctx, settings) != 0) {
 		RRR_MSG_0("Could not convert settings of perl5 instance %s to hash value\n",
@@ -341,7 +348,7 @@ static int perl5_process_callback (RRR_CMODULE_PROCESS_CALLBACK_ARGS) {
 	struct perl5_child_data *child_data = private_arg;
 	struct perl5_data *data = child_data->parent_data;
 	struct rrr_perl5_ctx *ctx = child_data->ctx;
-	struct rrr_cmodule_config_data *cmodule_config_data = &(INSTANCE_D_CMODULE(data->thread_data)->config_data);
+	const struct rrr_cmodule_config_data *cmodule_config_data = rrr_cmodule_helper_config_data_get(data->thread_data);
 
 	struct rrr_perl5_message_hv *hv_message = NULL;
 	struct rrr_msg_addr addr_msg_tmp = *message_addr;
@@ -362,11 +369,9 @@ static int perl5_process_callback (RRR_CMODULE_PROCESS_CALLBACK_ARGS) {
 	}
 
 	if (is_spawn_ctx) {
-		RRR_DBG_3("Perl5 spawning, sub is %s\n", cmodule_config_data->source_function);
 		ret = rrr_perl5_call_blessed_hvref(ctx, cmodule_config_data->source_function, "rrr::rrr_helper::rrr_message", hv_message->hv);
 	}
 	else {
-		RRR_DBG_3("Perl5 processing, sub is %s\n", cmodule_config_data->process_function);
 		ret = rrr_perl5_call_blessed_hvref(ctx, cmodule_config_data->process_function, "rrr::rrr_helper::rrr_message", hv_message->hv);
 	}
 
@@ -383,6 +388,44 @@ static int perl5_process_callback (RRR_CMODULE_PROCESS_CALLBACK_ARGS) {
 	return ret;
 }
 
+struct perl5_fork_callback_data {
+	struct rrr_instance_runtime_data *thread_data;
+};
+
+static int perl5_fork (void *arg) {
+	struct perl5_fork_callback_data *callback_data = arg;
+	struct rrr_instance_runtime_data *thread_data = callback_data->thread_data;
+	struct perl5_data *data = thread_data->private_data;
+
+	int ret = 0;
+
+	if (parse_config(data, thread_data->init_data.instance_config) != 0) {
+		ret = 1;
+		goto out;
+	}
+	if (rrr_cmodule_helper_parse_config(thread_data, "perl5", "sub") != 0) {
+		ret = 1;
+		goto out;
+	}
+
+	if (rrr_cmodule_helper_worker_forks_start (
+			thread_data,
+			perl5_init_wrapper_callback,
+			data,
+			perl5_configuration_callback,
+			NULL, // <-- in the init wrapper, this callback arg is set to child_data
+			perl5_process_callback,
+			NULL  // <-- in the init wrapper, this callback is set to child_data
+	) != 0) {
+		RRR_MSG_0("Error while starting perl5 worker fork for instance %s\n", INSTANCE_D_NAME(thread_data));
+		ret = 1;
+		goto out;
+	}
+
+	out:
+	return ret;
+}
+
 static void *thread_entry_perl5(struct rrr_thread *thread) {
 	struct rrr_instance_runtime_data *thread_data = thread->private_data;
 	struct perl5_data *data = thread_data->private_data = thread_data->private_memory;
@@ -394,42 +437,19 @@ static void *thread_entry_perl5(struct rrr_thread *thread) {
 
 	pthread_cleanup_push(data_cleanup, data);
 
-	rrr_thread_set_state(thread, RRR_THREAD_STATE_INITIALIZED);
-	rrr_thread_signal_wait(thread, RRR_THREAD_SIGNAL_START);
-	rrr_thread_set_state(thread, RRR_THREAD_STATE_RUNNING);
+	struct perl5_fork_callback_data fork_callback_data = {
+		thread_data
+	};
 
-	if (parse_config(data, thread_data->init_data.instance_config) != 0) {
+	if (rrr_thread_start_condition_helper_fork(thread, perl5_fork, &fork_callback_data) != 0) {
 		goto out_message;
 	}
-	if (rrr_cmodule_helper_parse_config(thread_data, "perl5", "sub") != 0) {
-		goto out_message;
-	}
-
-	pid_t fork_pid = 0;
-
-	if (rrr_cmodule_helper_start_worker_fork (
-			&fork_pid,
-			thread_data,
-			perl5_init_wrapper_callback,
-			data,
-			perl5_configuration_callback,
-			NULL, // <-- in the init wrapper, this callback arg is set to child_data
-			perl5_process_callback,
-			NULL  // <-- in the init wrapper, this callback is set to child_data
-	) != 0) {
-		RRR_MSG_0("Error while starting perl5 worker fork for instance %s\n", INSTANCE_D_NAME(thread_data));
-		goto out_message;
-	}
-
-	rrr_thread_set_state(thread, RRR_THREAD_STATE_RUNNING_FORKED);
 
 	RRR_DBG_1 ("perl5 instance %s started thread %p\n", INSTANCE_D_NAME(thread_data), thread_data);
 
 	rrr_cmodule_helper_loop (
 			thread_data,
-			INSTANCE_D_STATS(thread_data),
-			&thread_data->poll,
-			fork_pid
+			1 * 1000 * 1000 // 1 s
 	);
 
 	out_message:
@@ -458,8 +478,7 @@ void init(struct rrr_instance_module_data *data) {
 	data->module_name = module_name;
 	data->type = RRR_MODULE_TYPE_FLEXIBLE;
 	data->operations = module_operations;
-	data->start_priority = RRR_THREAD_START_PRIORITY_FORK;
-	data->dl_ptr = NULL;
+	data->event_functions = rrr_cmodule_helper_event_functions;
 }
 
 void unload(void) {
