@@ -30,6 +30,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <signal.h>
 
 #include "../lib/log.h"
+#include "../lib/allocator.h"
 #include "../lib/poll_helper.h"
 #include "../lib/instance_config.h"
 #include "../lib/instances.h"
@@ -38,12 +39,14 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../lib/array.h"
 #include "../lib/python3/python3_common.h"
 #include "../lib/python3/python3_config.h"
-#include "../lib/python3/python3_vl_message.h"
+#include "../lib/python3/python3_message.h"
 #include "../lib/python3/python3_cmodule.h"
 #include "../lib/messages/msg_msg.h"
 #include "../lib/messages/msg_addr.h"
 #include "../lib/cmodule/cmodule_helper.h"
 #include "../lib/cmodule/cmodule_main.h"
+#include "../lib/cmodule/cmodule_worker.h"
+#include "../lib/cmodule/cmodule_config_data.h"
 #include "../lib/stats/stats_instance.h"
 #include "../lib/message_holder/message_holder.h"
 
@@ -110,7 +113,7 @@ int python3_configuration_callback(RRR_CMODULE_CONFIGURATION_CALLBACK_ARGS) {
 	}
 
 	// NOTE : The python config object operates on the original settings structure
-	config = rrr_python3_config_new (worker->settings);
+	config = rrr_python3_config_new (rrr_cmodule_worker_get_settings(worker));
 
 	if (config == NULL) {
 		RRR_MSG_0("Could not create config object in __rrr_py_persistent_send_config \n");
@@ -142,7 +145,7 @@ int python3_process_callback(RRR_CMODULE_PROCESS_CALLBACK_ARGS) {
 
 	PyObject *arg_message = NULL;
 
-	arg_message = rrr_python3_rrr_msg_msg_new_from_message_and_address(message, (is_spawn_ctx ? NULL : message_addr));
+	arg_message = rrr_python3_rrr_message_new_from_message_and_address(message, (is_spawn_ctx ? NULL : message_addr));
 	if (arg_message == NULL) {
 		RRR_MSG_0("Could not create python3 message in python3_process_callback\n");
 		ret = 1;
@@ -185,7 +188,7 @@ int python3_init_wrapper_callback(RRR_CMODULE_INIT_WRAPPER_CALLBACK_ARGS) {
 	struct python3_data *data = private_arg;
 	struct python3_child_data child_data = {0};
 
-	struct rrr_cmodule_config_data *cmodule_config_data = &(INSTANCE_D_CMODULE(data->thread_data)->config_data);
+	const struct rrr_cmodule_config_data *cmodule_config_data = rrr_cmodule_helper_config_data_get(data->thread_data);
 
 	child_data.parent_data = data;
 
@@ -274,7 +277,9 @@ int python3_init_wrapper_callback(RRR_CMODULE_INIT_WRAPPER_CALLBACK_ARGS) {
 			configuration_callback,
 			&child_data,
 			process_callback,
-			&child_data
+			&child_data,
+			custom_tick_callback,
+			custom_tick_callback_arg
 	)) != 0) {
 		RRR_MSG_0("Error from worker loop in __rrr_cmodule_worker_loop_init_wrapper_default\n");
 		// Don't goto out, run cleanup functions
@@ -294,6 +299,45 @@ int python3_init_wrapper_callback(RRR_CMODULE_INIT_WRAPPER_CALLBACK_ARGS) {
 	return ret;
 }
 
+struct python3_fork_callback_data {
+	struct rrr_instance_runtime_data *thread_data;
+	pid_t *fork_pid;
+};
+
+static int python3_fork (void *arg) {
+	struct python3_fork_callback_data *callback_data = arg;
+	struct rrr_instance_runtime_data *thread_data = callback_data->thread_data;
+	struct python3_data *data = thread_data->private_data;
+
+	int ret = 0;
+
+	if (parse_config(data, thread_data->init_data.instance_config) != 0) {
+		ret = 1;
+		goto out;
+	}
+	if (rrr_cmodule_helper_parse_config(thread_data, "python3", "function") != 0) {
+		ret = 1;
+		goto out;
+	}
+
+	if (rrr_cmodule_helper_worker_forks_start (
+			thread_data,
+			python3_init_wrapper_callback,
+			data,
+			python3_configuration_callback,
+			NULL, // <-- in the init wrapper, this callback arg is set to child_data
+			python3_process_callback,
+			NULL  // <-- in the init wrapper, this callback is set to child_data
+	) != 0) {
+		RRR_MSG_0("Error while starting python3 worker fork for instance %s\n", INSTANCE_D_NAME(thread_data));
+		ret = 1;
+		goto out;
+	}
+
+	out:
+	return ret;
+}
+
 static void *thread_entry_python3 (struct rrr_thread *thread) {
 	struct rrr_instance_runtime_data *thread_data = thread->private_data;
 	struct python3_data *data = thread_data->private_data = thread_data->private_memory;
@@ -309,42 +353,21 @@ static void *thread_entry_python3 (struct rrr_thread *thread) {
 
 	RRR_DBG_1("python3 instance %s\n", INSTANCE_D_NAME(thread_data));
 
-	rrr_thread_set_state(thread, RRR_THREAD_STATE_INITIALIZED);
-	rrr_thread_signal_wait(thread, RRR_THREAD_SIGNAL_START);
-	rrr_thread_set_state(thread, RRR_THREAD_STATE_RUNNING);
-
-	if (parse_config(data, thread_data->init_data.instance_config) != 0) {
-		goto out_message;
-	}
-	if (rrr_cmodule_helper_parse_config(thread_data, "python3", "function") != 0) {
-		goto out_message;
-	}
-
 	pid_t fork_pid = 0;
 
-	if (rrr_cmodule_helper_start_worker_fork (
-			&fork_pid,
-			thread_data,
-			python3_init_wrapper_callback,
-			data,
-			python3_configuration_callback,
-			NULL, // <-- in the init wrapper, this callback arg is set to child_data
-			python3_process_callback,
-			NULL  // <-- in the init wrapper, this callback is set to child_data
-	) != 0) {
-		RRR_MSG_0("Error while starting perl5 worker fork for instance %s\n", INSTANCE_D_NAME(thread_data));
+	struct python3_fork_callback_data fork_callback_data = {
+		thread_data, &fork_pid
+	};
+
+	if (rrr_thread_start_condition_helper_fork(thread, python3_fork, &fork_callback_data) != 0) {
 		goto out_message;
 	}
 
-	rrr_thread_set_state(thread, RRR_THREAD_STATE_RUNNING_FORKED);
-
 	RRR_DBG_1 ("python3 instance %s started thread %p\n", INSTANCE_D_NAME(thread_data), thread_data);
 
 	rrr_cmodule_helper_loop (
 			thread_data,
-			INSTANCE_D_STATS(thread_data),
-			&thread_data->poll,
-			fork_pid
+			1 * 1000 * 1000 // 1 s
 	);
 
 	out_message:
@@ -372,8 +395,7 @@ void init(struct rrr_instance_module_data *data) {
 	data->module_name = module_name;
 	data->type = RRR_MODULE_TYPE_FLEXIBLE;
 	data->operations = module_operations;
-	data->dl_ptr = NULL;
-	data->start_priority = RRR_THREAD_START_PRIORITY_FORK;
+	data->event_functions = rrr_cmodule_helper_event_functions;
 }
 
 void unload(void) {
