@@ -2,7 +2,7 @@
 
 Read Route Record
 
-Copyright (C) 2019-2020 Atle Solbakken atle@goliathdns.no
+Copyright (C) 2019-2021 Atle Solbakken atle@goliathdns.no
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -32,6 +32,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "main.h"
 #include "lib/rrr_config.h"
 #include "lib/log.h"
+#include "lib/allocator.h"
+#include "lib/event/event.h"
 #include "lib/common.h"
 #include "lib/instances.h"
 #include "lib/instance_config.h"
@@ -48,6 +50,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "lib/map.h"
 #include "lib/fork.h"
 #include "lib/rrr_umask.h"
+#include "lib/allocator.h"
+#include "lib/rrr_mmap_stats.h"
 #include "lib/util/rrr_readdir.h"
 
 RRR_CONFIG_DEFINE_DEFAULT_LOG_PREFIX("rrr");
@@ -95,32 +99,67 @@ int rrr_signal_handler(int s, void *arg) {
 }
 
 static const struct cmd_arg_rule cmd_rules[] = {
-		{CMD_ARG_FLAG_NO_FLAG_MULTI,'\0',	"config",				"{CONFIGURATION FILE OR DIRECTORY}"},
-		{0,							'W',	"no_watchdog_timers",	"[-W|--no_watchdog_timers]"},
-		{0,							'T',	"no_thread_restart",	"[-T|--no_thread_restart]"},
-		{0,							's',	"stats",				"[-s|--stats]"},
-// Not implemented (yet). TTL check is present in duplicator and buffer modules
-//		{CMD_ARG_FLAG_HAS_ARGUMENT,	't',	"ttl",					"[-t|--time-to-live]"},
-		{0,							'l',	"loglevel-translation",	"[-l|--loglevel-translation]"},
-		{CMD_ARG_FLAG_HAS_ARGUMENT,	'd',	"debuglevel",			"[-d|--debuglevel[=]DEBUG FLAGS]"},
-		{CMD_ARG_FLAG_HAS_ARGUMENT,	'D',	"debuglevel_on_exit",	"[-D|--debuglevel_on_exit[=]DEBUG FLAGS]"},
-		{0,							'h',	"help",					"[-h|--help]"},
-		{0,							'v',	"version",				"[-v|--version]"},
-		{0,							'\0',	NULL,					NULL}
+		{CMD_ARG_FLAG_NO_FLAG_MULTI,   '\0',   "config",                "{CONFIGURATION FILE OR DIRECTORY}"},
+		{0,                            'W',    "no-watchdog-timers",    "[-W|--no-watchdog-timers]"},
+		{0,                            'T',    "no-thread-restart",     "[-T|--no-thread-restart]"},
+		{0,                            's',    "stats",                 "[-s|--stats]"},
+		{CMD_ARG_FLAG_HAS_ARGUMENT,    'r',    "run-directory",         "[-r|--run-directory[=]RUN DIRECTORY]"},
+		{0,                            'l',    "loglevel-translation",  "[-l|--loglevel-translation]"},
+		{0,                            'b',    "banner",                "[-b|--banner]"},
+		{CMD_ARG_FLAG_HAS_ARGUMENT,    'e',    "environment-file",      "[-e|--environment-file[=]ENVIRONMENT FILE]"},
+		{CMD_ARG_FLAG_HAS_ARGUMENT,    'd',    "debuglevel",            "[-d|--debuglevel[=]DEBUG FLAGS]"},
+		{CMD_ARG_FLAG_HAS_ARGUMENT,    'D',    "debuglevel-on-exit",    "[-D|--debuglevel-on-exit[=]DEBUG FLAGS]"},
+		{0,                            'h',    "help",                  "[-h|--help]"},
+		{0,                            'v',    "version",               "[-v|--version]"},
+		{0,                            'i',    "install-directories",   "[-i|--install-directories]"},
+		{0,                            '\0',    NULL,                   NULL}
 };
 
+#define DUMP_INSTALL_DIRECTORY(name,value) \
+	printf("%s:%s\n", name, value)
+
+void dump_install_directories (void) {
+	DUMP_INSTALL_DIRECTORY("module-dir", RRR_MODULE_PATH);
+	DUMP_INSTALL_DIRECTORY("cmodule-dir", RRR_CMODULE_PATH);
+}
+
 struct stats_data {
 	unsigned int handle;
 	struct rrr_stats_engine engine;
 };
 
-static int main_stats_post_sticky_text_message (struct stats_data *stats_data, const char *path, const char *text) {
-	struct rrr_stats_message message;
+static int main_stats_post_text_message (struct stats_data *stats_data, const char *path, const char *text, uint32_t flags) {
+	struct rrr_msg_stats message;
 
-	if (rrr_stats_message_init (
+	if (rrr_msg_stats_init (
 			&message,
 			RRR_STATS_MESSAGE_TYPE_TEXT,
-			RRR_STATS_MESSAGE_FLAGS_STICKY,
+			flags,
+			path,
+			text,
+			strlen(text) + 1
+	) != 0) {
+		RRR_BUG("Could not initialize main statistics message\n");
+	}
+
+	if (rrr_stats_engine_post_message(&stats_data->engine, stats_data->handle, "main", &message) != 0) {
+		RRR_MSG_0("Could not post main statistics message\n");
+		return EXIT_FAILURE;
+	}
+
+	return EXIT_SUCCESS;
+}
+
+static int main_stats_post_unsigned_message (struct stats_data *stats_data, const char *path, uint64_t value, uint32_t flags) {
+	struct rrr_msg_stats message;
+
+	char text[125];
+	sprintf(text, "%" PRIu64, value);
+
+	if (rrr_msg_stats_init (
+			&message,
+			RRR_STATS_MESSAGE_TYPE_BASE10_TEXT,
+			flags,
 			path,
 			text,
 			strlen(text) + 1
@@ -149,13 +188,13 @@ static int main_stats_post_sticky_messages (struct stats_data *stats_data, struc
 	if (snprintf (
 			msg_text,
 			RRR_STATS_MESSAGE_DATA_MAX_SIZE,
-			"RRR running with %u instances\n",
+			"RRR running with %u instances",
 			rrr_instance_collection_count(instances)
 	) >= RRR_STATS_MESSAGE_DATA_MAX_SIZE) {
 		RRR_BUG("Statistics message too long in main\n");
 	}
 
-	if (main_stats_post_sticky_text_message(stats_data, "status", msg_text) != 0) {
+	if (main_stats_post_text_message(stats_data, "status", msg_text, RRR_STATS_MESSAGE_FLAGS_STICKY) != 0) {
 		ret = EXIT_FAILURE;
 		goto out;
 	}
@@ -166,13 +205,13 @@ static int main_stats_post_sticky_messages (struct stats_data *stats_data, struc
 		char path[128];
 		sprintf(path, "instance_metadata/%u", i);
 
-		if (main_stats_post_sticky_text_message(stats_data, path, instance->module_data->instance_name) != 0) {
+		if (main_stats_post_text_message(stats_data, path, instance->module_data->instance_name, RRR_STATS_MESSAGE_FLAGS_STICKY) != 0) {
 			ret = EXIT_FAILURE;
 			goto out;
 		}
 
 		sprintf(path, "instance_metadata/%u/module", i);
-		if (main_stats_post_sticky_text_message(stats_data, path, instance->module_data->module_name) != 0) {
+		if (main_stats_post_text_message(stats_data, path, instance->module_data->module_name, RRR_STATS_MESSAGE_FLAGS_STICKY) != 0) {
 			ret = EXIT_FAILURE;
 			goto out;
 		}
@@ -180,7 +219,7 @@ static int main_stats_post_sticky_messages (struct stats_data *stats_data, struc
 		unsigned int j = 0;
 		RRR_LL_ITERATE_BEGIN(&instance->senders, struct rrr_instance_friend);
 			sprintf(path, "instance_metadata/%u/senders/%u", i, j);
-			if (main_stats_post_sticky_text_message(stats_data, path, node->instance->module_data->instance_name) != 0) {
+			if (main_stats_post_text_message(stats_data, path, node->instance->module_data->instance_name, RRR_STATS_MESSAGE_FLAGS_STICKY) != 0) {
 				ret = EXIT_FAILURE;
 				goto out;
 			}
@@ -194,6 +233,132 @@ static int main_stats_post_sticky_messages (struct stats_data *stats_data, struc
 	return ret;
 }
 
+struct main_loop_event_callback_data {
+	struct rrr_thread_collection **collection;
+	struct rrr_instance_collection *instances;
+	struct rrr_config *config;
+	struct cmd_data *cmd;
+	struct stats_data *stats_data;
+	struct rrr_message_broker *message_broker;
+	struct rrr_fork_handler *fork_handler;
+	const char *config_file;
+	struct rrr_event_queue *queue;
+};
+
+static void main_loop_close_sockets_except (
+		int stats_socket,
+		struct rrr_event_queue *queue
+) {
+	int fds[RRR_EVENT_QUEUE_FD_MAX + 1];
+	size_t fds_count;
+	rrr_event_queue_fds_get (fds, &fds_count, queue);
+
+	fds[fds_count++] = stats_socket;
+
+	rrr_socket_close_all_except_array_no_unlink (fds, fds_count);
+}
+
+static int main_loop_threads_restart (
+	struct main_loop_event_callback_data *callback_data
+) {
+	int ret = 0;
+
+	main_loop_close_sockets_except (callback_data->stats_data->engine.socket, callback_data->queue);
+
+	if ((ret = rrr_main_create_and_start_threads (
+			callback_data->collection,
+			callback_data->instances,
+			callback_data->config,
+			callback_data->cmd,
+			&callback_data->stats_data->engine,
+			callback_data->message_broker,
+			callback_data->fork_handler
+	)) != 0) {
+		goto out;
+	}
+
+	// This is messy. Handle gets registered inside of main_stats_post_sticky_messages
+	// and then gets unregistered here.
+	if (callback_data->stats_data->handle != 0) {
+		rrr_stats_engine_handle_unregister(&callback_data->stats_data->engine, callback_data->stats_data->handle);
+		callback_data->stats_data->handle = 0;
+	}
+
+	if (callback_data->stats_data->engine.initialized != 0) {
+		if (main_stats_post_sticky_messages(callback_data->stats_data, callback_data->instances) != 0) {
+			ret = 1;
+			goto out;
+		}
+	}
+
+	out:
+	return ret;
+}
+
+static int main_mmap_periodic (struct stats_data *stats_data) {
+	struct rrr_mmap_stats mmap_stats = {0};
+
+	rrr_allocator_maintenance(&mmap_stats);
+
+	int ret = 0;
+
+	if (stats_data != NULL && stats_data->handle != 0) {
+		ret |= main_stats_post_unsigned_message (stats_data, "mmap/count", mmap_stats.mmap_total_count, 0);
+		ret |= main_stats_post_unsigned_message (stats_data, "mmap/empty_count", mmap_stats.mmap_total_empty_count, 0);
+		ret |= main_stats_post_unsigned_message (stats_data, "mmap/bad_count", mmap_stats.mmap_total_bad_count, 0);
+		ret |= main_stats_post_unsigned_message (stats_data, "mmap/heap_size", mmap_stats.mmap_total_heap_size, 0);
+	}
+
+	return ret;
+}
+
+static int main_loop_periodic (RRR_EVENT_FUNCTION_PERIODIC_ARGS) {
+	if (!main_running) {
+		return RRR_EVENT_EXIT;
+	}
+
+	struct main_loop_event_callback_data *callback_data = arg;
+
+	rrr_config_set_debuglevel_orig();
+
+	if (*(callback_data->collection) == NULL) {
+		if (main_loop_threads_restart (callback_data) != 0) {
+			return RRR_EVENT_EXIT;
+		}
+	}
+
+	rrr_fork_handle_sigchld_and_notify_if_needed(callback_data->fork_handler, 0);
+
+	if (rrr_instance_check_threads_stopped(callback_data->instances) == 1) {
+		RRR_DBG_1 ("One or more threads have finished for configuration %s\n", callback_data->config_file);
+
+		rrr_config_set_debuglevel_on_exit();
+		rrr_main_threads_stop_and_destroy(*(callback_data->collection));
+		*(callback_data->collection) = NULL;
+
+		// Allow re-use of costumer names. Any ghosts currently using a handle will be detected
+		// as the handle usercount will be > 1. This handle will not be destroyed untill the
+		// ghost breaks out of it's hanged state. It's nevertheless not possible for anyone else
+		// to find the handle as it will be removed from the costumer handle list.
+		rrr_message_broker_unregister_all(callback_data->message_broker);
+
+		if (main_running && rrr_config_global.no_thread_restart == 0) {
+			rrr_posix_usleep(1000000); // 1s
+		}
+		else {
+			return RRR_EVENT_EXIT;
+		}
+	}
+
+	int count;
+	rrr_thread_cleanup_postponed_run(&count);
+	if (count > 0) {
+		RRR_MSG_0("Main cleaned up after %i ghost(s) (in loop) in configuration %s\n", count, callback_data->config_file);
+	}
+
+	return main_mmap_periodic(callback_data->stats_data);
+}
+
 // We have one loop per fork and one fork per configuration file
 // Parent fork only monitors child forks
 static int main_loop (
@@ -204,7 +369,8 @@ static int main_loop (
 	int ret = EXIT_SUCCESS;
 
 	struct stats_data stats_data = {0};
-	struct rrr_message_broker message_broker = {0};
+	struct rrr_message_broker *message_broker = NULL;
+	struct rrr_event_queue *queue = NULL;
 
 	struct rrr_config *config = NULL;
 	struct rrr_instance_collection instances = {0};
@@ -212,13 +378,18 @@ static int main_loop (
 
 	rrr_config_set_log_prefix(config_file);
 
-	if ((config = rrr_config_parse_file(config_file)) == NULL) {
-		RRR_MSG_0("Configuration file parsing failed for %s\n", config_file);
+	if (rrr_event_queue_new(&queue) != 0) {
 		ret = EXIT_FAILURE;
 		goto out;
 	}
 
-	RRR_DBG_1("RRR found %d instances in configuration file %s\n",
+	if (rrr_config_parse_file(&config, config_file) != 0) {
+		RRR_MSG_0("Configuration file parsing failed for %s\n", config_file);
+		ret = EXIT_FAILURE;
+		goto out_destroy_events;
+	}
+
+	RRR_DBG_1("RRR found %d instances in configuration file '%s'\n",
 			config->module_count, config_file);
 
 	if (RRR_DEBUGLEVEL_1) {
@@ -236,117 +407,73 @@ static int main_loop (
 	}
 
 	if (cmd_exists(cmd, "stats", 0)) {
-		if (rrr_stats_engine_init(&stats_data.engine) != 0) {
+		if (rrr_stats_engine_init(&stats_data.engine, queue) != 0) {
 			RRR_MSG_0("Could not initialize statistics engine\n");
 			ret = EXIT_FAILURE;
 			goto out_destroy_instance_metadata;
 		}
 	}
 
-	if (rrr_message_broker_init(&message_broker) != 0) {
+	if (rrr_message_broker_new(&message_broker) != 0) {
 		ret = EXIT_FAILURE;
-		goto out_destroy_instance_metadata;
+		goto out_destroy_stats_engine;
 	}
 
-	threads_restart:
-
-	rrr_socket_close_all_except(stats_data.engine.socket);
-
-	rrr_config_set_debuglevel_orig();
-	if ((ret = rrr_main_create_and_start_threads (
-			&collection,
-			&instances,
-			config,
-			cmd,
-			&stats_data.engine,
-			&message_broker,
-			fork_handler
-	)) != 0) {
-		goto out_unregister_stats_handle;
-	}
-
-	// This is messy. Handle gets registered inside of main_stats_post_sticky_messages
-	// and then gets unregistered here.
-	if (stats_data.handle != 0) {
-		rrr_stats_engine_handle_unregister(&stats_data.engine, stats_data.handle);
-		stats_data.handle = 0;
-	}
-
-	if (stats_data.engine.initialized != 0) {
-		if (main_stats_post_sticky_messages(&stats_data, &instances) != 0) {
-			goto out_stop_threads;
-		}
-	}
-
-	// Main loop
 	rrr_signal_handler_set_active(RRR_SIGNALS_ACTIVE);
-	while (main_running) {
-		rrr_posix_usleep (250000); // 250ms
 
-		rrr_fork_handle_sigchld_and_notify_if_needed(fork_handler, 0);
+	struct main_loop_event_callback_data event_callback_data = {
+		&collection,
+		&instances,
+		config,
+		cmd,
+		&stats_data,
+		message_broker,
+		fork_handler,
+		config_file,
+		queue
+	};
 
-		if (rrr_instance_check_threads_stopped(&instances) == 1) {
-			RRR_DBG_1 ("One or more threads have finished for configuration %s\n", config_file);
-
-			rrr_config_set_debuglevel_on_exit();
-			rrr_main_threads_stop_and_destroy(collection);
-
-			// Allow re-use of costumer names. Any ghosts currently using a handle will be detected
-			// as the handle usercount will be > 1. This handle will not be destroyed untill the
-			// ghost breaks out of it's hanged state. It's nevertheless not possible for anyone else
-			// to find the handle as it will be removed from the costumer handle list.
-			rrr_message_broker_unregister_all_hard(&message_broker);
-
-			if (main_running && rrr_config_global.no_thread_restart == 0) {
-				rrr_posix_usleep(1000000); // 1s
-				goto threads_restart;
-			}
-			else {
-				goto out_unload_modules;
-			}
-		}
-
-		if (stats_data.engine.initialized != 0) {
-			rrr_stats_engine_tick(&stats_data.engine);
-		}
-
-		int count;
-		rrr_thread_postponed_cleanup_run(&count);
-		if (count > 0) {
-			RRR_MSG_0("Main cleaned up after %i ghost(s) (in loop) in configuration %s\n", count, config_file);
-		}
-	}
+	rrr_event_dispatch (
+			queue,
+			250 * 1000, // 250 ms
+			main_loop_periodic,
+			&event_callback_data
+	);
 
 	RRR_DBG_1 ("Main loop finished\n");
 
-	out_stop_threads:
+	if (collection != NULL) {
 		rrr_main_threads_stop_and_destroy(collection);
-	out_unregister_stats_handle:
-		if (stats_data.handle != 0) {
-			rrr_stats_engine_handle_unregister(&stats_data.engine, stats_data.handle);
-		}
-		rrr_config_set_debuglevel_on_exit();
-		RRR_DBG_1("Debuglevel on exit is: %i\n", rrr_config_global.debuglevel);
-		int count;
+	}
 
-		rrr_thread_postponed_cleanup_run(&count);
-		if (count > 0) {
-			RRR_MSG_0("Main cleaned up after %i ghost(s) (after loop)\n", count);
-		}
-		rrr_socket_close_all();
+	if (stats_data.handle != 0) {
+		rrr_stats_engine_handle_unregister(&stats_data.engine, stats_data.handle);
+	}
+	rrr_config_set_debuglevel_on_exit();
+	RRR_DBG_1("Debuglevel on exit is: %i\n", rrr_config_global.debuglevel);
+	int count;
 
-	out_unload_modules:
-#		ifndef RRR_NO_MODULE_UNLOAD
-			rrr_instance_unload_all(&instances);
-#		endif
+	rrr_thread_cleanup_postponed_run(&count);
+	if (count > 0) {
+		RRR_MSG_0("Main cleaned up after %i ghost(s) (after loop)\n", count);
+	}
+
+#ifndef RRR_NO_MODULE_UNLOAD
+	rrr_instance_unload_all(&instances);
+#endif
+	rrr_message_broker_destroy(message_broker);
+
+	out_destroy_stats_engine:
 		rrr_stats_engine_cleanup(&stats_data.engine);
-		rrr_message_broker_cleanup(&message_broker);
 	out_destroy_instance_metadata:
 		rrr_signal_handler_set_active(RRR_SIGNALS_NOT_ACTIVE);
 		rrr_instance_collection_clear(&instances);
 	out_destroy_config:
 		rrr_config_destroy(config);
+	out_destroy_events:
+		rrr_event_queue_destroy(queue);
 	out:
+		rrr_allocator_cleanup();
 		return ret;
 }
 
@@ -474,6 +601,27 @@ static int get_config_files (struct rrr_map *target, struct cmd_data *cmd) {
 		return ret;
 }
 
+struct main_periodic_callback_data {
+	struct rrr_fork_handler *fork_handler;
+};
+
+static int main_periodic (RRR_EVENT_FUNCTION_PERIODIC_ARGS) {
+	struct main_periodic_callback_data *callback_data = arg;
+
+	if (!main_running) {
+		return RRR_EVENT_EXIT;
+	}
+
+	rrr_fork_handle_sigchld_and_notify_if_needed(callback_data->fork_handler, 0);
+
+	if (some_fork_has_stopped) {
+		RRR_MSG_0("One or more forks has exited\n");
+		return RRR_EVENT_EXIT;
+	}
+
+	return main_mmap_periodic(NULL);
+}
+
 int main (int argc, const char *argv[], const char *env[]) {
 	if (!rrr_verify_library_build_timestamp(RRR_BUILD_TIMESTAMP)) {
 		fprintf(stderr, "Library build version mismatch.\n");
@@ -489,6 +637,8 @@ int main (int argc, const char *argv[], const char *env[]) {
 
 	int is_child = 0;
 
+	struct rrr_event_queue *queue = NULL;
+
 	struct rrr_signal_handler *signal_handler_fork = NULL;
 	struct rrr_signal_handler *signal_handler = NULL;
 
@@ -521,7 +671,11 @@ int main (int argc, const char *argv[], const char *env[]) {
 		goto out_cleanup_signal;
 	}
 
-	if (rrr_main_print_help_and_version(&cmd, 2) != 0) {
+	if (rrr_main_print_banner_help_and_version(&cmd, 2) != 0) {
+		goto out_cleanup_signal;
+	}
+	else if (cmd_exists(&cmd, "install-directories", 0)) {
+		dump_install_directories();
 		goto out_cleanup_signal;
 	}
 
@@ -584,20 +738,31 @@ int main (int argc, const char *argv[], const char *env[]) {
 	RRR_MAP_ITERATE_END();
 
 	rrr_signal_handler_set_active(RRR_SIGNALS_ACTIVE);
-	while (main_running) {
-		rrr_fork_handle_sigchld_and_notify_if_needed(fork_handler, 0);
 
-		if (some_fork_has_stopped) {
-			RRR_MSG_0("One or more forks has exited\n");
-			goto out_cleanup_signal;
-		}
-
-		rrr_posix_usleep(250000); // 250 ms
+	// Create queue after forking to prevent it and it's FDs from existing in the forks
+	if (rrr_event_queue_new(&queue) != 0) {
+		ret = EXIT_FAILURE;
+		goto out_cleanup_signal;
 	}
 
+	struct main_periodic_callback_data callback_data = {
+		fork_handler
+	};
+
+	rrr_event_dispatch (
+			queue,
+			250 * 1000, // 250 ms
+			main_periodic,
+			&callback_data
+	);
+
 	out_cleanup_signal:
 		rrr_signal_handler_set_active(RRR_SIGNALS_NOT_ACTIVE);
 
+		if (queue != NULL) {
+			rrr_event_queue_destroy(queue);
+		}
+
 		rrr_signal_handler_remove(signal_handler);
 		rrr_signal_handler_remove(signal_handler_fork);
 
@@ -615,6 +780,7 @@ int main (int argc, const char *argv[], const char *env[]) {
 
 	out_run_cleanup_methods:
 		rrr_exit_cleanup_methods_run_and_free();
+		rrr_socket_close_all();
 		if (ret == 0) {
 			RRR_MSG_1("Exiting program without errors\n");
 		}
@@ -626,5 +792,6 @@ int main (int argc, const char *argv[], const char *env[]) {
 		rrr_strerror_cleanup();
 		rrr_log_cleanup();
 	out:
+		rrr_allocator_cleanup();
 		return ret;
 }
