@@ -25,6 +25,9 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <fcntl.h>
 #include <termios.h>
 
+#include "lib/log.h"
+#include "lib/allocator.h"
+
 #include "main.h"
 #include "../build_timestamp.h"
 #include "lib/rrr_config.h"
@@ -32,7 +35,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "lib/rrr_strerror.h"
 #include "lib/version.h"
 #include "lib/socket/rrr_socket.h"
-#include "lib/log.h"
 #include "lib/parse.h"
 #include "lib/passwd.h"
 #include "lib/util/linked_list.h"
@@ -47,8 +49,9 @@ static const struct cmd_arg_rule cmd_rules[] = {
 		{CMD_ARG_FLAG_NO_ARGUMENT,	's',	"stdin",				"[-s|--stdin]"},
 		{CMD_ARG_FLAG_HAS_ARGUMENT,	'p',	"permission",			"[-p|--permission[=]PERMISSION]"},
 		{CMD_ARG_FLAG_NO_ARGUMENT,	'l',	"loglevel-translation",	"[-l|--loglevel-translation]"},
+		{CMD_ARG_FLAG_HAS_ARGUMENT,	'e',	"environment-file",		"[-e|--environment-file[=]ENVIRONMENT FILE]"},
 		{CMD_ARG_FLAG_HAS_ARGUMENT,	'd',	"debuglevel",			"[-d|--debuglevel[=]DEBUG FLAGS]"},
-		{CMD_ARG_FLAG_HAS_ARGUMENT,	'D',	"debuglevel_on_exit",	"[-D|--debuglevel_on_exit[=]DEBUG FLAGS]"},
+		{CMD_ARG_FLAG_HAS_ARGUMENT,	'D',	"debuglevel-on-exit",	"[-D|--debuglevel-on-exit[=]DEBUG FLAGS]"},
 		{CMD_ARG_FLAG_NO_ARGUMENT,	'h',	"help",					"[-h|--help]"},
 		{CMD_ARG_FLAG_NO_ARGUMENT,	'v',	"version",				"[-v|--version]"},
 		{0,							'\0',	NULL,					NULL}
@@ -76,7 +79,7 @@ static int __rrr_auth_parse_config (struct rrr_auth_data *data, struct cmd_data
 
 	const char *file = cmd_get_value(cmd, "file", 0);
 	if (file != NULL && *file != '\0') {
-		data->filename = strdup(file);
+		data->filename = rrr_strdup(file);
 		if (data->filename == NULL) {
 			RRR_MSG_0("Could not allocate memory in __rrr_auth_parse_config\n");
 			ret = 1;
@@ -86,7 +89,7 @@ static int __rrr_auth_parse_config (struct rrr_auth_data *data, struct cmd_data
 
 	const char *username = cmd_get_value(cmd, "username", 0);
 	if (username != NULL && *username != '\0') {
-		data->username = strdup(username);
+		data->username = rrr_strdup(username);
 		if (data->username == NULL) {
 			RRR_MSG_0("Could not allocate memory in __rrr_auth_parse_config\n");
 			ret = 1;
@@ -102,7 +105,7 @@ static int __rrr_auth_parse_config (struct rrr_auth_data *data, struct cmd_data
 
 	const char *permission = cmd_get_value(cmd, "permission", 0);
 	if (permission != NULL && *permission != '\0') {
-		data->permission = strdup(permission);
+		data->permission = rrr_strdup(permission);
 		if (data->permission == NULL) {
 			RRR_MSG_0("Could not allocate memory in __rrr_auth_parse_config\n");
 			ret = 1;
@@ -149,7 +152,7 @@ int main (int argc, const char **argv, const char **env) {
 	}
 
 	// Don't require arguments here, separate check in parse_config
-	if (rrr_main_print_help_and_version(&cmd, 0) != 0) {
+	if (rrr_main_print_banner_help_and_version(&cmd, 0) != 0) {
 		ret = EXIT_FAILURE;
 		goto out;
 	}
@@ -199,5 +202,6 @@ int main (int argc, const char **argv, const char **env) {
 		rrr_strerror_cleanup();
 		rrr_log_cleanup();
 	out_final:
+		rrr_allocator_cleanup();
 		return ret;
 }
