@@ -2,7 +2,7 @@
 
 Read Route Record
 
-Copyright (C) 2020 Atle Solbakken atle@goliathdns.no
+Copyright (C) 2020-2021 Atle Solbakken atle@goliathdns.no
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -33,67 +33,73 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../build_timestamp.h"
 #include "main.h"
 #include "lib/log.h"
+#include "lib/allocator.h"
 #include "lib/cmdlineparser/cmdline.h"
 #include "lib/common.h"
 #include "lib/http/http_server.h"
 #include "lib/net_transport/net_transport_config.h"
 #include "lib/socket/rrr_socket.h"
-#include "lib/threads.h"
 #include "lib/version.h"
 #include "lib/rrr_config.h"
 #include "lib/rrr_strerror.h"
+#include "lib/event/event.h"
 #include "lib/util/macro_utils.h"
 #include "lib/util/rrr_time.h"
 
+#define RRR_HTTP_SERVER_FIRST_DATA_TIMEOUT_MS  3000
+#define RRR_HTTP_SERVER_IDLE_TIMEOUT_MS        RRR_HTTP_SERVER_FIRST_DATA_TIMEOUT_MS * 2
+#define RRR_HTTP_SERVER_SEND_CHUNK_COUNT_LIMIT 100000
+
 RRR_CONFIG_DEFINE_DEFAULT_LOG_PREFIX("rrr_http_server");
 
 static const struct cmd_arg_rule cmd_rules[] = {
-		{CMD_ARG_FLAG_HAS_ARGUMENT,	'p',	"port",					"[-p|--port[=]HTTP PORT]"},
-		{0,							'P',	"plain-disable",		"[-P|--plain-disable]"},
-		{CMD_ARG_FLAG_HAS_ARGUMENT,	's',	"ssl-port",				"[-s|--ssl-port[=]HTTPS PORT]"},
-		{CMD_ARG_FLAG_HAS_ARGUMENT,	'c',	"certificate",			"[-c|--certificate[=]PEM SSL CERTIFICATE]"},
-		{CMD_ARG_FLAG_HAS_ARGUMENT,	'k',	"key",					"[-k|--key[=]PEM SSL PRIVATE KEY]"},
-		{0,							'N',	"no-cert-verify",		"[-N|--no-cert-verify]"},
-		{CMD_ARG_FLAG_HAS_ARGUMENT,	'd',	"debuglevel",			"[-d|--debuglevel[=]DEBUG FLAGS]"},
-		{CMD_ARG_FLAG_HAS_ARGUMENT,	'D',	"debuglevel_on_exit",	"[-D|--debuglevel_on_exit[=]DEBUG FLAGS]"},
-		{0,							'h',	"help",					"[-h|--help]"},
-		{0,							'v',	"version",				"[-v|--version]"},
-		{0,							'\0',	NULL,					NULL}
+        {CMD_ARG_FLAG_HAS_ARGUMENT,    'p',    "port",                  "[-p|--port[=]HTTP PORT]"},
+#if defined(RRR_WITH_OPENSSL) || defined(RRR_WITH_LIBRESSL)
+        {0,                            'P',    "plain-disable",         "[-P|--plain-disable]"},
+        {CMD_ARG_FLAG_HAS_ARGUMENT,    's',    "ssl-port",              "[-s|--ssl-port[=]HTTPS PORT]"},
+        {CMD_ARG_FLAG_HAS_ARGUMENT,    'c',    "certificate",           "[-c|--certificate[=]PEM SSL CERTIFICATE]"},
+        {CMD_ARG_FLAG_HAS_ARGUMENT,    'k',    "key",                   "[-k|--key[=]PEM SSL PRIVATE KEY]"},
+        {0,                            'N',    "no-cert-verify",        "[-N|--no-cert-verify]"},
+#endif
+        {CMD_ARG_FLAG_HAS_ARGUMENT,    'e',    "environment-file",      "[-e|--environment-file[=]ENVIRONMENT FILE]"},
+        {CMD_ARG_FLAG_HAS_ARGUMENT,    'd',    "debuglevel",            "[-d|--debuglevel[=]DEBUG FLAGS]"},
+        {CMD_ARG_FLAG_HAS_ARGUMENT,    'D',    "debuglevel-on-exit",    "[-D|--debuglevel-on-exit[=]DEBUG FLAGS]"},
+        {0,                            'h',    "help",                  "[-h|--help]"},
+        {0,                            'v',    "version",               "[-v|--version]"},
+        {0,                            '\0',    NULL,                   NULL}
 };
 
 struct rrr_http_server_data {
+	uint16_t http_port;
+#if defined(RRR_WITH_OPENSSL) || defined(RRR_WITH_LIBRESSL)
 	char *certificate_file;
 	char *private_key_file;
-	uint16_t http_port;
 	uint16_t https_port;
 	int ssl_no_cert_verify;
 	int plain_disable;
+#endif
 };
 
-/*
-struct rrr_http_server_response {
-	int code;
-	char *argument;
-};
-
-static void __rrr_http_server_response_cleanup (struct rrr_http_server_response *response) {
-	RRR_FREE_IF_NOT_NULL(response->argument);
-	response->code = 0;
-}
-*/
-
 static void __rrr_http_server_data_init (struct rrr_http_server_data *data) {
 	memset (data, '\0', sizeof(*data));
 }
 
 static void __rrr_http_server_data_cleanup (struct rrr_http_server_data *data) {
+#if defined(RRR_WITH_OPENSSL) || defined(RRR_WITH_LIBRESSL)
 	RRR_FREE_IF_NOT_NULL(data->certificate_file);
 	RRR_FREE_IF_NOT_NULL(data->private_key_file);
+#else
+	(void)(data);
+#endif
 }
 
 static int __rrr_http_server_parse_config (struct rrr_http_server_data *data, struct cmd_data *cmd) {
 	int ret = 0;
 
+	uint64_t port_tmp;
+	const char *port;
+
+#if defined(RRR_WITH_OPENSSL) || defined(RRR_WITH_LIBRESSL)
 	// Certificate file
 	const char *certificate = cmd_get_value(cmd, "certificate", 0);
 	if (cmd_get_value (cmd, "certificate", 1) != NULL) {
@@ -102,7 +108,7 @@ static int __rrr_http_server_parse_config (struct rrr_http_server_data *data, st
 		goto out;
 	}
 	if (certificate != NULL) {
-		data->certificate_file = strdup(certificate);
+		data->certificate_file = rrr_strdup(certificate);
 		if (data->certificate_file == NULL) {
 			RRR_MSG_0("Could not allocate memory in __rrr_post_parse_config\n");
 			ret = 1;
@@ -118,7 +124,7 @@ static int __rrr_http_server_parse_config (struct rrr_http_server_data *data, st
 		goto out;
 	}
 	if (key != NULL) {
-		data->private_key_file = strdup(key);
+		data->private_key_file = rrr_strdup(key);
 		if (data->private_key_file == NULL) {
 			RRR_MSG_0("Could not allocate memory in __rrr_post_parse_config\n");
 			ret = 1;
@@ -149,31 +155,6 @@ static int __rrr_http_server_parse_config (struct rrr_http_server_data *data, st
 		goto out;
 	}
 
-	// HTTP port
-	const char *port = cmd_get_value(cmd, "port", 0);
-	uint64_t port_tmp = 0;
-	if (cmd_get_value (cmd, "port", 1) != NULL) {
-		RRR_MSG_0("Error: Only one 'port' argument may be specified\n");
-		ret = 1;
-		goto out;
-	}
-	if (port != NULL) {
-		if (cmd_convert_uint64_10(port, &port_tmp)) {
-			RRR_MSG_0("Could not understand argument 'port', must be and unsigned integer\n");
-			ret = 1;
-			goto out;
-		}
-	}
-	if (port_tmp == 0) {
-		port_tmp = 80;
-	}
-	if (port_tmp < 1 || port_tmp > 65535) {
-		RRR_MSG_0("HTTP port out of range (must be 1-65535, got %" PRIu64 ")\n", port_tmp);
-		ret = 1;
-		goto out;
-	}
-	data->http_port = port_tmp;
-
 	// HTTPS port
 	port = cmd_get_value(cmd, "ssl-port", 0);
 	port_tmp = 0;
@@ -198,6 +179,32 @@ static int __rrr_http_server_parse_config (struct rrr_http_server_data *data, st
 		goto out;
 	}
 	data->https_port = port_tmp;
+#endif
+
+	// HTTP port
+	port = cmd_get_value(cmd, "port", 0);
+	port_tmp = 0;
+	if (cmd_get_value (cmd, "port", 1) != NULL) {
+		RRR_MSG_0("Error: Only one 'port' argument may be specified\n");
+		ret = 1;
+		goto out;
+	}
+	if (port != NULL) {
+		if (cmd_convert_uint64_10(port, &port_tmp)) {
+			RRR_MSG_0("Could not understand argument 'port', must be and unsigned integer\n");
+			ret = 1;
+			goto out;
+		}
+	}
+	if (port_tmp == 0) {
+		port_tmp = 80;
+	}
+	else if (port_tmp > 65535) {
+		RRR_MSG_0("HTTP port out of range (must be 1-65535, got %" PRIu64 ")\n", port_tmp);
+		ret = 1;
+		goto out;
+	}
+	data->http_port = port_tmp;
 
 	out:
 	return ret;
@@ -208,6 +215,14 @@ int rrr_http_server_signal_handler(int s, void *arg) {
 	return rrr_signal_default_handler(&main_running, s, arg);
 }
 
+static int rrr_http_server_event_periodic (RRR_EVENT_FUNCTION_PERIODIC_ARGS) {
+	(void)(arg);
+
+	rrr_allocator_maintenance_nostats();
+
+	return (main_running ? 0 : RRR_EVENT_EXIT);
+}
+
 int main (int argc, const char **argv, const char **env) {
 	if (!rrr_verify_library_build_timestamp(RRR_BUILD_TIMESTAMP)) {
 		fprintf(stderr, "Library build version mismatch.\n");
@@ -221,24 +236,28 @@ int main (int argc, const char **argv, const char **env) {
 	}
 	rrr_strerror_init();
 
-	int count = 0;
 	struct cmd_data cmd;
 	struct rrr_http_server_data data;
 	struct rrr_signal_handler *signal_handler = NULL;
 	struct rrr_http_server *http_server = NULL;
-
+	struct rrr_event_queue *events = NULL;
 
 	cmd_init(&cmd, cmd_rules, argc, argv);
 	__rrr_http_server_data_init(&data);
 
 	signal_handler = rrr_signal_handler_push(rrr_http_server_signal_handler, NULL);
 
+	if (rrr_event_queue_new(&events) != 0) {
+		ret = EXIT_FAILURE;
+		goto out;
+	}
+
 	if (rrr_main_parse_cmd_arguments_and_env(&cmd, env, CMD_CONFIG_DEFAULTS) != 0) {
 		ret = EXIT_FAILURE;
 		goto out;
 	}
 
-	if (rrr_main_print_help_and_version(&cmd, 0) != 0) {
+	if (rrr_main_print_banner_help_and_version(&cmd, 0) != 0) {
 		ret = EXIT_FAILURE;
 		goto out;
 	}
@@ -248,24 +267,41 @@ int main (int argc, const char **argv, const char **env) {
 		goto out;
 	}
 
-	if (rrr_http_server_new(&http_server) != 0) {
+	struct rrr_http_server_callbacks callbacks = {
+			NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL
+	};
+
+	if (rrr_http_server_new (
+			&http_server,
+			0, // Don't disable http2
+			&callbacks
+	) != 0) {
 		ret = EXIT_FAILURE;
 		goto out;
 	}
 
+#if defined(RRR_WITH_OPENSSL) || defined(RRR_WITH_LIBRESSL)
 	int transport_count = 0;
 
 	if (data.plain_disable != 1) {
+#endif
 		if (rrr_http_server_start_plain (
 				http_server,
-				data.http_port
+				events,
+				data.http_port,
+				RRR_HTTP_SERVER_FIRST_DATA_TIMEOUT_MS,
+				RRR_HTTP_SERVER_IDLE_TIMEOUT_MS,
+				RRR_HTTP_SERVER_SEND_CHUNK_COUNT_LIMIT
 		) != 0) {
 			ret = EXIT_FAILURE;
 			goto out;
 		}
+#if defined(RRR_WITH_OPENSSL) || defined(RRR_WITH_LIBRESSL)
 		transport_count++;
 	}
+#endif
 
+#if defined(RRR_WITH_OPENSSL) || defined(RRR_WITH_LIBRESSL)
 	if (data.certificate_file != NULL && data.private_key_file != NULL) {
 		// DO NOT run config cleanup for this, memory managed elsewhere
 		struct rrr_net_transport_config net_transport_config_tls = {
@@ -285,7 +321,11 @@ int main (int argc, const char **argv, const char **env) {
 
 		if (rrr_http_server_start_tls (
 				http_server,
+				events,
 				data.https_port,
+				RRR_HTTP_SERVER_FIRST_DATA_TIMEOUT_MS,
+				RRR_HTTP_SERVER_IDLE_TIMEOUT_MS,
+				RRR_HTTP_SERVER_SEND_CHUNK_COUNT_LIMIT,
 				&net_transport_config_tls,
 				flags
 		) != 0) {
@@ -300,37 +340,15 @@ int main (int argc, const char **argv, const char **env) {
 		ret = EXIT_FAILURE;
 		goto out;
 	}
+#endif
 
 	rrr_signal_default_signal_actions_register();
 
-	uint64_t prev_stats_time = rrr_time_get_64();
-	int accept_count_total = 0;
-
 	rrr_signal_handler_set_active(RRR_SIGNALS_ACTIVE);
-	while (main_running) {
-		// We must do this here, the HTTP server library does not do this
-		// itself as it is also used by RRR modules for which this is performed
-		// by the main process
-		rrr_thread_postponed_cleanup_run(&count);
 
-		int accept_count = 0;
-		if (rrr_http_server_tick(&accept_count, http_server, 5, NULL, NULL, NULL, NULL, NULL, NULL) != 0) {
-			ret = EXIT_FAILURE;
-			break;
-		}
-
-		if (accept_count == 0) {
-			rrr_posix_usleep(10000); // 10 ms
-		}
-		else {
-			accept_count_total += accept_count;
-		}
-
-		if (rrr_time_get_64() > prev_stats_time + 1000000) {
-			RRR_DBG_1("Accepted HTTP connections: %i/s\n", accept_count_total);
-			accept_count_total = 0;
-			prev_stats_time = rrr_time_get_64();
-		}
+	if (rrr_event_dispatch(events, 100000, rrr_http_server_event_periodic, NULL) != 0) {
+		ret = EXIT_FAILURE;
+		goto out;
 	}
 
 	out:
@@ -342,8 +360,9 @@ int main (int argc, const char **argv, const char **env) {
 			http_server = NULL;
 		}
 
-		rrr_thread_postponed_cleanup_run(&count);
-		RRR_DBG_1("Cleaned up after %i ghost threads\n", count);
+		if (events != NULL) {
+			rrr_event_queue_destroy(events);
+		}
 
 		rrr_signal_handler_remove(signal_handler);
 
@@ -357,5 +376,6 @@ int main (int argc, const char **argv, const char **env) {
 		rrr_log_cleanup();
 
 	out_final:
+		rrr_allocator_cleanup();
 		return ret;
 }
