@@ -26,6 +26,9 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <sys/types.h>
 #include <sys/stat.h>
 
+#include "lib/log.h"
+#include "lib/allocator.h"
+
 #include "main.h"
 #include "../build_timestamp.h"
 #include "lib/cmdlineparser/cmdline.h"
@@ -33,7 +36,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "lib/rrr_strerror.h"
 #include "lib/version.h"
 #include "lib/socket/rrr_socket.h"
-#include "lib/log.h"
 #include "lib/parse.h"
 #include "lib/passwd.h"
 #include "lib/util/gnu.h"
@@ -55,8 +57,9 @@ static const struct cmd_arg_rule cmd_rules[] = {
 		 CMD_ARG_FLAG_ALLOW_EMPTY,	'P',	"password",				"[-P|--password[=]PASSWORD]"},
 		{CMD_ARG_FLAG_NO_ARGUMENT,	's',	"stdout",				"[-s|--stdout]"},
 		{CMD_ARG_FLAG_NO_ARGUMENT,	'l',	"loglevel-translation",	"[-l|--loglevel-translation]"},
+		{CMD_ARG_FLAG_HAS_ARGUMENT,	'e',	"environment-file",		"[-e|--environment-file[=]ENVIRONMENT FILE]"},
 		{CMD_ARG_FLAG_HAS_ARGUMENT,	'd',	"debuglevel",			"[-d|--debuglevel[=]DEBUG FLAGS]"},
-		{CMD_ARG_FLAG_HAS_ARGUMENT,	'D',	"debuglevel_on_exit",	"[-D|--debuglevel_on_exit[=]DEBUG FLAGS]"},
+		{CMD_ARG_FLAG_HAS_ARGUMENT,	'D',	"debuglevel-on-exit",	"[-D|--debuglevel-on-exit[=]DEBUG FLAGS]"},
 		{CMD_ARG_FLAG_NO_ARGUMENT,	'h',	"help",					"[-h|--help]"},
 		{CMD_ARG_FLAG_NO_ARGUMENT,	'v',	"version",				"[-v|--version]"},
 		{0,							'\0',	NULL,					NULL}
@@ -129,7 +132,7 @@ static int __rrr_passwd_parse_config (struct rrr_passwd_data *data, struct cmd_d
 
 	const char *file = cmd_get_value(cmd, "file", 0);
 	if (file != NULL && *file != '\0') {
-		data->filename = strdup(file);
+		data->filename = rrr_strdup(file);
 		if (data->filename == NULL) {
 			RRR_MSG_0("Could not allocate memory in __rrr_passwd_parse_config\n");
 			ret = 1;
@@ -139,7 +142,7 @@ static int __rrr_passwd_parse_config (struct rrr_passwd_data *data, struct cmd_d
 
 	const char *username = cmd_get_value(cmd, "username", 0);
 	if (username != NULL && *username != '\0') {
-		data->username = strdup(username);
+		data->username = rrr_strdup(username);
 		if (data->username == NULL) {
 			RRR_MSG_0("Could not allocate memory in __rrr_passwd_parse_config\n");
 			ret = 1;
@@ -150,7 +153,7 @@ static int __rrr_passwd_parse_config (struct rrr_passwd_data *data, struct cmd_d
 	// Empty password allowed
 	const char *password = cmd_get_value(cmd, "password", 0);
 	if (password != NULL) {
-		data->password = strdup(password);
+		data->password = rrr_strdup(password);
 		if (data->password == NULL) {
 			RRR_MSG_0("Could not allocate memory in __rrr_passwd_parse_config\n");
 			ret = 1;
@@ -459,7 +462,7 @@ int main (int argc, const char **argv, const char **env) {
 	}
 
 	// Don't require arguments here, separate check in parse_config
-	if (rrr_main_print_help_and_version(&cmd, 0) != 0) {
+	if (rrr_main_print_banner_help_and_version(&cmd, 0) != 0) {
 		goto out;
 	}
 
@@ -597,5 +600,6 @@ int main (int argc, const char **argv, const char **env) {
 		rrr_strerror_cleanup();
 		rrr_log_cleanup();
 	out_final:
+		rrr_allocator_cleanup();
 		return ret;
 }
