@@ -2,7 +2,7 @@
 
 Read Route Record
 
-Copyright (C) 2019-2020 Atle Solbakken atle@goliathdns.no
+Copyright (C) 2019-2021 Atle Solbakken atle@goliathdns.no
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -27,13 +27,13 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <errno.h>
-#include <lib/read.h>
 #include <signal.h>
 
 #include "main.h"
 #include "../build_timestamp.h"
 #include "lib/rrr_config.h"
 #include "lib/log.h"
+#include "lib/allocator.h"
 #include "lib/version.h"
 #include "lib/cmdlineparser/cmdline.h"
 #include "lib/array.h"
@@ -61,15 +61,17 @@ static const struct cmd_arg_rule cmd_rules[] = {
 		{CMD_ARG_FLAG_HAS_ARGUMENT,	'f',	"file",					"[-f|--file[=]FILENAME|-]"},
 		{CMD_ARG_FLAG_HAS_ARGUMENT |
 		 CMD_ARG_FLAG_SPLIT_COMMA,	'r',	"readings",				"[-r|--readings[=]reading1,reading2,...]"},
-		{CMD_ARG_FLAG_HAS_ARGUMENT,	'a',	"array_definition",		"[-a|--array_definition[=]ARRAY DEFINITION]"},
+		{CMD_ARG_FLAG_HAS_ARGUMENT,	'a',	"array-definition",		"[-a|--array-definition[=]ARRAY DEFINITION]"},
 		{CMD_ARG_FLAG_HAS_ARGUMENT,	'm',	"max-message-size",		"[-m|--max-message-size]"},
 		{CMD_ARG_FLAG_HAS_ARGUMENT,	'c',	"count",				"[-c|--count[=]MAX FILE ELEMENTS]"},
 		{CMD_ARG_FLAG_HAS_ARGUMENT,	't',	"topic",				"[-t|--topic[=]MQTT TOPIC]"},
 		{0,							's',	"sync",					"[-s|--sync]"},
+		{0,							'S',	"strip-separators",		"[-S|--strip-separators]"},
 		{0,							'q',	"quiet",				"[-q|--quiet]"},
 		{0,							'l',	"loglevel-translation",	"[-l|--loglevel-translation]"},
+		{CMD_ARG_FLAG_HAS_ARGUMENT,	'e',	"environment-file",		"[-e|--environment-file[=]ENVIRONMENT FILE]"},
 		{CMD_ARG_FLAG_HAS_ARGUMENT,	'd',	"debuglevel",			"[-d|--debuglevel[=]DEBUG FLAGS]"},
-		{CMD_ARG_FLAG_HAS_ARGUMENT,	'D',	"debuglevel_on_exit",	"[-D|--debuglevel_on_exit[=]DEBUG FLAGS]"},
+		{CMD_ARG_FLAG_HAS_ARGUMENT,	'D',	"debuglevel-on-exit",	"[-D|--debuglevel-on-exit[=]DEBUG FLAGS]"},
 		{0,							'h',	"help",					"[-h|--help]"},
 		{0,							'v',	"version",				"[-v|--version]"},
 		{0,							'\0',	NULL,					NULL}
@@ -94,6 +96,7 @@ struct rrr_post_data {
 	uint64_t elements_count;
 	struct rrr_array_tree *tree;
 
+	int strip_separators;
 	int sync_byte_by_byte;
 	int quiet;
 
@@ -130,7 +133,7 @@ static void __rrr_post_destroy_data (struct rrr_post_data *data) {
 	RRR_FREE_IF_NOT_NULL(data->filename);
 	RRR_FREE_IF_NOT_NULL(data->socket_path);
 	RRR_FREE_IF_NOT_NULL(data->topic);
-	RRR_LL_DESTROY(&data->readings, struct rrr_post_reading, free(node));
+	RRR_LL_DESTROY(&data->readings, struct rrr_post_reading, rrr_free(node));
 	if (data->tree != NULL) {
 		rrr_array_tree_destroy(data->tree);
 	}
@@ -148,7 +151,7 @@ static int __rrr_post_add_readings (struct rrr_post_data *data, struct cmd_data
 						RRR_MSG_0("Error in reading '%s', not an unsigned integer\n", reading);
 						return 1;
 					}
-					struct rrr_post_reading *reading_new = malloc(sizeof(*reading_new));
+					struct rrr_post_reading *reading_new = rrr_allocate(sizeof(*reading_new));
 					if (reading_new == NULL) {
 						RRR_MSG_0("Could not allocate memory in __rrr_post_add_readings\n");
 						return 1;
@@ -181,13 +184,21 @@ static int __rrr_post_parse_config (struct rrr_post_data *data, struct cmd_data
 		goto out;
 	}
 
-	data->socket_path = strdup(socket);
+	data->socket_path = rrr_strdup(socket);
 	if (data->socket_path == NULL) {
 		RRR_MSG_0("Could not allocate memory in __rrr_post_parse_config\n");
 		ret = 1;
 		goto out;
 	}
 
+	// Strip separators
+	if (cmd_exists(cmd, "strip-separators", 0)) {
+		data->strip_separators = 1;
+	}
+	else {
+		data->strip_separators = 0;
+	}
+
 	// Sync byte by byte
 	if (cmd_exists(cmd, "sync", 0)) {
 		data->sync_byte_by_byte = 1;
@@ -212,7 +223,7 @@ static int __rrr_post_parse_config (struct rrr_post_data *data, struct cmd_data
 		goto out;
 	}
 	if (filename != NULL) {
-		data->filename = strdup(filename);
+		data->filename = rrr_strdup(filename);
 		if (data->filename == NULL) {
 			RRR_MSG_0("Could not allocate memory in __rrr_post_parse_config\n");
 			ret = 1;
@@ -227,7 +238,7 @@ static int __rrr_post_parse_config (struct rrr_post_data *data, struct cmd_data
 		goto out;
 	}
 	if (topic != NULL) {
-		data->topic = strdup(topic);
+		data->topic = rrr_strdup(topic);
 		if (data->topic == NULL) {
 			RRR_MSG_0("Could not allocate memory in __rrr_post_parse_config\n");
 			ret = 1;
@@ -256,13 +267,13 @@ static int __rrr_post_parse_config (struct rrr_post_data *data, struct cmd_data
 	}
 
 	// Array definition
-	const char *array_definition = cmd_get_value(cmd, "array_definition", 0);
+	const char *array_definition = cmd_get_value(cmd, "array-definition", 0);
 
 	if (array_definition == NULL || *array_definition == '\0') {
 		array_definition = RRR_POST_DEFAULT_ARRAY_DEFINITION;
 	}
 
-	array_tree_tmp = malloc(strlen(array_definition) + 1 + 1); // plus extra ; plus \0
+	array_tree_tmp = rrr_allocate(strlen(array_definition) + 1 + 1); // plus extra ; plus \0
 	if (array_tree_tmp == NULL) {
 		RRR_MSG_0("Could not allocate temporary arry tree string in parse_config\n");
 		ret = 1;
@@ -277,7 +288,7 @@ static int __rrr_post_parse_config (struct rrr_post_data *data, struct cmd_data
 		goto out;
 	}
 
-	if (cmd_get_value (cmd, "array_definition", 1) != NULL) {
+	if (cmd_get_value (cmd, "array-definition", 1) != NULL) {
 		RRR_MSG_0("Error: Only one array_definition argument may be specified\n");
 		ret = 1;
 		goto out;
@@ -416,6 +427,10 @@ static int __rrr_post_read_callback (struct rrr_read_session *read_session, stru
 
 	struct rrr_msg_msg *message = NULL;
 
+	if (data->strip_separators) {
+		rrr_array_strip_type(array_final, &rrr_type_definition_sep);
+	}
+
 	if ((ret = rrr_array_new_message_from_collection (
 			&message,
 			array_final,
@@ -491,6 +506,8 @@ static int __rrr_post_read (struct rrr_post_data *data) {
 				data->tree,
 				data->sync_byte_by_byte,
 				4096,
+				0, // No ratelimit interval
+				0, // No ratelimit max bytes
 				data->max_message_size,
 				__rrr_post_read_callback,
 				&callback_data
@@ -541,7 +558,7 @@ int main (int argc, const char **argv, const char **env) {
 		goto out;
 	}
 
-	if (rrr_main_print_help_and_version(&cmd, 2) != 0) {
+	if (rrr_main_print_banner_help_and_version(&cmd, 2) != 0) {
 		goto out;
 	}
 
@@ -597,5 +614,6 @@ int main (int argc, const char **argv, const char **env) {
 		rrr_strerror_cleanup();
 		rrr_log_cleanup();
 	out_final:
+		rrr_allocator_cleanup();
 		return ret;
 }
