@@ -29,6 +29,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #endif
 
 #include "../../lib/log.h"
+#include "../../lib/allocator.h"
 
 #include "type_array.h"
 #include "../test.h"
@@ -50,6 +51,9 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../../lib/socket/rrr_socket.h"
 #include "../../lib/util/rrr_endian.h"
 
+// Set high to stop test from exiting. Set back to 200 when work is done.
+#define RRR_TEST_TYPE_ARRAY_LOOP_COUNT 200
+
 struct rrr_test_result {
 	int result;
 };
@@ -122,7 +126,6 @@ int test_anything_callback (RRR_MODULE_POLL_CALLBACK_SIGNATURE) {
 
 int test_do_poll_loop (
 		struct rrr_instance *self,
-		struct rrr_instance *output,
 		struct rrr_message_broker *broker,
 		int (*callback)(RRR_MODULE_POLL_CALLBACK_SIGNATURE),
 		struct rrr_test_callback_data *callback_data
@@ -130,36 +133,38 @@ int test_do_poll_loop (
 	int ret = 0;
 
 	struct rrr_test_result *test_result = callback_data->test_result;
-	rrr_message_broker_costumer_handle *handle = NULL;
+
+	struct rrr_message_broker_costumer *handle_self = NULL;
 
 	uint64_t limit = rrr_time_get_64() + 2000000; // 2 seconds (6 zeros)
 
-	while (rrr_time_get_64() < limit) {
-		if ((handle = rrr_message_broker_costumer_find_by_name(broker, INSTANCE_M_NAME(output))) != NULL) {
-			break;
-		}
+	while (rrr_time_get_64() < limit && (handle_self == NULL)) {
+		handle_self = rrr_message_broker_costumer_find_by_name(broker, INSTANCE_M_NAME(self));
 		rrr_posix_usleep(50000);
 	}
 
-	if (handle == NULL) {
-		TEST_MSG("Could not find message broker handle for output '%s' after 2 seconds in test_do_poll_loop\n",
-				INSTANCE_M_NAME(output));
+	if (handle_self == NULL) {
+		TEST_MSG("Could not find message broker handle for self after 2 seconds in test_do_poll_loop\n");
 		ret = 1;
 		goto out;
 	}
 
 	// Poll from output
-	for (int i = 1; i <= 200 && test_result->result != 2; i++) {
-		if (rrr_thread_check_encourage_stop(self->thread) == 1) {
+	for (int i = 1; i <= RRR_TEST_TYPE_ARRAY_LOOP_COUNT && test_result->result != 2; i++) {
+		if (rrr_thread_signal_encourage_stop_check(self->thread) != 0) {
 			break;
 		}
 
-		TEST_MSG("Test result polling from %s try: %i of 200\n",
-				INSTANCE_M_NAME(output), i);
+		rrr_thread_watchdog_time_update(self->thread);
 
+		TEST_MSG("Test result polling try: %i of %i\n",
+				i, RRR_TEST_TYPE_ARRAY_LOOP_COUNT);
+
+		uint16_t amount = 100;
 		ret = rrr_message_broker_poll_delete (
-				broker,
-				handle,
+				&amount,
+				handle_self,
+				0,
 				callback,
 				callback_data,
 				150
@@ -177,8 +182,8 @@ int test_do_poll_loop (
 			test_result->result = 1;
 		}
 		else {
-			TEST_MSG("Result of polling from %s: %i\n",
-					INSTANCE_M_NAME(output), test_result->result);
+			TEST_MSG("Result of polling: %i\n",
+					test_result->result);
 		}
 	}
 
@@ -244,7 +249,8 @@ int test_averager_callback (RRR_MODULE_POLL_CALLBACK_SIGNATURE) {
 	struct rrr_array array_tmp = {0};
 
 	if (MSG_IS_ARRAY(message)) {
-		if (rrr_array_message_append_to_collection(&array_tmp, message) != 0) {
+		uint16_t array_version_dummy;
+		if (rrr_array_message_append_to_collection(&array_version_dummy, &array_tmp, message) != 0) {
 			TEST_MSG("Could not create array collection in test_averager_callback\n");
 			ret = 1;
 			goto out;
@@ -306,6 +312,7 @@ int test_averager (
 		RRR_TEST_FUNCTION_ARGS
 ) {
 	(void)(test_function_data);
+	(void)(instances);
 
 	// Preconditions for this test:
 	// - Sender of the averager module is a voltmonitor module with configuration
@@ -315,32 +322,21 @@ int test_averager (
 
 	struct rrr_test_result test_result = {0};
 
-	struct rrr_instance *output = rrr_instance_find(instances, output_name);
-	if (output == NULL) {
-		TEST_MSG("Could not find output instances %s in test_averager\n",
-				output_name);
-		ret = 1;
-		goto out;
-	}
-
 	struct rrr_test_callback_data callback_data = { &test_result, NULL };
 
 	// Poll from first output
-	TEST_MSG("Polling from %s\n", INSTANCE_M_NAME(output));
 	ret |= test_do_poll_loop(
 			INSTANCE_D_INSTANCE(self_thread_data),
-			output,
 			INSTANCE_D_BROKER(self_thread_data),
 			test_averager_callback,
 			&callback_data
 	);
 	TEST_MSG("Result of test_averager, should be 2: %i\n", test_result.result);
 
-	out:
 	return ret;
 }
 
-#define TEST_DATA_ELEMENTS 12
+#define TEST_DATA_ELEMENTS 13
 
 struct rrr_test_type_array_callback_data {
 	const struct rrr_test_function_data *config;
@@ -373,19 +369,6 @@ int test_type_array_callback (RRR_MODULE_POLL_CALLBACK_SIGNATURE) {
 
 	TEST_MSG("Received a message in test_type_array_callback of class %" PRIu32 "\n", MSG_CLASS(message));
 
-	if (RRR_DEBUGLEVEL_3) {
-// TODO : Needs to be put in a buffer then written out
-/*		RRR_DBG("dump message: 0x");
-		for (unsigned int i = 0; i < MSG_TOTAL_SIZE(message); i++) {
-			char c = ((char*)message)[i];
-			if (c < 0x10) {
-				RRR_DBG("0");
-			}
-			RRR_DBG("%x", c);
-		}
-		RRR_DBG("\n");*/
-	}
-
 	if (!MSG_IS_ARRAY(message)) {
 		// Ignore non-array messages
 		TEST_MSG("Message received in test_type_array_callback was not an array, ignoring\n");
@@ -394,7 +377,8 @@ int test_type_array_callback (RRR_MODULE_POLL_CALLBACK_SIGNATURE) {
 		goto out;
 	}
 
-	if (rrr_array_message_append_to_collection(&collection, message) != 0) {
+	uint16_t array_version_dummy;
+	if (rrr_array_message_append_to_collection(&array_version_dummy, &collection, message) != 0) {
 		TEST_MSG("Error while parsing message from output function in test_type_array_callback\n");
 		ret = 1;
 		goto out;
@@ -411,7 +395,7 @@ int test_type_array_callback (RRR_MODULE_POLL_CALLBACK_SIGNATURE) {
 		final_length += node->total_stored_length;
 	RRR_LL_ITERATE_END();
 
-	const struct rrr_type_value *types[12];
+	const struct rrr_type_value *types[13];
 
 	// After the array has been assembled and then disassembled again, all
 	// short numbers become full length 8 bytes numbers
@@ -433,6 +417,8 @@ int test_type_array_callback (RRR_MODULE_POLL_CALLBACK_SIGNATURE) {
 
 	types[11] = rrr_array_value_get_by_index(&collection, 11);
 
+	types[12] = rrr_array_value_get_by_index(&collection, 12);
+
 	// In some tests chains, the integers become text strings
 	if (config->do_array_str_to_h_conversion) {
 		for (int i = 0; i < 9; i++) {
@@ -527,6 +513,9 @@ int test_type_array_callback (RRR_MODULE_POLL_CALLBACK_SIGNATURE) {
 	for (int i = 11; i < 12; i++) {
 		TEST_MSG("Type %i: %u (%s)\n", i, types[i]->definition->type, (types[i]->definition->type == RRR_TYPE_MSG ? "OK" : "NOT OK"));
 	}
+	for (int i = 12; i < 13; i++) {
+		TEST_MSG("Type %i: %u (%s)\n", i, types[i]->definition->type, (types[i]->definition->type == RRR_TYPE_STR ? "OK" : "NOT OK"));
+	}
 
 	if (!RRR_TYPE_IS_64(types[0]->definition->type) ||
 		!RRR_TYPE_IS_64(types[1]->definition->type) ||
@@ -539,14 +528,15 @@ int test_type_array_callback (RRR_MODULE_POLL_CALLBACK_SIGNATURE) {
 		!RRR_TYPE_IS_64(types[8]->definition->type) ||
 
 		!RRR_TYPE_IS_BLOB(types[10]->definition->type) ||
-		types[11]->definition->type != RRR_TYPE_MSG
+		types[11]->definition->type != RRR_TYPE_MSG ||
+		types[12]->definition->type != RRR_TYPE_STR
 	) {
 		TEST_MSG("Wrong types in collection in test_type_array_callback\n");
 		ret = 1;
 		goto out;
 	}
 
-	final_data_raw = malloc(sizeof(*final_data_raw));
+	final_data_raw = rrr_allocate(sizeof(*final_data_raw));
 
 	memset(final_data_raw, '\0', sizeof(*final_data_raw));
 
@@ -596,24 +586,21 @@ int test_type_array_callback (RRR_MODULE_POLL_CALLBACK_SIGNATURE) {
 		goto out;
 	}
 
+	if (types[12]->total_stored_length != 0) {
+		char *tmp = NULL;
+		types[12]->definition->to_str(&tmp, types[12]);
+		RRR_MSG_0("Returned empty string was not empty, value was '%s'\n",
+			(tmp != NULL ? tmp : "(conversion failed)")
+		);
+		ret = 1;
+		goto out;
+	}
+
 	strcpy(final_data_raw->blob_a, blob_a);
 	strcpy(final_data_raw->blob_b, blob_b);
 
 	memcpy (&final_data_raw->msg, types[11]->data, types[11]->total_stored_length);
 
-	if (RRR_DEBUGLEVEL_3) {
-		// TODO : This needs to be put in a buffer then written out
-/*		RRR_DBG("dump final_data_raw: 0x");
-		for (unsigned int i = 0; i < sizeof(*final_data_raw); i++) {
-			char c = ((char*)final_data_raw)[i];
-			if (c < 0x10) {
-				RRR_DBG("0");
-			}
-			RRR_DBG("%x", c);
-		}
-		RRR_DBG("\n");*/
-	}
-
 	if (final_data_raw->le1 == final_data_raw->le3 ||
 		final_data_raw->le3 == final_data_raw->le4 ||
 		final_data_raw->le4 == final_data_raw->le1
@@ -667,25 +654,18 @@ int test_type_array_callback (RRR_MODULE_POLL_CALLBACK_SIGNATURE) {
 int test_array (
 		RRR_TEST_FUNCTION_ARGS
 ) {
+	(void)(instances);
+
 	int ret = 0;
 
 	struct rrr_test_result test_result_1 = {1};
 
-	struct rrr_instance *output_1 = rrr_instance_find(instances, output_name);
-	if (output_1 == NULL) {
-		TEST_MSG("Could not find output instance %s in test_type_array\n",
-				output_name);
-		return 1;
-	}
-
 	struct rrr_test_type_array_callback_data array_callback_data = { test_function_data };
 	struct rrr_test_callback_data callback_data = { &test_result_1, &array_callback_data };
 
 	// Poll from first output
-	TEST_MSG("Polling from %s\n", INSTANCE_M_NAME(output_1));
 	ret |= test_do_poll_loop(
 			INSTANCE_D_INSTANCE(self_thread_data),
-			output_1,
 			INSTANCE_D_BROKER(self_thread_data),
 			test_type_array_callback,
 			&callback_data
@@ -707,25 +687,17 @@ int test_anything (
 		RRR_TEST_FUNCTION_ARGS
 ) {
 	(void)(test_function_data);
+	(void)(instances);
 
 	int ret = 0;
 
 	struct rrr_test_result test_result_1 = {1};
 
-	struct rrr_instance *output_1 = rrr_instance_find(instances, output_name);
-	if (output_1 == NULL) {
-		TEST_MSG("Could not find output instance %s in test_type_array\n",
-				output_name);
-		return 1;
-	}
-
 	struct rrr_test_callback_data callback_data = { &test_result_1, NULL };
 
 	// Poll from first output
-	TEST_MSG("Polling from %s\n", INSTANCE_M_NAME(output_1));
 	ret |= test_do_poll_loop(
 			INSTANCE_D_INSTANCE(self_thread_data),
-			output_1,
 			INSTANCE_D_BROKER(self_thread_data),
 			test_anything_callback,
 			&callback_data
@@ -749,7 +721,7 @@ int test_type_array_mysql_and_network_callback (RRR_MODULE_POLL_CALLBACK_SIGNATU
 
 	int ret = 0;
 
-	RRR_DBG_4("Received message_1 in test_type_array_mysql_and_network_callback\n");
+	RRR_DBG_2("Received message_1 in test_type_array_mysql_and_network_callback\n");
 
 	test_result->result = 2;
 
@@ -879,15 +851,6 @@ int test_type_array_mysql (
 	struct test_type_array_mysql_data mysql_data = {NULL, NULL, NULL, NULL, 0};
 	struct rrr_msg_holder *entry = NULL;
 
-	struct rrr_instance *tag_buffer = rrr_instance_find(instances, output_name);
-
-	if (tag_buffer == NULL) {
-		TEST_MSG("Could not find output instance %s in test_type_array_mysql_and_network\n",
-				output_name);
-		ret = 1;
-		goto out;
-	}
-
 	struct rrr_instance *mysql = NULL;
 	RRR_LL_ITERATE_BEGIN(instances, struct rrr_instance);
 		struct rrr_instance *instance = node;
@@ -922,7 +885,6 @@ int test_type_array_mysql (
 	TEST_MSG("Polling MySQL\n");
 	ret |= test_do_poll_loop(
 			INSTANCE_D_INSTANCE(self_thread_data),
-			tag_buffer,
 			INSTANCE_D_BROKER(self_thread_data),
 			test_type_array_mysql_and_network_callback,
 			&callback_data
