@@ -44,6 +44,13 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "test_usleep.h"
 #include "test_fixp.h"
 #include "test_inet.h"
+#ifdef RRR_WITH_JSONC
+#	include "test_json.h"
+#endif
+#include "test_conversion.h"
+#include "test_msgdb.h"
+#include "test_nullsafe.h"
+#include "test_increment.h"
 
 RRR_CONFIG_DEFINE_DEFAULT_LOG_PREFIX("test");
 
@@ -97,13 +104,17 @@ int signal_interrupt (int s, void *arg) {
 }
 
 static const struct cmd_arg_rule cmd_rules[] = {
-		{CMD_ARG_FLAG_NO_FLAG,		'\0',	"config",		"{CONFIGURATION FILE}"},
-		{CMD_ARG_FLAG_HAS_ARGUMENT,	'd',	"debuglevel",	"-d|--debuglevel DEBUGLEVEL"},
-		{CMD_ARG_FLAG_NO_ARGUMENT,	'l',	"library-tests","-l|--library-tests"},
-		{0,							'\0',	NULL, 			""}
+        {CMD_ARG_FLAG_NO_FLAG,        '\0',   "config",                "{CONFIGURATION FILE}"},
+        {0,                           'W',    "no-watchdog-timers",    "[-W|--no-watchdog-timers]"},
+        {0,                           'T',    "no-thread-restart",     "[-T|--no-thread-restart]"},
+	{CMD_ARG_FLAG_HAS_ARGUMENT,   'r',    "run-directory",         "[-r|--run-directory[=]RUN DIRECTORY]"},
+        {CMD_ARG_FLAG_HAS_ARGUMENT,   'e',    "environment-file",      "[-e|--environment-file[=]ENVIRONMENT FILE]"},
+        {CMD_ARG_FLAG_HAS_ARGUMENT,   'd',    "debuglevel",            "[-d|--debuglevel DEBUGLEVEL]"},
+        {CMD_ARG_FLAG_NO_ARGUMENT,    'l',    "library-tests",         "[-l|--library-tests]"},
+        {0,                           '\0',    NULL,                   ""}
 };
 
-int rrr_test_library_functions (void) {
+int rrr_test_library_functions (struct rrr_fork_handler *fork_handler) {
 	int ret = 0;
 	int ret_tmp = 0;
 
@@ -133,6 +144,38 @@ int rrr_test_library_functions (void) {
 
 	ret |= ret_tmp;
 
+#ifdef RRR_WITH_JSONC
+	TEST_BEGIN("JSON parsing") {
+		ret_tmp = rrr_test_json();
+	} TEST_RESULT(ret_tmp == 0);
+
+	ret |= ret_tmp;
+#endif
+
+	TEST_BEGIN("type conversion") {
+		ret_tmp = rrr_test_conversion();
+	} TEST_RESULT(ret_tmp == 0);
+
+	ret |= ret_tmp;
+
+	TEST_BEGIN("message database") {
+		ret_tmp = rrr_test_msgdb(fork_handler);
+	} TEST_RESULT(ret_tmp == 0);
+
+	ret |= ret_tmp;
+
+	TEST_BEGIN("nullsafe") {
+		ret_tmp = rrr_test_nullsafe();
+	} TEST_RESULT(ret_tmp == 0);
+
+	ret |= ret_tmp;
+
+	TEST_BEGIN("increment") {
+		ret_tmp = rrr_test_increment();
+	} TEST_RESULT(ret_tmp == 0);
+
+	ret |= ret_tmp;
+
 	return ret;
 }
 
@@ -153,7 +196,7 @@ int main (int argc, const char **argv, const char **env) {
 
 	// TODO : Implement stats engine for test program
 	struct rrr_stats_engine stats_engine = {0};
-	struct rrr_message_broker message_broker = {0};
+	struct rrr_message_broker *message_broker = NULL;
 	struct rrr_config *config = NULL;
 	struct rrr_fork_handler *fork_handler = NULL;
 
@@ -165,7 +208,7 @@ int main (int argc, const char **argv, const char **env) {
 
 	rrr_signal_default_signal_actions_register();
 
-	if (rrr_message_broker_init(&message_broker) != 0) {
+	if (rrr_message_broker_new(&message_broker) != 0) {
 		ret = EXIT_FAILURE;
 		goto out_cleanup_signal;
 	}
@@ -194,7 +237,7 @@ int main (int argc, const char **argv, const char **env) {
 		goto out_cleanup_cmd;
 	}
 
-	if (rrr_main_print_help_and_version(&cmd, 2) != 0) {
+	if (rrr_main_print_banner_help_and_version(&cmd, 2) != 0) {
 		goto out_cleanup_cmd;
 	}
 
@@ -202,7 +245,7 @@ int main (int argc, const char **argv, const char **env) {
 
 	if (cmd_exists(&cmd, "library-tests", 0)) {
 		TEST_MSG("Library tests requested by argument, doing that now.\n");
-		ret = rrr_test_library_functions();
+		ret = rrr_test_library_functions(fork_handler);
 		goto out_cleanup_cmd;
 	}
 
@@ -213,17 +256,9 @@ int main (int argc, const char **argv, const char **env) {
 		goto out_cleanup_cmd;
 	}
 
-	TEST_BEGIN("non-existent config file") {
-		config = rrr_config_parse_file("nonexistent_file");
-	} TEST_RESULT(config == NULL);
-
-	if (config != NULL) {
-		goto out_cleanup_config;
-	}
-
-	TEST_BEGIN("true configuration loading") {
-		config = rrr_config_parse_file(config_file);
-	} TEST_RESULT(config != NULL);
+	TEST_BEGIN("configuration loading") {
+		ret = rrr_config_parse_file(&config, config_file);
+	} TEST_RESULT(ret == 0);
 
 	if (config == NULL) {
 		ret = 1;
@@ -254,7 +289,7 @@ int main (int argc, const char **argv, const char **env) {
 				config,
 				&cmd,
 				&stats_engine,
-				&message_broker,
+				message_broker,
 				fork_handler
 		) != 0) {
 			ret = 1;
@@ -306,7 +341,7 @@ int main (int argc, const char **argv, const char **env) {
 		cmd_destroy(&cmd);
 
 	out_cleanup_message_broker:
-		rrr_message_broker_cleanup(&message_broker);
+		rrr_message_broker_destroy(message_broker);
 
 //	out_cleanup_fork_handler:
 		rrr_fork_send_sigusr1_and_wait(fork_handler);
